-- ---------------------------------------------------------------------------
-- Система:   "Электронный архив"
-- Модуль:    Структуры таблиц базы данных "Электронный архив"
-- Автор:     Моргунов Е. П.
-- Дата:      15.04.2012
-- Версия:    0.6
-- ---------------------------------------------------------------------------

-- ПРИМЕЧАНИЕ. 
-- 1. Типовые фрагменты имен полей:
--    _fname = _full_name
--    _sname = _short_name

-- 2. При работе с таблицами, имеющими так называемые "цепочки" эквивалентных
--    объектов (что объясняется наличием разночтений, написаний на различных
--    языках, переименованиями и т.п.) следует помнить, что при удалении
--    записи, которая является "родоначальницей" цепочки, возможно разрушение
--    цепочки. В настоящее время забота о переадресации указателей
--    (они хранятся в полях с типовым именем *_orig) с удаляемой записи на
--    другую запись лежит на операторе-пользователе. 

-- 3. Для каждой таблицы предусмотрена команда удаления
--                      DROP TABLE <table_name> CASCADE;
--    эти команды закомментированы. Они могут потребоваться, например, при
--    повторном создании таблиц без удаления самой базы данных.

-- 4. Определения таблиц сгруппированы в две группы: справочные таблицы и 
--    основные таблицы. В пределах каждой группы они расположены, как правило,
--    в алфавитном порядке их имен. Отклонения от этого правила объясняются
--    наличием внешних ключей (таблица A, на которую ссылается внешний ключ
--    таблицы B, должна уже быть создана к моменту попытки создания таблицы B).
--    Кроме того, определения нескольких справочных таблиц (например, Persons)
--    помещены в группе основных таблиц по историческим причинам.


-- ---------------------------------------------------------------------------
-- СПРАВОЧНЫЕ ТАБЛИЦЫ
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- Таблица "Языки"
-- ---------------------------------------------------------------------------
-- DROP TABLE Langs CASCADE;

CREATE TABLE Langs                    -- Languages
  ( lang_code int2,                   -- Код языка
    lang_fname text                   -- Наименование языка полное
      UNIQUE NOT NULL,
    lang_sname text                   -- Наименование языка краткое 
      UNIQUE NOT NULL,
    comment text,                     -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,        -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,      -- Когда добавлена (изменена) запись
    PRIMARY KEY ( lang_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Каналы поступления"
-- ---------------------------------------------------------------------------
-- DROP TABLE Acq_ways CASCADE;

CREATE TABLE Acq_ways             -- Acquisition_ways
  ( acq_way_code int2,            -- Код канала поступления
    acq_way_fname text NOT NULL,  -- Наименование канала поступления полное
    acq_way_sname text NOT NULL,  -- Наименование канала поступления краткое 
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( acq_way_code )
  );
-- Internet
-- книжный магазин
-- подарок
-- книги родителей

-- ---------------------------------------------------------------------------
-- Таблица "Коды видов деятельности организаций, учреждений"
-- ---------------------------------------------------------------------------
-- DROP TABLE Activity_types CASCADE;

CREATE TABLE Activity_types
  ( act_type_code int2,             -- Код вида деятельности
    act_type_fname text             -- Наименование вида деятельности полное
      UNIQUE NOT NULL,
    act_type_sname text             -- Наименование вида деятельности краткое
      UNIQUE NOT NULL,
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( act_type_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Виды статуса мероприятий"
-- ---------------------------------------------------------------------------
-- DROP TABLE Conf_status_types CASCADE;

CREATE TABLE Conf_status_types        -- Conferences_status_types
  ( conf_status_code int2,            -- Код статуса мероприятия
    conf_status_fname text NOT NULL,  -- Наименование статуса мероприятия
                                      -- полное
    conf_status_sname text NOT NULL,  -- Наименование статуса мероприятия
                                      -- краткое
    comment text,                     -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,        -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,      -- Когда добавлена (изменена) запись
    PRIMARY KEY ( conf_status_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Виды мероприятий"
-- ---------------------------------------------------------------------------
-- DROP TABLE Conf_types CASCADE;

CREATE TABLE Conf_types              -- Conferences_types
  ( conf_type_code int2,             -- Код вида мероприятия
    conf_type_fname text             -- Наименование вида мероприятия полное
      UNIQUE NOT NULL,
    conf_type_sname text             -- Наименование вида мероприятия краткое
      UNIQUE NOT NULL,
    comment text,                    -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,       -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,     -- Когда добавлена (изменена) запись
    PRIMARY KEY ( conf_type_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Страны"
-- ---------------------------------------------------------------------------
-- DROP TABLE Countries CASCADE;

CREATE TABLE Countries
  ( country_code int2,                -- Код страны
    country_fname text NOT NULL,      -- Наименование страны полное
    country_sname text NOT NULL,      -- Наименование страны краткое 
    country_code_orig int2,           -- Код страны, являющейся началом цепочки
                                      -- (для учета переименований,
                                      --  разночтений, написаний на различных
                                      --  языках)
    lang_code int2                    -- Код языка (1 - русский язык)
      NOT NULL DEFAULT 1,
    comment text,                     -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,        -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,      -- Когда добавлена (изменена) запись
    PRIMARY KEY ( country_code ),
    UNIQUE ( country_fname, lang_code ),
    UNIQUE ( country_sname, lang_code ),
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( country_code_orig )
      REFERENCES Countries ( country_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Города"
-- ПРИМЕЧАНИЕ. Если страна, в которой находится город, меняла свое
--             наименование (или распадалась и т. п.), то можно ввести более
--             одной записи для данного города. Если название города
--             приводится на разных языках, то следует ввести в таблицу записи
--             для каждого языка, а в таблицу Countries также следует ввести
--             записи на этих языках.
-- ---------------------------------------------------------------------------
-- DROP TABLE Cities CASCADE;

CREATE TABLE Cities
  ( city_code int2,                -- Код города
    country_code int2 NOT NULL,    -- Код страны
    city_region text,              -- Регион (или другая информация о
                                   -- местонахождении города -- это может
                                   -- быть важно при наличии в одной стране
                                   -- городов с одинаковыми названиями)
    city_fname text NOT NULL,      -- Наименование города полное
    city_sname text NOT NULL,      -- Наименование города краткое 
    city_code_orig int2,           -- Код города, являющегося началом цепочки
                                   -- (для учета переименований,
                                   --  разночтений, написаний на различных
                                   --  языках)
    -- язык должен быть таким же, как и у наименования страны, код которой
    -- указан в поле country_code
    -- ПРИМЕЧАНИЕ. В настоящее время обеспечение этого соответствия будет
    --             возложено на оператора-пользователя.
    -- TODO        В дальнейшем возможно какое-либо программное решение
    --             или решение средствами СУБД.
    lang_code int2                 -- Код языка (1 - русский язык)
      NOT NULL DEFAULT 1,
    comment text,                  -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,     -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,   -- Когда добавлена (изменена) запись
    PRIMARY KEY ( city_code ),
    -- --------------------------------------------------------
    -- TODO Необходимость этих двух уникальных ключей - под вопросом
    --      (следует вернуться к этому фрагменту после некоторго времени
    --       эксплуатации системы)
    UNIQUE ( city_fname, city_region, country_code, lang_code ),
    UNIQUE ( city_sname, city_region, country_code, lang_code ),
    -- ---------------------------------------------------------
    FOREIGN KEY ( country_code )
      REFERENCES Countries ( country_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( city_code_orig )
      REFERENCES Cities ( city_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Валюты"
-- ---------------------------------------------------------------------------
-- DROP TABLE Currencies CASCADE;

CREATE TABLE Currencies
  ( currency_code int2,            -- Код валюты
    currency_fname text            -- Наименование валюты полное
      NOT NULL,
    currency_sname text            -- Наименование валюты краткое
      NOT NULL,
    country_code int2 NOT NULL,    -- Наименование страны
    currency_date date,            -- Дата введения данной валюты
    currency_code_orig int2,       -- Код валюты, являющейся началом цепочки
                                   -- (для учета переименований,
                                   --  разночтений, написаний на различных
                                   --  языках)
    lang_code int2                 -- Код языка, на котором приведено
      NOT NULL DEFAULT 1,          -- наименование валюты (1 - русский язык)
    comment text,                  -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,     -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,   -- Когда добавлена (изменена) запись
    PRIMARY KEY ( currency_code ),
    FOREIGN KEY ( country_code )
      REFERENCES Countries ( country_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( currency_code_orig )
      REFERENCES Currencies ( currency_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Типы связей между объектами библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Dep_types CASCADE;

CREATE TABLE Dep_types              -- Objects_dependencies_types
  ( dep_type_code int2,             -- Код типа зависимости
    dep_type_fname text             -- Наименование типа зависимости полное
      NOT NULL UNIQUE,
    dep_type_sname text             -- Наименование типа зависимости краткое
      NOT NULL UNIQUE,
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( dep_type_code )
  );
-- ПРИМЕЧАНИЕ. Для обратимых связей нужно вводить по 2 записи.
--             Например, "Книга является переводом книги такой-то" и
--             "Книга является оригинальным изданием для такого-то переводного
--             издания".

-- ---------------------------------------------------------------------------
-- Таблица "Виды характеристик объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Det_char_types CASCADE;

CREATE TABLE Det_char_types         -- Detailed_characteristics_types
  ( det_char_code int2,             -- Код характеристики (аспекта)
    det_char_fname text             -- Описание характеристики (аспекта)
      NOT NULL UNIQUE,              -- полное
    det_char_sname text             -- Описание характеристики (аспекта)
      NOT NULL UNIQUE,              -- краткое
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( det_char_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Виды тиражей"
-- ---------------------------------------------------------------------------
-- DROP TABLE Edition_types CASCADE;

CREATE TABLE Edition_types
  ( edition_code int2,            -- Код вида тиража
    edition_name text             -- Наименование вида тиража
      NOT NULL UNIQUE,
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( edition_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Группы признаков объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Feature_groups CASCADE;

CREATE TABLE Feature_groups
  ( feature_group_code int2,        -- Код группы признаков
    feature_group_name text         -- Описание группы признаков
       NOT NULL UNIQUE,
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( feature_group_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Виды признаков объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Feature_types CASCADE;

CREATE TABLE Feature_types
  ( feature_code int2,              -- Код признака
    feature_group_code int2,        -- Код группы признаков
    feature_name text               -- Описание признака
       NOT NULL UNIQUE,
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( feature_code ),
    FOREIGN KEY ( feature_group_code )
      REFERENCES Feature_groups ( feature_group_code )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Коды общего обозначения материала"
-- ---------------------------------------------------------------------------
-- DROP TABLE Gen_mater_types CASCADE;

CREATE TABLE Gen_mater_types     -- General_material_designation_types
  ( gen_mat_code int2,           -- Код общего обозначения материала
    gen_mat_name text NOT NULL,  -- Наименование общего обозначения материала
    gen_mat_code_orig int2,      -- Код общего обозначения материала,
                                 -- являющийся началом цепочки
                                 -- (для учета написаний на различных
                                 --  языках)
    lang_code int2               -- Код языка (1 - русский язык)
      NOT NULL DEFAULT 1,
    comment text,                -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,   -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP, -- Когда добавлена (изменена) запись
    PRIMARY KEY ( gen_mat_code ),
    UNIQUE ( gen_mat_name, lang_code ),
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( gen_mat_code_orig )
      REFERENCES Gen_mater_types ( gen_mat_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Коды видов упаковки"
-- ---------------------------------------------------------------------------
-- DROP TABLE Pack_types CASCADE;

CREATE TABLE Pack_types
  ( pack_code int2,                 -- Код вида упаковки
    pack_fname text                 -- Наименование вида упаковки полное
      UNIQUE NOT NULL,
    pack_sname text                 -- Наименование вида упаковки краткое 
      UNIQUE NOT NULL,
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( pack_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Коды физической формы, в которой присутствуют единицы хранения"
-- ---------------------------------------------------------------------------
-- DROP TABLE Phys_form_types CASCADE;

CREATE TABLE Phys_form_types
  ( phys_form_code int2,             -- Код физической формы
    phys_form_fname text             -- Наименование физической формы полное
      UNIQUE NOT NULL,
    phys_form_sname text             -- Наименование физической формы краткое
      UNIQUE NOT NULL,
    comment text,                    -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,       -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,     -- Когда добавлена (изменена) запись
    PRIMARY KEY ( phys_form_code )
  );

-- в оригинальном виде
-- ксерокопия
-- распечатка из файла (сканированного и распознанного текста)
-- распечатка из файла (сканированного текста)
-- файл текстовый (сканированный и распознанный)
-- файл текстовый (набранный вручную)
-- магнитофонная запись с оригинальной грампластинки
-- рукописная выписка
-- копия диска CD-ROM

-- ---------------------------------------------------------------------------
-- Таблица "Виды языковой обработки элементов библиографических описаний"
-- ---------------------------------------------------------------------------
-- DROP TABLE Process_types CASCADE;

CREATE TABLE Process_types
  ( process_code int2,                -- Код вида обработки элемента
                                      -- (1 - оригинальное,
                                      --  2 - в транслитерации,
                                      --  3 - в транскрипции,
                                      --  4 - в переводе)
    process_fname text NOT NULL,      -- Наименование вида обработки полное
    process_sname text NOT NULL,      -- Наименование вида обработки краткое 
    comment text,                     -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,        -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,      -- Когда добавлена (изменена) запись
    PRIMARY KEY ( process_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Коды источников библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Recs_sources CASCADE;

CREATE TABLE Recs_sources        -- Records_sources
  ( source_code int2,            -- Код источника библиографического описания
    source_name text             -- Описание источника библиографического
      NOT NULL UNIQUE,           -- описания
    comment text,                -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,   -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP, -- Когда добавлена (изменена) запись
    PRIMARY KEY ( source_code )
  );
-- книга, план изд-ва, прайс-лист ...

-- ---------------------------------------------------------------------------
-- Таблица "Виды справочных материалов"
-- ---------------------------------------------------------------------------
-- DROP TABLE Ref_types CASCADE;

CREATE TABLE Ref_types              -- References_types
  ( ref_type_code int2,             -- Код вида справочного материала
    ref_type_fname text             -- Описание вида справочного материала
      NOT NULL UNIQUE,              -- полное
    ref_type_sname text             -- Описание вида справочного материала
      NOT NULL UNIQUE,              -- краткое
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( ref_type_code )
  );

-- указатель предметный
-- указатель именной
-- словарь терминологический
-- список иллюстраций (часто приводится в зарубежных научных книгах)
-- список таблиц (часто приводится в зарубежных научных книгах)
-- список обозначений

-- ---------------------------------------------------------------------------
-- Таблица "Роли"
-- ---------------------------------------------------------------------------
-- DROP TABLE Roles CASCADE;

CREATE TABLE Roles
  ( role_code int2,                -- Код роли
    role_fname text                -- Наименование роли полное
      NOT NULL UNIQUE,
    role_sname text                -- Наименование роли краткое 
      NOT NULL UNIQUE,
    role_code_orig int2,           -- Код роли, являющейся началом цепочки
                                   -- (для учета разночтений, написаний
                                   -- на различных языках)
    lang_code int2                 -- Код языка, на котором приведено написание
      NOT NULL DEFAULT 1,          -- роли (1 - русский язык)
    process_code int2              -- Код вида обработки наименования роли
      NOT NULL DEFAULT 1,          -- (1 - оригинальное, т.е. в том виде, как
                                   -- приведено в источнике библиограф.
                                   -- описания)
    comment text,                  -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,     -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,   -- Когда добавлена (изменена) запись
    PRIMARY KEY ( role_code ),
    FOREIGN KEY ( role_code_orig )
      REFERENCES Roles ( role_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Отрасли науки"
-- ---------------------------------------------------------------------------
-- DROP TABLE Sci_branches CASCADE;

CREATE TABLE Sci_branches
  ( sci_code int2,                  -- Код отрасли науки
    sci_fname text                  -- Наименование отрасли науки полное
      UNIQUE NOT NULL,              -- (например, "физико-математические")
    sci_sname text                  -- Наименование отрасли науки краткое 
      UNIQUE NOT NULL,              -- (например, "физ.-мат.")
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( sci_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Виды ученых степеней"
-- ---------------------------------------------------------------------------
-- DROP TABLE Sci_degrees CASCADE;

CREATE TABLE Sci_degrees          -- Scientific_degrees
  ( sci_deg_code int2,            -- Код ученой степени
    sci_deg_fname text            -- Наименование ученой степени полное
      UNIQUE NOT NULL,            -- (например, "кандидат")
    sci_deg_sname text            -- Наименование ученой степени краткое
      UNIQUE NOT NULL,            -- (например, "канд.")
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( sci_deg_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Научные специальности"
-- ---------------------------------------------------------------------------
-- DROP TABLE Sci_specs CASCADE;

CREATE TABLE Sci_specs               -- Scientific_specialities
  ( sci_spec_code int2,              -- Код научной специальности
    sci_spec_cipher text NOT NULL,   -- Шифр научной специальности
    sci_spec_name text NOT NULL,     -- Наименование научной специальности
    sci_spec_date date NOT NULL,     -- Дата введения научной специальности
    sci_spec_term date NOT NULL,     -- Срок действия научной специальности
    comment text,                    -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,       -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,     -- Когда добавлена (изменена) запись
    PRIMARY KEY ( sci_spec_code ),
    UNIQUE ( sci_spec_cipher, sci_spec_date )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Группы специфических обозначений материала"
-- ПРИМЕЧАНИЕ. См. библиотечную систему IRBIS.
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_mater_groups CASCADE;

CREATE TABLE Spec_mater_groups    -- Specific_material_designation_groups
  ( spec_mat_group_code int2,     -- Код группы специфического обозначения
                                  -- материала
    spec_mat_group_fname text     -- Полное наименование группы
      NOT NULL,
    spec_mat_group_sname text     -- Краткое наименование группы
      NOT NULL,
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( spec_mat_group_code )
  );

-- ПЕЧАТЬ, АУДИО, ВИДЕО ... (См. библиотечную систему IRBIS).


-- ---------------------------------------------------------------------------
-- Таблица "Специфические обозначения материала"
-- ПРИМЕЧАНИЕ. См. библиотечную систему IRBIS.
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_mater_types CASCADE;

CREATE TABLE Spec_mater_types     -- Specific_material_designation_types
  ( spec_mat_code int2,           -- Код специфического обозначения материала
    spec_mat_group_code int2      -- Код группы специфического обозначения
      NOT NULL,                   -- материала
    spec_mat_fname text           -- Полное наименование материала
      NOT NULL,
    spec_mat_sname text           -- Краткое наименование материала
      NOT NULL,
    spec_mat_code_orig int2,      -- Код специфического обозначения материала,
                                  -- являющийся началом цепочки
                                  -- (для учета написаний на различных
                                  --  языках)
    lang_code int2                -- Код языка (1 - русский язык)
      NOT NULL DEFAULT 1,
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( spec_mat_code ),
    FOREIGN KEY ( spec_mat_group_code )
      REFERENCES Spec_mater_groups ( spec_mat_group_code )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( spec_mat_code_orig )
      REFERENCES Spec_mater_types ( spec_mat_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Виды стандартных номеров (ISBN, ISSN и др.)"
-- ---------------------------------------------------------------------------
-- DROP TABLE Std_num_types CASCADE;

CREATE TABLE Std_num_types        -- Standard_numbers_types
  ( std_num_code int2,            -- Код вида стандартного номера
    std_num_name text             -- Наименование вида стандартного номера
      NOT NULL UNIQUE,            -- ("ISBN", "ISSN", ...)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( std_num_code )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Виды заглавий"
-- ---------------------------------------------------------------------------
-- DROP TABLE Title_types CASCADE;

CREATE TABLE Title_types
  ( title_type_code int2,            -- Код вида заглавия
    title_type_fname text            -- Наименование вида заглавия полное
      UNIQUE NOT NULL,
    title_type_sname text            -- Наименование вида заглавия краткое 
      UNIQUE NOT NULL,
    comment text,                    -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,       -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,     -- Когда добавлена (изменена) запись
    PRIMARY KEY ( title_type_code )
  );



-- --------------------------------------------------------------------------
-- ОСНОВНЫЕ ТАБЛИЦЫ
-- --------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- Таблица "Общий список объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Bib_objs CASCADE;

CREATE TABLE Bib_objs            -- Bibliographic_objects
  ( obj_ID int4,                 -- Уникальный идентификатор объекта
                                 -- библиографического описания
    flags text,                  -- Поле флагов
                                 -- (в расчете на непредвиденные случаи)
    comment text,                -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,   -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP, -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Библиографические описания (записи)"
-- ---------------------------------------------------------------------------
-- DROP TABLE Bib_recs CASCADE;

CREATE TABLE Bib_recs          -- Bibliographic_records
  ( obj_ID int4,               -- Уникальный идентификатор объекта
                               -- библиографического описания
    bib_rec_num int2           -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0       -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    gen_mat_code int2          -- Код общего обозначения материала
      NOT NULL DEFAULT 1,      -- (1 - "текст")
                               -- ПРИМЕЧАНИЕ. Это поле не перенесено в таблицу
                               -- Bib_objs, т.к. при составлении нескольких
                               -- описаний для одного объекта возможно
                               -- использование различных значений этого поля
                               -- (например, из-за использования различных
                               -- языков: "Текст" и "Text" и т.п.)
    bib_rec_head text,         -- Заголовок библиографической записи
    bib_rec_full text,         -- Полная библиографическая запись
    bib_rec_short text,        -- Краткая библиографическая запись
    ord_unit_num text,         -- Порядковая единица для сериальных
                               -- и многотомных изданий
    source_code int2 NOT NULL, -- Код источника библиографического описания
    details_flag bool          -- Флаг наличия "деталировки" в базе данных
      NOT NULL,
    lang_code int2             -- Код языка, на котором составлено
      NOT NULL DEFAULT 1,      -- библиографическое описание (1 - русский язык)
    process_code int2          -- Код вида обработки библиограф. описания
      NOT NULL DEFAULT 1,      -- (1 - оригинальное, т.е. в том виде, как
                               -- приведено в источнике библиограф. описания)
    reason_add_rec text,       -- Назначение дополнит. описания (или причина
                               -- его составления)
    flags text,                -- Поле флагов
                               -- (в расчете на непредвиденные случаи)
    comment text,              -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    -- нужно ввести хотя бы одно описание (либо полное, либо краткое, либо оба)
    CHECK ( NOT ( bib_rec_full IS NULL and bib_rec_short IS NULL ) ),
    PRIMARY KEY ( obj_ID, bib_rec_num ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( gen_mat_code )
      REFERENCES Gen_mater_types ( gen_mat_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( source_code )
      REFERENCES Recs_sources ( source_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Условия доступности"
-- ---------------------------------------------------------------------------
-- DROP TABLE Avail_terms;

CREATE TABLE Avail_terms          -- Availability_terms
  ( obj_ID int4 NOT NULL,         -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    avail_num int2                -- Порядковый номер условий доступности
      NOT NULL DEFAULT 1
      CHECK ( avail_num >= 1 ),
    avail_terms text NOT NULL,    -- Условия доступности
    er_flag bool                  -- Флаг электронного ресурса
      NOT NULL DEFAULT 'no',
    protocol text,                -- Протокол (HTTP, FTP)
    URL text,                     -- URL (адрес) электронного ресурса
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, avail_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Содержание (оглавление) объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Contents;

CREATE TABLE Contents
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    order_num int2,               -- Порядковый номер записи в содержании
                                  -- (оглавлении) данной книги (журнала)
                                  -- Примечание. Это поле нужно для
                                  --             сортировки записей по порядку,
                                  --             поскольку сортировка по полю
                                  --             unit_num может дать неверное
                                  --             упорядочение (например, после
                                  --             главы 1 будет идти глава 10,
                                  --             а не глава 2)
    unit_num text,                -- Номер главы, части, параграфа, ...
    authors text,                 -- Авторы, исполнители, композиторы, ...
    unit_title text NOT NULL,     -- Наименование, заглавие
    class_codes text,             -- Коды классификаций УДК, ББК ...
    spec_mat_code int2,           -- Код специфического обозначения материала
                                  -- (т.е. код единицы измерения: страницы,
                                  -- минуты, ...)
    location text,                -- Место расположения в пределах объекта
                                  -- библиограф. описания (номера страниц, ...)
    amount text,                  -- Объем материала: например, 24 (страницы),
                                  -- 11 (штук) (единицы измерения не указывать)
                                  -- ПРИМЕЧАНИЕ. Тип данных text выбран на
                                  --             всякий случай.
    duration text,                -- Длительность звучания (для аудио-
                                  -- и видеозаписей)
    side numeric( 1 )             -- Номер стороны (для грампластинок)
      CHECK ( side IS NULL or side = 1 or side = 2 ),
    analytic_obj_ID int4,         -- Идентификатор объекта библиографического
                                  -- описания (если на составную часть
                                  -- составлено аналитическое описание)
                                  -- ПРИМЕЧАНИЕ. Отсылаем к объекту, а не к
                                  --             конкретному варианту биб.
                                  --             описания.
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, order_num ),
    -- если номера глав в книге не указываются, то unit_num будет NULL
    -- (и это не помешает ввести более одной записи для конкретных значений
    -- полей obj_ID, bib_rec_num)
    UNIQUE ( obj_ID, bib_rec_num, unit_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( spec_mat_code )
      REFERENCES Spec_mater_types ( spec_mat_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( analytic_obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Депонированные работы"
-- ---------------------------------------------------------------------------
-- DROP TABLE Deposits;

CREATE TABLE Deposits
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    depo_date date NOT NULL,      -- Дата депонирования
    depo_site text NOT NULL,      -- Место депонирования (город и организация)
                                  -- (вводить непосредственно символьные
                                  --  строки)
    depo_code text NOT NULL,      -- Шифр
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Детальные характеристики объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Det_chars;

CREATE TABLE Det_chars            -- Detailed_characteristics
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    det_char_code int2,           -- Код характеристики (аспекта)
    assessment_date date,         -- Дата выставления оценки
    det_char_mark numeric( 2 )    -- Оценка в баллах по 10-балльной шкале
      NOT NULL
      CHECK ( det_char_mark >= 1 and det_char_mark <= 10 ),
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, det_char_code, assessment_date ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( det_char_code )
      REFERENCES Det_char_types ( det_char_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Диссертации и авторефераты диссертаций"
-- ---------------------------------------------------------------------------
-- DROP TABLE Diss;

CREATE TABLE Diss                 -- Dissertations
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    rec_num int2                  -- Номер записи о данной диссертации
      NOT NULL DEFAULT 1          -- (автореферате)
      CHECK ( rec_num = 1 or      -- Примечание. Если диссертация выполнена
              rec_num = 2 ),      --             на стыке наук, то нужно вводить
                                  --             две записи.
    diss_abstr_flag numeric( 1 )  -- Признак "диссертация/автореферат"
      NOT NULL DEFAULT 1          -- (1 - диссертация; 2 - автореферат)
      CHECK ( diss_abstr_flag = 1 or diss_abstr_flag = 2 ),
    sci_deg_code int2,            -- Код ученой степени    
    sci_code int2,                -- Код отрасли науки
    sci_spec_code int2,           -- Код научной специальности
    def_date date,                -- Дата защиты
    conf_date date,               -- Дата утверждения
    diss_reg_num text,            -- Номер гос. регистрации
    -- ПРИМЕЧАНИЕ. Эти сведения можно помещать в сведения об ответственности
    -- Научный руководитель (руководители)
    -- Оппоненты
    -- Диссертационный совет
    -- Ведущая организация
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    -- в первой записи о диссертации (автореферате) больше обязательных полей,
    -- а во второй записи (если диссертация выполнена на стыке наук) таких
    -- полей меньше
    CHECK ( ( rec_num = 1 and sci_deg_code IS NOT NULL and
              sci_code IS NOT NULL and sci_spec_code IS NOT NULL and
              def_date IS NOT NULL ) or
            ( rec_num = 2 and sci_deg_code IS NULL and sci_code IS NULL and
              sci_spec_code IS NOT NULL and def_date IS NULL and 
              conf_date IS NULL and diss_reg_num IS NULL )
          ),
    PRIMARY KEY ( obj_ID, bib_rec_num, rec_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( sci_deg_code )
      REFERENCES Sci_degrees ( sci_deg_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( sci_code )
      REFERENCES Sci_branches ( sci_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( sci_spec_code )
      REFERENCES Sci_specs ( sci_spec_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Тиражи объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Editions;

CREATE TABLE Editions
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    edition_quant text            -- Объем тиража (тип данных text, т.к.
      NOT NULL,                   -- у журналов возможно изменение тиража
                                  -- от выпуска к выпуску, поэтому тираж будет
                                  -- указываться таким образом: 500-750);
                                  -- если издание вышло в переплете и мягкой
                                  -- обложке (тираж разделен на две части), то
                                  -- указать это в поле comment
    edition_code int2 NOT NULL,   -- Код вида тиража
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID ), 
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( edition_code )
      REFERENCES Edition_types ( edition_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Сведения об издании"
-- ---------------------------------------------------------------------------
-- DROP TABLE Edition_stat;

CREATE TABLE Edition_stat         -- Edition_statement
  ( obj_ID int4 NOT NULL,         -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    ed_stat_type numeric( 1 )     -- Тип сведений об издании
      NOT NULL DEFAULT 1             -- (1 - основные сведения;
      CHECK ( ed_stat_type >= 1 and  --  2 - параллельные сведения;
              ed_stat_type <= 3 ),   --  3 - дополнительные сведения)
    ed_stat_num int2              -- Порядковый номер сведений
      NOT NULL DEFAULT 1          -- (нумерация ведется в пределах каждого
      CHECK ( ed_stat_num >= 1 ), --  типа сведений)
    ed_stat text NOT NULL,        -- Текст сведений
    lang_code int2 NOT NULL       -- Код языка сведений
      DEFAULT 1,                  -- (1 - это должен быть код русского языка)
    process_code int2             -- Код вида обработки заглавия
      NOT NULL DEFAULT 1,         -- (1 - оригинальное)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, ed_stat_type, ed_stat_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Признаки объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Features;

CREATE TABLE Features
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    feature_code int2,            -- Код признака
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, feature_code ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( feature_code )
      REFERENCES Feature_types ( feature_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- DROP INDEX Features_ind_1;
CREATE INDEX Features_ind_1 ON Features ( feature_code, obj_ID );

-- ---------------------------------------------------------------------------
-- Таблица "Общие характеристики объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Gen_chars;

CREATE TABLE Gen_chars            -- General_characteristics
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    gen_char text NOT NULL,       -- Общая характеристика источника 
    useful_fragments text,        -- Информация о фрагментах текста,
                                  -- заинтересовавших меня (номера страниц,
                                  -- абзацев, строк...)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Ключевые слова"
-- ---------------------------------------------------------------------------
-- DROP TABLE Keywords CASCADE;

CREATE TABLE Keywords
  ( keyword_code int4,            -- Код ключевого слова
    keyword_name text NOT NULL,   -- Ключевое слово (выражение)
    keyword_code_orig int4,       -- Код ключевого слова, являющегося началом
                                  -- цепочки (для учета переименований,
                                  -- разночтений, сокращений, написаний
                                  -- на различных языках)
    lang_code int2                -- Код языка (1 - русский язык)
      NOT NULL DEFAULT 1,
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( keyword_code ),
    UNIQUE( keyword_name, lang_code ),
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( keyword_code_orig )
      REFERENCES Keywords ( keyword_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Иерархии ключевых слов"
-- Примечание. Наличие этой таблицы вызвано возможностью включения ключевого
--             слова в более чем одну иерархию. Например, "Операционные
--             системы" могут быть включены в иерархию "Системы" и в иерархию
--             "Информационные технологии". Причем в первую иерархию
--             "Операционные системы" разумно включить БЕЗ их субиерархии,
--             в во вторую иерархию их стоит включить со всей субиерархией.
-- ---------------------------------------------------------------------------
-- DROP TABLE Keywords_h;

CREATE TABLE Keywords_h            -- Keywords_hierarchies
  ( keyword_code int4,             -- Код ключевого слова
    keyword_code_parent int4,      -- Код вышестоящего ключевого слова
    include_all_flag bool          -- Признак включения данного ключевого
      NOT NULL DEFAULT 'yes',      -- слова в данную иерархию со всей
                                   -- субиерархией (это поле имеет приоритет
                                   -- над полем hier_depth)
    hier_depth int2                -- Уровень субиерархии, до которого
      NOT NULL DEFAULT 0           -- следует включать слова в вышестоящую
      CHECK ( hier_depth >= 0 ),   -- иерархию (в случае, когда include_all_flag
                                   -- равен 'no')
                                   -- ПРИМЕЧАНИЕ. Если include_all_flag равен
                                   --             'no', то hier_depth = 0
                                   --             означает включение только
                                   --             данного ключевого слова
                                   --             без зависимых от него слов.
    comment text,                  -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,     -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,   -- Когда добавлена (изменена) запись
    PRIMARY KEY ( keyword_code, keyword_code_parent ),
    FOREIGN KEY ( keyword_code )
      REFERENCES Keywords ( keyword_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    -- TODO Рассмотреть возможность переключения зависимого ключевого слова
    --      на другое ключевое слово на том же или более высоком уровне
    --      иерархии (для этого потребуется программное решение, возможно,
    --      триггер). Тогда роль внешнего ключа должна быть пересмотрена.
    FOREIGN KEY ( keyword_code_parent )
      REFERENCES Keywords ( keyword_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Примечания"
-- Примечание. Для одного объекта библиографического описания может вводиться
--             более одной записи о примечаниях.
-- ---------------------------------------------------------------------------
-- DROP TABLE Notes;

CREATE TABLE Notes
  ( obj_ID int4 NOT NULL,         -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    note_num int2                 -- Порядковый номер примечания
      NOT NULL DEFAULT 1,
    note_text text NOT NULL,      -- Текст примечания (область в биб. описании)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, note_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Объекты-контейнеры"
-- ---------------------------------------------------------------------------
-- DROP TABLE Obj_containers;

CREATE TABLE Obj_containers       -- Objects_containers
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
                                  -- (составной части)
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    part_num int2 NOT NULL        -- Порядковый номер фрагмента
      DEFAULT 1
      CHECK ( part_num >= 1 ),
    obj_ID_cont int4,             -- Уникальный идентификатор объекта
                                  -- библиографического описания, в котором
                                  -- помещается составная часть
                                  -- (объект-контейнер)
    bib_rec_num_cont int2         -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num_cont >= 0 ),
    part_place text,              -- Номера страниц, на которых размещается
                                  -- составная часть (для газет до 8 стр.
                                  -- не указывается)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, obj_ID_cont, bib_rec_num_cont ),
    UNIQUE ( obj_ID, bib_rec_num, part_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( obj_ID_cont, bib_rec_num_cont )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Связи объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Obj_deps;

CREATE TABLE Obj_deps             -- Objects_dependencies
  ( base_obj_ID int4,             -- Уникальный идентификатор объекта
                                  -- библиографического описания,
                                  -- являющегося базовым
    dep_obj_ID int4,              -- Уникальный идентификатор объекта
                                  -- библиографического описания,
                                  -- являющегося зависимым
    dep_type_code int2,           -- Код типа зависимости
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( base_obj_ID, dep_obj_ID, dep_type_code ),
    FOREIGN KEY ( base_obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( dep_obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( dep_type_code )
      REFERENCES Dep_types ( dep_type_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Описания временных группировок (списков) объектов
-- библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Obj_groups_descrs CASCADE;

CREATE TABLE Obj_groups_descrs
  ( tmp_group_code int2,            -- Уникальный код временной группировки
    tmp_group_name text NOT NULL,   -- Наименование временной группировки
    tmp_group_descr text NOT NULL,  -- Описание временной группировки
    tmp_group_date date NOT NULL,   -- Дата рождения временной группировки
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( tmp_group_code ),
    UNIQUE ( tmp_group_name )
  );

-- ---------------------------------------------------------------------------
-- Таблица "Состав временных группировок (списков) объектов
-- библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Obj_groups;

CREATE TABLE Obj_groups
  ( tmp_group_code int2,          -- Уникальный код временной группировки
    obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    tmp_group_inc_date date       -- Дата внесения объекта
      NOT NULL,                   -- во временную группировку
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( tmp_group_code, obj_ID, bib_rec_num ),
    FOREIGN KEY ( tmp_group_code )
      REFERENCES Obj_groups_descrs ( tmp_group_code )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Ключевые слова, назначенные объектам библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Obj_keywords;

CREATE TABLE Obj_keywords
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    keyword_code int4,            -- Код ключевого слова
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, keyword_code ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( keyword_code )
      REFERENCES Keywords ( keyword_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- DROP INDEX Obj_keywords_ind_1;
CREATE INDEX Obj_keywords_ind_1 ON Obj_keywords ( keyword_code, obj_ID );

-- ---------------------------------------------------------------------------
-- Таблица "Языки текста объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Obj_langs;

CREATE TABLE Obj_langs
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    lang_code int2                -- Код языка (1 - русский язык)
      NOT NULL DEFAULT 1,
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, lang_code ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Организации, учреждения, временные коллективы"
-- ---------------------------------------------------------------------------
-- DROP TABLE Orgs CASCADE;

CREATE TABLE Orgs               -- Organizations
  ( org_ID int4,                -- Уникальный идентификатор организации
    org_fname text NOT NULL,    -- Наименование организации полное
    org_sname text NOT NULL,    -- Наименование организации краткое
    org_city_code int2          -- Местонахождение организации (город)
      NOT NULL,
    org_addr text,              -- Адрес организации
    org_email text,             -- Электронный адрес организации
    org_descr text,             -- Краткое описание организации, доп. сведения
    org_ID_orig int4,           -- Код организациии, являющейся началом цепочки
                                -- (для учета разночтений, написаний
                                -- на различных языках)
    org_ID_parent int4,         -- Код вышестоящей организации
                                -- Примечание. Для упрощения отслеживания
                                -- иерархических связей в организациях можно
                                -- в поля "Наименование полное" и "Наименование
                                -- краткое" вводить и наименование вышестоящей
                                -- организации, но заключать его в квадратные
                                -- скобки: 
                                -- [РАН.Ин-т всеобщей истории]Отдел такой-то
    org_tmp_flag bool NOT NULL  -- Признак временного коллектива (конференции)
      DEFAULT 'no',
    lang_code int2              -- Код языка, на котором приведено наименование
      NOT NULL DEFAULT 1,       -- организации (1 - русский язык)
    process_code int2           -- Код вида обработки наименования организации
      NOT NULL DEFAULT 1,       -- (1 - оригинальное, т.е. в том виде, как
                                -- приведено в источнике библиограф. описания)
    comment text,               -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    -- не допускается делать ссылку на вышестоящую запись из записи,
    -- не являющейся началом цепочки эквивалентных записей (т.е. цепочки
    -- разночтений, записей на различных языках и т.п.) - из каждой такой
    -- группы записей, описывающих один и тот же объект, на верхний уровень
    -- иерархии может вести только одна "ниточка", иначе будет путаница;
    -- запись, которая является началом цепочки эквивалентных записей,
    -- должна иметь NULL в качестве значения поля org_ID_orig
    -- ПРИМЕЧАНИЕ. При удалении записей, которые ссылаются на вышестоящий
    --             уровень (поле org_ID_parent имеет значение не NULL) нужно
    --             следить за тем, чтобы не возникало разрушений цепочек и
    --             разрывов в иерархиях.
    -- TODO        Видимо, нужно написать какой-нибудь триггер для передачи
    --             "прав родоначальницы" цепочки от удаляемой записи к другой
    --             записи из этой же цепочки.
    CHECK ( NOT ( org_ID_orig IS NOT NULL and org_ID_parent IS NOT NULL ) ),
    PRIMARY KEY ( org_ID ),
    FOREIGN KEY ( org_city_code )
      REFERENCES Cities ( city_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    -- см. ПРИМЕЧАНИЕ выше (возможно, что этот внешний ключ будет дополнен
    -- триггером)
    FOREIGN KEY ( org_ID_orig )
      REFERENCES Orgs ( org_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    -- см. ПРИМЕЧАНИЕ выше (возможно, что этот внешний ключ будет дополнен
    -- триггером)
    FOREIGN KEY ( org_ID_parent )
      REFERENCES Orgs ( org_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Виды деятельности организаций, учреждений"
-- ---------------------------------------------------------------------------
-- DROP TABLE Orgs_act;

CREATE TABLE Orgs_act             -- Orgs_activities
  ( org_ID int4,                  -- Уникальный идентификатор организации
    act_type_code int2,           -- Код вида деятельности
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( org_ID, act_type_code ),
    FOREIGN KEY ( org_ID )
      REFERENCES Orgs ( org_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( act_type_code )
      REFERENCES Activity_types ( act_type_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Конференции, съезды, симпозиумы, семинары, выставки ..."
-- Примечание. Это - временные коллективы.
-- ---------------------------------------------------------------------------
-- DROP TABLE Confs CASCADE;

CREATE TABLE Confs
  ( conf_code int4,                  -- Уникальный код мероприятия
    conf_fname text NOT NULL,        -- Полное наименование (тема) мероприятия
    conf_place text NOT NULL,        -- Место проведения (писать словами, т.к.
                                     -- может проводиться в нескольких местах
                                     -- одновременно)
    conf_date_beg date NOT NULL,     -- Дата начала работы мероприятия
    conf_date_end date NOT NULL,     -- Дата окончания работы мероприятия
    conf_status_code int2 NOT NULL,  -- Код статуса мероприятия (Всероссийский,
                                     -- международный ...)
    conf_foreign_flag bool           -- Признак зарубежного мероприятия
      NOT NULL DEFAULT 'no',         -- (т.е., проводимого за рубежом)
    conf_type_code int2 NOT NULL,    -- Код типа мероприятия (конференция,
                                     -- семинар, выставка ...)
    conf_num text,                   -- Порядковый номер (тип поля text
                                     -- на всякий случай)
    conf_code_orig int4,             -- Код главной записи (в случае переводов,
                                     -- разночтений)
    comment text,                    -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,       -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,     -- Когда добавлена (изменена) запись
    CHECK ( conf_date_beg <= conf_date_end ),
    PRIMARY KEY ( conf_code ),
    FOREIGN KEY ( conf_code )
      REFERENCES Orgs ( org_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( conf_status_code )
      REFERENCES Conf_status_types ( conf_status_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( conf_type_code )
      REFERENCES Conf_types ( conf_type_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    -- внешний ключ уазывает на поле из этой же таблицы
    FOREIGN KEY ( conf_code_orig )
      REFERENCES Confs ( conf_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Материалы конференций, съездов, симпозиумов, семинаров,
-- выставок ..."
-- ПРИМЕЧАНИЕ. Эта таблица, возможно, и не нужна.
-- TODO        Принять решение по результатам опытной эксплуатации.
-- ---------------------------------------------------------------------------
-- DROP TABLE Confs_maters;

CREATE TABLE Confs_maters         -- Conferences_materials
  ( conf_code int4,               -- Уникальный код мероприятия
    obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, conf_code ),
    FOREIGN KEY ( conf_code )
      REFERENCES Confs ( conf_code )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Персоналии"
-- ---------------------------------------------------------------------------
-- DROP TABLE Persons CASCADE;

CREATE TABLE Persons
  ( pers_ID int4,               -- Уникальный идентификатор персоналии
    pers_name text NOT NULL,    -- Полное имя, отчество, фамилия
                                -- ПРИМЕЧАНИЕ. В поле записывать первой
                                --             фамилию и ставить запятую.
    pers_life_years text,       -- Годы жизни
    pers_add_info text,         -- Дополнительные сведения (младший/старший,
                                -- отец/сын, ... род занятий, ...)
    pers_ID_orig int4,          -- Код персоналии, являющейся началом цепочки
                                -- (для учета разночтений, написаний
                                -- на различных языках)
    lang_code int2              -- Код языка, на котором приведено написание
      NOT NULL DEFAULT 1,       -- Ф. И. О. персоналии (1 - русский язык)
    process_code int2           -- Код вида обработки имени персоналии
      NOT NULL DEFAULT 1,       -- (1 - оригинальное, т.е. в том виде, как
                                -- приведено в источнике библиограф. описания)
    pers_addr text,             -- Адрес
    pers_email text,            -- Электронный адрес
    comment text,               -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,   -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP, -- Когда добавлена (изменена) запись
    PRIMARY KEY ( pers_ID ),
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( pers_ID_orig )
      REFERENCES Persons ( pers_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- DROP INDEX Persons_ind_1;
CREATE INDEX Persons_ind_1 ON Persons ( pers_name );

-- ---------------------------------------------------------------------------
-- Таблица "Цены объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Prices;

CREATE TABLE Prices
  ( obj_ID int4 NOT NULL,          -- Уникальный идентификатор объекта
                                   -- библиографического описания
    price numeric( 14,2 ),         -- Цена:
                                   --  NULL - цена не указана (не известна);
                                   --  0 - издание бесплатное
                                   -- ПРИМЕЧАНИЕ. Если издание вышло в переплете
                                   --             и мягкой обложке (тираж
                                   --             разделен на две части), то
                                   --             указать все цены в поле
                                   --             comment.
    price_flag numeric( 1 )        -- Признак вида цены:
      NOT NULL DEFAULT 1           --  0 - "обычная" цена;
      CHECK ( price_flag >= 0 and  --  1 - договорная цена;
              price_flag <= 3 ),   --  2 - "Б. ц." ("без цены");
                                   --  3 - бесплатное издание
    currency_code int2,            -- Код валюты
    exchange_rate numeric( 14,2 ), -- Обменный курс валюты к рублю
    price_note text,               -- Дополнительные сведения к цене
    comment text,                  -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,     -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,   -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID ), 
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( currency_code )
      REFERENCES Currencies ( currency_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Дата издания, печатания, изготовления"
-- ---------------------------------------------------------------------------
-- DROP TABLE Pub_print_date;

CREATE TABLE Pub_print_date
  ( obj_ID int4 NOT NULL,         -- Уникальный идентификатор объекта
                                  -- библиографического описания
    pub_date_begin text,          -- Дата начала издания
                                  -- ПРИМЕЧАНИЕ. Для дат выбран тип данных 
                                  --             text, т.к. возможны даты вида
                                  --             [1988], 198?, 197-, ...
    dash_flag bool NOT NULL       -- Флаг наличия тире, которое ставится между
      DEFAULT 'no',               -- датой начала и окончания издания
                                  -- (если дата только одна, то такое тире
                                  -- не требуется)
    pub_date_end text,            -- Дата окончания издания
    print_date text,              -- Дата печатания, изготовления
    sine_anno_flag bool NOT NULL  -- Признак "без года (sine anno)"
      DEFAULT 'no',
    pub_print_date_sure_flag bool -- Флаг точно установленной даты
      NOT NULL DEFAULT 'yes',     -- (когда дата известна не точно, при 
                                  -- указании даты используются скобки - [])
                                  -- (когда дат две - начальная дата и
                                  -- конечная дата, то этот флаг устанавливается
                                  -- в значение 'no', если хотя бы одна из дат
                                  -- не является точной)
                                  -- ПРИМЕЧАНИЕ. По умолчанию дата издания
                                  --             считается установленной точно.
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    -- если не известна дата начала, а дата окончания известна, то в поле
    -- pub_date_begin записывается символ "?"
    CHECK ( NOT ( pub_date_begin IS NULL and pub_date_end IS NOT NULL ) ),
    CHECK ( ( sine_anno_flag = 'yes' and
              pub_date_begin IS NULL and dash_flag = 'no' and 
              pub_date_end IS NULL and print_date IS NULL
            ) or
            ( sine_anno_flag = 'no' and
              ( pub_date_begin IS NOT NULL or pub_date_end IS NOT NULL or
                print_date IS NOT NULL
              )
            )
          ),
    PRIMARY KEY ( obj_ID ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Место издания, изготовления"
-- ---------------------------------------------------------------------------
-- DROP TABLE Pub_print_place;

CREATE TABLE Pub_print_place
  ( obj_ID int4,                   -- Уникальный идентификатор объекта
                                   -- библиографического описания
    bib_rec_num int2               -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0           -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    group_num int2                 -- Номер группы сведений о месте издания
      DEFAULT 1
      CHECK ( group_num >= 1 ),
    place_num int2                 -- Номер места издания в пределах группы
      DEFAULT 1
      CHECK ( place_num >= 1 ),
    -- Этот флаг нужен потому, что когда не известен город (место издания),
    -- указывают страну издания
    city_country_flag numeric( 1 ) -- Флаг "город/страна"
      DEFAULT 1                    -- (1 - город; 2 - страна)
      CHECK ( city_country_flag = 1 or city_country_flag = 2 or
              city_country_flag IS NULL ),
    city_code int2                 -- Код города (код места издания)
      NOT NULL DEFAULT 0,
    country_code int2              -- Код страны (он нужен, когда не известно
      NOT NULL DEFAULT 0,          -- место издания)

    pub_print_flag numeric( 1 )    -- Признак "издатель/изготовитель"
      NOT NULL DEFAULT 1           -- (1 - издатель; 2 - изготовитель)
      CHECK ( pub_print_flag = 1 or pub_print_flag = 2 ),
    sine_loco_flag bool NOT NULL   -- Признак "без места (sine loco)"
      DEFAULT 'no',
    place_sure_flag bool NOT NULL  -- Флаг точно установленного места издания
      DEFAULT 'yes',               -- (когда место установлено не точно,
                                   -- используются скобки - [])
                                   -- Примечание. По умолчанию место издания
                                   --             считается установленным точно.
    place_add_info text,           -- Дополнительные сведения к месту издания/
                                   -- печатания (например, когда одноименные
                                   -- города есть в разных странах)
    comment text,                  -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,     -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,   -- Когда добавлена (изменена) запись
    -- если место издания (либо город, либо страна) указано, то флаг
    -- sine_loco_flag должен быть 'no' и при этом значения полей city_code и
    -- country_code должны соответствовать значению флага city_country_flag
    CHECK ( ( sine_loco_flag = 'yes' and city_country_flag IS NULL ) or
            ( sine_loco_flag = 'no' and 
              -- город
              ( ( city_country_flag = 1 and city_code <> 0 and
                                            country_code = 0
                ) or
              -- страна 
                ( city_country_flag = 2 and city_code = 0 and
                                            country_code <> 0
                )
              )
            )
          ),
    PRIMARY KEY ( obj_ID, bib_rec_num, group_num, place_num ),
    -- UNIQUE ( obj_ID, bib_rec_num, group_num, place_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( city_code )
      REFERENCES Cities ( city_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( country_code )
      REFERENCES Countries ( country_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- DROP INDEX Pub_print_place_ind_1;
CREATE INDEX Pub_print_place_ind_1 ON Pub_print_place ( city_code, obj_ID );

-- ---------------------------------------------------------------------------
-- Таблица "Издатель, изготовитель, распространитель"
-- ---------------------------------------------------------------------------
-- DROP TABLE Pub_printer;

CREATE TABLE Pub_printer          -- Publisher_printer
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    group_num int2                -- Номер группы сведений об издателе
      DEFAULT 1
      CHECK ( group_num >= 1 ),
    pub_num int2                  -- Номер издателя, изготовителя
      DEFAULT 1                   -- в пределах группы
      CHECK ( pub_num >= 1 ),
    pers_org_flag numeric( 1 )    -- Флаг "персоналия/организация"
      DEFAULT 2                   -- (1 - персоналия; 2 - организация)
      CHECK ( pers_org_flag = 1 or pers_org_flag = 2 or 
              pers_org_flag IS NULL ),
    pub_org_ID int4               -- Код издателя, изготовителя ...
      NOT NULL DEFAULT 0,         -- (организация)
    pub_pers_ID int4              -- Код издателя, изготовителя ...
      NOT NULL DEFAULT 0,         -- (персоналия)
    pub_print_flag numeric( 1 )   -- Признак "издатель/изготовитель"
      NOT NULL DEFAULT 1          --
      CHECK ( pub_print_flag = 1 or  -- 1 - издатель;
              pub_print_flag = 2 or  -- 2 - изготовитель;
              pub_print_flag = 3 ),  -- 3 - другое
    pub_func text,                 -- Функция издателя (изготовителя)
    sine_nomine_flag bool NOT NULL -- Признак "без издателя (sine nomine)"
      DEFAULT 'no',
    pub_print_sure_flag bool      -- Флаг точно установленного издателя
      NOT NULL DEFAULT 'yes',     -- (когда издатель установлен не точно,
                                  -- используются скобки - [])
                                  -- Примечание. По умолчанию издатель
                                  --             считается установленным точно.
    pub_print_add_info text,      -- Дополнительные сведения к имени издателя/
                                  -- изготовителя (например, когда одноименные
                                  -- издатели есть в разных странах)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    -- если издатель (либо персоналия, либо организация) указан, то флаг
    -- sine_nomine_flag должен быть 'no' и при этом значения полей pub_org_ID
    -- и pub_pers_ID должны соответствовать значению флага pers_org_flag
    CHECK ( ( sine_nomine_flag = 'yes' and pers_org_flag IS NULL ) or
            ( sine_nomine_flag = 'no' and 
              -- персоналия
              ( ( pers_org_flag = 1 and pub_pers_ID <> 0 and
                                        pub_org_ID = 0
                ) or
              -- организация
                ( pers_org_flag = 2 and pub_pers_ID = 0 and
                                        pub_org_ID <> 0
                )
              )
            )
          ),
    PRIMARY KEY ( obj_ID, bib_rec_num, group_num, pub_num ),
    -- UNIQUE ( obj_ID, bib_rec_num, group_num, pub_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( pub_pers_ID )
      REFERENCES Persons ( pers_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( pub_org_ID )
      REFERENCES Orgs ( org_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- DROP INDEX Pub_printer_ind_1;
CREATE INDEX Pub_printer_ind_1 ON Pub_printer ( pub_org_ID, obj_ID );

-- DROP INDEX Pub_printer_ind_2;
CREATE INDEX Pub_printer_ind_2 ON Pub_printer ( pub_pers_ID, obj_ID );

-- ---------------------------------------------------------------------------
-- Таблица "Серии/подсерии"
-- ---------------------------------------------------------------------------
-- DROP TABLE Series CASCADE;

CREATE TABLE Series
  ( series_ID int2,               -- Уникальный идентификатор серии/подсерии
    series_ISSN text              -- ISSN, присвоенный серии/подсерии
      UNIQUE,
    subser_flag bool              -- Признак подсерии
      NOT NULL DEFAULT 'no',
    series_ID_parent int2,        -- Идентификатор вышестоящей серии (иерархия)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( series_ID ),
    FOREIGN KEY ( series_ID_parent )
      REFERENCES Series ( series_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Описания серий/подсерий"
-- ---------------------------------------------------------------------------
-- DROP TABLE Ser_descrs;

CREATE TABLE Ser_descrs          -- Series_descriptions
  ( series_ID int2,              -- Уникальный идентификатор серии/подсерии
    ser_descr_num int2           -- Номер описания серии/подсерии
      NOT NULL DEFAULT 0         -- (это поле, аналогичное полю bib_rec_num)
      CHECK ( ser_descr_num >= 0 ), -- (основное описание имеет номер 0)
    ser_descr text NOT NULL,     -- Описание серии/подсерии
    lang_code int2               -- Код языка, на котором составлено
      NOT NULL DEFAULT 1,        -- описание серии/подсерии (1 - русский язык)
    process_code int2            -- Код вида обработки описания серии/подсерии
      NOT NULL DEFAULT 1,        -- (1 - оригинальное, т.е. в том виде, как
                                 -- приведено в источнике библиограф. описания)
    reason_add_descr text,       -- Назначение дополнит. описания (или причина
                                 -- его составления)
    comment text,                -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,   -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP, -- Когда добавлена (изменена) запись
    PRIMARY KEY ( series_ID, ser_descr_num ),
    FOREIGN KEY ( series_ID )
      REFERENCES Series ( series_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Заглавия серий/подсерий"
-- ---------------------------------------------------------------------------
-- DROP TABLE Ser_titles;

CREATE TABLE Ser_titles
  ( series_ID int2,                 -- Уникальный идентификатор серии/подсерии
    ser_descr_num int2              -- Номер описания серии/подсерии
      NOT NULL DEFAULT 0            -- (это поле, аналогичное полю bib_rec_num)
      CHECK ( ser_descr_num >= 0 ), -- (основное описание имеет номер 0)
    ser_title_num int2              -- Номер заглавия
      NOT NULL DEFAULT 0            -- (0 - основное; 1, 2, ... - параллельные)
      CHECK ( ser_title_num >= 0 ),
    ser_title text NOT NULL,        -- Заглавие серии/подсерии
    ser_title_info text,            -- Сведения, относящиеся к заглавию
                                    -- (разделять знаком ":")
    lang_code int2                  -- Код языка, на котором записано заглавие
      NOT NULL DEFAULT 1,           -- (1 - русский язык)
    process_code int2               -- Код вида обработки заглавия
      NOT NULL DEFAULT 1,           -- (1 - оригинальное)
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( series_ID, ser_descr_num, ser_title_num ),
    FOREIGN KEY ( series_ID, ser_descr_num )
      REFERENCES Ser_descrs ( series_ID, ser_descr_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Номера выпусков в сериях"
-- ---------------------------------------------------------------------------
-- DROP TABLE Obj_series;

CREATE TABLE Obj_series
  ( obj_ID int4,                    -- Уникальный идентификатор объекта
                                    -- библиографического описания
    bib_rec_num int2                -- Номер библиогр. записи для данного
      NOT NULL DEFAULT 0            -- объекта (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    series_ID int2,                 -- Уникальный идентификатор серии/подсерии
    ser_descr_num int2              -- Номер описания серии/подсерии
      NOT NULL DEFAULT 0            -- (это поле, аналогичное полю bib_rec_num)
      CHECK ( ser_descr_num >= 0 ), -- (основное описание имеет номер 0)
    issue_num text,                 -- Номер выпуска в серии/подсерии
                                    -- (он может быть не указан, т.е. выпуски
                                    -- могут быть ненумерованными)
    comment text,                   -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,      -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,    -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, series_ID, ser_descr_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( series_ID, ser_descr_num )
      REFERENCES Ser_descrs ( series_ID, ser_descr_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Описания справочных материалов объектов библиографического 
-- описания"
-- Примечание. Для одного объекта библиографического описания может вводиться
--             более одной записи о справочных материалах.
-- ---------------------------------------------------------------------------
-- DROP TABLE Refs_descrs;

CREATE TABLE Refs_descrs          -- Obj_references_descriptions
  ( obj_ID int4 NOT NULL,         -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    ref_num int2                  -- Порядковый номер (уникальный код)
      NOT NULL DEFAULT 1          -- справочного материала в пределах данного
      CHECK ( ref_num >= 1 ),     -- объекта биб. описания
    ref_type_code int2 NOT NULL,  -- Код вида справочного материала
    ref_descr text NOT NULL,      -- Описание (название) справочного материала,
                                  -- взятое из объекта биб. описания
                                  -- (если материал сформирован пользователем,
                                  --  то и описание должен сформировать он сам)
    ref_vol text,                 -- Объем справочного материала или номера
                                  -- страниц, на которых он размещен
                                  -- ПРИМЕЧАНИЕ. Если материал сформирован
                                  --             пользователем, то это поле
                                  --             можно не заполнять.
    ref_original_flag bool        -- Признак оригинального материала
      NOT NULL DEFAULT 'yes',     -- (если материал сформирован пользователем,
                                  --  то будет значение 'no')
    ref_complete_flag bool        -- Признак полноты представления справочного
      NOT NULL DEFAULT 'no',      -- материала в базе данных
                                  -- (если введены не все элементы, например,
                                  --  именного указателя, то значение 'no')
    lang_code int2                -- Код языка, на котором приведено описание
      NOT NULL DEFAULT 1,         -- справочного материала (1 - русский язык)
    process_code int2             -- Код вида языковой обработки описания
      NOT NULL DEFAULT 1,         -- справочного материала (1 - оригинальное,
                                  -- т.е. в том виде, как приведено в источнике
                                  -- биб. описания или как сформировано
                                  -- пользователем)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, ref_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( ref_type_code )
      REFERENCES Ref_types ( ref_type_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Содержание справочных материалов объектов библиографического
-- описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Refs_contents;

CREATE TABLE Refs_contents        -- Obj_references_contents
  ( obj_ID int4 NOT NULL,         -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    ref_num int2                  -- Порядковый номер (уникальный код)
      NOT NULL DEFAULT 1          -- справочного материала в пределах данного
      CHECK ( ref_num >= 1 ),     -- объекта биб. описания
    ref_elem text NOT NULL,       -- Элемент справочного материала
                                  -- (например, термин из предметного
                                  --  указателя)
    ref_elem_descr text,          -- Описание элемента справочного материала
                                  -- (например, номера страниц, на которых
                                  --  упоминается персоналия, или толкование
                                  --  термина из глоссария)
    lang_code int2                -- Код языка, на котором записан элемент
      NOT NULL DEFAULT 1,         -- справочного материала (1 - русский язык)
                                  -- ПРИМЕЧАНИЕ. Язык описания элемента может
                                  --             быть другим. Например, термин
                                  --             англозычный, а толкование --
                                  --             на русском языке.
    process_code int2             -- Код вида языковой обработки элемента
      NOT NULL DEFAULT 1,         -- справочного материала (1 - оригинальный,
                                  -- т.е. в том виде, как он приведен 
                                  -- в источнике биб. описания)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, ref_num, ref_elem ),
    FOREIGN KEY ( obj_ID, bib_rec_num, ref_num )
      REFERENCES Refs_descrs ( obj_ID, bib_rec_num, ref_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Сведения об ответственности"
-- Примечание. Наличие флага pers_org_flag позволит при выборке соединять
--             эту таблицу сразу с таблицей персоналий Persons и с таблицей
--             организаций Orgs (в обоих случаях - связи типа 1:1), а затем
--             по значению этого флага решать, какое поле из выборки
--             использовать: имя персоналии либо наименование организации.
-- ---------------------------------------------------------------------------
-- DROP TABLE Resp_stat;

CREATE TABLE Resp_stat            -- Responsibility_statement
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    resp_type numeric( 1 )        -- Тип ответственности
      NOT NULL DEFAULT 1          -- (1 - сведения об ответственности;
      CHECK ( resp_type >= 1 and  --  2 - свед. об ответ., относящиеся
              resp_type <= 5 ),   --      к данному изданию;
                                  --  3 - свед. об ответ., относящиеся
                                  --      к доп. сведениям о данном издании;
                                  --  4 - ... к серии;
                                  --  5 - ... к подсерии)
    resp_num_in_type int2         -- Номер группы сведений в пределах типа
      NOT NULL DEFAULT 1
      CHECK ( resp_num_in_type >= 1 ),
    resp_num int2                 -- Порядковый номер в списке ответственных
      NOT NULL DEFAULT 1          -- (нумерация ведется в пределах каждой
      CHECK ( resp_num >= 1 ),    --  группы сведений внутри каждого типа
                                  --  ответственности)
    role_code int2 NOT NULL,      -- Код роли
    pers_org_flag numeric( 1 )    -- Флаг "персоналия/организация"
      NOT NULL DEFAULT 1          -- (1 - персоналия; 2 - организация)
      CHECK ( pers_org_flag = 1 or pers_org_flag = 2 ),
    resp_pers_ID int4             -- Код ответственного лица (персоналии)
      NOT NULL DEFAULT 0,
    resp_org_ID int4              -- Код ответственного лица (организации)
      NOT NULL DEFAULT 0,

    -- ПРИМЕЧАНИЕ. Поскольку поле series_num не включено в состав первичного
    --             ключа, то в случае, когда книга входит в состав двух
    --             и более серий, для разделения ответственных между сериями
    --             можно номера в поле resp_num_in_type сдвигать (увеличивать)
    --             на 100 для второй серии, на 200 - для третьей и т.д.

    series_num int2               -- Номер серии/подсерии в пределах
      CHECK ( series_num IS NULL  -- данного издания
              or series_num >= 1 ),
    series_ID int2,               -- Уникальный идентификатор серии/подсерии
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    CHECK ( ( pers_org_flag = 1 and resp_pers_ID > 0 and resp_org_ID = 0 )
            or 
            ( pers_org_flag = 2 and resp_pers_ID = 0 and resp_org_ID > 0 )
          ),
    PRIMARY KEY ( obj_ID, bib_rec_num, resp_type, resp_num_in_type, resp_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( role_code )
      REFERENCES Roles ( role_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( resp_pers_ID )
      REFERENCES Persons ( pers_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( resp_org_ID )
      REFERENCES Orgs ( org_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( series_ID )
      REFERENCES Series ( series_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Специфические сведения - электронные ресурсы"
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_er;

CREATE TABLE Spec_er              -- Specific_electronic_resources
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    spec_er_num int2              -- Номер специфических сведений
      DEFAULT 1                   -- Примечание. Они могут повторяться.
      CHECK ( spec_er_num >= 1 ),
    spec_er_text text NOT NULL,   -- Текст специфических сведений
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, spec_er_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Специфические сведения - картографические документы"
-- Примечание. Не знаю точно, какие поля являются обязательными, а какие -
--             нет, поэтому не указывал ограничение NOT NULL.
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_maps;

CREATE TABLE Spec_maps
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    map_scale text,               -- Масштаб
    map_projection text,          -- Картографическая проекция
    map_coord text,               -- Координаты
    map_equinox text,             -- Сведения о равноденствии или эпохе
                                  -- (в звездных картах)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    -- все эти поля не могут быть пустыми одновременно
    CHECK ( map_scale IS NOT NULL or map_projection IS NOT NULL or
            map_coord IS NOT NULL or map_equinox IS NOT NULL ),
    PRIMARY KEY ( obj_ID, bib_rec_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Специфическое обозначение материала и объем; сопроводительный
-- материал"
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_mater;

CREATE TABLE Spec_mater           -- Specific_material_designation
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    spec_mat_num int2             -- Порядковый номер записи о материале
      NOT NULL DEFAULT 1          -- для данного объекта библиограф. описания
      CHECK ( spec_mat_num >= 1 ),-- (для основного материала и для 
                                  -- сопроводительного материала вводить
                                  -- отдельные записи в эту таблицу)
    spec_mat_code int2,           -- Код специфического обозначения материала
    total_extent numeric( 6 ),    -- Суммарный объем материала данного вида
                                  -- (например, "378" (страниц и т.д.))
    orig_extent text NOT NULL,    -- Объем в том виде, как он представлен
                                  -- в источнике (например, XXV с., 354 с.,
                                  -- [10 с.]) 
    phys_add_info text,           -- Другие сведения о физической характеристике
    orig_dimensions text,         -- Размеры (как они представлены в источнике)
    size_sheet text,              -- Размер листа (для книг) (например, 60x90)
    part_sheet text,              -- Доля листа (для книг) (например, 1/16)
    accomp_flag bool NOT NULL     -- Признак сопроводительного материала
      DEFAULT 'no',
    accomp_title text,            -- Наименование сопроводительного материала
    accomp_obj_ID int4,           -- Уникальный идентификатор объекта
                                  -- библиографического описания, если на
                                  -- сопроводительный материал составлено
                                  -- самостоятельное описание
    pack_code int2,               -- Код вида упаковки
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, spec_mat_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( spec_mat_code )
      REFERENCES Spec_mater_types ( spec_mat_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    -- при удалении отдельной записи, составленной на сопроводительный
    -- материал, запись в таблице Spec_mater должна сохраниться, хотя
    -- данный материал и перестал рассматриваться в качестве самостоятельного
    -- объекта биб. описания
    FOREIGN KEY ( accomp_obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( pack_code )
      REFERENCES Pack_types ( pack_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Специфические сведения - нотные документы"
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_music;

CREATE TABLE Spec_music
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    music_info text NOT NULL,     -- Партитура, партии (голоса), дирекционы
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Специфические сведения - патентные документы"
-- Примечание. Не знаю точно, какие поля являются обязательными, а какие -
--             нет, поэтому не указывал ограничение NOT NULL.
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_patents;

CREATE TABLE Spec_patents
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    pat_request_reg_num text,     -- Регистрационный номер заявки на
                                  -- патентный документ
    pat_request_date date,        -- Дата подачи (поступления) заявки
    pat_pub_date date,            -- Дата публикации
    pat_edition text,             -- Сведения об официальном издании, в котором
                                  -- опубликованы сведения о патентном документе
    -- Сведения о конвенционном приоритете
    -- TODO Обязательны ли эти сведения? Это влияет на использование NOT NULL.
    pat_request_date_conv date,   -- Дата подачи заявки
    pat_request_num_conv text,    -- Номер заявки
    pat_country_code int2,        -- Код страны конвенционного приоритета
    pat_class_index text,         -- Индексы национальной патентной 
                                  -- классификации
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( pat_country_code )
      REFERENCES Countries ( country_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Специфические сведения - сериальные документы"
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_serials;

CREATE TABLE Spec_serials
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    numer_area text NOT NULL,     -- Область нумерации
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Специфические сведения - нормативные документы (стандарты и ТУ)"
-- ---------------------------------------------------------------------------
-- DROP TABLE Spec_stds;

CREATE TABLE Spec_stds            -- Spec_standards
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    std_prev text,                -- Обозначение ранее действовавшего документа
    std_date text,                -- Даты введения
    std_term text,                -- Сроки действия
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    -- все эти поля не могут быть пустыми одновременно
    CHECK ( std_prev IS NOT NULL or std_date IS NOT NULL or 
            std_term IS NOT NULL ),
    PRIMARY KEY ( obj_ID, bib_rec_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Стандартные номера (ISBN, ISSN и др.), присвоенные объектам
-- библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Std_nums;

CREATE TABLE Std_nums             -- Standard_numbers
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    std_num_code int2 NOT NULL,   -- Код вида стандартного номера
    std_num_val text,             -- Стандартный номер (ISBN, ISSN ...)
    std_num_note text,            -- Дополнительные сведения к стандартному
                                  -- номеру
    std_num_wrong bool            -- Флаг неверного стандартного номера
      DEFAULT 'no',               -- (ISBN или ISSN)
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, std_num_val ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( std_num_code )
      REFERENCES Std_num_types ( std_num_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- DROP INDEX Std_nums_ind_1;
CREATE INDEX Std_nums_ind_1 ON Std_nums ( std_num_val );

-- ---------------------------------------------------------------------------
-- Таблица "Экземпляры (единицы хранения)"
-- ---------------------------------------------------------------------------
-- DROP TABLE Stor_objs;

CREATE TABLE Stor_objs            -- Storage_objects
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    stor_obj_num int2             -- Порядковый номер единицы хранения для
      DEFAULT 1                   -- данного объекта библиогр. описания
      CHECK ( stor_obj_num >= 1 ),-- (возможно наличие нескольких единиц
                                  -- хранения, например, это могут быть
                                  -- ксерокопии и файл, а также дубликаты книг,
                                  -- а также книга может иметься в различных
                                  -- библиотеках и т.п.)
    -- Информация о владельце данной единицы хранения 
    pers_org_flag numeric( 1 )    -- Флаг "персоналия/организация"
      NOT NULL DEFAULT 1          -- (1 - персоналия; 2 - организация)
      CHECK ( pers_org_flag = 1 or pers_org_flag = 2 ),
    owner_pers_ID int4            -- Код владельца (персоналии)
      NOT NULL DEFAULT 0,
    owner_org_ID int4             -- Код владельца (организации)
      NOT NULL DEFAULT 0,
    -- ----------------------------------------------------------
    phys_form_code int2 NOT NULL, -- Код физической формы, в которой
                                  -- присутствует единица хранения
    stor_obj_code text,           -- Код (шифр) единицы хранения
                                  -- (например, номер экземпляра книги
                                  -- в библиотеке)
                                  -- (этого кода может и не быть)
    stor_place text,              -- Место хранения (например, файл на CD-ROM
                                  -- таком-то (идентификатор этого диска)
    stor_obj_add_info text,       -- Доп. сведения о физической форме и объеме
                                  -- имеющейся части объекта описания
                                  -- (например, формат файла и его размер;
                                  -- полнота ксерокопии (т.е. весь документ или
                                  -- его часть))
    stor_obj_defects text,        -- Дефекты единицы хранения (указывать,
                                  -- например, порванные страницы, наличие
                                  -- пометок в тексте, отсутствие страниц
                                  -- и т.п.)
    stor_obj_phys_cond numeric( 2 ) -- Оценка общего физического состояния
      NOT NULL                    -- единицы хранения по 10-балльной шкале
      CHECK ( stor_obj_phys_cond >= 1 and stor_obj_phys_cond <= 10 ),
    stor_obj_complete bool        -- Признак полноты комплектации
      NOT NULL DEFAULT 'yes',     -- ('no' - если, например, отсутствует
                                  -- часть страниц. Детальные сведения
                                  -- отражаются в поле stor_obj_defects)
    -- Информация о приобретении
    acq_date date,                -- Дата приобретения
                                  -- (если дата известна с точностью до
                                  -- месяца, то ставить первое число этого
                                  -- месяца, если с точностью до года - 
                                  -- первое число этого года, если она
                                  -- не известна, то оставить NULL)
    acq_date_precise bool,        -- Признак точной даты приобретения
                                  -- (если дата известна точно - 'yes',
                                  -- если известно, что дата неточна, - 'no',
                                  -- если дата не известна - NULL)
    acq_date_note text,           -- Пояснения к дате приобретения
    acq_place_city int2,          -- Место приобретения (код населенного пункта)
    acq_place_country int2,       -- Место приобретения (код страны)
                                  -- (указывается, когда не известно точное
                                  --  наименование населенного пункта)
    acq_way_code int2,            -- Код канала поступления
    stor_obj_price_curr           -- Цена (фактическая) в иностранной
      numeric( 14,2 ),            -- валюте (если эта единица хранения
                                  -- приобретена за валюту)
    currency_code int2,           -- Код валюты
    exchange_rate                 -- Обменный курс валюты к рублю
      numeric( 14,5 ),            -- (т.е. количество рублей, которое
                                  --  приравнивается к 1 единице иностранной
                                  --  валюты)
    stor_obj_price                -- Цена в рублях 
      numeric( 14,2 ),            -- (фактическая цена за данную единицу
                                  --  хранения)
    stor_obj_price_note text,     -- Дополнительные сведения к цене
    acq_note text,                -- Доп. сведения о приобретении
    remove_stor_obj_flag bool     -- Флаг выбытия единицы хранения
      NOT NULL DEFAULT 'no',
    remove_stor_obj_note text,    -- Доп. информация о выбытии единицы хранения
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    CHECK ( ( pers_org_flag = 1 and owner_pers_ID > 0 and owner_org_ID = 0 )
            or 
            ( pers_org_flag = 2 and owner_pers_ID = 0 and owner_org_ID > 0 )
          ),
    PRIMARY KEY ( obj_ID, stor_obj_num ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( owner_pers_ID )
      REFERENCES Persons ( pers_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( owner_org_ID )
      REFERENCES Orgs ( org_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( phys_form_code )
      REFERENCES Phys_form_types ( phys_form_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( acq_place_city )
      REFERENCES Cities ( city_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( acq_way_code )
      REFERENCES Acq_ways ( acq_way_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( currency_code )
      REFERENCES Currencies ( currency_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Экземпляры, отданные/взятые во временное пользование"
-- ---------------------------------------------------------------------------
-- DROP TABLE Lend_borrow;

CREATE TABLE Lend_borrow
  ( obj_ID int4 NOT NULL,         -- Уникальный идентификатор объекта
                                  -- библиографического описания
    stor_obj_num int2 NOT NULL,   -- Порядковый номер единицы хранения
                                  -- для данного объекта библиогр. описания
    -- Информация о владельце (или временном пользователе) данной единицы
    -- хранения
    pers_org_flag numeric( 1 )    -- Флаг "персоналия/организация"
      NOT NULL DEFAULT 1          -- (1 - персоналия; 2 - организация)
      CHECK ( pers_org_flag = 1 or pers_org_flag = 2 ),
    borr_lend_pers_ID int4        -- Код владельца (или временного пользователя)
      NOT NULL DEFAULT 0,         -- (персоналии)
    borr_lend_org_ID int4         -- Код владельца (или временного пользователя)
      NOT NULL DEFAULT 0,         -- (организации)
    -- ----------------------------------------------------------
    borr_lend_flag numeric( 1 )   -- Признак "взятый/отданный" объект
      NOT NULL DEFAULT 1,         -- (1 - взятый, 2 - отданный)
    borr_lend_date date           -- Дата взятия/отдачи объекта
      NOT NULL,
    return_date_supp date         -- Дата возврата объекта - предполагаемая
      NOT NULL,
    return_date_fact date,        -- Дата возврата объекта - фактическая
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    CHECK ( borr_lend_date <= return_date_supp ),
    CHECK ( return_date_fact IS NULL or return_date_fact >= borr_lend_date ),
    FOREIGN KEY ( obj_ID, stor_obj_num )
      REFERENCES Stor_objs ( obj_ID, stor_obj_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- ---------------------------------------------------------------------------
-- Таблица "Проработка содержания объектов библиографического описания"
-- ---------------------------------------------------------------------------
-- DROP TABLE Studying;

CREATE TABLE Studying
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    study_begin date,             -- Дата начала проработки
    study_end date NOT NULL,      -- Дата окончания проработки
    study_descr text,             -- Описание вида проработки
    study_result text,            -- Описание результата проработки
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    CHECK ( study_begin <= study_end ),
    PRIMARY KEY ( obj_ID, study_begin ),
    FOREIGN KEY ( obj_ID )
      REFERENCES Bib_objs ( obj_ID )
      ON DELETE CASCADE
      ON UPDATE CASCADE
  );

-- DROP INDEX Studying_ind_1;
CREATE INDEX Studying_ind_1 ON Studying ( study_begin );

-- ---------------------------------------------------------------------------
-- Таблица "Заглавия"
-- ---------------------------------------------------------------------------
-- DROP TABLE Titles;

CREATE TABLE Titles
  ( obj_ID int4,                  -- Уникальный идентификатор объекта
                                  -- библиографического описания
    bib_rec_num int2              -- Номер библиогр. записи для данного объекта
      NOT NULL DEFAULT 0          -- (основная запись имеет номер 0)
      CHECK ( bib_rec_num >= 0 ),
    title_type_code int2          -- Код вида заглавия (1 - основное,
      DEFAULT 1,                  -- 2 - параллельное, 3 - другое,
                                  -- 4 - ключевое (для сериального издания))
    title_num int2                -- Номер заглавия (для параллельных
      DEFAULT 1                   -- заглавий, для сборников произведений
      CHECK ( title_num >= 1 ),   -- разных авторов)
    obj_title text NOT NULL,      -- Заглавие
    lang_code int2 NOT NULL       -- Код языка заглавия
      DEFAULT 1,                  -- (1 - это должен быть код русского языка)
    process_code int2             -- Код вида обработки заглавия
      NOT NULL DEFAULT 1,         -- (1 - оригинальное)
    private_title bool NOT NULL   -- Признак частного заглавия тома (выпуска)
      DEFAULT 'no',    
    obj_title_info text,          -- Сведения, относящиеся к заглавию
                                  -- (разделять знаком ":")
    comment text,                 -- Примечание
    -- служебная информация
    who_chg text DEFAULT USER,    -- Кем добавлена (изменена) запись
    when_chg timestamp 
      DEFAULT CURRENT_TIMESTAMP,  -- Когда добавлена (изменена) запись
    PRIMARY KEY ( obj_ID, bib_rec_num, title_type_code, title_num ),
    FOREIGN KEY ( obj_ID, bib_rec_num )
      REFERENCES Bib_recs ( obj_ID, bib_rec_num )
      ON DELETE CASCADE
      ON UPDATE CASCADE,
    FOREIGN KEY ( title_type_code )
      REFERENCES Title_types ( title_type_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( lang_code )
      REFERENCES Langs ( lang_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE,
    FOREIGN KEY ( process_code )
      REFERENCES Process_types ( process_code )
      ON DELETE SET NULL
      ON UPDATE CASCADE
  );
