#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:   "Электронный архив"
# Программа: вспомогательная программа для формирования правил базы данных
#            для "теневых" таблиц
# Автор:     Моргунов Е. П.
# Дата:      15.04.2012
# Версия:    0.6
# -----------------------------------------------------------------------------

# использование программы
# ./mk_rules.pl < shadows.sql > rules.sql
# shadows.sql - файл с описаниями структур "теневых" таблиц
# rules.sql - файл с описаниями правил


use strict;

my $title;         # наименование таблицы (текстовый заголовок)
my $table;         # имя таблицы
my @struct = ();   # имена полей таблицы
my $field_name;    # имя поля

# флаг, указывающий, нужно ли формировать команду DROP RULE
# (если нужно ее формировать, то надо назначить значение 1)
my $drop = 0;

# флаг, означающий, что мы находимся внутри описания таблицы
my $in_table = 0;

# выведем информацию о текущем времени и дате
print "-- Этот файл создан программой " . $0 . "  " . localtime() . "\n\n\n";

print <<EOF;
-- ---------------------------------------------------------------------------
-- Правила для таблиц базы данных "Электронный архив"
-- ---------------------------------------------------------------------------

EOF

# обработаем файл, содержащий описания структур "теневых" таблиц
while ( <STDIN> )
{
  chomp;             # уберем символ новой строки
  next if $_ eq '';  # пустые строки пропускаем

  # заголовок таблицы
  if ( /^-- Таблица/ )
  {
    $title = $_;
  }
  # раздел в описании структуры таблицы, содержащий типовые поля
  elsif ( /^\s*-- служебная/ )
  {
    $in_table = 0;     # больше поля брать не нужно
    create_rules();    # формируем правила
  }
  elsif ( /^\s*--/ )
  {
    next;     # строки-комментарии пропускаем
  }
  # команда на создание таблицы
  elsif ( /^CREATE\s+TABLE\s+(.+)\s*/ )
  {
    ( $table ) = $_ =~ /^CREATE\s+TABLE\s+(.+)\s*/;
    $table =~ s/_2//;  # уберем символы "_2"
    $in_table = 1;     # флаг - мы внутри описания таблицы
    @struct = ();      # очищаем временный массив
    next;
  }
  # если это строка, описывающая поле таблицы, и при этом данное поле находится
  # не в разделе "служебная информация"
  elsif ( /^\s+\(*\s+(.+)\s+/ && $in_table == 1 )
  {
    # получим имя поля и поместим его во временный массив
    # ПРИМЕЧАНИЕ. Оператор делаем "нежадным" (т.е. до первого пробела) 
    #             и учитываем, что на строке первого поля есть левая скобка.
    ( $field_name ) = $_ =~ /^\s+\(*\s+(.+?)\s+/;
    push( @struct, $field_name );
  }
}

exit( 0 );

# ----------------------------------------------------------------------------
# Формирование команд для создания правил
# ----------------------------------------------------------------------------
sub create_rules
{
  my $f_name;    # имя поля
  
  # заголовок для описания правил данной таблицы
  print "-- " . "-" x 77 . "\n";
  print "$title\n";
  print "-- " . "-" x 77 . "\n";

  print "-- сохраним добавленную запись\n";

  # эту команду нужно формировать, если задан специальный флаг
  print "DROP RULE ${table}_rule_1 ON ${table};\n\n" if ( $drop );

  print "CREATE RULE ${table}_rule_1 AS\n";
  print "  ON INSERT TO ${table}\n";
  print "  DO INSERT INTO ${table}_2 ( \n";

  # помещаем все поля в описание правила
  foreach $f_name ( @struct )
  {
    print "            $f_name,\n";
  }
  # добавим поле, в котором будет храниться наименование операции
  print "            db_operation )\n";

  print "     VALUES ( \n";

  # помещаем все поля в описание правила
  foreach $f_name ( @struct )
  {
    print "            NEW.$f_name,\n";
  }
  # добавим значение для поля, в котором будет храниться наименование операции
  print "            'INSERT' );\n\n";

  # ----------------------------------------------------------------------
  print "-- сохраним обновленную запись\n";

  # эту команду нужно формировать, если задан специальный флаг
  print "DROP RULE ${table}_rule_1 ON ${table};\n\n" if ( $drop );
  
  print "CREATE RULE ${table}_rule_2 AS\n";
  print "  ON UPDATE TO ${table}\n";
  print "  DO INSERT INTO ${table}_2 ( \n";

  # помещаем все поля в описание правила
  foreach $f_name ( @struct )
  {
    print "            $f_name,\n";
  }
  # добавим поле, в котором будет храниться наименование операции
  print "            db_operation )\n";

  print "     VALUES ( \n";

  # помещаем все поля в описание правила
  foreach $f_name ( @struct )
  {
    print "            NEW.$f_name,\n";
  }
  # добавим значение для поля, в котором будет храниться наименование операции
  print "            'UPDATE' );\n\n";

  # ----------------------------------------------------------------------
  print "-- сохраним удаленную запись\n";

  # эту команду нужно формировать, если задан специальный флаг
  print "DROP RULE ${table}_rule_1 ON ${table};\n\n" if ( $drop );

  print "CREATE RULE ${table}_rule_3 AS\n";
  print "  ON DELETE TO ${table}\n";
  print "  DO INSERT INTO ${table}_2 ( \n";

  # помещаем все поля в описание правила
  foreach $f_name ( @struct )
  {
    print "            $f_name,\n";
  }
  # добавим поле, в котором будет храниться наименование операции
  print "            db_operation )\n";

  print "     VALUES ( \n";

  # помещаем все поля в описание правила
  foreach $f_name ( @struct )
  {
    print "            OLD.$f_name,\n";
  }
  # добавим значение для поля, в котором будет храниться наименование операции
  print "            'DELETE' );\n\n";

  return;
}
