#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:   "Электронный архив"
# Программа: вспомогательная программа для формирования структур
#            "теневых" таблиц на основе структур основных таблиц
# Автор:     Моргунов Е. П.
# Дата:      15.04.2012
# Версия:    0.6
# -----------------------------------------------------------------------------

# использование программы
# ./mk_shad_tables.pl < source.sql > shadows.sql
# source.sql - файл с описаниями структур основных таблиц
# shadows.sql - файл с описаниями структур "теневых" таблиц

use strict;

my $title;         # наименование таблицы (текстовый заголовок)
my $table;         # имя таблицы
my @struct = ();   # массив описаний полей таблицы
my $field_name;    # имя поля
my $field_type;    # тип данных поля

# описание текущего поля
# включаем в описание только имя поля и тип данных, а все ограничения -
# NOT NULL, DEFAULT... - отбрасываем
my $current_field = ''; 

# флаг, означающий, что мы находимся внутри описания таблицы
my $in_table = 0;

# выведем информацию о текущем времени и дате
print "-- Этот файл создан программой " . $0 . "  " . localtime() . "\n\n\n";

print <<EOF;
-- ---------------------------------------------------------------------------
-- Структуры таблиц базы данных "Электронный архив"
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- "Теневые" таблицы
-- ---------------------------------------------------------------------------

EOF

# обработаем файл, содержащий описания структур таблиц базы данных
while ( <STDIN> )
{
  chomp;             # уберем символ новой строки
  next if $_ eq '';  # пустые строки пропускаем

  # заголовок таблицы
  if ( /^--\s*Таблица/ )
  {
    $title = $_;
    next;
  }
  
  # раздел в описании структуры таблицы, содержащий типовые поля
  if ( /^\s*--\s*служебная/ )
  {
    $in_table = 0;     # больше поля брать не нужно
    create_struct();   # формируем структуру одной таблицы
    next;
  }

  # строки-комментарии пропускаем
  # ПРИМЕЧАНИЕ. Важно, чтобы эта проверка проводилась после проверок
  #             if ( /^--\s*Таблица/ ) и if ( /^\s*--\s*служебная/ )
  if ( /^\s*--/ )
  {
    next;
  }
  
  # если встретилась команда на создание таблицы
  if ( /^CREATE\s+TABLE\s+\w+\s*/ )
  {
    # берем имя таблицы
    ( $table ) = $_ =~ /^CREATE\s+TABLE\s+(\w+).*/;

    $in_table = 1;     # флаг - мы внутри описания таблицы
    @struct = ();      # очищаем временный массив
    next;
  }

  # если описания интересующих нас элементов в таблице закончились
  if ( $in_table == 0 )
  {
    next;
  }

  # собираем информацию для текущего поля
  
  s/^\s*\(//;          # уберем левую скобку, которая стоит перед описанием
                       # первого поля
  s/^\s*//;            # уберем все пробелы в начале строки
  s/--.*$//;           # отбрасываем комментарий до конца строки, если он есть
  s/\s*$//;            # также удаляем все пробелы между комментарием и
                       # описанием поля
  $_ = $_ . ' ';       # добавим один пробел, чтобы между фрагментами описания
                       # текущего поля гарантированно был пробел
  
  $current_field .= $_;     # формируем описание текущего поля
  
  # если это последний фрагмент описания текущего поля (он определяется
  # по наличию запятой в самом конце описания поля)
  # ПРИМЕЧАНИЕ. Внутри описания поля также может оказаться запятая.
  #             Это возможно при описании поля с типом данных numeric,
  #             например: "numeric( 14,2 )".
  if ( $current_field =~ /,\s*$/ )
  {
    # заменим последнюю запятую на пробел
    # (возможно, ее лучше просто удалять, но тогда нужно проверить
    # регулярное выражение для получения имени поля и типа данных)
    $current_field =~ s/,\s*$/ /;
    

    # возьмем из описания только имя поля и тип данных
    # ОПИСАНИЕ РЕГУЛЯРНОГО ВЫРАЖЕНИЯ.
    # (\w+)  - первое слово в анализируемой строке, т.е. имя поля
    # \s+    - пробел(ы) между именем поля и его типом данных
    # (\w+\(*\s*\d*,?\d*\s*\)*?) - тип данных поля
    #   \w+  - первое слово (если тип данных не содержит цифр в скобках,
    #          то следующий фрагмент рег. выражения ничего "не ловит")
    #   \(*\s*\d*,?\d*\s*\)*? - этот фрагмент нужен для обнаружения цифр
    #          в скобках, например: "numeric( 14,2 )" (очевидно, что данный
    #          фрагмент очень "рыхлый" и может быть значительно улучшен)
    # ПРИМЕЧАНИЕ. Следует помнить, что \w означает буквы, цифры и символ "_".
    ( $field_name, $field_type ) = $current_field =~
                   /^\s*(\w+)\s+(\w+\(*\s*\d*,?\d*\s*\)*)\s+/;

    # поместим описание текущего поля во временный массив
    push( @struct, "$field_name $field_type" );
    $current_field = '';
  }
}
  
exit( 0 );

# ----------------------------------------------------------------------------
# Формирование структуры одной таблицы
# ----------------------------------------------------------------------------
sub create_struct
{
  my $field_descr;   # описание поля
  my $bracket = 0;   # признак печати левой скобки
  
  print "-- " . "-" x 75 . "\n";
  print $title . "\n";
  print "-- " . "-" x 75 . "\n";

  # вставим эту команду, но в закомментированном виде
  print "-- DROP TABLE ${table}_2;\n\n";

  print "CREATE TABLE ${table}_2\n";
  
  foreach $field_descr ( @struct )
  {
    if ( $bracket == 0 )   # первое поле таблицы
    {
      print "  ( ";
      $bracket = 1;
    }
    else                   # для последующих полей делаем отступ в 4 пробела
    {
      print " " x 4;
    }

    print $field_descr . ",\n";
  }

  # выведем обязательные поля
  print <<EOF;
    who_chg text,
    when_chg timestamp,
    -- служебная информация
    db_operation text,                            -- операция над БД
    who_add text DEFAULT USER,                    -- Кем добавлена запись
    when_add timestamp DEFAULT CURRENT_TIMESTAMP  -- Когда добавлена запись
EOF

  print "  );\n\n";
  return;
}
