#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Библиографические описания (записи)"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'BIB_RECS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Bib_recs
my @f_names =
   ( 'obj_id',         # Уникальный идентификатор объекта
                       # библиографического описания
     'bib_rec_num',    # Номер библиогр. записи для данного объекта
                       # (основная запись имеет номер 0)
     'gen_mat_code',   # Код общего обозначения материала (1 - "текст")
     'bib_rec_head',   # Заголовок библиографической записи
     'bib_rec_full',   # Полная библиографическая запись
     'bib_rec_short',  # Краткая библиографическая запись
     'ord_unit_num',   # Порядковая единица для сериальных
                       # и многотомных изданий
     'source_code',    # Код источника библиографического описания
     'details_flag',   # Флаг наличия "деталировки" в базе данных
     'lang_code',      # Код языка, на котором составлено
                       # библиографическое описание (1 - русский язык)
     'process_code',   # Код вида обработки библиограф. описания
                       # (1 - оригинальное, т.е. в том виде, как
                       # приведено в источнике библиограф. описания)
     'reason_add_rec', # Назначение дополнит. описания (или причина
                       # его составления)
     'flags',          # Поле флагов (в расчете на непредвиденные случаи)
     'comment'         # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# проверим, есть ли у пользователя право на вход в этот модуль
if ( ! can_work( $user, $module, 'main', $query ) )
{
  exit( 0 );
}

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# первое условие -- для запуска скрипта из головного модуля;
# второе условие -- для запуска скрипта "из самого себя" (например,
# при выборе конкретной буквы, с которой должны начинаться биб. описания);
# третье условие -- использование модуля в качестве справочника;
# четвертое условие -- для запуска скрипта из головного модуля с целью
# просмотра БД (схемы) экспорта/импорта (число параметров должно быть равно 1,
# при этом имя параметра должно быть 'exp_imp')
if ( ( ! $query->path_info && ! $query->param ) ||
     $query->path_info =~ /^\/query$/ ||
     $query->path_info =~ /^\/select$/ ||
     ( ! $query->path_info && scalar $query->param() == 1 && 
       $query->param( 'exp_imp' )
     )
   )
{
  do_query();
}

# выбираем одну запись из БД для указанного объекта библиографического
# описания в виде формы с полной информацией
elsif ( $query->path_info =~ /^\/view$/ &&
        have_access( 'module' => $module, 'procedure' => 'VIEW',
                     'user' => $user )
      )
{
  do_view();
}

# добавление записи (организация формы для ввода данных)
elsif ( $query->path_info =~ /^\/add$/ &&
        have_access( 'module' => $module, 'procedure' => 'ADD',
                     'user' => $user )
      )
{
  do_add( query => $query,
          create_top => \&create_top_full,
          make_form => \&make_form,
          action => $script_name,
          target => "_self",
          submit_name => "submit_add",
          submit_value => "Добавить"
        );
}

# удаление записи
elsif ( $query->path_info =~ /^\/delete$/ &&
        have_access( 'module' => $module, 'procedure' => 'DELETE',
                     'user' => $user )
      )
{
  do_del();
}

# поиск записей по критерию
elsif ( $query->path_info =~ /^\/find$/ )
{
  do_find();
}

# выборка всех биб. описаний объектов, "СОДЕРЖАЩИХСЯ" в данном объекте
elsif ( $query->path_info =~ /^\/list_contained_objs$/ )
{
  do_list_contained_objs();
}

# экспортирование записи
elsif ( $query->path_info =~ /^\/export$/ &&
        have_access( 'module' => $module, 'procedure' => 'EXPORT',
                     'user' => $user )
      ) 
{
  do_export();
}

# импортирование записи
elsif ( $query->path_info =~ /^\/import$/ &&
        have_access( 'module' => $module, 'procedure' => 'IMPORT',
                     'user' => $user )
      ) 
{
  do_import();
}

# непосредственный ввод записи (с проверкой правильности ввода)
# используем нестандартную функцию, т.к. необходимо ввести запись не только
# в таблицу Bib_recs, но также и в таблицу Bib_objs
if ( $query->param( 'submit_add' ) &&
     have_access( 'module' => $module, 'procedure' => 'ADD', 'user' => $user )
   )
{
  add_rec_non_std( $query, \&create_top_full, \&check_param, \&make_command );
}

# непосредственное обновление записи (с проверкой правильности ввода)
elsif ( $query->param( 'submit_update' ) &&
        have_access( 'module' => $module, 'procedure' => 'UPDATE',
                     'user' => $user )
      )
{
  update_rec( 'query' => $query, 
              'create_top' => \&create_top_short,
              'check_param' => \&check_param,
              'make_command' => \&make_command );
}

exit( 0 );


# -----------------------------------------------------------------------------
# добавление в глобальный хеш-массив %CFG параметров для реализации
# "быстрой" вставки последних персоналий, введенных в таблицу Persons
# -----------------------------------------------------------------------------
sub add_conf_params_for_pers
{
  # эта функция дополняет глобальный хеш-массив %CFG таким образом, как будто
  # список персоналий был введен в конфигурационный файл e_arch.cfg

  # параметр - имя поля, для которого формируются дополнительные 
  # конфигурационные параметры
  my $field_name = shift;

  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД

  my $param_name;                 # имя параметра из хеша %CFG
  my $param_value;                # формируемое значение параметра
  
  # если НЕ задано значение конфигурационного параметра для числа последних
  # записей, введенных в таблицу Persons
  if ( ! $CFG{ PERSONS_NUM_LAST_RECS } )
  {
    return ( 0 );
  }  

  # значение конфигурационного параметра задано

  # выбираем ПОСЛЕДНИЕ записи о персоналиях
  # ПРИМЕЧАНИЕ. Если даже требуемое число последних введенных записей будет
  #             больше числа всех записей, имеющихся в таблице, этот 
  #             оператор сработает. Если какие-то записи были удалены,
  #             вследствие чего образовались "дырки" в нумерации pers_id,
  #             то просто будет выведено меньшее число записей.
  $command = "SELECT pers_name " .
             "FROM ${db_schema}persons " .
             "WHERE pers_id >= ( SELECT max( pers_id ) " .
                                "FROM ${db_schema}persons ) - " .
                   "$CFG{ PERSONS_NUM_LAST_RECS } + 1 " .
             "ORDER BY pers_name";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # определим максимальный номер уже созданных "быстрых" вставок
    # для поля $field_name

    my $max_index = -1;
    
    foreach ( keys %CFG )
    {
      # если этот параметр - заголовок "быстрой" вставки для поля $field_name
      if ( /^${field_name}_HELP_(LINK|BTN)_TITLE(\d+)$/ )
      {
        $max_index = $2 if ( $2 > $max_index );
      }
    }

    $max_index++;         # используем следующий номер для нашего параметра
    
    # пока что будем вызывать эту "быструю" подсказку только по HTML-ссылке
    $param_name = $field_name . '_HELP_LINK_TITLE' . $max_index;

    # текст HTML-ссылки пока что также назначим в программном модуле
    # TODO. Подумать над заданием этого текста в конфиг. файле.
    $CFG{ $param_name } = 'Персоналии';

    $i = 1;

    # выбираем все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # формируем имя параметра и записываем его значение в хеш %CFG
      $param_name = $field_name . '_HELP_VALUE' . $max_index . '_' . $i;

      # формируемое значение параметра зависит от имени поля
      if ( $field_name eq "BIB_RECS_BIB_REC_HEAD" )
      {
        # берем значение из таблицы Persons без изменений
        $param_value = $row[ $result->fnumber( 'pers_name' ) ];

        # если в конце полученного значения нет точки, поставим ее,
        # т. к. в заголовке биб. описания нужна точка
        $param_value .= '.' if ( $param_value !~ /\.$/ )
      }
      elsif ( $field_name eq "BIB_RECS_BIB_REC_FULL" )
      {
        # фамилия, запятая, пробелы (если есть), инициал имени, остальные буквы
        # имени (если есть), пробел(ы), инициал отчества или второго имени,
        # остальные буквы отчества (если есть), пробелы (если есть)
        if ( $row[ $result->fnumber( 'pers_name' ) ] =~
             /^(\S+),\s*(\S)\S*\s+(\S)\S*\s*$/ )
        {
          # представим значение поля в виде: инициалы и фамилия
          $param_value = $2 . '. ' . $3 . '. ' . $1;
        }
        # аналогично первому случаю, но только два компонента
        elsif ( $row[ $result->fnumber( 'pers_name' ) ] =~
                /^(\S+),\s*(\S)\S*\s*$/ )
        {
          # представим значение поля в виде: первый инициал и фамилия
          # (этот вариант -- для многих, но не всех, иностранных персоналий)
          $param_value = $2 . '. ' .  $1;
        }
        # рассматриваем всего два компонента: первый - фамилия, а второй
        # включает в себя все остальное (таким образом, многокомпонентные
        # случаи должны обрабатываться в этой ветви)
        elsif ( $row[ $result->fnumber( 'pers_name' ) ] =~
                /^(\S+),\s*(.+)\s*$/ )
        {
          $param_value = $2 . ' ' . $1;
        }
        else    # все более сложные случаи пока что не обрабатываем
        {
          # TODO. Сделать обработку частиц типа von, van der и т. д.
          
          # берем значение из таблицы Persons без изменений
          $param_value = $row[ $result->fnumber( 'pers_name' ) ];
        }
      }

      $CFG{ $param_name } = $param_value;
      $i++;
    }
  }
  return ( 0 );
}

# -----------------------------------------------------------------------------
# непосредственное добавление записи в БД
# ПРИМЕЧАНИЕ. В этой процедуре выполняется добавление записи не только
#             в таблицу Bib_recs, но также и в таблицу Bib_objs
# -----------------------------------------------------------------------------
sub add_rec_non_std
{
  my $query = shift;        # параметр - CGI-объект 

  # параметр - ссылка на функцию для формирования верхней части окна
  # со списком "команд" для пользователя
  my $create_top = shift;

  my $check_param = shift;  # параметр - ссылка на функцию для проверки
                            # значений, введенных в поля формы
  my $make_command = shift; # параметр - ссылка на функцию для формирования
                            # строки команды для работы с БД

  my( $conn, $result );     # для работы с БД
  my $command;              # строка команды для добавления записи в таблицу

  # уникальный идентификатор объекта биб. описания
  # (значение этой переменной формируется в процедуре make_form())
  my $obj_id = $query->param( 'obj_id' );

  # формируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();          # начинаем оформление верхней части окна
  &{ $create_top }();  # формируем верхнюю часть окна со списком "команд" 
                       # для пользователя
  close_top();         # завершаем оформление верхней части окна

  open_bottom();       # начинаем оформление нижней части окна

  # если проверка введенных значений прошла успешно, выполним добавление
  if ( &{ $check_param }() )
  {
    $conn = exec_connect_db();          # подключаемся к БД
    exec_transaction( $conn, "BEGIN" ); # выполним операции в рамках транзакции

    # сначала сформируем строку-команду ...
    $command = "INSERT INTO Bib_objs ( obj_id, flags, comment, who_chg ) " .
               "VALUES ( $obj_id, null, null, '$user' )"; 
    # DEBUG 
    print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

    # ... и вставим запись в таблицу Bib_objs
    $result = $conn->exec( $command );

    # если произошла ошибка при вставке записи в базу данных
    if ( PGRES_COMMAND_OK ne $result->resultStatus )
    {
      # сообщение для пользователя
      print "<p class=\"error\">$CFG{ ERROR_INSERT }</p>\n";
      print STDERR $command . "\n";           # выводим строку команды в лог
      die $conn->errorMessage;
    }
    else
    {
      print "<p class=\"info\">";
      print( "Запись добавлена в базу данных (таблица \"Общий список " .
             "объектов библиографического описания\")</p>\n" );
    }

    # теперь проделаем то же самое с таблицей Bib_recs

    # сформируем строку-команду для вставки записи
    $command = &{ $make_command }( 'APPEND' );

    # DEBUG 
    print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

    # выполним команду вставки записи в БД
    $result = $conn->exec( $command );

    # если произошла ошибка при вставке записи в базу данных
    if ( PGRES_COMMAND_OK ne $result->resultStatus )
    {
      # сообщение для пользователя
      print "<p class=\"error\">$CFG{ ERROR_INSERT }</p>\n";
      print STDERR $command . "\n";           # выводим строку команды в лог
      die $conn->errorMessage;
    }
    else
    {
      print "<p class=\"info\">";
      print( "Запись добавлена в базу данных (таблица \"Библиографические " .
             "описания (записи)\")</p>\n" );
    }
    exec_transaction( $conn, "END" );     # конец транзакции
  }

  close_bottom();      # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # должны быть только цифры (пустое поле не допускается)
  if ( $query->param( 'obj_id' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный уникальный " .
          "идентификатор объекта библиографического описания" );
  }

  # должны быть только цифры (пустое поле не допускается)
  if ( $query->param( 'bib_rec_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный номер библиогр. записи " .
          "для данного объекта библиографического описания" );
  }

  if ( $query->param( 'gen_mat_code' ) eq '' )
  {
    push( @error_messages, "Не указан код общего обозначения материала" );
  }

  if ( $query->param( 'source_code' ) eq '' )
  {
    push( @error_messages, "Не указан код источника библиографического " .
          "описания" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  if ( $query->param( 'process_code' ) eq '' )
  {
    push( @error_messages, "Не указан код вида обработки библиографического " .
          "описания" );
  }

  if ( $query->param( 'bib_rec_full' ) eq '' &&
       $query->param( 'bib_rec_short' ) eq '' )
  {
    push( @error_messages, "Нужно ввести хотя бы одно описание: " .
          "либо полное, либо краткое, либо оба" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя -- полный вариант 
# -----------------------------------------------------------------------------
sub create_top_full
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    make_letters => 'yes',
                    num_last_recs => $CFG{ BIB_RECS_NUM_LAST_RECS },
                    full_top => 'yes'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя -- сокращенный вариант 
# -----------------------------------------------------------------------------
sub create_top_short
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my( $conn, $result );      # для работы с БД
  my @row;                   # для хранения одной записи из выборки
  my $obj_id;                # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;           # номер библиогр. записи для данного объекта
  my $command;               # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();          # начинаем оформление верхней части окна
  create_top_full();   # формируем верхнюю часть окна со списком
                       # "команд" для пользователя
  close_top();         # завершаем оформление верхней части окна

  open_bottom();       # начинаем оформление нижней части окна

  $conn = exec_connect_db();           # подключаемся к БД
  exec_transaction( $conn, "BEGIN" );  # выполним операции в рамках транзакции

  # должна быть удалена только ОДНА запись с таким ключом
  # из таблицы Bib_recs (из всех других таблиц, связанных с таблицей 
  # Bib_recs, записи удалит СУБД)
  $command = "DELETE FROM ${db_schema}bib_recs " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num'";
  # DEBUG 
  print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

  $result = $conn->exec( $command );   # теперь выполняем удаление из БД

  # если произошла ошибка при удалении записей из базы данных
  if ( PGRES_COMMAND_OK ne $result->resultStatus )
  {
    # сообщение для пользователя
    print "<p class=\"error\">$CFG{ ERROR_DELETE }</p>\n";
    print STDERR $command . "\n";           # выводим строку команды в лог
    die $conn->errorMessage;
  }

  # проверим число фактически удаленных записей
  if ( $result->cmdTuples == 1 )
  {
    print "<p class=\"info\">";
    print "Библиографическое описание удалено из базы данных</p>\n";
  }
  elsif ( $result->cmdTuples == 0 )
  {
    # число записей может оказаться равным 0, если пользователь удалил
    # конретное биб. описание для данного объекта, а затем с помощью
    # кнопки "Назад" браузера отобразил ту страницу, на которой была
    # отображена web-ссылка для удаления этого биб. описания
    # (т. е. устаревшую страницу), и попытался снова выполнить удаление
    print "<p class=\"info\">";
    print "Библиографическое описание из базы данных НЕ удалено</p>\n";

    exec_transaction( $conn, "END" );      # конец транзакции
    print $query->end_html();
    return ( 0 );    
  }
  # невероятный случай -- но предусмотрим и его
  else          # ( $result->cmdTuples > 1 )
  {
    print "<p class=\"error\">";
    print "Из базы данных могли быть удалены " . $result->cmdTuples .
          " записей. Это ошибка! Обратитесь к программисту</p>\n";

    exec_transaction( $conn, "ROLLBACK" );      # откат транзакции
    print $query->end_html();
    return ( 0 );
  }

  # ---------------------------------------------------------------------
  # ВАЖНО! Если удаляется последнее биб. описание для данного объекта, 
  #        то нужно удалять также и запись из таблицы Bib_objs.
  # ---------------------------------------------------------------------  

  # в том случае, если еще остались биб. описания для данного объекта,
  # эта команда сработает корректно, но не будет удалено ни одной записи
  # из таблицы Bib_objs
  $command = "DELETE FROM ${db_schema}bib_objs o " .
             "WHERE o.obj_id = '$obj_id' AND " .
                   "NOT EXISTS ( SELECT * FROM ${db_schema}bib_recs r " .
                                "WHERE r.obj_id = '$obj_id' )";
  # DEBUG 
  print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

  $result = $conn->exec( $command );    # теперь выполняем удаление из БД

  # если произошла ошибка при удалении записей из базы данных
  if ( PGRES_COMMAND_OK ne $result->resultStatus )
  {
    # сообщение для пользователя
    print "<p class=\"error\">$CFG{ ERROR_DELETE }</p>\n";
    print STDERR $command . "\n";           # выводим строку команды в лог
    die $conn->errorMessage;
  }

  # проверим число фактически удаленных записей
  if ( $result->cmdTuples == 1 )
  {
    print "<p class=\"info\">";
    print "Объект библиографического описания удален из базы данных</p>\n";
  }
  elsif ( $result->cmdTuples == 0 )
  {
    print "<p class=\"info\">";
    print "В базе данных еще осталось библиографическое описание для " .
          "данного объекта</p>\n";
  }
  else
  {
    ;   # считаем, что СУБД не позволит удалить больше одной записи
  }

  exec_transaction( $conn, "END" );      # конец транзакции
  close_bottom();      # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );    
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  # ВАЖНО!!! Будем экспортировать только текущее (выбранное пользователем)
  #          биб. описание для данного объекта. 
  # TODO. Конечно, возникнет проблема в том случае, когда впоследствии придется
  #       импортировать эти два (или более) биб. описаний одного и того же 
  #       объекта в целевую базу данных.
  
  my $obj_id;                # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;           # номер библиогр. записи для данного объекта
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее
                             # TODO. Где используется именно массив? Не помню.

  $obj_id = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );

  # сформируем строку-команду для главной таблицы и поместим ее в массив 
  # (используем ссылку на массив из двух элементов, первым из которых является
  # имя таблицы)
  $command = "SELECT * FROM Bib_recs ". 
             "WHERE obj_id = $obj_id AND " .
                   "bib_rec_num = $bib_rec_num";

  push( @table_cmd_pairs, [ 'Bib_recs', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top_full,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();        # начинаем оформление верхней части окна
  create_top_full(); # формируем верхнюю часть окна
  close_top();       # завершаем оформление верхней части окна
  
  open_bottom();     # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();    # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  # ВАЖНО!!! Будем импортировать только текущее (выбранное пользователем)
  #          биб. описание для данного объекта. 

  # TODO. Конечно, возникает проблема в том случае, когда нужно импортировать
  #       второе (или более) биб. описание одного и того же объекта в целевую
  #       базу данных.
  
  my $obj_id;                # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;           # номер библиогр. записи для данного объекта
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $obj_id = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );

  # сформируем строку-команду для главной таблицы и поместим ее в массив 
  # (используем ссылку на массив из двух элементов, первым из которых является
  # имя таблицы)
  $command = "SELECT * FROM exp_imp.Bib_recs " .
             "WHERE obj_id = $obj_id AND " .
                   "bib_rec_num = $bib_rec_num";

  push( @table_cmd_pairs, [ 'Bib_recs', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top_full,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# выборка и вывод биб. описаний всех объектов, "содержащихся" в данном объекте
# -----------------------------------------------------------------------------
sub do_list_contained_objs
{
  my @row;                    # для хранения одной записи из выборки
  my $command;                # строка команды для работы с БД
  my $obj_id;                 # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;            # номер библиогр. записи для данного объекта
  my $bib_rec;                # биб. описание
  my $target_win;             # имя окна браузера для вывода HTML-страницы
  my $headline;               # заголовок списка биб. описаний
  
  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();         # начинаем оформление верхней части окна
  create_top_short(); # формируем верхнюю часть окна
  close_top();        # завершаем оформление верхней части окна

  open_bottom();      # начинаем оформление нижней части окна

  # выведем уникальный идентификатор объекта биб. описания и номер библиогр.
  # записи для данного объекта в качестве заголовка
  print "<p class=\"bib_rec\">";
  print "Идентификатор объекта биб. описания: $obj_id\n";
  print "&nbsp;&nbsp;\n";
  print "Номер библиогр. записи для данного объекта: $bib_rec_num</p>\n";

  # если имеются проблемы с русскими буквами в HTML-ссылках
  $bib_rec = get_bib_rec( $obj_id, $bib_rec_num, $db_schema )
    if ( $CFG{ HIDE_RUS_LETTERS } );

  # выведем также биб. описание
  print "<p class=\"bib_rec\">$bib_rec</p>\n";

  # сформируем заголовок модуля
  $headline = "<h2 class=\"module_title\">Объекты, \"содержащиеся\" " .
              "в данном объекте</h2>\n";

  # выбираем все записи об объектах, для которых значения полей obj_id_cont
  # и bib_rec_num_cont в таблице Obj_containers равны значениям полей obj_id
  # и bib_rec_num, переданным в качестве параметров
  # ПРИМЕЧАНИЕ. Используем возможность PostgreSQL, позволяющую выбирать
  #             имя поля в зависимости от какого-то условия (это удобно,
  #             так как может быть заполнено только одно из полей
  #             'bib_rec_full' или 'bib_rec_short').
  $command = "SELECT b.obj_id, b.bib_rec_num, " .
                    "CASE WHEN b.bib_rec_short IS NOT NULL " .
                         "THEN b.bib_rec_short " .
                         "ELSE b.bib_rec_full " .
                    "END AS bib_rec " .
             "FROM ${db_schema}obj_containers o, ${db_schema}bib_recs b " .
             "WHERE o.obj_id_cont = '$obj_id' AND " .
                   "o.bib_rec_num_cont = '$bib_rec_num' AND " .
                   "o.obj_id = b.obj_id AND " .
                   "o.bib_rec_num = b.bib_rec_num " .
             "ORDER BY bib_rec";

  # вывод биб. описаний всех объектов, выбранных по запросу
  list_bib_recs_by_query( $query, $command, $headline );

  close_bottom();     # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка тех библиографических описаний (записей), которые 
# начинаются с указанной буквы
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $letter;                     # первая буква биб. описания
  my $letter_lower;               # первая буква -- в нижнем регистре
  my @avail_objs;                 # список физ. форм, в которых присутствуют
                                  # наличные экземпляры объекта биб. описания,
                                  # хранимые у персоналии, имеющей
                                  # идентификатор, указанный в конфиг. файле

  # число последних записей, введенных в таблицу, которые нужно выводить
  # при обращении по специальной ссылке
  my $num_last_recs;

  # условие WHERE для команды выборки
  my $where;

  # имя поля для биб. описания: bib_rec_short или bib_rec_full
  # ПРИМЕЧАНИЕ. А когда поле bib_rec_short может быть пустым?... Видимо, эта
  #             конструкция введена на всякий случай: мало ли что бывает
  #             на практике...
  my $bib_rec_fname = "CASE WHEN bib_rec_short IS NOT NULL " .
                           "THEN bib_rec_short " .
                           "ELSE bib_rec_full " .
                      "END ";

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;
  
  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки
  
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();        # начинаем оформление верхней части окна
  create_top_full(); # формируем верхнюю часть окна
  close_top();       # завершаем оформление верхней части окна

  open_bottom();     # начинаем оформление нижней части окна

  $letter        = $query->param( 'letter' );
  $num_last_recs = $query->param( 'num_last_recs' );

  # если параметры не переданы, то просто завершаем работу досрочно - 
  # это происходит при первоначальном запуске скрипта
  # ПРИМЕЧАНИЕ. Проверка как на undef, так и на пустое значение '' выполнена
  #             на случай каких-либо проблем в CGI.pm.
  if ( ( $letter eq undef || $letter eq '' ) &&
       ( $num_last_recs eq undef || $num_last_recs eq '' )
     )
  {
    close_bottom();   # завершаем оформление нижней части окна

    print $query->end_html();
    return ( 0 );
  }

  # если пользователь выбрал ссылку для вывода только последних записей,
  # введенных в таблицу
  if ( $num_last_recs )
  {
    # ПРИМЕЧАНИЕ. Если даже требуемое число последних введенных записей будет
    #             больше числа всех записей, имеющихся в таблице, этот 
    #             оператор сработает. Если какие-то записи были удалены,
    #             вследствие чего образовались "дырки" в нумерации obj_id,
    #             то просто будет выведено меньшее число записей.
    # ПРИМЕЧАНИЕ. В этом условии не учтен тот факт, что есть еще поле
    #             bib_rec_num, значение которого практически всегда равно
    #             нулю. Но если значение этого поля для каких-то описаний
    #             окажется не равным нулю (т. е. для некоторого объекта имеется
    #             более одного описания), тогда возможны два исхода:
    #             - запись, введенная в таблицу одной из ПОСЛЕДНИХ, не попадает
    #             в выборку, если значение поля obj_id для этого описания
    #             НЕ ПОПАДАЕТ в диапазон, формируемый параметром
    #             $num_last_recs, однако число записей в выборке будет таким,
    #             какого требует параметр $num_last_recs;
    #             - число записей в выборке будет больше, чем требует параметр
    #             $num_last_recs, если значение поля obj_id для этого
    #             описания ПОПАДАЕТ в диапазон, формируемый параметром
    #             $num_last_recs
    $where = "WHERE obj_id >= ( SELECT max( obj_id ) " .
                               "FROM ${db_schema}bib_recs ) - " .
                    "$num_last_recs + 1 ";
  }
  # пользователь выбрал конкретную букву или другой символ
  elsif ( $letter )
  {
    # ВАЖНО! Не удалять этот фрагмент. Пусть останется в исторических целях.
    
    # При переходе на версию KDE 4.4.5 оказалось, что браузер Konqueror
    # кодирует русские составляющие html-ссылок, передаваемых методом GET,
    # в кодировке UTF-8 (двумя байтами). В результате этого выборки по буквам
    # русского алфавита перестали работать. Принятое решение исходит из того,
    # что html-страница и база данных используют кодировку KOI8-R.
    # ПРИМЕЧАНИЕ. Другие браузеры (их новые версии, например, IE8, Chrome)
    #             поступают аналогично.
    # TODO. Нужно придумать более общее решение (когда, например, html-страница
    #       представлена в кодировке UTF-8, а база данных - в кодировке 
    #       CP1251).
    # DEBUG
    # print "letter " . $letter . "<br />\n";
    #{
    #  use Encode qw( from_to );
    #  # если длина переменной $letter более одного байта (в частности, два 
    #  # байта), то считаем, что это UTF-8
    #  from_to( $letter, "utf-8", "koi8-r" ) if ( length( $letter ) > 1 );
    #}

    # для русских букв может потребоваться особая предварительная обработка
    $letter = reveal_rus_letter( $letter );

    # воспользуемся возможностями локализации для перевода буквы, выбранной
    # пользователем, в нижний регистр
    $letter_lower = change_case( $letter, 'lower' );

    # выбираем все записи, в которых первая буква поля 'bib_rec_full' или
    # 'bib_rec_short' совпадает с указанной в качестве параметра, при этом
    # регистр символов не имеет значения (первая буква может быть и строчной)
    # если первый символ -- двойные кавычки, то не учитываем его
    $where = "WHERE $bib_rec_fname ~ '^($letter|\"$letter|" .
                                       "$letter_lower|\"$letter_lower)' ";
  }

  # ПРИМЕЧАНИЕ. Используем возможность PostgreSQL, позволяющую выбирать
  #             имя поля в зависимости от какого-то условия (это удобно,
  #             так как может быть заполнено только одно из полей
  #             'bib_rec_full' или 'bib_rec_short').
  #             К сожалению, пришлось продублировать конструкцию CASE еще
  #             два раза (в WHERE и ORDER BY): использование псевдонима 
  #             bib_rec не сработало (возможно, я где-то ошибся?).
  $command = "SELECT obj_id, bib_rec_num, $bib_rec_fname AS bib_rec " .
             "FROM ${db_schema}bib_recs " .
             $where .
  # если первый символ в поле -- кавычки, то не учитываем его, а берем
  # значение поля, начиная со второго символа;
  # переведем значение поля в нижний регистр для того, чтобы регистр символов
  # не влиял на порядок сортировки (наверное, можно было переводить и 
  # в верхний регистр -- все равно)
             "ORDER BY CASE WHEN $bib_rec_fname ~ '^\"' " .
                           "THEN lower( substr( $bib_rec_fname, 2, 200 ) ) " .
                           "ELSE lower( substr( $bib_rec_fname, 1, 200 ) ) " .
                      "END";
  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если это список записей, введенных в таблицу последними, то укажем это
    if ( $num_last_recs )
    {
      print "<h3>Записи, введенные в базу данных последними (" . 
            $num_last_recs . " шт.)</h3>\n";
    }
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>ID</th>\n";
    print "<th>Номер биб. записи</th>\n";
    print "<th>Краткое описание</th>\n";
    print "<th>Наличие</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретной биб. записи ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Bib_recs полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'OBJ_CONTAINERS' )
      {
        # ... полям obj_id_cont, bib_rec_num_cont и bib_rec_cont
        # в модуле OBJ_CONTAINERS
        create_assign_values_func( 'obj_id_cont', 'bib_rec_num_cont',
                                   'bib_rec_cont' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим идентификатор объекта, номер биб. описания и краткое описание
      # (если краткого описания нет, то выводится полное описание -
      # см. строку команды SELECT)
      print "<td class=\"center\">$row[ $result->fnumber( 'obj_id' ) ]</td>\n";
      print "<td class=\"center\">" .
            $row[ $result->fnumber( 'bib_rec_num' ) ] . "</td>\n";

      # будем заменять двойной дефис, которым мы имитируем тире при вводе
      # данных, символом тире &mdash;
      # см. TODO в функции list_bib_recs_by_query() модуля E_ARCH.pm.
      my $bib_rec_for_display = $row[ $result->fnumber( 'bib_rec' ) ];
      $bib_rec_for_display =~ s/--/&mdash;/g;

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретной биб. записи, то значение поля
      # bib_rec выведем в виде ссылки
      if ( $query->param( 'enable_select' ) )
      {
        # экранируем кавычки и другие "неудобные" символы
        # (см. комментарий в модуле obj_groups_descrs.pl)
        # выполним добавочную обработку значения переменной, т. к. того, что
        # делает функция escapeHTML(), недостаточно
        my $bib_rec = $row[ $result->fnumber( 'bib_rec' ) ];
        $bib_rec =~ s/\'/\\\'/g;
        $bib_rec = CGI::escapeHTML( $bib_rec );

        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'obj_id' ) ],
                                  $row[ $result->fnumber( 'bib_rec_num' ) ],
                                  '$bib_rec' )">
                  $bib_rec_for_display</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$bib_rec_for_display</td>\n";
      }

      # наличие экземпляров данного объекта у владельца, идентификатор
      # которого указан в конфиг. файле
      print "<td>\n";
      @avail_objs = get_available( $row[ $result->fnumber( 'obj_id' ) ],
                                   $db_schema );

      # выведем все найденные экземпляры (точнее, физ. формы, в которых эти
      # экземпляры присутствуют) в одной ячейке
      foreach ( @avail_objs )
      {
        print $_ . "<br />\n";
      }
      print "</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "obj_id=" . $row[ $result->fnumber( 'obj_id' ) ] .
              "&bib_rec_num=" . $row[ $result->fnumber( 'bib_rec_num' ) ];

      # просмотр описания в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_blank">Просмотр</a> } . "\n"  if $can_view;

      # удаление записи 
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete"} .
            qq{ target="_self">Удаление</A> } . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();    # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;                # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;           # номер библиогр. записи для данного объекта
  my $target_win;            # имя окна браузера для вывода HTML-страницы
  my $command;               # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $target_win  = $query->param( 'target_win' );

  $target_win  = "_self" unless $target_win;

  # выбираем одно библиографическое описание (запись)" 
  $command = "SELECT * " .
             "FROM ${db_schema}bib_recs " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num'"; 

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top_short,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с " .
                     "идентификатором объекта " . $obj_id .
                     " и номером биб. записи " . $bib_rec_num,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# получение информации о наличии "деталировки" для данного биб. описания
# в подчиненной таблице
# -----------------------------------------------------------------------------
sub get_details
{
  # параметры процедуры
  my $table = shift;        # имя таблицы БД, в которой производится поиск
  my $obj_id = shift;       # уникальный идентификатор объекта биб. описания
  my $bib_rec_num = shift;  # номер библиогр. записи для данного объекта

  # имена полей в таблице БД для уникального идентификатора объекта биб. 
  # описания и номера библиогр. записи
  # ПРИМЕЧАНИЕ. Эти параметры были введены для реализации режима просмотра
  #             всех биб. описаний объектов, "СОДЕРЖАЩИХСЯ" в данном объекте.
  #             Например, в биб. описании многотомного издания "содержатся"
  #             биб. описания всех его томов (выпусков).
  my $field_for_obj_id = shift;
  my $field_for_bib_rec_num = shift;

  my $result;               # для работы с БД (результат выборки) 
  my $command;              # строка команды для выборки из таблицы
  my @row;                  # для хранения одной записи из выборки
  my $mark;                 # символ, которым отмечается наличие "деталировки"

  # назначим имена полей по умолчанию, если параметры не переданы
  $field_for_obj_id = 'obj_id' unless $field_for_obj_id;
  $field_for_bib_rec_num = 'bib_rec_num' unless $field_for_bib_rec_num;
 
  $command = "SELECT count( * ) FROM ${db_schema}$table " .
             "WHERE $field_for_obj_id = '$obj_id'";

  # для нескольких таблиц параметр bib_rec_num не применяется, поэтому
  # его значение может быть равно undef
  if ( $bib_rec_num ne undef )
  {
    $command .= " AND $field_for_bib_rec_num = '$bib_rec_num'";
  }

  $result = exec_select( $command );    # сделаем выборку из БД

  # возьмем запись из полученной выборки
  # (функция count() всегда дает одну запись, даже если значение, возвращаемое
  # этой функцией, равно 0)
  @row = $result->fetchrow();
  
  if ( $row[ 0 ] == 0 )  # "деталировочных" записей нет
  {
    # отметки на экране не будет (возвратим "пробел" для выравнивания
    # при выводе на экран)
    $mark = '&nbsp;&nbsp;';
  }
  else
  {
    # если в конфиг. файле предусмотрена специальная отметка, то используем ее
    if ( $CFG{ BIB_RECS_DETAILS_MARK } ne undef )
    {
      $mark = $CFG{ BIB_RECS_DETAILS_MARK };
    }
    # если отметка не предусмотрена, то ...
    else
    {
      # ... будем выводить число "деталировочных" записей в указанной таблице
      $mark = $row[ 0 ];
    }
  }

  return ( $mark );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "bib_recs",
                       key => "obj_id, bib_rec_num",
                       f_names => \@f_names,
                       f_special => [ 'details_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();      # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # общих обозначений материала)
  my( $ref_values_maters, $ref_labels_maters ) = 
    popup_gen_mater_types_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # источников библиографического описания)
  my( $ref_values_sources, $ref_labels_sources ) = 
    popup_recs_sources_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых выполнено биб. описание)
  my( $ref_values_langs, $ref_labels_langs ) = popup_langs_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов языковой обработки биб. описания)
  my( $ref_values_procs, $ref_labels_procs ) = 
    popup_process_types_list( $db_schema );

  # типовая часть HTML-ссылок для запуска режимов работы с "деталировками"
  # биб. описаний
  my $href;

  # отметка на экране, которая показывает наличие "деталировочных" записей
  # для каждого из модулей ("Заглавия", "Сведения об ответственности" и т. д.)
  my $details_mark;

  # имя окна браузера, в которое должны будут выводиться HTML-страницы,
  # сгенерированные "деталировочными" модулями ("Заглавия", "Сведения об
  # ответственности" и др.)
  # ПРИМЕЧАНИЕ. Необходимость в этой переменной и в CGI-параметре target_win
  #             возникла из-за того, что в случае просмотра биб. описания,
  #             включенного в список биб. описаний, сформированный по какому-то
  #             запросу (например, все биб. описания для такой-то персоналии,
  #             представленной в области сведений об ответственности -- 
  #             см. модуль Persons.pl), это описание выводится в новом окне
  #             (_blank). Согласно выбранной мною политике, все деталировочные
  #             модули ("Заглавия", "Сведения об ответственности" и др.)
  #             должны выводить свою информацию также в это новое окно, а не
  #             в то окно, которое было сформировано ранее и которое уже
  #             содержит информацию (список), необходимую пользователю
  #             для удобной работы. Например, в исходном окне может содержаться
  #             HTML-страница со списком биб. описаний, к которым имеет
  #             отношение персоналия "Бир, Стаффорд". Поэтому было бы удобно
  #             выводить HTML-страницу с подробной информацией о конкретном
  #             объекте биб. описания не в этом же окне, а в новом
  #             окне, тем самым оставляя список на экране, а не замещая его
  #             новой HTML-страницей.
  my $target_win = $query->param( 'target_win' );
  $target_win = "_self" unless $target_win;

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    # ПРИМЕЧАНИЕ. Может быть, правильнее было бы брать требуемое значение
    #             из таблицы bib_objs? Хотя, с другой стороны, значения поля
    #             obj_id в этих таблицах должны быть синхронизированы,
    #             поэтому принципиальной разницы вроде бы и нет... Подумать!
    $fields{ 'obj_id' } = get_next_key_value( 'bib_recs', 'obj_id' );

    # значение поля "Номер библиогр. записи для данного объекта"
    # будем назначать по умолчанию равным 0
    # TODO Нужно добавить еще один режим работы - ввод дополнительного
    #      биб. описания для данного объекта. В этом случае поле 'bib_rec_num'
    #      будет получать значение на 1 большее, чем максимальное значение
    #      этого поля для текущего объекта.
    $fields{ 'bib_rec_num' } = 0;

    # код общего обозначения материала
    $fields{ 'gen_mat_code' }   = $CFG{ BIB_RECS_GEN_MAT_CODE };

    # заголовок библиографической записи
    $fields{ 'bib_rec_head' }   = $CFG{ BIB_RECS_BIB_REC_HEAD };
    
    # полная библиографическая запись
    $fields{ 'bib_rec_full' }   = $CFG{ BIB_RECS_BIB_REC_FULL };

    # краткая библиографическая запись
    $fields{ 'bib_rec_short' }  = $CFG{ BIB_RECS_BIB_REC_SHORT };

    # порядковая единица для сериальных и многотомных изданий
    $fields{ 'ord_unit_num' }   = $CFG{ BIB_RECS_ORD_UNIT_NUM };

    # код источника библиографического описания
    $fields{ 'source_code' }    = $CFG{ BIB_RECS_SOURCE_CODE };

    # флаг наличия "деталировки" в базе данных
    $fields{ 'details_flag' }   = $CFG{ BIB_RECS_DETAILS_FLAG };

    # код языка
    $fields{ 'lang_code' }      = $CFG{ BIB_RECS_LANG_CODE };

    # код вида обработки библиограф. описания
    $fields{ 'process_code' }   = $CFG{ BIB_RECS_PROCESS_CODE };

    # назначение дополнит. описания (или причина его составления)
    $fields{ 'reason_add_rec' } = $CFG{ BIB_RECS_REASON_ADD_REC };

    # поле флагов
    $fields{ 'flags' }          = $CFG{ BIB_RECS_FLAGS };

    # комментарий
    $fields{ 'comment' }        = $CFG{ BIB_RECS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи для данного
  # объекта" формируются автоматически с использованием функции
  # get_next_key_value(), не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа
  print $query->hidden( -name => 'obj_id',
                        -default => $fields{ 'obj_id' }
                      ), "\n";
  print $query->hidden( -name => 'bib_rec_num',
                        -default => $fields{ 'bib_rec_num' }
                      ), "\n";
  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";
  
  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Уникальный идентификатор объекта " .
            "библиографического описания</th>\n";
  print "<td>", $fields{ 'obj_id' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номер библиогрaфической записи для данного объекта</th>\n";
  print "<td>", $fields{ 'bib_rec_num' }, "</td>\n";
  print "</tr>\n";
  
  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_gen_mater_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Общее обозначения материала</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'gen_mat_code',
                            -default => $fields{ 'gen_mat_code' },
                            -values => $ref_values_maters,
                            -labels => { @{ $ref_labels_maters } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Заголовок библиографической записи</th>\n";
  print "<td>";

  # добавим в глобальный хеш-массив %CFG параметры для реализации
  # "быстрой" вставки последних персоналий, введенных в таблицу Persons
  add_conf_params_for_pers( "BIB_RECS_BIB_REC_HEAD" ) 
    if $CFG{ HOT_HELP_IS_ON };

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].bib_rec_head",
                   "BIB_RECS_BIB_REC_HEAD" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'bib_rec_head',
                          -default => $fields{ 'bib_rec_head' },
                          -rows => $CFG{ BIB_RECS_BIB_REC_HEAD_ROWS },
                          -columns => $CFG{ BIB_RECS_BIB_REC_HEAD_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Полная библиографическая запись</th>\n";
  print "<td>";

  # добавим в глобальный хеш-массив %CFG параметры для реализации
  # "быстрой" вставки последних персоналий, введенных в таблицу Persons
  add_conf_params_for_pers( "BIB_RECS_BIB_REC_FULL" ) 
    if $CFG{ HOT_HELP_IS_ON };
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].bib_rec_full",
                   "BIB_RECS_BIB_REC_FULL" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'bib_rec_full',
                          -default => $fields{ 'bib_rec_full' },
                          -rows => $CFG{ BIB_RECS_BIB_REC_FULL_ROWS },
                          -columns => $CFG{ BIB_RECS_BIB_REC_FULL_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Краткая библиографическая запись</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].bib_rec_short",
                   "BIB_RECS_BIB_REC_SHORT" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'bib_rec_short',
                          -default => $fields{ 'bib_rec_short' },
                          -rows => $CFG{ BIB_RECS_BIB_REC_SHORT_ROWS },
                          -columns => $CFG{ BIB_RECS_BIB_REC_SHORT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Порядковая единица для сериальных и многотомных изданий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].ord_unit_num",
                   "BIB_RECS_ORD_UNIT_NUM" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'ord_unit_num',
                           -default => $fields{ 'ord_unit_num' },
                           -size => $CFG{ BIB_RECS_ORD_UNIT_NUM_SIZE },
                           -maxlength => 
                             $CFG{ BIB_RECS_ORD_UNIT_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";
  
  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_recs_sources_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Источник библиографического описания</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'source_code',
                            -default => $fields{ 'source_code' },
                            -values => $ref_values_sources,
                            -labels => { @{ $ref_labels_sources } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак наличия \"деталировки\" в базе данных</th>\n";
  print "<td>",
        $query->checkbox( -name => 'details_flag',
                          -checked => 
                             $fields{ 'details_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";
  
  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Язык</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_process_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид языковой обработки</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'process_code',
                            -default => $fields{ 'process_code' },
                            -values => $ref_values_procs,
                            -labels => { @{ $ref_labels_procs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Назначение дополнительного описания " .
            "(или причина его составления)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].reason_add_rec",
                   "BIB_RECS_REASON_ADD_REC" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'reason_add_rec',
                           -default => $fields{ 'reason_add_rec' },
                           -size => $CFG{ BIB_RECS_REASON_ADD_REC_SIZE },
                           -maxlength => 
                             $CFG{ BIB_RECS_REASON_ADD_REC_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Специфические признаки</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].flags", "BIB_RECS_FLAGS" ) 
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'flags',
                           -default => $fields{ 'flags' },
                           -size => $CFG{ BIB_RECS_FLAGS_SIZE },
                           -maxlength => $CFG{ BIB_RECS_FLAGS_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "BIB_RECS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ BIB_RECS_COMMENT_ROWS },
                          -columns => $CFG{ BIB_RECS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, значит, мы находимся в режиме просмотра/корректировки
  # и можем создать ссылки для ввода записей в таблицы, связанные с таблицей
  # Bib_recs
  if ( $ref_row ne undef )
  {
    # будем заменять двойной дефис, которым мы имитируем тире при вводе
    # данных, символом тире &mdash;
    # см. TODO в функции list_bib_recs_by_query() модуля E_ARCH.pm.
    my $bib_rec_for_display = ( ( $fields{ 'bib_rec_full' } ne undef ) ?
                                  $fields{ 'bib_rec_full' } :
                                  $fields{ 'bib_rec_short' } );
    $bib_rec_for_display =~ s/--/&mdash;/g;

    # типовая часть HTML-ссылок для запуска режимов работы с "деталировками"
    # биб. описаний (будем включать в ссылку не только ключевые поля, но
    # также и биб. описание - для наглядности и удобства ввода "деталировок")
    # ПРИМЕЧАНИЕ. В биб. описании могут быть символы, которые нужно
    #             заэкранировать - что мы и делаем.
    $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
            "obj_id=$fields{ 'obj_id' }" .
            "&bib_rec_num=$fields{ 'bib_rec_num' }" .
            "&bib_rec=" . CGI::escape( $bib_rec_for_display );

    # НЕ УДАЛЯТЬ пока что этот фрагмент. Он использовался для проверки влияния
    # порядка следования параметров в URL, т.к. были сомнения в том, что при
    # помещении параметра bib_rec в середину списка (на НЕ последнее место)
    # CGI.pm сможет корректно распознать пары name=value для тех параметров,
    # которые следуют за ним (поскольку этот параметр - bib_rec - может
    # содержать, например, кавычки). Также были сомнения и в корректности
    # работы механизма escapeHTML/unescapeHTML. Вроде бы все работает
    # корректно и предсказуемо, но все же нужно дополнительно уточнить,
    # когда срабатывает механизм escapeHTML/unescapeHTML. Эти функции есть
    # в CGI.pm, но unescapeHTML нигде в версии 2.46 CGI.pm явно не вызывается.
    # А в версии 2.70 CGI.pm есть вызов этой функции. Почему? 
    # ПРИМЕЧАНИЕ. В настоящее время мои программы используют версию 2.46
    #             пакета CGI.pm.
    # $href = "obj_id=$fields{ 'obj_id' }" .
    #         "&bib_rec=" . ( ( $fields{ 'bib_rec_short' } ne undef ) ?
    #                           $fields{ 'bib_rec_short' } :
    #                           $fields{ 'bib_rec_full' } ) .
    #         "&bib_rec_num=$fields{ 'bib_rec_num' }";

    # НЕ УДАЛЯТЬ пока что этот фрагмент. Он показывает путь моих исканий.
    # поскольку в биб. описании имеются пробелы, заменим их символами "%20",
    # а также заэкранируем символы, которые имеют специальное значение в HTML:
    # в биб. описаниях могут быть, например, кавычки
    # (для упрощения кода обрабатываем всю ссылку $href)
    # ВАЖНО. Обрабатывать всю ссылу целиком оказалось нельзя, т.к. символы,
    #        например, "&" используются в качестве разделителей пар name=value,
    #        но могут также входить буквально в значение поля Bib_rec.
    # $href =~ s/[ ]+/%20/g;
    # $href = CGI::escapeHTML( $href );
    # $href = CGI::escape( $href );
    # $href = uri_escape( $href );      # из пакета URI::Escape

    print "<table class=\"details_links\">\n";
    print "<tr>\n";

    print "<td>\n";

    print "<h4 class=\"bib_descr_area\">" .
          "Область заглавия и сведений об ответственности</h4>\n";

    # заглавия
    $details_mark = get_details( 'titles', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/titles.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Заглавия</a>},
          "<br />\n";

    # сведения об ответственности
    $details_mark = get_details( 'resp_stat', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/resp_stat.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Сведения об ответственности</a>},
          "<br />\n";

    print "<h4 class=\"bib_descr_area\">Область издания</h4>\n";

    # сведения об издании
    $details_mark = get_details( 'edition_stat', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/edition_stat.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Сведения об издании</a>},
          "<br />\n";

    print "<h4 class=\"bib_descr_area\">Область специфических сведений</h4>\n";

    # специфические сведения
    
    # спец. сведения - электронные ресурсы
    $details_mark = get_details( 'spec_er', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/spec_er.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Электронные ресурсы</a>},
          "<br />\n";

    # спец. сведения - сериальные документы
    $details_mark = get_details( 'spec_serials', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/spec_serials.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Сериальные документы</a>},
          "<br />\n";

    # спец. сведения - нормативные документы
    $details_mark = get_details( 'spec_stds', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/spec_stds.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Нормативные документы (стандарты и ТУ)</a>},
          "<br />\n";

    print "<h4 class=\"bib_descr_area\">Область выходных данных</h4>\n";

    # место издания
    $details_mark = get_details( 'pub_print_place', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/pub_print_place.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Место издания, изготовления</a>},
          "<br />\n";

    # издатели/изготовители
    $details_mark = get_details( 'pub_printer', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/pub_printer.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Издатель, изготовитель, распространитель} .
          "</a>",
          "<br />\n";

    # даты издания
    $details_mark = get_details( 'pub_print_date', $fields{ 'obj_id' },
                                 undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/pub_print_date.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Дата издания, печатания, изготовления</a>},
          "<br />\n";

    print "<h4 class=\"bib_descr_area\">" .
          "Область физической характеристики</h4>\n";

    # специфическое обозначение материала
    $details_mark = get_details( 'spec_mater', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/spec_mater.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Специфическое обозначение материала } .
                                  "и объем; сопроводительный материал</a>",
          "<br />\n";

    print "<h4 class=\"bib_descr_area\">Область серии</h4>\n";

    # серии/подсерии
    $details_mark = get_details( 'obj_series', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/obj_series.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Серии</a>},
          "<br />\n";

    print "<h4 class=\"bib_descr_area\">Область примечания</h4>\n";

    # примечания
    $details_mark = get_details( 'notes', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/notes.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Примечания</a>},
          "<br />\n";

    # тираж
    $details_mark = get_details( 'editions', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/editions.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Тираж</a>},
          "<br />\n";

    print "<h4 class=\"bib_descr_area\">" .
          "Область стандартного номера и условий доступности</h4>\n";

    # стандартные номера
    $details_mark = get_details( 'std_nums', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/std_nums.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Стандартные номера (ISBN, ISSN, ...)</a>},
          "<br />\n";

    # условия доступности
    $details_mark = get_details( 'avail_terms', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/avail_terms.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Условия доступности</a>},
          "<br />\n";

    # цена
    $details_mark = get_details( 'prices', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/prices.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Цена</a>},
          "<br />\n";

    print "</td>\n";

    print "<td>\n";

    print "<h4 class=\"bib_descr_area\">" .
          "Дополнительные библиографические сведения</h4>\n";

    # объекты-контейнеры
    $details_mark = get_details( 'obj_containers', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/obj_containers.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Объекты-контейнеры</a>},
          "<br />\n";

    # все биб. описания объектов, "СОДЕРЖАЩИХСЯ" в данном объекте
    $details_mark = get_details( 'obj_containers', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' },
                                 'obj_id_cont', 'bib_rec_num_cont' );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="$script_name/list_contained_objs?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Объекты, "содержащиеся" в объекте</a>},
          "<br />\n";

    # языки текста
    $details_mark = get_details( 'obj_langs', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/obj_langs.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Языки текста</a>},
          "<br />\n";

    # признаки объектов
    $details_mark = get_details( 'features', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/features.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Признаки объекта</a>},
          "<br />\n";

    print "<br />\n";

    print "<h4 class=\"bib_descr_area\">Использование</h4>\n";

    # проработка содержания
    $details_mark = get_details( 'studying', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/studying.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Проработка содержания</a>},
          "<br />\n";

    # общие характеристики объектов
    $details_mark = get_details( 'gen_chars', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/gen_chars.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Общая характеристика</a>},
          "<br />\n";

    # детальные характеристики объектов
    $details_mark = get_details( 'det_chars', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/det_chars.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Детальные характеристики</a>},
          "<br />\n";

    # ключевые слова
    $details_mark = get_details( 'obj_keywords', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/obj_keywords.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Ключевые слова</a>},
          "<br />\n";

    # содержание (оглавление)
    $details_mark = get_details( 'contents', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/contents.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Содержание (оглавление)</a>},
          "<br />\n";

    # справочные материалы
    $details_mark = get_details( 'refs_descrs', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/refs_descrs.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Справочные материалы</a>},
          "<br />\n";

    print "<br />\n";

    # временные группировки
    $details_mark = get_details( 'obj_groups', $fields{ 'obj_id' },
                                 $fields{ 'bib_rec_num' } );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/obj_groups.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Участие в тематических списках</a>},
          "<br />\n";

    print "<br />\n";

    print "<h4 class=\"bib_descr_area\">Наличие</h4>\n";

    # хранимые объекты
    $details_mark = get_details( 'stor_objs', $fields{ 'obj_id' }, undef );

    print "<span id=\"details_mark\">$details_mark" . "&nbsp;</span>" .
          qq{<a class="command_link" } .
          qq{href="/$CFG{ HOME_WEB_DIR }/stor_objs.pl/query?$href} .
          qq{&target_win=$target_win"} .
          qq{ target="$target_win">Экземпляры (единицы хранения)</a>},
          "<br />\n";

    print "</td>\n";
    print "</tr>\n";

    print "</table>\n";
  }

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Флаг наличия "деталировки" в базе данных
  if ( $fname eq 'details_flag' )
  {
    # это поле имеет тип bool, поэтому проверяем значение параметра
    # только на непустоту
    $value = $query->param( $fname ) ? 'yes' : 'no';
  }

  return ( $value );  
}
