#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   справочник "Города"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'CITIES';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Cities
my @f_names =
   ( 
     'city_code',         # Код города
     'country_code',      # Код страны
     'city_region',       # Регион
     'city_fname',        # Наименование города полное
     'city_sname',        # Наименование города краткое 
     'city_code_orig',    # Код города, являющегося началом цепочки
                          # (для учета переименований, разночтений, написаний
                          # на различных языках)
     'lang_code',         # Код языка (1 - русский язык)
     'comment'            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе
  
  if ( $query->param( 'city_code' ) eq '' )
  {
    push( @error_messages, "Не указан код города" );
  }

  if ( $query->param( 'country_code' ) eq '' )
  {
    push( @error_messages, "Не указан код страны" );
  }

  if ( $query->param( 'city_fname' ) eq '' )
  {
    push( @error_messages, "Не указано полное наименование города" );
  }

  if ( $query->param( 'city_sname' ) eq '' )
  {
    push( @error_messages, "Не указано краткое наименование города" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Города',
                    user => $user
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $city_code;             # код города
  my $command;               # строка команды для работы с БД

  $city_code = $query->param( 'city_code' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}cities " .
             "WHERE city_code = '$city_code'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $city_code;             # код города
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $city_code = $query->param( 'city_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Cities WHERE city_code = $city_code";
  push( @table_cmd_pairs, [ 'Cities', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $city_code;             # код города
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $city_code = $query->param( 'city_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Cities WHERE city_code = $city_code";
  push( @table_cmd_pairs, [ 'Cities', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех городов
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выбираем все записи о городах (а также краткое наименование страны и
  # краткое наименование языка, на котором записано название города)
  $command = "SELECT c1.*, c2.city_fname AS city_fname_orig, " .
                    "cc.country_sname, l.lang_sname " .
             "FROM ( ${db_schema}cities c1 LEFT JOIN ${db_schema}cities c2 " .
                  "ON c1.city_code_orig = c2.city_code ), " .
                  "${db_schema}countries cc, ${db_schema}langs l " .
             "WHERE c1.lang_code = l.lang_code and " .
                   "c1.country_code = cc.country_code " .
             "ORDER BY c1.city_fname";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если этот модуль вызван в режиме справочника, и при этом он вызван
    # "сам из себя", то запретим операции просмотра и удаления записей
    # (поскольку режим корректировки выполняется из режима просмотра,
    # то и он будет недоступен)
    if ( $query->param( 'enable_select' ) &&
         $query->param( 'caller_name') eq $module )
    {
      $can_view = 0;
      $can_delete = 0;
    }
    else
    {
      $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                               'user' => $user );
      $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                                 'user' => $user );
    }

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }
 
    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код города</th>\n";
    print "<th>Наименование полное</th>\n";
    print "<th>Город &mdash; начало цепочки (код и наименование)</th>\n";
    print "<th>Страна</th>\n";
    print "<th>Регион</th>\n";
    print "<th>Язык</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретного города (места издания) ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Cities полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'PUB_PRINT_PLACE' )
      {
        # ... полям city_code, city_sname и country_sname_city
        # в модуле PUB_PRINT_PLACE
        create_assign_values_func( 'city_code', 'city_sname',
                                   'country_sname_city' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'STOR_OBJS' )
      {
        # ... полям acq_place_city, city_sname и country_sname_city
        # в модуле STOR_OBJS
        create_assign_values_func( 'acq_place_city', 'city_sname',
                                   'country_sname_city' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'CITIES' )
      {
        # ... полям city_code_orig, city_sname_orig и country_sname_orig
        # в модуле CITIES
        create_assign_values_func( 'city_code_orig', 'city_sname_orig',
                                   'country_sname_orig' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'ORGS' )
      {
        # ... полям org_city_code, org_city_sname и org_country_sname_city
        # в модуле CITIES
        create_assign_values_func( 'org_city_code', 'org_city_sname',
                                   'org_country_sname_city' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код города, полное наименование, код и наименование города --
      # начала цепочки, страну, регион, язык, на котором написано наименование
      # города
      # (добавим символ &nbsp; к коду города, являющегося началом цепочки,
      # а также к наименованию региона, из соображений эстетики: если значение
      # поля 'city_code_orig' или 'city_region' равно null, то не отображаются
      # горизонтальные строки-разделители)
      print "<td class=\"center\">" .
            $row[ $result->fnumber( 'city_code' ) ] . "</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретного города (места издания), то значение
      # поля city_fname выведем в виде ссылки
      if ( $query->param( 'enable_select' ) )
      {
        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'city_code' ) ],
                                  '$row[ $result->fnumber( 'city_sname' ) ]',
                                  '$row[ $result->fnumber( 'country_sname' ) ]'
                                )">
                   $row[ $result->fnumber( 'city_fname' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'city_fname' ) ]</td>\n";
      }

      # если код города, являющегося началом цепочки, отсутствует, то выведем
      # лишь символ пробела "&nbsp;"
      print "<td>",
            ( $row[ $result->fnumber( 'city_code_orig' ) ] ne undef ) ?
              "(" . $row[ $result->fnumber( 'city_code_orig' ) ] . ") " .
              $row[ $result->fnumber( 'city_fname_orig' ) ] :
            "&nbsp;",
            "</td>\n";

      print "<td>$row[ $result->fnumber( 'country_sname' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'city_region' ) ]&nbsp;</td>\n";
      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "city_code=" . $row[ $result->fnumber( 'city_code' ) ] .
              $add_params;

      # просмотр описания города в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи о городе
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $city_code;                  # код города
  my $command;                    # строка команды для работы с БД

  $city_code = $query->param( 'city_code' );

  # выбираем одну запись о городе
  # выберем также краткое название города, являющегося началом цепочки,
  # и название страны для него
  $command = "SELECT c1.*, c2.city_sname AS city_sname_orig, " .
                    "c3.country_sname AS country_sname_orig " .
             "FROM ( ${db_schema}cities c1 LEFT JOIN ${db_schema}cities c2 " .
                  "ON c1.city_code_orig = c2.city_code ) " .
                  "LEFT JOIN ${db_schema}countries c3 " .
                  "ON c2.country_code = c3.country_code " .
             "WHERE c1.city_code = '$city_code'"; 

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "города " . $city_code,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "cities",
                       key => "city_code",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();      # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # стран, в которых находятся города)
  my( $ref_values_countries, $ref_labels_countries ) =
    popup_countries_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны наименования городов)
  my( $ref_values_langs, $ref_labels_langs ) = popup_langs_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'city_code' } = get_next_key_value( 'cities', 'city_code' );

    # значения по умолчанию

    # код страны
    $fields{ 'country_code' }   = $CFG{ CITIES_COUNTRY_CODE };

    # регион
    $fields{ 'city_region' }    = $CFG{ CITIES_CITY_REGION };

    # наименование города полное
    $fields{ 'city_fname' }     = $CFG{ CITIES_CITY_FNAME };

    # наименование города краткое 
    $fields{ 'city_sname' }     = $CFG{ CITIES_CITY_SNAME };

    # код города, являющегося началом цепочки
    # (для учета переименований, разночтений, написаний на различных языках)
    $fields{ 'city_code_orig' } = $CFG{ CITIES_CITY_CODE_ORIG };

    # код языка (1 - русский язык)
    $fields{ 'lang_code' }      = $CFG{ CITIES_LANG_CODE };

    # примечание
    $fields{ 'comment' }        = $CFG{ CITIES_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Код города" формируется автоматически
  # с использованием функции get_next_key_value(), не будем позволять его
  # редактировать, но совсем убрать его нельзя, т.к. оно используется
  # при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'city_code',
                        -default => $fields{ 'city_code' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. Повторный вызов скрипта, уже вызванного в режиме справочника,
  #             возможен для выполнения корректировки записи в этом справочнике
  #             или для добавления в него новой записи. Когда после выполнения
  #             корректировки или добавления записи мы вновь выберем ссылку
  #             "Список", в этом списке у нас будут HTML-ссылки, "кликая"
  #             по одной из которых, мы и сможем осуществить выбор нужного
  #             города. Если бы этих двух скрытых полей не было, то в списке
  #             городов их наименования были бы представлены не в виде
  #             HTML-ссылок для выбора значений, а в виде обычных текстовых
  #             строк. Для того чтобы понять назначение этих двух полей,
  #             нужно их закомментировать, а затем вызвать модуль в качестве
  #             справочника, откорректировать или добавить запись, вновь
  #             выбрать ссылку "Список" и увидеть, что получится.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код города</th>\n";
  print "<td>", $fields{ 'city_code' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Полное наименование</th>\n";
  print "<td>",
        $query->textfield( -name => 'city_fname',
                           -default => $fields{ 'city_fname' },
                           -size => $CFG{ CITIES_CITY_FNAME_SIZE },
                           -maxlength => $CFG{ CITIES_CITY_FNAME_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Краткое наименование</th>\n";
  print "<td>",
        $query->textfield( -name => 'city_sname',
                           -default => $fields{ 'city_sname' },
                           -size => $CFG{ CITIES_CITY_SNAME_SIZE },
                           -maxlength => $CFG{ CITIES_CITY_SNAME_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_countris_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Страна</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'country_code',
                            -default => $fields{ 'country_code' },
                            -values => $ref_values_countries,
                            -labels => { @{ $ref_labels_countries } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Регион</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].city_region",
                   "CITIES_CITY_REGION" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'city_region',
                           -default => $fields{ 'city_region' },
                           -size => $CFG{ CITIES_CITY_REGION_SIZE },
                           -maxlength => $CFG{ CITIES_CITY_REGION_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Город &mdash; начало цепочки (код, наименование, страна)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник cities.pl
  create_selection_func( func_name => 'select_orig_city',
                         module_name => 'cities.pl',
                         caller_name => $module,
                         window_name => 'orig_cities',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также название города и страны для города, являющегося
  # началом цепочки (без возможности редактирования)
  print "<td>",
        $query->textfield( -name => 'city_code_orig',
                           -default => $fields{ 'city_code_orig' },
                           -size => $CFG{ CITIES_CITY_CODE_ORIG_SIZE },
                           -maxlength => 
                             $CFG{ CITIES_CITY_CODE_ORIG_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'city_sname_orig',
                           -default => $fields{ 'city_sname_orig' },
                           -size => $CFG{ CITIES_CITY_SNAME_ORIG_SIZE },
                           -maxlength => 
                             $CFG{ CITIES_CITY_SNAME_ORIG_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'country_sname_orig',
                           -default => $fields{ 'country_sname_orig' },
                           -size => $CFG{ CITIES_COUNTRY_SNAME_ORIG_SIZE },
                           -maxlength => 
                             $CFG{ CITIES_COUNTRY_SNAME_ORIG_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" onclick=} .
        qq{"select_orig_city()">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";
  
  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Язык</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";
  
  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "CITIES_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };
  
  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ CITIES_COMMENT_ROWS },
                          -columns => $CFG{ CITIES_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
    
  return ( 0 );
}
