#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Конференции, съезды, симпозиумы, семинары, выставки ..."
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'CONFS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Confs
my @f_names =
   ( 'conf_code',         # Уникальный код мероприятия
     'conf_fname',        # Полное наименование (тема) мероприятия
     'conf_place',        # Место проведения (писать словами, т.к. может
                          # проводиться в нескольких местах одновременно)
     'conf_date_beg',     # Дата начала работы мероприятия
     'conf_date_end',     # Дата окончания работы мероприятия
     'conf_status_code',  # Код статуса мероприятия (Всероссийский,
                          # международный ...)
     'conf_foreign_flag', # Признак зарубежного мероприятия
                          # (т.е. проводимого за рубежом)
     'conf_type_code',    # Код типа мероприятия (конференция,
                          # семинар, выставка ...)
     'conf_num',          # Порядковый номер
     'conf_code_orig',    # Код главной записи (в случае переводов,
                          # разночтений)
     'comment'            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # НЕ допускается пустое поле - должны быть только цифры
  if ( $query->param( 'conf_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный уникальный код " .
          "мероприятия" );
  }

  if ( $query->param( 'conf_fname' ) eq '' )
  {
    push( @error_messages, "Не указано полное наименование мероприятия" );
  }

  if ( $query->param( 'conf_place' ) eq '' )
  {
    push( @error_messages, "Не указано место проведения мероприятия" );
  }

  # пустое поле НЕ допускается
  if ( ! date_is_correct( $query->param( 'conf_date_beg' ) ) )
  {
    push( @error_messages, "Не введена либо неверная дата начала работы " .
          "мероприятия" );
  }

  # пустое поле НЕ допускается
  if ( ! date_is_correct( $query->param( 'conf_date_end' ) ) )
  {
    push( @error_messages, "Не введена либо неверная дата окончания работы " .
          "мероприятия" );
  }

  if ( $query->param( 'conf_status_code' ) eq '' )
  {
    push( @error_messages, "Не указан код статуса мероприятия" );
  }

  if ( $query->param( 'conf_type_code' ) eq '' )
  {
    push( @error_messages, "Не указан код типа мероприятия" );
  }

  # допускается либо пустое поле, либо только цифры
  if ( $query->param( 'conf_code_orig' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный код мероприятия -- начала цепочки" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Конференции, съезды, симпозиумы, ' .
                                    'семинары, выставки ...',
                    user => $user,
                    make_letters => 'yes',
                    num_last_recs => $CFG{ CONFS_NUM_LAST_RECS },
                    full_top => 'yes'
                  );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $conf_code;             # код мероприятия
  my $command;               # строка команды для работы с БД

  $conf_code = $query->param( 'conf_code' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}confs " .
             "WHERE conf_code = '$conf_code'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $conf_code;             # уникальный код мероприятия
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $conf_code = $query->param( 'conf_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Confs WHERE conf_code = $conf_code";
  push( @table_cmd_pairs, [ 'Confs', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $conf_code;             # уникальный код мероприятия
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $conf_code = $query->param( 'conf_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Confs WHERE conf_code = $conf_code";
  push( @table_cmd_pairs, [ 'Confs', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех мероприятий, наименования которых начинаются
# с указанной буквы
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $letter;                     # первая буква наименования мероприятия
  my $letter_lower;               # первая буква -- в нижнем регистре

  # число последних записей, введенных в таблицу, которые нужно выводить
  # при обращении по специальной ссылке
  my $num_last_recs;

  # условие WHERE для команды выборки
  my $where;

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  $letter        = $query->param( 'letter' );
  $num_last_recs = $query->param( 'num_last_recs' );

  # если параметры не переданы, то просто завершаем работу досрочно - 
  # это происходит при первоначальном запуске скрипта
  # ПРИМЕЧАНИЕ. Проверка как на undef, так и на пустое значение '' выполнена
  #             на случай каких-либо проблем в CGI.pm.
  if ( ( $letter eq undef || $letter eq '' ) &&
       ( $num_last_recs eq undef || $num_last_recs eq '' )
     )
  {
    close_bottom();   # завершаем оформление нижней части окна
    print $query->end_html();
    return ( 0 );
  }

  # если пользователь выбрал ссылку для вывода только последних записей,
  # введенных в таблицу
  if ( $num_last_recs )
  {
    # ПРИМЕЧАНИЕ. Если даже требуемое число последних введенных записей будет
    #             больше числа всех записей, имеющихся в таблице, этот 
    #             оператор сработает. Если какие-то записи были удалены,
    #             вследствие чего образовались "дырки" в нумерации conf_code,
    #             то просто будет выведено меньшее число записей.
    $where = "WHERE c1.conf_code >= " .
                   "( SELECT max( conf_code ) FROM ${db_schema}confs ) - " .
                   "$num_last_recs + 1 ";
  }
  # пользователь выбрал конкретную букву или другой символ
  elsif ( $letter )
  {
    # для русских букв может потребоваться особая предварительная обработка
    $letter = reveal_rus_letter( $letter );

    # воспользуемся возможностями локализации для перевода буквы, выбранной
    # пользователем, в нижний регистр
    $letter_lower = change_case( $letter, 'lower' );

    # если первый символ -- двойные кавычки, то не учитываем его
    $where = "WHERE c1.conf_fname ~ '^($letter|\"$letter|" .
                                      "$letter_lower|\"$letter_lower)' ";
  }

  # выбираем все записи о мероприятиях, наименования которых удовлетворяют
  # условию $where (т. е. начинаются с указанной буквы или были введены 
  # в таблицу последними), а также код и наименование мероприятия, являющегося
  # началом цепочки
  $command = "SELECT c1.*, c2.conf_fname AS conf_fname_orig " .
             "FROM ( ${db_schema}confs c1 LEFT JOIN ${db_schema}confs c2 " .
                  "ON c1.conf_code_orig = c2.conf_code ) " .
             $where .
             "ORDER BY c1.conf_date_beg";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если это список записей, введенных в таблицу последними, то укажем это
    if ( $num_last_recs )
    {
      print "<h3>Записи, введенные в базу данных последними (" . 
            $num_last_recs . " шт.)</h3>\n";
    }

    # если этот модуль вызван в режиме справочника, и при этом он вызван
    # "сам из себя", то запретим операции просмотра и удаления записей
    # (поскольку режим корректировки выполняется из режима просмотра,
    # то и он будет недоступен)
    if ( $query->param( 'enable_select' ) &&
         $query->param( 'caller_name') eq $module )
    {
      $can_view = 0;
      $can_delete = 0;
    }
    else
    {
      $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                               'user' => $user );
      $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                                 'user' => $user );
    }

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код мероприятия</th>\n";
    print "<th>Наименование полное</th>\n";
    print "<th>Мероприятие &mdash; начало цепочки (код и наименование)</th>\n";
    print "<th>Дата начала</th>\n";
    print "<th>Дата окончания</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе мероприятия -- начала цепочки ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Confs полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'CONFS' )
      {
        # ... полям conf_code_orig и conf_fname_orig в модуле CONFS
        create_assign_values_func( 'conf_code_orig', 'conf_fname_orig' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код мероприятия, полное наименование, код и наименование
      # мероприятия -- начало цепочки, даты начала и окончания
      # (добавим символ &nbsp; к коду мероприятия, являющемуся началом цепочки,
      # из соображений эстетики: если значение поля 'conf_code_orig'
      # равно null, то не отображаются горизонтальные строки-разделители)
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'conf_code' ) ]</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе наименования мероприятия -- начала цепочки,
      # то значение поля conf_fname выведем в виде ссылки

      # экранируем кавычки и другие "неудобные" символы
      # (см. комментарий в модуле obj_groups_descrs.pl)
      # выполним добавочную обработку значения переменной, т. к. того, что
      # делает функция escapeHTML(), недостаточно
      my $conf_fname = $row[ $result->fnumber( 'conf_fname' ) ];
      $conf_fname =~ s/\'/\\\'/g;
      $conf_fname = CGI::escapeHTML( $conf_fname );

      if ( $query->param( 'enable_select' ) )
      {
        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'conf_code' ) ],
                                  '$conf_fname' )">
                   $row[ $result->fnumber( 'conf_fname' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'conf_fname' ) ]</td>\n";
      }

      print "<td>",
            ( $row[ $result->fnumber( 'conf_code_orig' ) ] ne undef ) ?
              "(" . $row[ $result->fnumber( 'conf_code_orig' ) ] . ") " .
              $row[ $result->fnumber( 'conf_fname_orig' ) ] :
            "&nbsp;",
            "</td>\n";
      print "<td>" .
            date_for_display( $row[ $result->fnumber( 'conf_date_beg' ) ] ) .
            "</td>\n";
      print "<td>" .
            date_for_display( $row[ $result->fnumber( 'conf_date_end' ) ] ) .
            "</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "city_code=" . $row[ $result->fnumber( 'city_code' ) ] .
              $add_params;

      # просмотр описания мероприятия в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи о мероприятии
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $conf_code;             # код мероприятия
  my $command;               # строка команды для работы с БД

  $conf_code = $query->param( 'conf_code' );

  # выбираем одну запись о мероприятии
  $command = "SELECT c1.*, c2.conf_fname AS conf_fname_orig " .
             "FROM ( ${db_schema}confs c1 LEFT JOIN ${db_schema}confs c2 " .
                  "ON c1.conf_code_orig = c2.conf_code ) " .
             "WHERE c1.conf_code = '$conf_code'"; 

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "мероприятия " . $conf_code,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "confs",
                       key => "conf_code:conf_code_begin",
                       f_names => \@f_names,
                       f_special => [ 'conf_date_beg', 'conf_date_end',
                                      'conf_foreign_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # видов статуса мероприятий)
  my( $ref_values_stats, $ref_labels_stats ) =
    popup_conf_status_types_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов мероприятий)
  my( $ref_values_types, $ref_labels_types ) =
    popup_conf_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # значения по умолчанию

    # полное наименование (тема) мероприятия
    $fields{ 'conf_fname' }        = $CFG{ CONFS_CONF_FNAME };

    # место проведения (может проводиться в нескольких местах одновременно)
    $fields{ 'conf_place' }        = $CFG{ CONFS_CONF_PLACE };

    # дата начала работы мероприятия
    $fields{ 'conf_date_beg' }     = $CFG{ CONFS_CONF_DATE_BEG };

    # дата окончания работы мероприятия
    $fields{ 'conf_date_end' }     = $CFG{ CONFS_CONF_DATE_END };

    # код статуса мероприятия (Всероссийский, международный ...)
    $fields{ 'conf_status_code' }  = $CFG{ CONFS_CONF_STATUS_CODE };

    # признак зарубежного мероприятия, т. е. проводимого за рубежом)
    $fields{ 'conf_foreign_flag' } = $CFG{ CONFS_CONF_FOREIGN_FLAG };

    # код типа мероприятия (конференция, семинар, выставка ...)
    $fields{ 'conf_type_code' }    = $CFG{ CONFS_CONF_TYPE_CODE };

    # порядковый номер
    $fields{ 'conf_num' }          = $CFG{ CONFS_CONF_NUM };

    # код главной записи (в случае переводов, разночтений)
    $fields{ 'conf_code_orig' }    = $CFG{ CONFS_CONF_CODE_ORIG };

    # примечание
    $fields{ 'comment' }           = $CFG{ CONFS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # создадим скрытое поле для "Уникального кода мероприятия"
  # ПРИМЕЧАНИЕ. Идентификатор скрытого поля имеет постфикс 'begin' для того,
  #             чтобы можно было обновлять значение этого поля, поскольку оно
  #             является ключевыми (см. функцию make_cmd в пакете E_ARCH.pm.
  #             Т.е. нам нужны и исходное значение этого поля, и его новое
  #             значение (которое, возможно, было изменено пользователем).
  print $query->hidden( -name => 'conf_code_begin',
                        -default => $fields{ 'conf_code' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код мероприятия</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль orgs.pl в качестве модуля-справочника  
  create_selection_func( func_name => 'select_org',
                         module_name => 'orgs.pl',
                         caller_name => $module,
                         window_name => 'orgs',
                         exp_imp => $query->param( 'exp_imp' ) );
  print "<td>",
        $query->textfield( -name => 'conf_code',
                           -default => $fields{ 'conf_code' },
                           -size => $CFG{ CONFS_CONF_CODE_SIZE },
                           -maxlength => $CFG{ CONFS_CONF_CODE_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )"} .
        qq{ onclick="select_org()">Список</a>},
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Полное наименование</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].conf_fname", "CONFS_CONF_FNAME" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'conf_fname',
                          -default => $fields{ 'conf_fname' },
                          -rows => $CFG{ CONFS_CONF_FNAME_ROWS },
                          -columns => $CFG{ CONFS_CONF_FNAME_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Место проведения</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].conf_place", "CONFS_CONF_PLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'conf_place',
                           -default => $fields{ 'conf_place' },
                           -size => $CFG{ CONFS_CONF_PLACE_SIZE },
                           -maxlength => $CFG{ CONFS_CONF_PLACE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата начала мероприятия</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].conf_date_beg", "CONFS_CONF_DATE_BEG",
                   'REPLACE' ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'conf_date_beg',
                           -default =>
                              date_for_display( $fields{ 'conf_date_beg' } ),
                           -size => $CFG{ CONFS_CONF_DATE_BEG_SIZE },
                           -maxlength => $CFG{ CONFS_CONF_DATE_BEG_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата окончания мероприятия</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].conf_date_end", "CONFS_CONF_DATE_END",
                   'REPLACE' ) if $CFG{ HOT_HELP_IS_ON };
  
  print $query->textfield( -name => 'conf_date_end',
                           -default =>
                              date_for_display( $fields{ 'conf_date_end' } ),
                           -size => $CFG{ CONFS_CONF_DATE_END_SIZE },
                           -maxlength => $CFG{ CONFS_CONF_DATE_END_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_conf_status_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Статус мероприятия</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'conf_status_code',
                            -default => $fields{ 'conf_status_code' },
                            -values => $ref_values_stats,
                            -labels => { @{ $ref_labels_stats } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак зарубежного мероприятия</th>\n";
  print "<td>",
        $query->checkbox( -name => 'conf_foreign_flag',
                          -checked => 
                             $fields{ 'conf_foreign_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_conf_types_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Тип мероприятия</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'conf_type_code',
                            -default => $fields{ 'conf_type_code' },
                            -values => $ref_values_types,
                            -labels => { @{ $ref_labels_types } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Порядковый номер</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].conf_num", "CONFS_CONF_NUM" )
    if $CFG{ HOT_HELP_IS_ON };
  
  print $query->textfield( -name => 'conf_num',
                           -default => $fields{ 'conf_num' },
                           -size => $CFG{ CONFS_CONF_NUM_SIZE },
                           -maxlength => $CFG{ CONFS_CONF_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Мероприятие &mdash; начало цепочки (код)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль orgs.pl в качестве модуля-справочника  
  create_selection_func( func_name => 'select_conf',
                         module_name => 'confs.pl',
                         caller_name => $module,
                         window_name => 'confs',
                         exp_imp => $query->param( 'exp_imp' ) );
  print "<td>",
        $query->textfield( -name => 'conf_code_orig',
                           -default => $fields{ 'conf_code_orig' },
                           -size => $CFG{ CONFS_CONF_CODE_ORIG_SIZE },
                           -maxlength => $CFG{ CONFS_CONF_CODE_ORIG_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )"} .
        qq{ onclick="select_conf()">Список</a>},
        "</td>\n";
  print "</tr>\n";

  # выведем также наименование мероприятия -- начала цепочки
  # (без возможности редактирования)
  print "<tr>\n";
  print "<th>Мероприятие &mdash; начало цепочки (наименование)</th>\n";
  print "<td>", 
        $query->textarea( -name => 'conf_fname_orig',
                          -default => $fields{ 'conf_fname_orig' },
                          -rows => $CFG{ CONFS_CONF_FNAME_ORIG_ROWS },
                          -columns => $CFG{ CONFS_CONF_FNAME_ORIG_COLS },
                          -readonly => 'readonly'
                        ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "CONFS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ CONFS_COMMENT_ROWS },
                          -columns => $CFG{ CONFS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Дата введения мероприятия
  if ( $fname eq 'conf_date_beg' || $fname eq 'conf_date_end' )
  {
    $value = date_for_db( $query->param( $fname ) );
  }
  elsif ( $fname eq 'conf_foreign_flag' )
  {
    # это поле имеет тип bool, поэтому проверяем значение параметра
    # только на непустоту
    $value = $query->param( $fname ) ? 'yes' : 'no';
  }

  return ( $value );  
}
