#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Содержание (оглавление) объектов библиографического описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'CONTENTS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Contents
my @f_names =
   ( 'obj_id',            # Уникальный идентификатор объекта
                          # библиографического описания
     'bib_rec_num',       # Номер библиогр. записи для данного объекта
                          # (основная запись имеет номер 0)
     'order_num',         # Порядковый номер записи в содержании
                          # (оглавлении) данной книги (журнала)
     'unit_num',          # Номер главы, части, параграфа, ...
     'authors',           # Авторы, исполнители, композиторы, ...
     'unit_title',        # Наименование, заглавие
     'class_codes',       # Коды классификаций УДК, ББК ...
     'spec_mat_code',     # Код специфического обозначения материала
                          # (т.е. код единицы измерения: страницы, минуты, ...)
     'location',          # Место расположения в пределах объекта
                          # библиограф. описания (номера страниц, ...)
     'amount',            # Объем материала: например, 24 (страницы),
                          # 11 (штук) (единицы измерения не указывать)
     'duration',          # Длительность звучания (для аудио- и видеозаписей)
     'side',              # Номер стороны (для грампластинок)
     'analytic_obj_id',   # Идентификатор объекта библиографического
                          # описания (если на составную часть
                          # составлено аналитическое описание)
     'comment'            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'order_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указано или неверное значение порядкового " .
          "номера записи в содержании (оглавлении)" );
  }

  if ( $query->param( 'unit_title' ) eq '' )
  {
    push( @error_messages, "Не указано наименование, заглавие" );
  }

  if ( $query->param( 'spec_mat_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код специфического " .
          "обозначения материала" );
  }

  if ( ! ( $query->param( 'side' ) eq '' ||
           $query->param( 'side' ) eq '1' ||
           $query->param( 'side' ) eq '2'
         )
     )
  {
    push( @error_messages, "Неверный номер стороны (для грампластинок)" );
  }

  # допускается пустое поле или цифры
  if ( $query->param( 'analytic_obj_id' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный уникальный идентификатор объекта " .
          "биб. описания (если на составную часть составлено аналитическое " .
          "описание)" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $order_num;          # порядковый номер записи в содержании (оглавлении)
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $order_num   = $query->param( 'order_num' );
  $target_win  = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}contents " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "order_num = '$order_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод всех записей для оглавления (содержания) данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи для содержания (оглавления) данного объекта
  $command = "SELECT * " .
             "FROM ${db_schema}contents " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' " .
             "ORDER BY order_num";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Порядковый номер записи в содержании</th>\n";
    print "<th>Номер главы, части, параграфа</th>\n";
    print "<th>Авторы, исполнители, композиторы, ...</th>\n";
    print "<th>Наименование, заглавие</th>\n";
    print "<th>Страницы, ...</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим порядковый номер записи в содержании, номер главы (части,
      # параграфа), авторов (исполнителей и т.д.), наименование (заглавие),
      # страницы (место расположения)
      # (добавим символ &nbsp; к полям "Номер главы (части)", "Авторы, ...",
      # "Место расположения ..." из соображений эстетики: если значения
      # этих полей равны null, то не отображаются горизонтальные
      # строки-разделители)
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'order_num' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'unit_num' ) ]&nbsp;</td>\n";
      print "<td>$row[ $result->fnumber( 'authors' ) ]&nbsp;</td>\n";
      print "<td>$row[ $result->fnumber( 'unit_title' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'location' ) ]&nbsp;</td>\n";

      # просмотр данной записи в оглавлении в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&order_num=" . $row[ $result->fnumber( 'order_num' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи из оглавления
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&order_num=" . $row[ $result->fnumber( 'order_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи в оглавление
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $order_num;          # порядковый номер записи в содержании (оглавлении)
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $order_num   = $query->param( 'order_num' );
  $target_win  = $query->param( 'target_win' );

  # выбираем одну запись о содержании (оглавлении) данного объекта
  $command = "SELECT * " .
             "FROM ${db_schema}contents " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "order_num = '$order_num'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "порядковый номер записи в оглавлении - $order_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "contents",
                       key => "obj_id, bib_rec_num, " .
                              "order_num:order_num_begin",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # видов специфич. обознач. материала)
  my( $ref_values, $ref_labels ) = popup_spec_mater_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
    # "Номер стороны (для грампластинок)" требует особой обработки, если
    # значение этого поля в таблице равно null
    $fields{ 'side' } = '' if $fields{ 'side' } eq undef;
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );

    # значения по умолчанию

    # порядковый номер записи в содержании (оглавлении) данной книги (журнала)
    if ( $CFG{ CONTENTS_ORDER_NUM } )  # есть параметр в конфиг. файле
    {
      $fields{ 'order_num' } = $CFG{ CONTENTS_ORDER_NUM };
    }
    else
    {
      # будем предлагать пользователю значение поля order_num для новой
      # записи, которое на 1 больше максимального значения этого поля 
      # для данного номера биб. записи данного объекта библиогр. описания
      # (для значений полей obj_id и bib_rec_num в таблице Contents)
      $fields{ 'order_num' } = 
        get_next_key_value( 'contents', 'order_num',
                            'obj_id = ' . $query->param( 'obj_id' ) . ' AND ' .
                            'bib_rec_num = ' . $query->param( 'bib_rec_num' )
                          );
    }

    # номер главы, части, параграфа, ...
    $fields{ 'unit_num' }        = $CFG{ CONTENTS_UNIT_NUM };

    # авторы, исполнители, композиторы, ...
    $fields{ 'authors' }         = $CFG{ CONTENTS_AUTHORS };

    # наименование, заглавие
    $fields{ 'unit_title' }      = $CFG{ CONTENTS_UNIT_TITLE };

    # коды классификаций УДК, ББК ...
    $fields{ 'class_codes' }     = $CFG{ CONTENTS_CLASS_CODES };

    # код специфического обозначения материала
    # (т.е. код единицы измерения: страницы, минуты, ...)
    $fields{ 'spec_mat_code' }   = $CFG{ CONTENTS_SPEC_MAT_CODE };

    # место расположения в пределах объекта библиограф. описания
    # (номера страниц, ...)
    $fields{ 'location' }        = $CFG{ CONTENTS_LOCATION };

    # объем материала: например, 24 (страницы), 11 (штук) (единицы измерения
    # не указывать)
    $fields{ 'amount' }          = $CFG{ CONTENTS_AMOUNT };

    # длительность звучания (для аудио- и видеозаписей)
    $fields{ 'duration' }        = $CFG{ CONTENTS_DURATION };
    
    # номер стороны (для грампластинок)
    $fields{ 'side' }            = $CFG{ CONTENTS_SIDE };
    
    # идентификатор объекта библиографического описания (если на составную
    # часть составлено аналитическое описание)
    $fields{ 'analytic_obj_id' } = $CFG{ CONTENTS_ANALYTIC_OBJ_ID };

    # примечание
    $fields{ 'comment' }         = $CFG{ CONTENTS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи для данного
  # объекта" не подлежат редактированию, не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа, а также при выполнении операции INSERT
  # (значения этих полей сохраняются такими же, какими они были при
  # предыдущем запуске скрипта, поэтому можно не присваивать им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";

  # создадим скрытое поле для "Порядкового номера записи в содержании"
  # ПРИМЕЧАНИЕ. Идентификатор скрытого поля имеет постфикс 'begin' для того,
  #             чтобы можно было обновлять значение этого поля, поскольку оно
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужно и исходное значение этого поля, и его новое
  #             значение (которое, возможно, было изменено пользователем).
  print $query->hidden( -name => 'order_num_begin',
                        -default => $fields{ 'order_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();           # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Порядковый номер записи в содержании</th>\n";
  print "<td>",
        $query->textfield( -name => 'order_num',
                           -default => $fields{ 'order_num' },
                           -size => $CFG{ CONTENTS_ORDER_NUM_SIZE },
                           -maxlength => $CFG{ CONTENTS_ORDER_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номер главы, части, параграфа, ...</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].unit_num", "CONTENTS_UNIT_NUM" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'unit_num',
                           -default => $fields{ 'unit_num' },
                           -size => $CFG{ CONTENTS_UNIT_NUM_SIZE },
                           -maxlength => $CFG{ CONTENTS_UNIT_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Авторы, исполнители, композиторы, ...</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].authors", "CONTENTS_AUTHORS" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'authors',
                          -default => $fields{ 'authors' },
                          -rows => $CFG{ CONTENTS_AUTHORS_ROWS },
                          -columns => $CFG{ CONTENTS_AUTHORS_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Наименование, заглавие</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].unit_title", "CONTENTS_UNIT_TITLE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'unit_title',
                          -default => $fields{ 'unit_title' },
                          -rows => $CFG{ CONTENTS_UNIT_TITLE_ROWS },
                          -columns => $CFG{ CONTENTS_UNIT_TITLE_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Коды классификаций УДК, ББК ...</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].class_codes", "CONTENTS_CLASS_CODES" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'class_codes',
                           -default => $fields{ 'class_codes' },
                           -size => $CFG{ CONTENTS_CLASS_CODES_SIZE },
                           -maxlength => $CFG{ CONTENTS_CLASS_CODES_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_spec_mater_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Специфическое обозначение материала</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'spec_mat_code',
                            -default => $fields{ 'spec_mat_code' },
                            -values => $ref_values,
                            -labels => { @{ $ref_labels } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Место расположения в пределах объекта библиографического " .
            "описания (номера страниц, ...)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].location", "CONTENTS_LOCATION" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'location',
                           -default => $fields{ 'location' },
                           -size => $CFG{ CONTENTS_LOCATION_SIZE },
                           -maxlength => $CFG{ CONTENTS_LOCATION_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Объем материала (без указания единиц измерения)<br />" .
            "например, 24 (страницы), 11 (штук) </th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].amount", "CONTENTS_AMOUNT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'amount',
                           -default => $fields{ 'amount' },
                           -size => $CFG{ CONTENTS_AMOUNT_SIZE },
                           -maxlength => $CFG{ CONTENTS_AMOUNT_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Длительность звучания (для аудио- и видеозаписей)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].duration", "CONTENTS_DURATION" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'duration',
                           -default => $fields{ 'duration' },
                           -size => $CFG{ CONTENTS_DURATION_SIZE },
                           -maxlength => $CFG{ CONTENTS_DURATION_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номер стороны (для грампластинок)</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'side',
                            -default => $fields{ 'side' },
                            -values => [ '1', '2', '' ],
                            -labels => { '1' => 'сторона 1',
                                         '2' => 'сторона 2',
                                         ''  => '-' }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Уникальный идентификатор объекта библиографического описания " .
            "(если на составную часть составлено аналитическое описание)" .
            "</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].analytic_obj_id",
                   "CONTENTS_ANALYTIC_OBJ_ID", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'analytic_obj_id',
                           -default => $fields{ 'analytic_obj_id' },
                           -size => $CFG{ CONTENTS_ANALYTIC_OBJ_ID_SIZE },
                           -maxlength => 
                             $CFG{ CONTENTS_ANALYTIC_OBJ_ID_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "CONTENTS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ CONTENTS_COMMENT_ROWS },
                          -columns => $CFG{ CONTENTS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Содержание (оглавление)" );
  
  return ( 0 );
}
