#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   справочник "Страны"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'COUNTRIES';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Countries
my @f_names =
   ( 
     'country_code',      # Код страны
     'country_fname',     # Наименование страны полное
     'country_sname',     # Наименование страны краткое 
     'country_code_orig', # Код страны, являющейся началом цепочки
                          # (для учета переименований, разночтений, написаний
                          # на различных языках)
     'lang_code',         # Код языка (1 - русский язык)
     'comment'            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'country_code' ) eq '' )
  {
    push( @error_messages, "Не указан код страны" );
  }

  if ( $query->param( 'country_fname' ) eq '' )
  {
    push( @error_messages, "Не указано полное наименование страны" );
  }

  if ( $query->param( 'country_sname' ) eq '' )
  {
    push( @error_messages, "Не указано краткое наименование страны" );
  }

  # допускается пустое поле или цифры
  if ( $query->param( 'country_code_orig' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный код страны -- начала цепочки" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Страны',
                    user => $user
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $country_code;          # код страны
  my $command;               # строка команды для работы с БД

  $country_code = $query->param( 'country_code' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}countries " .
             "WHERE country_code = '$country_code'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $country_code;          # код страны
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $country_code = $query->param( 'country_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Countries WHERE country_code = $country_code";
  push( @table_cmd_pairs, [ 'Countries', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $country_code;          # код страны
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $country_code = $query->param( 'country_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Countries " .
             "WHERE country_code = $country_code";
  push( @table_cmd_pairs, [ 'Countries', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех стран
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выбираем все записи о странах (а также краткое наименование страны,
  # являющейся началом цепочки, и краткое наименование языка,
  # на котором записано название страны)
  $command = "SELECT c1.*, c2.country_sname AS country_sname_orig, " .
                    "l.lang_sname " .
             "FROM ( ${db_schema}countries c1 " .
                  "LEFT JOIN ${db_schema}countries c2 " .
                  "ON c1.country_code_orig = c2.country_code ), " .
                  "${db_schema}langs l " .
             "WHERE c1.lang_code = l.lang_code " .
             "ORDER BY c1.country_sname";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если этот модуль вызван в режиме справочника, и при этом он вызван
    # "сам из себя", то запретим операции просмотра и удаления записей
    # (поскольку режим корректировки выполняется из режима просмотра,
    # то и он будет недоступен)
    if ( $query->param( 'enable_select' ) &&
         $query->param( 'caller_name') eq $module )
    {
      $can_view = 0;
      $can_delete = 0;
    }
    else
    {
      $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                               'user' => $user );
      $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                                 'user' => $user );
    }

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код страны</th>\n";
    print "<th>Наименование краткое</th>\n";
    print "<th>Страна &mdash; начало цепочки (код и наименование)</th>\n";
    print "<th>Язык</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретной страны ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Countries полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'PUB_PRINT_PLACE' )
      {
        # ... полям country_code и country_sname_pub в модуле PUB_PRINT_PLACE
        create_assign_values_func( 'country_code', 'country_sname_pub' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'STOR_OBJS' )
      {
        # ... полям acq_place_country и country_sname_acq в модуле STOR_OBJS
        create_assign_values_func( 'acq_place_country', 'country_sname_acq' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'COUNTRIES' )
      {
        # ... полям country_code_orig и country_sname_orig в модуле COUNTRIES
        create_assign_values_func( 'country_code_orig', 'country_sname_orig' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код страны, краткое наименование, код и краткое наименование
      # страны -- начала цепочки, язык, на котором написано наименование
      # (добавим символ &nbsp; к коду страны, являющейся началом цепочки,
      # из соображений эстетики: если значение поля 'country_code_orig' равно
      # null, то не отображаются горизонтальные строки-разделители)
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'country_code' ) ]</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретной страны, то значение поля
      # country_sname выведем в виде ссылки
      if ( $query->param( 'enable_select' ) )
      {
        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'country_code' ) ],
                                  '$row[ $result->fnumber( 'country_sname' ) ]'
                                )">
                   $row[ $result->fnumber( 'country_sname' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'country_sname' ) ]</td>\n";
      }

      # если страна, являющаяся началом цепочки, отсутствует, то выведем
      # лишь символ пробела "&nbsp;"
      print "<td>",
            ( $row[ $result->fnumber( 'country_code_orig' ) ] ne undef ) ?
              "(" . $row[ $result->fnumber( 'country_code_orig' ) ] . ") " .
              $row[ $result->fnumber( 'country_sname_orig' ) ] :
            "&nbsp;",
            "</td>\n";

      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "country_code=" . $row[ $result->fnumber( 'country_code' ) ] .
              $add_params;

      # просмотр описания страны в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи о стране
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $country_code;               # код страны
  my $command;                    # строка команды для работы с БД

  $country_code = $query->param( 'country_code' );

  # выбираем одну запись о стране
  # выберем также название страны, являющейся началом цепочки
  $command = "SELECT c1.*, c2.country_sname AS country_sname_orig " .
             "FROM ${db_schema}countries c1 " .
                  "LEFT JOIN ${db_schema}countries c2 " .
                  "ON c1.country_code_orig = c2.country_code " .
             "WHERE c1.country_code = '$country_code'"; 

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "страны " . $country_code,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "countries",
                       key => "country_code",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны наименования стран)
  my( $ref_values, $ref_labels ) = popup_langs_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'country_code' } = get_next_key_value( 'countries',
                                                    'country_code' );
    # значения по умолчанию

    # наименование страны полное
    $fields{ 'country_fname' }     = $CFG{ COUNTRIES_COUNTRY_FNAME };

    # наименование страны краткое 
    $fields{ 'country_sname' }     = $CFG{ COUNTRIES_COUNTRY_SNAME };

    # код страны, являющейся началом цепочки
    # (для учета переименований, разночтений, написаний на различных языках)
    $fields{ 'country_code_orig' } = $CFG{ COUNTRIES_COUNTRY_CODE_ORIG };

    # код языка (1 - русский язык)
    $fields{ 'lang_code' }         = $CFG{ COUNTRIES_LANG_CODE };

    # примечание
    $fields{ 'comment' }           = $CFG{ COUNTRIES_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Код страны" формируется автоматически
  # с использованием функции get_next_key_value(), не будем позволять его
  # редактировать, но совсем убрать его нельзя, т.к. оно используется
  # при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'country_code',
                        -default => $fields{ 'country_code' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код страны</th>\n";
  print "<td>", $fields{ 'country_code' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Полное наименование</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].country_fname",
                   "COUNTRIES_COUNTRY_FNAME" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'country_fname',
                          -default => $fields{ 'country_fname' },
                          -rows => $CFG{ COUNTRIES_COUNTRY_FNAME_ROWS },
                          -columns => $CFG{ COUNTRIES_COUNTRY_FNAME_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Краткое наименование</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].country_sname",
                   "COUNTRIES_COUNTRY_SNAME" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'country_sname',
                           -default => $fields{ 'country_sname' },
                           -size => $CFG{ COUNTRIES_COUNTRY_SNAME_SIZE },
                           -maxlength =>
                             $CFG{ COUNTRIES_COUNTRY_SNAME_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Страна &mdash; начало цепочки (код и наименование)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник countries.pl
  create_selection_func( func_name => 'select_orig_country',
                         module_name => 'countries.pl',
                         caller_name => $module,
                         window_name => 'orig_countries',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также название страны, являющейся началом цепочки
  # (без возможности редактирования)
  print "<td>",
        $query->textfield( -name => 'country_code_orig',
                           -default => $fields{ 'country_code_orig' },
                           -size => $CFG{ COUNTRIES_COUNTRY_CODE_ORIG_SIZE },
                           -maxlength =>
                             $CFG{ COUNTRIES_COUNTRY_CODE_ORIG_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'country_sname_orig',
                           -default => $fields{ 'country_sname_orig' },
                           -size => $CFG{ COUNTRIES_COUNTRY_SNAME_ORIG_SIZE },
                           -maxlength =>
                             $CFG{ COUNTRIES_COUNTRY_SNAME_ORIG_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" onclick=} .
        qq{"select_orig_country()">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  
  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<th>Язык</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values,
                            -labels => { @{ $ref_labels } }
                          ),
        "</td>\n";
  print "</tr>\n";
  
  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment",
                   "COUNTRIES_COMMENT" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ COUNTRIES_COMMENT_ROWS },
                          -columns => $CFG{ COUNTRIES_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}
