#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Сведения об издании"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'EDITION_STAT';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Edition_stat
my @f_names =
   ( 'obj_id',           # Уникальный идентификатор объекта
                         # библиографического описания
     'bib_rec_num',      # Номер библиогр. записи для данного объекта
                         # (основная запись имеет номер 0)
     'ed_stat_type',     # Тип сведений об издании (1 - основные сведения;
                         # 2 - параллельные сведения; 3 - дополнительные
                         # сведения)
     'ed_stat_num',      # Порядковый номер сведений (нумерация ведется
                         # в пределах каждого типа сведений)
     'ed_stat',          # Текст сведений
     'lang_code',        # Код языка сведений (1 - код русского языка)
     'process_code',     # Код вида обработки сведений (1 - оригинальное)
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

# коды типов сведений об издании
my %ed_stat_types = ( '1' => 'основные',
                      '2' => 'параллельные',
                      '3' => 'дополнительные'
                    );

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # код типа сведений об издании (допускаются значения от 1 до 3)
  if ( $query->param( 'ed_stat_type' ) !~ /^[123]$/ )
  {
    push( @error_messages, "Не указан или неверный код типа сведений " .
          "об издании<br />. Допускаются значения от 1 до 3" );
  }

  if ( $query->param( 'ed_stat_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный номер сведений" );
  }

  if ( $query->param( 'ed_stat' ) eq '' )
  {
    push( @error_messages, "Не указан текст сведений" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  if ( $query->param( 'process_code' ) eq '' )
  {
    push( @error_messages, "Не указан код вида обработки сведений" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $ed_stat_type;       # код типа сведений об издании
  my $ed_stat_num;        # порядковый номер сведений
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id       = $query->param( 'obj_id' );
  $bib_rec_num  = $query->param( 'bib_rec_num' );
  $ed_stat_type = $query->param( 'ed_stat_type' );
  $ed_stat_num  = $query->param( 'ed_stat_num' );
  $target_win   = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}edition_stat " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "ed_stat_type = '$ed_stat_type' AND " .
                   "ed_stat_num = '$ed_stat_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех сведений об издании для данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о сведениях об издании для данного объекта
  # (получим также краткое наименования языка, на котором записаны сведения)
  $command = "SELECT e.*, l.lang_sname " .
             "FROM ${db_schema}edition_stat e, ${db_schema}langs l " .
             "WHERE e.obj_id = $obj_id AND " .
                   "e.bib_rec_num = $bib_rec_num AND " .
                   "e.lang_code = l.lang_code " .
             "ORDER BY e.ed_stat_type, e.ed_stat_num";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Тип сведений об издании</th>\n";
    print "<th>Порядковый номер сведений</th>\n";
    print "<th>Текст сведений</th>\n";
    print "<th>Язык сведений</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим наименование типа сведений об издании, номер сведений об
      # издании, текст сведений и язык сведений
      print "<td>" .
            $ed_stat_types{ $row[ $result->fnumber( 'ed_stat_type' ) ] } .
            "</td>\n";
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'ed_stat_num' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'ed_stat' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # просмотр данной записи о сведениях об издании в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&ed_stat_type=" . $row[ $result->fnumber( 'ed_stat_type' ) ] .
            "&ed_stat_num=" . 
              $row[ $result->fnumber( 'ed_stat_num' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о сведениях об издании
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&ed_stat_type=" . $row[ $result->fnumber( 'ed_stat_type' ) ] .
            "&ed_stat_num=" . 
              $row[ $result->fnumber( 'ed_stat_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о сведениях об издании
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $ed_stat_type;       # код типа сведений об издании
  my $ed_stat_num;        # порядковый номер сведений
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id       = $query->param( 'obj_id' );
  $bib_rec_num  = $query->param( 'bib_rec_num' );
  $ed_stat_type = $query->param( 'ed_stat_type' );
  $ed_stat_num  = $query->param( 'ed_stat_num' );
  $target_win   = $query->param( 'target_win' );

  # выбираем одну запись о сведениях об издании
  $command = "SELECT * " .
             "FROM ${db_schema}edition_stat " .
             "WHERE obj_id = $obj_id AND " .
                   "bib_rec_num = $bib_rec_num AND " .
                   "ed_stat_type = '$ed_stat_type' AND " .
                   "ed_stat_num = '$ed_stat_num'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "код типа сведений об издании - $ed_stat_type, " .
                     "номер сведений об издании - $ed_stat_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "edition_stat",
                       key => "obj_id, bib_rec_num, " .
                              "ed_stat_type:ed_stat_type_begin, " .
                              "ed_stat_num:ed_stat_num_begin",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны сведения об издании)
  my( $ref_values_langs, $ref_labels_langs ) = popup_langs_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов языковой обработки сведения об издании)
  my( $ref_values_procs, $ref_labels_procs ) =
    popup_process_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );

    # значения по умолчанию

    # тип сведений об издании
    $fields{ 'ed_stat_type' } = $CFG{ EDITION_STAT_ED_STAT_TYPE };

    # порядковый номер сведений
    $fields{ 'ed_stat_num' }  = $CFG{ EDITION_STAT_ED_STAT_NUM };

    # текст сведений
    $fields{ 'ed_stat' }      = $CFG{ EDITION_STAT_ED_STAT };

    # код языка сведений
    $fields{ 'lang_code' }    = $CFG{ EDITION_STAT_LANG_CODE };

    # код вида обработки сведений
    $fields{ 'process_code' } = $CFG{ EDITION_STAT_PROCESS_CODE };

    # комментарий
    $fields{ 'comment' }      = $CFG{ EDITION_STAT_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи для данного
  # объекта" не подлежат редактированию, не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа, а также при выполнении операции INSERT
  # (значения этих полей сохраняются такими же, какими они были при
  # предыдущем запуске скрипта, поэтому можно не присваивать им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";

  # создадим скрытые поля для "Типа сведений об издании" и "Порядкового
  # номера сведений"
  # ПРИМЕЧАНИЕ. Идентификаторы скрытых полей имеют постфикс 'begin' для того,
  #             чтобы можно было обновлять значения этих полей, поскольку они
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm.
  #             Т.е. нам нужны и исходные значения этих полей, и их новые
  #             значения (которые, возможно, были изменены пользователем).
  print $query->hidden( -name => 'ed_stat_type_begin',
                        -default => $fields{ 'ed_stat_type' }
                      ), "\n";
  print $query->hidden( -name => 'ed_stat_num_begin',
                        -default => $fields{ 'ed_stat_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();      # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Тип сведений об издании</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'ed_stat_type',
                            -default => $fields{ 'ed_stat_type' },
                            -values => [ sort keys %ed_stat_types ],
                            -labels => \%ed_stat_types
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Порядковый номер сведений</th>\n";
  print "<td>",
        $query->textfield( -name => 'ed_stat_num',
                           -default => $fields{ 'ed_stat_num' },
                           -size => $CFG{ EDITION_STAT_ED_STAT_NUM_SIZE },
                           -maxlength => 
                             $CFG{ EDITION_STAT_ED_STAT_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Текст сведений</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].ed_stat", "EDITION_STAT_ED_STAT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'ed_stat',
                           -default => $fields{ 'ed_stat' },
                           -size => $CFG{ EDITION_STAT_ED_STAT_SIZE },
                           -maxlength => 
                             $CFG{ EDITION_STAT_ED_STAT_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Язык сведений об издании</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_process_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид языковой обработки сведений об издании</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'process_code',
                            -default => $fields{ 'process_code' },
                            -values => $ref_values_procs,
                            -labels => { @{ $ref_labels_procs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "EDITION_STAT_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ EDITION_STAT_COMMENT_ROWS },
                          -columns => $CFG{ EDITION_STAT_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Сведения об издании" );
  
  return ( 0 );
}
