#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Экспорт/импорт"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module;

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# имя модуля назначим в зависимости от значения path_info
# (вызов этого модуля без path_info выполняться не должен)
$module = ( $query->path_info =~ /^\/export$/ ) ? 'EXPORT' : 'IMPORT';

# проверим, есть ли у пользователя право на вход в этот модуль
if ( ! can_work( $user, $module, 'main', $query ) )
{
  exit( 0 );
}

# выводим начальную форму для выполнения экспорта/импорта
if ( ( $query->path_info =~ /^\/export$/ || $query->path_info =~ /^\/import$/ )
     && ! $query->param )
{
  create_main_form();
}

# начинаем экспорт
elsif ( $query->path_info =~ /^\/export$/ && $query->param( 'start' ) &&
        have_access( 'module' => $module, 'procedure' => 'START',
                     'user' => $user )
   )
{
  do_start_export();
}

# экспортируем всю базу данных
elsif ( $query->path_info =~ /^\/export$/ && $query->param( 'export_all' ) &&
        have_access( 'module' => $module, 'procedure' => 'EXPORT_ALL',
                     'user' => $user )
   )
{
  do_export_db();
}

# завершаем экспорт
elsif ( $query->path_info =~ /^\/export$/ && $query->param( 'finish' ) &&
        have_access( 'module' => $module, 'procedure' => 'FINISH',
                     'user' => $user )
      )
{
  do_finish_export();
}

# начинаем импорт
elsif ( $query->path_info =~ /^\/import$/ && $query->param( 'start' ) &&
        have_access( 'module' => $module, 'procedure' => 'START',
                     'user' => $user )
      )
{
  do_start_import();
}

# завершаем импорт
elsif ( $query->path_info =~ /^\/import$/ && $query->param( 'finish' ) &&
        have_access( 'module' => $module, 'procedure' => 'FINISH',
                     'user' => $user )
      )
{
  do_finish_import();
}

exit( 0 );


# -----------------------------------------------------------------------------
# выполнение некоторых предварительных проверок в схеме БД "экспорт/импорт"
# непосредственно после "закачки" данных из файла экспорта/импорта в схему БД
# -----------------------------------------------------------------------------
sub check_before_import
{
  # параметр
  my $conn = shift;          # дескриптор соединения с БД

  my $result;                # результат выборки из БД
  my $command;               # строка SQL-команды
  my @row;                   # для хранения одной записи из выборки
  my $series_issn_null;      # признак наличия пустых полей series_issn

  # сделаем проверку наличия в таблице Series записей с пустым значением
  # поля series_issn (это поле CHECK_FIELDS)
  # ПРИМЕЧАНИЕ. В настоящее время пользователь должен ввести в эти пустые
  #             поля какие-либо искусственные значения. Причем, он должен
  #             визуально сопоставить импортируемые описания серий/подсерий
  #             с теми, которые уже есть в целевой БД. При наличии совпадений
  #             необходимо ввести искусственные значения и в поле series_issn
  #             в записи целевой БД. Эти значения должны совпадать с теми,
  #             которые пользователь ввел в импортируемые записи. Цель этих
  #             манипуляций -- имитация наличия ISSN у серии. После завершения
  #             процедуры импорта нужно удалить псевдо-ISSN в записях таблиц
  #             Series целевой БД.
  $command = "SELECT * FROM exp_imp.Series";

  # сделаем выборку из БД
  # (второй параметр -- строка пробелов, это нужно для того, чтобы
  # не выводилось сообщение в случае, когда записи не найдены)
  $result = exec_select( $command, "  ", $conn );
  
  # записи есть
  if ( $result )
  {
    # просматриваем все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      if ( $row[ $result->fnumber( 'series_issn' ) ] eq undef )
      {
        $series_issn_null = 1;
      }
    }

    if ( $series_issn_null )     # есть хоть одна запись без ISSN
    {
      print "<p class=\"info\">";
      print( "В таблице \"" . $EXP_CFG{ TABLES }{ 'Series' }{ TABLE_NAME } .
             "\" есть записи с отсутствующим значением поля ISSN. " .
             "Необходимо ввести искусственные значения в импортируемые " .
             "записи. Если в целевой базе данных есть описания таких же " .
             "серий, нужно ввести такие же значения и в целевую базу " .
             "данных. После завершения импортирования данных эти " .
             "искусственные значения ISSN нужно удалить</p>\n" );
    }
  }
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление всех записей из всех таблиц схемы БД "экспорт/импорт" и схемы 
# "импорт"
# -----------------------------------------------------------------------------
sub clear_tables
{
  # параметр
  my $conn = shift;              # дескриптор соединения с БД

  my $result;                    # результат выборки из БД
  my $table;                     # таблица из общего списка таблиц БД
  my $deleted_count_exp_imp = 0; # счетчик удаленных записей (схема exp_imp)
  my $deleted_count_imp = 0;     # счетчик удаленных записей (схема imp)
  my $i;
  
  # обрабатываем все таблицы в ОБРАТНОМ порядке относительно того, в котором
  # они определены в SQL-файле создания БД (e_arch.sql), это позволяет
  # соблюсти требования внешних ключей
  # ПРИМЕЧАНИЕ. При выбранном способе удаления записей не используется
  #             возможность каскадного удаления записей из таблиц, связанных
  #             по внешним ключам. Например, можно было бы удалять записи из 
  #             таблиц Bib_objs и Bib_recs, а из таблиц, соответствующих 
  #             областям биб. описания, связанные записи удалила бы СУБД.
  #             Однако в этом случае было бы сложнее подсчитать общее число
  #             удаленных записей. Какое решение лучше -- покажет время.
  for ( $i = scalar @{ $EXP_CFG{ TABLES_ORDER } } - 1; $i >= 0; $i-- )
  {
    $table = $EXP_CFG{ TABLES_ORDER }->[ $i ]; # у нас же ссылка на массив

    # будем подсчитывать число записей, удаленных из экспортной схемы БД
    $deleted_count_exp_imp += exec_delete( $conn, $table, 'exp_imp' );
  }
  
  # удалим записи из таблиц "импортной" схемы БД
  for ( $i = scalar @{ $EXP_CFG{ TABLES_ORDER } } - 1; $i >= 0; $i-- )
  {
    $table = $EXP_CFG{ TABLES_ORDER }->[ $i ]; # у нас же ссылка на массив

    # будем подсчитывать число записей, удаленных из "импортной" схемы БД
    $deleted_count_imp += exec_delete( $conn, $table, 'imp' );
  }

  # если удалена хотя бы одна запись из экспортной схемы БД
  if ( $deleted_count_exp_imp > 0 )
  {
    print "<p class=\"info\">";
    print( "Записи удалены из базы данных экспорта: " .
           "$deleted_count_exp_imp шт.</p>\n" );
  }
  else
  {
    print "<p class=\"info\">";
    print( "Записей в базе данных экспорта не было</p>\n" );
  }

  # если удалена хотя бы одна запись из "импортной" схемы БД
  if ( $deleted_count_imp > 0 )
  {
    print "<p class=\"info\">";
    print( "Записи удалены из базы данных импорта: " .
           "$deleted_count_imp шт.</p>\n" );
  }
  else
  {
    print "<p class=\"info\">";
    print( "Записей в базе данных импорта не было</p>\n" );
  }
  
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод начальной формы
# -----------------------------------------------------------------------------
sub create_main_form
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_blank" );

  open_top();           # начинаем оформление верхней части окна
  create_top();         # формируем верхнюю часть окна
  close_top();          # завершаем оформление верхней части окна
  open_bottom();        # начинаем оформление нижней части окна
  close_bottom();       # завершаем оформление нижней части окна

  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # заголовок зависит от условного имени модуля
  my $module_title = ( $module eq 'EXPORT' ) ? 'Экспорт данных' :
                                               'Импорт данных';
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => $module_title,
                    user => $user,
                    full_top => 'yes'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование всей базы данных (всех записей из всех таблиц схемы public)
# -----------------------------------------------------------------------------
sub do_export_db
{
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее
  my $table;                 # имя таблицы БД
  my $i;

  # обрабатываем все таблицы в том порядке, в котором они определены 
  # в SQL-файле создания БД (e_arch.sql), это позволяет соблюсти требования
  # внешних ключей
  for ( $i = 0; $i < scalar @{ $EXP_CFG{ TABLES_ORDER } }; $i++ )
  {
    $table = $EXP_CFG{ TABLES_ORDER }->[ $i ]; # у нас же ссылка на массив

    # если зависимую таблицу экспортировать не нужно -- к началу цикла
    next if $EXP_CFG{ TABLES }{ $table }{ TABLE_EXPORT } eq 'disabled';

    # сформируем строку-команду для выборки данных из таблицы и поместим
    # ее в массив (используем ссылку на массив из двух элементов, первым
    # из которых является имя таблицы)
    $command = "SELECT * FROM $table";
    push( @table_cmd_pairs, [ $table, $command ] );

    # TODO. Оптимизировать эту процедуру за счет исключения (по возможности)
    #       проверок на дублирование и т. п.
    # ПРИМЕЧАНИЕ. Не удалять пока что этот фрагмент! Это была грубая попытка
    #             оптимизации. Она не работает, поскольку без обработки
    #             внешних ключей, в которых ссылающаяся и ссылочная таблицы
    #             совпадают, невозможно корректно обработать цепочки иерархий
    #             в таблицах -- команды UPDATE не формируются.
#    # если для данной таблицы существуют параметры, наличие которых приводит
#    # к рекурсивным вызовам ряда процедур, то удалим эти параметры, поскольку
#    # в данном случае они не нужны
#    delete( $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS } )
#      if exists( $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS } );
#
#    delete( $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS_EXCLUDE } )
#      if exists( $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS_EXCLUDE } );
#
#    delete( $EXP_CFG{ TABLES }{ $table }{ DEPENDENT_TABLES } )
#      if exists( $EXP_CFG{ TABLES }{ $table }{ DEPENDENT_TABLES } );
  }

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# завершить экспорт
# -----------------------------------------------------------------------------
sub do_finish_export
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_blank" );

  open_top();        # начинаем оформление верхней части окна
  create_top();      # формируем верхнюю часть окна
  close_top();       # завершаем оформление верхней части окна
  
  open_bottom();     # начинаем оформление нижней части окна

  # экспортируем все записи из всех таблиц схемы БД "экспорт/импорт"
  export_tables();
  
  print "<p class=\"info\">Экспортирование данных завершено</p>\n";

  close_bottom();    # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# завершить импорт
# -----------------------------------------------------------------------------
sub do_finish_import
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_blank" );

  open_top();        # начинаем оформление верхней части окна
  create_top();      # формируем верхнюю часть окна
  close_top();       # завершаем оформление верхней части окна
  
  open_bottom();     # начинаем оформление нижней части окна

  # импортируем все записи из всех таблиц схемы БД "импорт" в схему public
  # (основную схему)
  import_tables();
  
  # теперь удаляем признак нахождения системы в состоянии ИМПОРТИРОВАНИЯ 
  # данных, чтобы предотвратить возможное повторное импортирование тех данных,
  # которые обрабатываются по схеме 'adjust_import' и 'import' (это касается
  # таблиц Bib_objs, Bib_recs и др.)
  # ПРИМЕЧАНИЕ. После удаления этого файла не будут отображаться ссылки
  #             "Импортируемые данные" и "Завершить импорт" в меню модуля
  #             "Импорт".
  # TODO. Поскольку процедура create_top() выполняется ДО удаления файла
  #       ./exp_imp/import, ссылки "Импортируемые данные" и "Завершить импорт"
  #       БУДУТ созданы. Нужно либо выполнять операцию удаления файла
  #       перед вызовом процедуры create_top() (что не совсем правильно,
  #       поскольку ведь импорт еще не завершен, а мы уже убрали признак;
  #       а что если сбой?), либо нужно удалять эти ссылки, используя какой-то
  #       способ, возможно, JavaScript.
  if ( scalar stat( "./exp_imp/import" ) > 0 )  # если файл существует
  {
    if ( ! unlink( "./exp_imp/import" ) )
    {
      my $msg = "Не удалось удалить файл ./exp_imp/import. Ошибка: $!";
      # сообщение для пользователя
      print "<p class=\"error\">$msg</p>\n";
      print STDERR $msg . "\n";     # выводим сообщение в лог
      die $msg;
    }
  }

  # удалим ссылки на операции 'Импортируемые данные' и 'Завершить импорт';
  # сделаем это без всякого изящества -- путем присваивания ссылкам
  # пустых значений
  # ВАЖНО. Выполнять операцию завершения импорта (в отличие от операции
  #        завершения экспорта) повторно НЕЛЬЗЯ, поскольку биб. описания
  #        импортируются по схеме 'import', в результате возникнет дублирование
  #        биб. описаний в целевой базе данных.
  # TODO. Нужно найти более грамотное решение!
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript">
  <!--
  var list = document.getElementsByTagName( "A" );
  var i;

  for( i = 0; i < list.length; i++ )
  {
    if ( list[ i ].innerHTML == 'Завершить импорт' ||
         list[ i ].innerHTML == 'Импортируемые данные'
       )
    {
      list[ i ].innerHTML = '';
    }
  }
  //-->
  </script>
END_OF_SCRIPT
  ;

  print "<p class=\"info\">Импортирование данных завершено</p>\n";

  close_bottom();    # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# начать экспорт
# -----------------------------------------------------------------------------
sub do_start_export
{
  my $conn;          # дескриптор соединения с БД

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_blank" );

  open_top();        # начинаем оформление верхней части окна
  create_top();      # формируем верхнюю часть окна
  close_top();       # завершаем оформление верхней части окна
  
  open_bottom();     # начинаем оформление нижней части окна

  # сформируем признак того, что наша система будет находиться
  # в состоянии ЭКСПОРТИРОВАНИЯ данных

  # сначала удаляем признак нахождения системы в состоянии 
  # ИМПОРТИРОВАНИЯ данных
  if ( scalar stat( "./exp_imp/import" ) > 0 )  # если файл существует
  {
    if ( ! unlink( "./exp_imp/import" ) )
    {
      my $msg = "Не удалось удалить файл ./exp_imp/import. Ошибка: $!";
      # сообщение для пользователя
      print "<p class=\"error\">$msg</p>\n";
      print STDERR $msg . "\n";     # выводим строку команды в лог
      die $msg;
    }
  }
  # создаем файл -- признак ЭКСПОРТИРОВАНИЯ
  if ( ! open( EXP, "> ./exp_imp/export" ) )
  {
    my $msg = "Не удалось создать файл ./exp_imp/export. Ошибка: $!";
    # сообщение для пользователя
    print "<p class=\"error\">$msg</p>\n";
    print STDERR $msg . "\n";     # выводим строку команды в лог
    die $msg;
  }
  close( EXP );  

  $conn = exec_connect_db();           # подключаемся к БД
  exec_transaction( $conn, "BEGIN" );  # выполним операции в рамках транзакции
  
  # удалим все записи из всех таблиц схемы БД "экспорт/импорт"
  # (эта процедура удаляет также и все записи из всех таблиц схемы "импорт")
  clear_tables( $conn );

  exec_transaction( $conn, "END" );    # конец транзакции
  
  print "<p class=\"info\">Система подготовлена к экспортированию данных" .
        "</p>\n";

  close_bottom();    # завершаем оформление нижней части окна

  # TODO. Здесь нужно вставить код, который добавляет html-ссылки для
  #       режимов "Экспортируемые данные" и "Завершить экспорт". В настоящее
  #       время для того, чтобы эти ссылки появились, нужно выйти из этой
  #       страницы и зайти на нее снова.
  #       См. TODO в процедуре do_start_import().

  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# начать импорт
# -----------------------------------------------------------------------------
sub do_start_import
{
  my $conn;          # дескриптор соединения с БД
  my $result;        # результат выполнения SQL-команды
  my $fill_ok;       # признак успешного заполнения данными таблиц импорта
  
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_blank" );

  open_top();        # начинаем оформление верхней части окна
  create_top();      # формируем верхнюю часть окна
  close_top();       # завершаем оформление верхней части окна
  
  open_bottom();     # начинаем оформление нижней части окна

  # сначала удаляем признак нахождения системы в состоянии 
  # ЭКСПОРТИРОВАНИЯ данных
  if ( scalar stat( "./exp_imp/export" ) > 0 )  # если файл существует
  {
    if ( ! unlink( "./exp_imp/export" ) )
    {
      my $msg = "Не удалось удалить файл ./exp_imp/export. Ошибка: $!";
      # сообщение для пользователя
      print "<p class=\"error\">$msg</p>\n";
      print STDERR $msg . "\n";     # выводим строку команды в лог
      die $msg;
    }
  }

  $conn = exec_connect_db();           # подключаемся к БД
  exec_transaction( $conn, "BEGIN" );  # выполним операции в рамках транзакции
  
  # удалим все записи из всех таблиц схем БД "экспорт/импорт" и "импорт"
  clear_tables( $conn );

  # введем записи в таблицы схемы БД "экспорт/импорт", используя
  # файл экспорта (SQL-команды)
  $fill_ok = fill_tables( $conn );

  # выполним некоторые предварительные проверки только что введенных данных
  check_before_import( $conn );

  exec_transaction( $conn, "END" );    # конец транзакции

  if ( $fill_ok )       # успешное заполнение таблиц
  {  
    # сформируем признак того, что наша система будет находиться
    # в состоянии ИМПОРТИРОВАНИЯ данных

    # создаем файл -- признак ИМПОРТИРОВАНИЯ
    if ( ! open( IMP, "> ./exp_imp/import" ) )
    {
      my $msg = "Не удалось создать файл ./exp_imp/import. Ошибка: $!";
      # сообщение для пользователя
      print "<p class=\"error\">$msg</p>\n";
      print STDERR $msg . "\n";     # выводим строку команды в лог
      die $msg;
    }
    close( IMP );  

    print "<p class=\"info\">Система подготовлена к импортированию данных" .
          "</p>\n";
  }

  # TODO. Поскольку процедура create_top() выполняется ДО создания файла
  #       ./exp_imp/import, ссылки "Импортируемые данные" и "Завершить импорт"
  #       НЕ БУДУТ созданы. Нужно добавлять эти ссылки, используя какой-то
  #       способ, возможно, JavaScript.
  
  close_bottom();    # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление всех записей из указанной таблицы схемы БД "экспорт/импорт" или
# схемы БД "импорт"
# -----------------------------------------------------------------------------
sub exec_delete
{
  # параметры процедуры
  my $conn = shift;             # дескриптор соединения с БД
  my $table = shift;            # имя таблицы
  my $schema = shift;           # схема БД

  my $result;                   # результат выполнения SQL-команды

  # строка SQL-команды
  # не будем использовать каскадное удаление, т. к. нам нужно подсчитывать
  # число удаленных записей для каждой таблицы
  my $command = "DELETE FROM $schema.$table";

  # DEBUG 
  print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

  $result = $conn->exec( $command );   # теперь выполняем удаление из БД

  # если произошла ошибка при удалении записей из базы данных
  if ( PGRES_COMMAND_OK ne $result->resultStatus )
  {
    # сообщение для пользователя
    print "<p class=\"error\">$CFG{ ERROR_DELETE }</p>\n";
    print STDERR $command . "\n";     # выводим строку команды в лог
    die $conn->errorMessage;
  }

  # проверим число фактически удаленных записей и выведем информацию
  # (в режиме отладки)
  if ( $DEBUG )
  {
    if ( $result->cmdTuples == 0 )
    {
      print_debug_info( "Записей в таблице $table (" .
                        $EXP_CFG{ TABLES }{ $table }{ TABLE_NAME } .
                        ") схемы $schema не было</p>\n"
                      );
    }
    else          # ( $result->cmdTuples > 1 )
    {
      print_debug_info( "Записи из таблицы $table (" .
                        $EXP_CFG{ TABLES }{ $table }{ TABLE_NAME } .
                        ") схемы $schema удалены: " .
                        $result->cmdTuples . " шт." . "</p>\n"
                      );
    }
  }
  # возвратим число удаленных записей 
  # (мы их подсчитываем в вызывающей процедуре)
  return ( $result->cmdTuples );
}

# -----------------------------------------------------------------------------
# экспортирование всех записей из указанной таблицы схемы БД "экспорт/импорт"
# в текстовый файл (в виде SQL-команд INSERT и UPDATE)
# -----------------------------------------------------------------------------
sub exec_export
{
  # параметры процедуры
  my $conn = shift;        # дескриптор соединения с БД
  my $table = shift;       # имя таблицы

  my $result;              # результат выполнения SQL-команды
  my @row;                 # для хранения одной записи из выборки
  my $f_key_name;          # имя внешнего ключа
  my @f_key_fields;        # массив ключевых полей внешнего ключа
  my @key_fields;          # массив ключевых полей
  my %rec = ();            # хеш-массив, содержащий значения полей текущей
                           # записи из выборки  
  my $fname;               # имя поля
  my $value;               # значение поля

  # строка SQL-команды
  my $command = "SELECT * FROM exp_imp.$table";

  my @insert_commands = ();  # массив SQL-команд INSERT
  my @update_commands = ();  # массив SQL-команд UPDATE

  my $i;

  # сделаем выборку из БД
  # (второй параметр -- строка пробелов, это нужно для того, чтобы
  # не выводилось сообщение в случае, когда записи не найдены)
  $result = exec_select( $command, "  ", $conn );
  
  # записей нет
  if ( ! $result )
  {
    # DEBUG 
    print_debug_info( "Записей в таблице $table (" .
                      $EXP_CFG{ TABLES }{ $table }{ TABLE_NAME } .
                      ") не было" ) if $DEBUG;
    return ( 0 );          # экспортировано 0 записей
  }

  # обрабатываем все записи из полученной выборки
  while ( @row = $result->fetchrow )
  {
    %rec = ();          # "обнулим" хеш-массив для новой записи

    # поместим в него значения всех полей текущей записи из выборки
    for ( $i = 0; $i < scalar @row; $i++ )
    {
      # используем эти переменные для удобства
      $fname = $result->fname( $i );  # по номеру поля получим его имя
      $value = $row[ $i ];

      $rec{ $fname } = $value;        # запишем значение поля в хеш-массив
    }

    # для последующей реализации импорта нам сейчас необходимо увеличить
    # на определенную величину значения ключевых полей данной таблицы,
    # а также ключевых полей всех внешних ключей таблицы

    # увеличим значения всех регулируемых полей таблицы, если они не равны
    # undef (NULL) или 0
    foreach $fname ( @{ $EXP_CFG{ TABLES }{ $table }{ ADJUSTABLE_FIELDS } } )
    {
      if ( ! ( $rec{ $fname } eq undef || $rec{ $fname } == 0 ) )
      {
        $rec{ $fname } += $EXP_CFG{ PARAMS }{ ADJUSTABLE_FIELDS_INCREASE };
      }
    }

    # если есть внешний ключ, который ссылается на эту же таблицу,
    # то, возможно, потребуется сформировать не только команду INSERT,
    # но и команду UPDATE

    # если внешних ключей вообще нет
    if ( ! exists $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS } )
    {
      make_insert_cmd_for_export( \@insert_commands, $table, \%rec );
      next;
    }

    # для всех внешних ключей таблицы $table
    # (если у таблицы есть два и более внешних ключей, ссылающихся не нее саму,
    # то команд UPDATE будет столько же, хотя, в принципе, можно бы формировать
    # только одну команду UPDATE, которая включала бы все поля, являющиеся
    # членами этих внешних ключей, но это более сложный в реализации вариант)
    foreach $f_key_name ( keys %{ $EXP_CFG{ TABLES }{ $table }
                                          { FOREIGN_KEYS } } )
    {
      # ТОЛЬКО если ключ ссылается на ЭТУ же таблицу...
      if ( $table eq $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS }
                             { $f_key_name }{ REF_TABLE } )
      {
        # ... нужно сформировать команду UPDATE
        make_update_cmd_for_export( \@update_commands, $table, \%rec, 
                                    $f_key_name );
      }
    }
    # наконец формируем команду INSERT
    make_insert_cmd_for_export( \@insert_commands, $table, \%rec );
  }

  # открываем в режиме дополнения файл для вывода SQL-команд
  if ( ! open( EXPORT_FILE, ">> ./exp_imp/exported_data" ) )
  {
    my $msg = "Не удалось открыть файл exported_data. Ошибка: $!";
    # сообщение для пользователя
    print "<p class=\"error\">$msg</p>\n";
    print STDERR $msg . "\n";     # выводим строку команды в лог
    die $msg;
  } 

  # сначала выводим все команды INSERT
  foreach ( @insert_commands )
  {
    print EXPORT_FILE $_ . ";\n";
  }

  # затем -- все команды UPDATE, тогда внешние ключи будут довольны
  foreach ( @update_commands )
  {
    print EXPORT_FILE $_ . ";\n";
  }

  close( EXPORT_FILE );

  # возвратим число экспортированных записей 
  # (мы их подсчитываем в вызывающей процедуре)
  return ( scalar @insert_commands );
}

# -----------------------------------------------------------------------------
# импортирование записей из указанной таблицы схемы БД "экспорт/импорт"
# в схему public (основную схему)
# -----------------------------------------------------------------------------
sub exec_import
{
  # параметры процедуры
  my $conn = shift;        # дескриптор соединения с БД
  my $table = shift;       # имя таблицы

  my $result;              # результат выполнения SQL-команды
  my @row;                 # для хранения одной записи из выборки
  my $command;             # строка SQL-команды
  my @update_cmds = ();    # массив SQL-команд UPDATE
  my @insert_cmds = ();    # массив SQL-команд INSERT
  my $imported_count;      # счетчик импортированных записей

  # если данные из таблицы импортировать не нужно, завершаем работу
  return if ( $EXP_CFG{ TABLES }{ $table }{ TABLE_IMPORT } eq 'disabled' );

  # таблицы могут относиться к одному из нескольких "классов" импорта, каждый
  # из которых отличается набором операций, производимых с таблицей

  # сформируем команду для выполнения выборки из таблицы $table
  $command = make_select_cmd( $table );
  
  # сделаем выборку из БД
  # (второй параметр -- строка пробелов, это нужно для того, чтобы
  # не выводилось сообщение в случае, когда записи не найдены)
  $result = exec_select( $command, "  ", $conn );
  
  # записей нет
  if ( ! $result )
  {
    print_debug_info( "Записей в таблице импорта $table (" .
                      $EXP_CFG{ TABLES }{ $table }{ TABLE_NAME } .
                      ") не было</p>\n"
                    ) if $DEBUG;
    return ( 0 );          # импортировано 0 записей
  }

  # режим для всех справочников и, возможно, для нескольких других таблиц
  if ( $EXP_CFG{ TABLES }{ $table }{ IMPORT_MODE } eq 'check_adjust_import' )
  {
    process_check_adjust_import( $table, $result, \@update_cmds,
                                 \@insert_cmds );
  }  
  # режим для таблицы Bib_objs (возможно, в будущем и для других таблиц)
  elsif ( $EXP_CFG{ TABLES }{ $table }{ IMPORT_MODE } eq 'adjust_import' )
  {
    process_adjust_import( $table, $result, \@update_cmds, \@insert_cmds );
  }  
  # режим для таблиц Orgs_act, Ser_descrs, Ser_titles (возможно, в будущем
  # и для других таблиц)
  # этот режим применяется тогда, когда в целевой БД может существовать
  # такая же запись в главной таблице, однако связанной с ней записи
  # в подчиненной таблице -- нет
  # ПРИМЕР. В таблице Orgs как в исходной БД, так и в целевой БД есть запись
  #         для организации с именем "Алгоритм", однако в целевой БД
  #         в таблице Orgs_act нет записи для вида деятельности "Научная",
  #         а в исходной БД такая запись есть. Мы должны добавить такую
  #         запись в целевую БД. Образно говоря, при импортировании
  #         биб. описания мы импортируем все дерево, т.е. все записи
  #         из всех областей биб. описания для ДАННОГО объекта. Мы не можем
  #         "доимпортировать", например, какую-нибудь строчку из оглавления
  #         или один из стандартных номеров (ББК или УДК). В случае же
  #         с видами деятельности организации мы можем "доимпортировать"
  #         те виды деятельности, которых нет у данной организации в целевой
  #         базе данных.
  elsif ( $EXP_CFG{ TABLES }{ $table }{ IMPORT_MODE } eq 'check_import' )
  {
    process_check_import( $table, $result, \@insert_cmds );
  }  
  elsif ( $EXP_CFG{ TABLES }{ $table }{ IMPORT_MODE } eq 'import' )
  {
    process_import( $table, $result, \@insert_cmds );
  }  

  # выполним команды обновления и вставки записей (подсчитываем только
  # число добавленных записей)
  exec_update_insert( $conn, \@update_cmds );
  $imported_count = exec_update_insert( $conn, \@insert_cmds );

  return ( $imported_count );
}

# -----------------------------------------------------------------------------
# выполнение команд обновления (UPDATE) и вставки (INSERT) при импортировании
# записей из указанной таблицы схемы БД "импорт" в схему public (основную
# схему)
# -----------------------------------------------------------------------------
sub exec_update_insert
{
  # параметры процедуры
  my $conn = shift;        # дескриптор соединения с БД
  my $ref_cmds = shift;    # ссылка на массив SQL-команд UPDATE или INSERT

  my $cmd_type;            # тип команды
  my $result;              # результат выполнения SQL-команды
  my $command;             # строка SQL-команды

  # если массив пустой, завершаем работу
  return ( 0 ) if ( scalar @{ $ref_cmds } == 0 );

  # тип всех команд в массиве -- один и тот же ('UPDATE' или 'INSERT')
  # оператор делаем нежадным, т.к. нам нужно только первое слово
  ( $cmd_type ) = $$ref_cmds[ 0 ] =~ /^(.+?)\s+/;

  foreach $command ( @{ $ref_cmds } )
  {
    # DEBUG 
    print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

    # выполним команду вставки (схема public) или обновления (схема "импорт")
    $result = $conn->exec( $command );

    # если произошла ошибка при вставке/обновлении записей в БД
    if ( PGRES_COMMAND_OK ne $result->resultStatus )
    {
      # сообщение для пользователя
      print "<p class=\"error\">" .
            ( ( $cmd_type eq 'INSERT' ) ? $CFG{ ERROR_INSERT } :
                                          $CFG{ ERROR_UPDATE } ) . "</p>\n";
      print STDERR $command . "\n";             # выводим строку команды в лог
      die $conn->errorMessage;
    }
  }  
  # если не было сбоя, то все команды из массива считаем успешно выполненными
  return ( scalar @{ $ref_cmds } );
}

# -----------------------------------------------------------------------------
# экспортирование всех записей из всех таблиц схемы БД "экспорт/импорт" в файл
# -----------------------------------------------------------------------------
sub export_tables
{
  my $conn;                  # дескриптор соединения с БД
  my $result;                # результат выполнения SQL-команды
  my $table;                 # таблица из общего списка таблиц БД
  my $exported_count = 0;    # счетчик экспортированных записей
  my $i;

  # сначала удаляем файл экспорта, если он существует
  if ( scalar stat( "./exp_imp/exported_data" ) > 0 )
  {
    if ( ! unlink( "./exp_imp/exported_data" ) )
    {
      my $msg = "Не удалось удалить файл ./exp_imp/exported_data. Ошибка: $!";
      # сообщение для пользователя
      print "<p class=\"error\">$msg</p>\n";
      print STDERR $msg . "\n";     # выводим строку команды в лог
      die $msg;
    }
  }

  $conn = exec_connect_db();           # подключаемся к БД
  exec_transaction( $conn, "BEGIN" );  # выполним операции в рамках транзакции

  # обрабатываем все таблицы в том порядке, в котором они определены 
  # в SQL-файле создания БД (e_arch.sql), это позволяет соблюсти требования
  # внешних ключей
  for ( $i = 0; $i < scalar @{ $EXP_CFG{ TABLES_ORDER } }; $i++ )
  {
    $table = $EXP_CFG{ TABLES_ORDER }->[ $i ]; # у нас же ссылка на массив

    # будем подсчитывать число записей, экспортированных из экспортной схемы БД
    $exported_count += exec_export( $conn, $table );
  }

  exec_transaction( $conn, "END" );    # конец транзакции

  # если экспортирована хотя бы одна запись из экспортной схемы БД
  if ( $exported_count > 0 )
  {
    print "<p class=\"info\">";
    print( "Экспортированные записи сохранены в файле exported_data: " .
           "$exported_count шт.</p>\n" );
  }
  else
  {
    print "<p class=\"info\">";
    print( "Не было выбрано ни одной записи для экспорта</p>\n" );
  }

  return ( 0 );
}

# -----------------------------------------------------------------------------
# заполнение таблиц схемы БД "экспорт/импорт" записями, используя
# файл SQL-команд
# -----------------------------------------------------------------------------
sub fill_tables
{
  # параметр
  my $conn = shift;          # дескриптор соединения с БД

  my $result;                # результат выборки из БД
  my $inserted_count = 0;    # счетчик введенных записей
  my $command;               # строка SQL-команды

  # проверим наличие файла импорта
  if ( scalar stat( "./exp_imp/exported_data" ) == 0 )
  {
    # сообщение для пользователя
    print "<p class=\"error\">Нет файла с импортируемыми данными: " .
          "/exp_imp/exported_data</p>\n";
    return ( 0 );
  }

  # открываем файл, содержащий команды для ввода данных
  if ( ! open( IMPORT_FILE, "< ./exp_imp/exported_data" ) )
  {
    my $msg = "Не удалось открыть файл exported_data. Ошибка: $!";
    # сообщение для пользователя
    print "<p class=\"error\">$msg</p>\n";
    print STDERR $msg . "\n";     # выводим строку команды в лог
    die $msg;
  }

  # ... и читаем его построчно
  while ( <IMPORT_FILE> )
  {
    chomp;               # уберем символ конца строки
    next if /^$/;        # пустые строки отбрасываем
    next if /^\s*--/;    # строки-комментарии пропускаем

    s/^\s*//;            # уберем все пробелы в начале строки
    s/\s*$//;            # также удаляем все пробелы в конце строки
   
    # если встретилась команда INSERT или UPDATE
    if ( /^INSERT\s+INTO/ || /^UPDATE\s+/ )
    {
      $command = $_;   # начинаем формирование строки SQL-команды
    }
    # команда занимает более одной строки (это возможно, т. к. в некоторых
    # полях записей вполне могут оказаться символы новой строки, например,
    # в поле "Комментарий")
    # ПРИМЕЧАНИЕ. Крайне маловероятно, что в поле, например, "Комментарий",
    #             будет введено слово "INSERT", которому предшествует
    #             символ новой строки. В таком случае получилась бы имитация
    #             новой команды INSERT в рамках не до конца сформированной
    #             "исходной" команды INSERT.
    else
    {
      # формируем строку SQL-команды
      # добавим один пробел, чтобы между фрагментами команды гарантированно
      # был пробел
      $command .= ' ' . $_;
    }

    # если в текущей строке файла есть символ ";", которыми завершается
    # команда
    if ( /;$/ )
    {
      $command =~ s/;$//;    # отбросим символ ";" из строки SQL-команды

      # DEBUG 
      print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

      # выполним команду вставки/обновления записи в БД
      $result = $conn->exec( $command );

      # если произошла ошибка при вставке/обновлении записи в БД
      if ( PGRES_COMMAND_OK ne $result->resultStatus )
      {
        # сообщение для пользователя
        print "<p class=\"error\">";
        print /^INSERT\s+INTO/ ? $CFG{ ERROR_INSERT } : $CFG{ ERROR_UPDATE };
        print "</p>\n";
        print STDERR $command . "\n";   # выводим строку команды в лог
        die $conn->errorMessage;
      }

      # будем подсчитывать только число записей, введенных в схему
      # "экспорт/импорт"
      $inserted_count++ if ( $command =~ /^INSERT\s+INTO/ );
    }
  }

  close( IMPORT_FILE );

  # если не было введено ни одной записи в экспортную схему БД
  if ( $inserted_count == 0 )
  {
    print "<p class=\"error\">";
    print( "В файле экспорта/импорта не было ни одной записи</p>\n" );
    return ( 0 );
  }
  
  # если введена хотя бы одна запись в экспортную схему БД
  print "<p class=\"info\">";
  print( "Записи введены в базу данных экспорта/импорта: " .
         "$inserted_count шт.</p>\n" );
  
  return ( 1 );  # 1 -- успешное завершение процедуры
}

# -----------------------------------------------------------------------------
# импортирование всех записей из всех таблиц схемы БД "импорт"
# в схему public (основную схему)
# -----------------------------------------------------------------------------
sub import_tables
{
  my $conn;                  # дескриптор соединения с БД
  my $table;                 # таблица из общего списка таблиц БД
  my $imported_count = 0;    # счетчик импортированных записей
  my $i;

  $conn = exec_connect_db();           # подключаемся к БД
  exec_transaction( $conn, "BEGIN" );  # выполним операции в рамках транзакции

  # обрабатываем все таблицы в том порядке, в котором они определены 
  # в SQL-файле создания БД (e_arch.sql), это позволяет соблюсти требования
  # внешних ключей
  for ( $i = 0; $i < scalar @{ $EXP_CFG{ TABLES_ORDER } }; $i++ )
  {
    $table = $EXP_CFG{ TABLES_ORDER }->[ $i ]; # у нас же ссылка на массив

    # будем подсчитывать число записей, импортированных из "импортной" схемы БД
    # в схему public (основную схему)
    $imported_count += exec_import( $conn, $table );
  }

  exec_transaction( $conn, "END" );    # конец транзакции

  # если импортирована хотя бы одна запись из "импортной" схемы БД
  if ( $imported_count > 0 )
  {
    print "<p class=\"info\">";
    print( "Записи импортированы из базы данных импорта " .
           "в основную базу данных: $imported_count шт.</p>\n" );
  }
  else
  {
    print "<p class=\"info\">";
    print( "Записей в базе данных импорта не было</p>\n" );
  }

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование команды INSERT при экспортировании одной записи из таблицы
# схемы БД "экспорт/импорт" в файл
# -----------------------------------------------------------------------------
sub make_insert_cmd_for_export
{
  # параметры
  my $ref_insert_commands = shift;  # ссылка на массив SQL-команд INSERT
  my $table = shift;                # имя таблицы БД
  my $ref_rec = shift;              # ссылка на хеш-массив, содержащий поля
                                    # одной записи из таблицы

  my $fname;                        # имя поля
  my $value;                        # значение поля
  my $fields;                       # список имен полей (в виде строки)
  my $values;                       # список значений полей (в виде строки)
  my $command;                      # строка SQL-команды

  foreach $fname ( keys %{ $ref_rec } )
  {
    # используем эту переменную для удобства
    $value = $$ref_rec{ $fname };

    # включаем имя текущего поля в список имен полей
    $fields .= $fname . ",";

    # TODO. Нужно сделать обработку "нехороших" символов более основательно.
    $value =~ s/\'/\'\'/g;

    # формируем строку значений полей (поэтому нужны одинарные кавычки!!!)
    # если значение параметра пустое, то записываем null, но в этом
    # случае одинарные кавычки НЕ НУЖНЫ
    # ПРИМЕЧАНИЕ. Хотя для числовых полей кавычки не требуются, 
    #             мы их ставим только из соображений единообразия 
    #             и простоты данной функции.
    $values .= $value ne '' ? "'$value'," : "null," ;
  }

  $fields =~ s/,$//;         # уберем последнюю запятую
  $values =~ s/,$//;         # уберем последнюю запятую

  # сформируем команду вставки записи в экспортную схему БД окончательно  
  $command = "INSERT INTO exp_imp.$table ( $fields ) VALUES ( $values )";

  push( @{ $ref_insert_commands }, $command );   # поместим ее в массив

  return( 0 );
}

# -----------------------------------------------------------------------------
# формирование команды INSERT при импортировании одной записи из таблицы
# схемы БД "импорт" в таблицу схемы public
# -----------------------------------------------------------------------------
sub make_insert_cmd_for_import
{
  # параметры
  my $ref_insert_commands = shift;  # ссылка на массив SQL-команд INSERT
  my $table = shift;                # имя таблицы БД
  my $ref_rec = shift;              # ссылка на хеш-массив, содержащий поля
                                    # одной записи из таблицы

  my $fname;                        # имя поля
  my $value;                        # значение поля
  my $fields;                       # список имен полей (в виде строки)
  my $values;                       # список значений полей (в виде строки)
  my $command;                      # строка SQL-команды
  my $f_key_name;                   # имя внешнего ключа

  # Мною принята стратегия: связи, имеющиеся между записями в одной и той же
  # таблице, возникающие вследствие наличия разночтений и переименований
  # (например, фамилия автора-иностранца), в целевую базу данных НЕ ПЕРЕНОСИМ.
  # Поэтому нужно значения полей, входящих в те внешние ключи, которые
  # ссылаются на эту же таблицу, "обнулить".
  # ПРИМЕЧАНИЕ. Эта процедура не нужна для таблиц, которые имеют режим импорта,
  #             отличный от 'check_adjust_import', потому что они не имеют
  #             внешних ключей, ссылающихся на эту же таблицу. Но все же
  #             не будем вводить проверку условия if ... 'check_adjust_import',
  #             чтобы в будущем, при изменении этой ситуации, случайно не
  #             забыть убрать проверку.
  # TODO. Можно этот цикл просмотра внешних ключей вынести на более высокий
  #       уровень иерархии процедур. А в эту процедуру можно передавать уже
  #       сформированный список имен полей, значения которых нужно "обнулить".

  # для всех внешних ключей таблицы $table
  foreach $f_key_name ( keys %{ $EXP_CFG{ TABLES }{ $table }
                                        { FOREIGN_KEYS } } )
  {
    # если внешний ключ ссылается на эту же таблицу
    if ( $table eq $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS }
                           { $f_key_name }{ REF_TABLE } )
    {
      # "обнулим" значение каждого поля, входящего во внешний ключ
      foreach $fname ( @{ $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS }
                                  { $f_key_name }{ F_KEY_FIELDS } } )
      {
        # не забываем про префикс 'imp_' (см. процедуру make_select_cmd())
        $$ref_rec{ 'imp_' . $fname } = undef;
      }
    }
  }

  # теперь приступаем к формированию команды INSERT
  foreach $fname ( keys %{ $ref_rec } )
  {
    next if ( $fname !~ /^imp_/ );   # не забываем про префикс 'imp_'

    # используем эту переменную для удобства
    $value = $$ref_rec{ $fname };

    $fname =~ s/^imp_//;
    
    # включаем имя текущего поля в список имен полей
    $fields .= $fname . ",";

    # TODO. Нужно сделать обработку "нехороших" символов более основательно.
    $value =~ s/\'/\'\'/g;

    # формируем строку значений полей (поэтому нужны одинарные кавычки!!!)
    # если значение параметра пустое, то записываем null, но в этом
    # случае одинарные кавычки НЕ НУЖНЫ
    # ПРИМЕЧАНИЕ. Хотя для числовых полей кавычки не требуются, 
    #             мы их ставим только из соображений единообразия 
    #             и простоты данной функции.
    $values .= $value ne '' ? "'$value'," : "null," ;
  }

  $fields =~ s/,$//;         # уберем последнюю запятую
  $values =~ s/,$//;         # уберем последнюю запятую

  # сформируем команду вставки записи в схему БД public окончательно  
  $command = "INSERT INTO $table ( $fields ) VALUES ( $values )";

  push( @{ $ref_insert_commands }, $command );   # поместим ее в массив

  return( 0 );
}

# -----------------------------------------------------------------------------
# формирование команды SELECT при импортировании записей из указанной таблицы
# схемы БД "импорт" в схему public (основную схему)
# -----------------------------------------------------------------------------
sub make_select_cmd
{
  # параметры процедуры
  my $table = shift;       # имя таблицы

  my $fname;               # имя поля
  my $value;               # значение поля
  my $command;             # строка SQL-команды

  # режим для всех справочников и, возможно, для нескольких других таблиц
  if ( $EXP_CFG{ TABLES }{ $table }{ IMPORT_MODE } =~ /^check/ )
  {
    # Сделаем выборку на основе левого внешнего соединения, в котором слева
    # будет стоять таблица $table из "импортной" схемы БД, а справа -- таблица
    # $table из схемы БД public. Соединение сделаем на основе полей
    # CHECK_FIELDS из конфиг. файла экспорта. Тем самым мы получим информацию
    # о том, каких сведений нет в основной схеме БД, либо они есть, но запись
    # имеет другое значение ключевого поля (полей).

    $command = "SELECT ";

    # в список имен полей из таблицы "импортной" схемы БД включаем ВСЕ поля,
    # кроме who_chg и when_chg (которых просто нет в списках полей для всех
    # таблиц в конфиг. файле экспорта/импорта)
    foreach $fname ( @{ $EXP_CFG{ TABLES }{ $table }{ FIELDS } } )
    {
      $command .= "imp.$fname AS imp_$fname, ";
    }

    # в список имен полей из таблицы схемы БД public включаем только КЛЮЧЕВЫЕ
    # поля
    foreach $fname ( @{ $EXP_CFG{ TABLES }{ $table }{ KEY_FIELDS } } ) 
    {
      $command .= "pub.$fname AS $fname, ";
    }

    $command =~ s/,\s*$//;       # уберем последнюю запятую (с пробелом)

    $command .= " FROM imp.$table AS imp LEFT JOIN $table AS pub ON ";

    # для соединения используем поля CHECK_FIELDS
    foreach $fname ( @{ $EXP_CFG{ TABLES }{ $table }{ CHECK_FIELDS } } )
    {
      $command .= "imp.$fname = pub.$fname AND ";
    }

    $command =~ s/\s*AND\s*$//;  # уберем последний оператор AND (с пробелами)
  }
  else 
  {
    # Сделаем обычную выборку из таблицы $table из "импортной" схемы БД
  
    $command = "SELECT ";

    foreach $fname ( @{ $EXP_CFG{ TABLES }{ $table }{ FIELDS } } )
    {
      # ПРИМЕЧАНИЕ. Здесь также нужен префикс imp_ -- для единообразия,
      #             соблюдение которого поможет в процедуре 
      #             make_insert_cmd_for_import().
      $command .= "imp.$fname AS imp_$fname, ";
    }
    $command =~ s/,\s*$//;       # уберем последнюю запятую (с пробелом)

    $command .= " FROM imp.$table AS imp";
  }

  return ( $command );
}

# -----------------------------------------------------------------------------
# формирование команды UPDATE при экспортировании одной записи из таблицы
# схемы БД "экспорт/импорт" в файл
# -----------------------------------------------------------------------------
sub make_update_cmd_for_export
{
  # параметры
  my $ref_update_commands = shift;  # ссылка на массив SQL-команд UPDATE
  my $table = shift;                # имя таблицы БД
  my $ref_rec = shift;              # ссылка на хеш-массив, содержащий поля
                                    # одной записи из таблицы
  my $f_key_name = shift;           # имя внешнего ключа

  my @f_key_fields;                 # массив ключевых полей внешнего ключа
  my @ref_fields;                   # массив ссылочных полей внешнего ключа
  my @key_fields;                   # массив ключевых полей таблицы
  my $command;                      # строка SQL-команды
  my $i;

  # возьмем параметры внешнего ключа из хеш-массива (в принципе, можно
  # работать и с хеш-массивом, но это приведет к слишком громоздким
  # конструкциям)
  @f_key_fields = @{ $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS }
                             { $f_key_name }{ F_KEY_FIELDS } };
  @ref_fields = @{ $EXP_CFG{ TABLES }{ $table }{ FOREIGN_KEYS }
                           { $f_key_name }{ REF_FIELDS } };

  # если значения полей внешнего ключа равны NULL (undef), то для этого
  # внешнего ключа не нужно формировать команду UPDATE
  # ПРИМЕЧАНИЕ. Будем считать, что если первое ключевое поле имеет
  #             значение NULL (undef), то и остальные поля будут иметь
  #             такое же значение. В системе "Электронный архив" 
  #             не используются внешние ключи, у которых лишь часть полей
  #             нулевые (NULL).
  return if ( $$ref_rec{ $f_key_fields[ 0 ] } eq undef );

  # формируем команду UPDATE

  # сформируем начало команды
  # ВАЖНО! Обновление производим в таблице схемы экспорта/импорта.
  $command = "UPDATE exp_imp.$table SET ";

  # обработаем все поля внешнего ключа
  for ( $i = 0; $i < scalar @f_key_fields; $i++ )
  {
    # формируем пару "имя_поля + значение" и добавляем ее к общей строке
    $command .= $f_key_fields[ $i ] . " = '" .
                $$ref_rec{ $f_key_fields[ $i ] } . "', ";

    # а теперь "обнулим" значение этого поля в хеш-массиве, сформированном
    # на основе записи из таблицы $table, чтобы при формировании строки-команды
    # INSERT значение этого поля в этой команде было пустым и гарантированно
    # не нарушило требование внешнего ключа;
    # это поле будет заполнено при выполнении операции UPDATE, которая
    # выполняется после операции INSERT, а при выполнении операции INSERT оно
    # будет пустым и гарантированно не нарушит требования внешнего ключа
    $$ref_rec{ $f_key_fields[ $i ] } = undef;
  }
        
  $command =~ s/,\s*$//;  # уберем последнюю запятую

  # завершаем формирование команды
  $command .= " WHERE ";

  @key_fields = @{ $EXP_CFG{ TABLES }{ $table }{ KEY_FIELDS } };

  # обработаем все поля первичного ключа таблицы
  for ( $i = 0; $i < scalar @key_fields; $i++ )
  {
    $command .= $key_fields[ $i ] . " = '" .
                $$ref_rec{ $key_fields[ $i ] } . "' AND ";
  }

  # сформируем команду вставки записи в экспортную схему БД окончательно  
  $command =~ s/\s*AND\s*$//;

  push( @{ $ref_update_commands }, $command );   # поместим ее в массив

  return( 0 );
}

# -----------------------------------------------------------------------------
# формирование команды UPDATE при импортировании одной записи из таблицы
# схемы БД "импорт" в таблицу схемы public
# -----------------------------------------------------------------------------
sub make_update_cmd_for_import
{
  # параметры
  my $ref_update_commands = shift;  # ссылка на массив SQL-команд UPDATE
  my $table = shift;                # имя таблицы БД
  my $ref_rec = shift;              # ссылка на хеш-массив, содержащий поля
                                    # одной записи из таблицы
  my $fname;                        # имя поля
  my $command;                      # строка SQL-команды
  my $i;

  # сформируем начало команды
  # ВАЖНО! Обновление производим в таблице схемы импорта.
  $command = "UPDATE imp.$table SET ";

  # берем значение ключевого поля 
  # ПРИМЕЧАНИЕ. Команда UPDATE формируется только при работе по схемам
  #             'check_adjust_import' и 'adjust_import'. А у таблиц,
  #             обрабатываемых по этим схемам, только одно ключевое поле.
  #             Поэтому обходимся без цикла (пока, по крайней мере).
  $fname = $EXP_CFG{ TABLES }{ $table }{ KEY_FIELDS }->[ 0 ];

  # формируем пару "имя_ключевого_поля + значение" и добавляем ее 
  # к общей строке
  $command .= $fname . " = '" . $$ref_rec{ $fname } . "' ";

  # завершаем формирование команды
  $command .= " WHERE $fname" . " = '" . $$ref_rec{ 'imp_' . $fname } . "'";

  push( @{ $ref_update_commands }, $command );   # поместим ее в массив

  return( 0 );
}

# -----------------------------------------------------------------------------
# обработка записей, выбранных из таблицы схемы imp базы данных, по методике
# 'adjust_import'
# -----------------------------------------------------------------------------
sub process_adjust_import
{
  # параметры
  my $table = shift;            # имя таблицы БД
  my $result = shift;           # результат выполнения SQL-команды
  my $ref_update_cmds = shift;  # ссылка на массив SQL-команд UPDATE
  my $ref_insert_cmds = shift;  # ссылка на массив SQL-команд INSERT

  my @row;                # для хранения одной записи из выборки
  my %rec;                # хеш-массив, содержащий значения полей одной записи
  my $fname;              # имя поля
  my $value;              # значение поля
  my $new_key_value;      # следующее значение ключевого поля
  my $i;
  
  # обрабатываем все записи из полученной выборки
  while ( @row = $result->fetchrow )
  {
    %rec = ();          # "обнулим" хеш-массив для новой записи

    # поместим в него значения всех полей текущей записи из выборки
    for ( $i = 0; $i < scalar @row; $i++ )
    {
      # используем эти переменные для удобства
      $fname = $result->fname( $i );  # по номеру поля получим его имя
                                      # (с префиксом 'imp_')
      $value = $row[ $i ];
      $rec{ $fname } = $value;        # запишем значение поля в хеш-массив
    }

    # возьмем имя ключевого поля для таблицы $table 
    # ПРИМЕЧАНИЕ. По методике 'adjust_import' обрабатывается таблица Bib_objs.
    $fname = $EXP_CFG{ TABLES }{ $table }{ KEY_FIELDS }->[ 0 ];

    # получим следующее значение ключевого поля из целевой таблицы
    # (оно будет равно максимальному значению + 1)
    # если мы уже обращались к таблице, то теперь нужно просто брать
    # значение, сохраненное в переменной, и увеличивать его на 1
    # ПРИМЕЧАНИЕ. Если обращаться каждый раз к таблице БД, то мы будем
    #             получать одно и то же значение, т. к. выполнение операций 
    #             INSERT производится не сразу же для каждой обработанной
    #             строки из выборки, а после обработки всей выборки.
    if ( $new_key_value )
    {
      $new_key_value++;
    }
    else
    {
      # TODO. Транзакция организована в процедуре import_tables(), однако
      #       в процедуре get_next_key_value() выполняется новое подключение
      #       к БД. Это нормально? Все работает, но нет ли подводных камней?
      $new_key_value = get_next_key_value( $table, $fname );
    }

    # сформируем команду UPDATE для обновления записи в "импортной" таблице
    # (а уже СУБД обновит значения всех внешних ключей в "импортных" таблицах)
    # ПРИМЕЧАНИЕ. Цель этой операции -- согласовать значения ключевых полей
    #             в "импортной" и целевой таблицах.
    # Предварительно добавим в хеш-массив поле $fname -- без префикса 'imp_'.
    # Это позволит нам использовать один и тот же алгоритм в процедуре
    # make_update_cmd_for_import() как для работы по схеме 'adjust_import',
    # так и по схеме 'check_adjust_import'.
    $rec{ $fname } = $new_key_value;
    make_update_cmd_for_import( $ref_update_cmds, $table, \%rec );

    # поля с префиксом 'imp_' -- из таблицы схемы imp
    # при формировании команды INSERT это ключевое поле должно иметь новое
    # значение
    $rec{ 'imp_' . $fname } = $new_key_value;
    make_insert_cmd_for_import( $ref_insert_cmds, $table, \%rec );
  }
  return ( 0 );
}

# -----------------------------------------------------------------------------
# обработка записей, выбранных из таблицы схемы imp базы данных, по методике
# 'check_adjust_import'
# -----------------------------------------------------------------------------
sub process_check_adjust_import
{
  # параметры
  my $table = shift;            # имя таблицы БД
  my $result = shift;           # результат выполнения SQL-команды
  my $ref_update_cmds = shift;  # ссылка на массив SQL-команд UPDATE
  my $ref_insert_cmds = shift;  # ссылка на массив SQL-команд INSERT

  my @row;                # для хранения одной записи из выборки
  my %rec;                # хеш-массив, содержащий значения полей одной записи
  my $fname;              # имя поля
  my $value;              # значение поля
  my $new_key_value;      # следующее значение ключевого поля
  my $diff;               # признак наличия различий значений ключевых полей
  my $i;
  
  # обрабатываем все записи из полученной выборки
  while ( @row = $result->fetchrow )
  {
    %rec = ();          # "обнулим" хеш-массив для новой записи

    # поместим в него значения всех полей текущей записи из выборки
    for ( $i = 0; $i < scalar @row; $i++ )
    {
      # используем эти переменные для удобства
      $fname = $result->fname( $i );  # по номеру поля получим его имя
      $value = $row[ $i ];

      $rec{ $fname } = $value;        # запишем значение поля в хеш-массив
    }

    # если импортируемая запись уже есть в целевой таблице
    # (т. е. первое же ключевое поле имеет непустое значение)
    # ПРИМЕЧАНИЕ. Почти у всех справочников ключевое поле одно. Исключение --
    #             таблица Orgs_act. Что с ней делать?
    $fname = $EXP_CFG{ TABLES }{ $table }{ KEY_FIELDS }->[ 0 ];

    # ВАЖНО. Здесь имя поля берется без префикса 'imp_' (см. процедуру
    #        make_select_cmd()).
    if ( $rec{ $fname } ne undef )
    {
      # такая запись уже есть в целевой таблице в схеме public

      $diff = 0;
      # если значение хотя бы одного из полей KEY_FIELDS в "импортной" таблице
      # отличается от значения в целевой таблице...
      # ПРИМЕЧАНИЕ. Все таблицы, обрабатываемые по схеме 'check_adjust_import',
      #             имеют только ОДНО ключевое поле. Поэтому цикл здесь,
      #             видимо, не нужен. Пока не нужен...
      foreach $fname ( @{ $EXP_CFG{ TABLES }{ $table }{ KEY_FIELDS } } )
      {
        if ( $rec{ 'imp_' . $fname } != $rec{ $fname } )
        { 
          $diff = 1;
          last;
        } 
      }

      if ( $diff )
      {
        # ...то сформируем команду UPDATE для обновления записи в "импортной"
        # таблице (а уже СУБД обновит значения всех внешних ключей в "импортных"
        # таблицах)
        # ПРИМЕЧАНИЕ. Цель этой операции -- согласовать значения ключевых полей
        #             в "импортной" и целевой таблицах.
        make_update_cmd_for_import( $ref_update_cmds, $table, \%rec );
      }
      else          # значения ключевых полей совпадают
      {
        ;           # ничего не делаем
      }
    }
    else            # такой записи в целевой таблице схемы public нет
    {
      # нужно добавить запись в целевую таблицу

      # получим следующее значение ключевого поля из целевой таблицы
      # (оно будет равно максимальному значению + 1)
      # если мы уже обращались к таблице, то теперь нужно просто брать
      # значение, сохраненное в переменной, и увеличивать его на 1
      # ПРИМЕЧАНИЕ. Если обращаться каждый раз к таблице БД, то мы будем
      #             получать одно и то же значение, т. к. выполнение операций 
      #             INSERT производится не сразу же для каждой обработанной
      #             строки из выборки, а после обработки всей выборки.
      if ( $new_key_value )
      {
        $new_key_value++;
      }
      else
      {
        $new_key_value = get_next_key_value( $table, $fname );
      }

      # сначала сформируем команду UPDATE для обновления записи в "импортной"
      # таблице (а уже СУБД обновит значения всех внешних ключей в "импортных"
      # таблицах)
      # ПРИМЕЧАНИЕ. Цель этой операции -- согласовать значения ключевых полей
      #             в "импортной" и целевой таблицах.
      # Предварительно заменим в хеш-массиве значение поля $fname -- без 
      # префикса 'imp_', поскольку его значение в данном случае равно undef.
      $rec{ $fname } = $new_key_value;
      make_update_cmd_for_import( $ref_update_cmds, $table, \%rec );

      # насчет префикса 'imp_' см. процедуру make_select_cmd()
      $rec{ 'imp_' . $fname } = $new_key_value;
      make_insert_cmd_for_import( $ref_insert_cmds, $table, \%rec );
    }
  }
  return ( 0 );
}

# -----------------------------------------------------------------------------
# обработка записей, выбранных из таблицы схемы imp базы данных, по методике
# 'check_import'
# -----------------------------------------------------------------------------
sub process_check_import
{
  # параметры
  my $table = shift;            # имя таблицы БД
  my $result = shift;           # результат выполнения SQL-команды
  my $ref_insert_cmds = shift;  # ссылка на массив SQL-команд INSERT

  my @row;                # для хранения одной записи из выборки
  my %rec;                # хеш-массив, содержащий значения полей одной записи
  my $fname;              # имя поля
  my $value;              # значение поля
  my $i;
  
  # обрабатываем все записи из полученной выборки
  while ( @row = $result->fetchrow )
  {
    %rec = ();          # "обнулим" хеш-массив для новой записи

    # поместим в него значения всех полей текущей записи из выборки
    for ( $i = 0; $i < scalar @row; $i++ )
    {
      # используем эти переменные для удобства
      $fname = $result->fname( $i );  # по номеру поля получим его имя
      $value = $row[ $i ];

      $rec{ $fname } = $value;        # запишем значение поля в хеш-массив
    }

    # если импортируемая запись уже есть в целевой таблице
    # (т. е. первое же ключевое поле имеет непустое значение)
    # ПРИМЕЧАНИЕ. По методике 'check_import' обрабатывается таблица Orgs_act.
    $fname = $EXP_CFG{ TABLES }{ $table }{ KEY_FIELDS }->[ 0 ];

    if ( $rec{ $fname } ne undef )
    {
      # такая запись уже есть в целевой таблице в схеме public
      ;           # ничего не делаем
    }
    else          # такой записи в целевой таблице схемы public нет
    {
      # нужно добавить запись в целевую таблицу
      make_insert_cmd_for_import( $ref_insert_cmds, $table, \%rec );
    }
  }
  return ( 0 );
}

# -----------------------------------------------------------------------------
# обработка записей, выбранных из таблицы схемы imp базы данных, по методике
# 'import'
# -----------------------------------------------------------------------------
sub process_import
{
  # параметры
  my $table = shift;            # имя таблицы БД
  my $result = shift;           # результат выполнения SQL-команды
  my $ref_insert_cmds = shift;  # ссылка на массив SQL-команд INSERT

  my @row;                # для хранения одной записи из выборки
  my %rec;                # хеш-массив, содержащий значения полей одной записи
  my $fname;              # имя поля
  my $value;              # значение поля
  my $i;
  
  # обрабатываем все записи из полученной выборки
  while ( @row = $result->fetchrow )
  {
    %rec = ();          # "обнулим" хеш-массив для новой записи

    # поместим в него значения всех полей текущей записи из выборки
    for ( $i = 0; $i < scalar @row; $i++ )
    {
      # используем эти переменные для удобства
      $fname = $result->fname( $i );  # по номеру поля получим его имя
      $value = $row[ $i ];

      $rec{ $fname } = $value;        # запишем значение поля в хеш-массив
    }

    # нужно добавить запись в целевую таблицу
    make_insert_cmd_for_import( $ref_insert_cmds, $table, \%rec );
  }
  return ( 0 );
}
