#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   справочник "Виды признаков объектов библиографического описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'FEATURE_TYPES';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Feature_types
my @f_names =
   ( 'feature_code',       # Код признака
     'feature_group_code', # Код группы признаков
     'feature_name',       # Описание признака
     'comment'             # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'feature_code' ) eq '' )
  {
    push( @error_messages, "Не указан код признака" );
  }

  if ( $query->param( 'feature_group_code' ) eq '' )
  {
    push( @error_messages, "Не указан код группы признаков" );
  }

  if ( $query->param( 'feature_name' ) eq '' )
  {
    push( @error_messages, "Не указано описание признака" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Виды признаков объектов ' .
                                    'библиографического описания',
                    user => $user
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $feature_code;          # код признака
  my $command;               # строка команды для работы с БД

  $feature_code = $query->param( 'feature_code' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}feature_types " .
             "WHERE feature_code = '$feature_code'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $feature_code;          # код признака
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $feature_code = $query->param( 'feature_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Feature_types WHERE feature_code = $feature_code";
  push( @table_cmd_pairs, [ 'Feature_types', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $feature_code;          # код признака
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $feature_code = $query->param( 'feature_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Feature_types " .
             "WHERE feature_code = $feature_code";
  push( @table_cmd_pairs, [ 'Feature_types', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех видов признаков
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выбираем все записи о видах признаков (а также наименование группы,
  # к которой относится признак)
  $command = "SELECT f.*, ff.feature_group_name " .
             "FROM ${db_schema}feature_types f, " .
                  "${db_schema}feature_groups ff " .
             "WHERE f.feature_group_code = ff.feature_group_code " .
             "ORDER BY f.feature_code";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код признака</th>\n";
    print "<th>Описание группы признаков</th>\n";
    print "<th>Описание признака</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретного вида признаков ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Feature_types полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'FEATURES' )
      {
        # ... полям feature_code, feature_name и feature_group_name
        # в модуле FEATURES
        create_assign_values_func( 'feature_code', 'feature_name',
                                   'feature_group_name' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код признака, описание группы, описание признака
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'feature_code' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'feature_group_name' ) ]</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретного вида признаков, то значение поля
      # feature_name выведем в виде ссылки

      # экранируем кавычки и другие "неудобные" символы
      # (см. комментарий в модуле obj_groups_descrs.pl)
      # выполним добавочную обработку значений переменных, т. к. того, что
      # делает функция escapeHTML(), недостаточно
      my $feature_name = $row[ $result->fnumber( 'feature_name' ) ];
      $feature_name =~ s/\'/\\\'/g;
      $feature_name = CGI::escapeHTML( $feature_name );

      my $feature_group_name = 
         $row[ $result->fnumber( 'feature_group_name' ) ];
      $feature_group_name =~ s/\'/\\\'/g;
      $feature_group_name = CGI::escapeHTML( $feature_group_name );

      if ( $query->param( 'enable_select' ) )
      {
        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'feature_code' ) ],
                                  '$feature_name',
                                  '$feature_group_name' )">
                   $row[ $result->fnumber( 'feature_name' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'feature_name' ) ]</td>\n";
      }

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "feature_code=" . $row[ $result->fnumber( 'feature_code' ) ] .
              $add_params;

      # просмотр описания признака в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n"  if $can_view;

      # удаление записи о признаке
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } . "\n"  if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $feature_code;               # код признака

  my $command;                    # строка команды для работы с БД

  $feature_code = $query->param( 'feature_code' );

  # выбираем одну запись о признаке
  $command = "SELECT * " .
             "FROM ${db_schema}feature_types " .
             "WHERE feature_code = '$feature_code'"; 

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "признака " . $feature_code,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "feature_types",
                       key => "feature_code",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # групп, к которым относятся признаки)
  my( $ref_values, $ref_labels ) = popup_feature_groups_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'feature_code' } = get_next_key_value( 'feature_types',
                                                    'feature_code' );
  }
  # -------------------------------------------------------------------

  # поскольку значение поля "Код признака" формируется автоматически
  # с использованием функции get_next_key_value(), не будем позволять его
  # редактировать, но совсем убрать его нельзя, т.к. оно используется
  # при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'feature_code',
                        -default => $fields{ 'feature_code' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код признака</th>\n";
  print "<td>", $fields{ 'feature_code' }, "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_feature_groups_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Описание группы признаков</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'feature_group_code',
                            -default => $fields{ 'feature_group_code' },
                            -values => $ref_values,
                            -labels => { @{ $ref_labels } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Описание признака</th>\n";
  print "<td>",
        $query->textfield( -name => 'feature_name',
                           -default => $fields{ 'feature_name' },
                           -size => $CFG{ FEATURE_TYPES_FEATURE_NAME_SIZE },
                           -maxlength =>
                             $CFG{ FEATURE_TYPES_FEATURE_NAME_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";
  
  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>",
        $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ FEATURE_TYPES_COMMENT_ROWS },
                          -columns => $CFG{ FEATURE_TYPES_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}
