#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Признаки объектов библиографического описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'FEATURES';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Features
my @f_names =
   ( 'obj_id',           # Уникальный идентификатор объекта
                         # библиографического описания
     'feature_code',     # Код признака
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

# добавление группы записей (организация формы для ввода данных)
if ( $query->path_info =~ /^\/add_group$/ &&
     have_access( 'module' => $module, 'procedure' => 'ADD', 'user' => $user )
   )
{
  do_add_group();
}

# непосредственный ввод группы записей (с проверкой правильности ввода)
if ( $query->param( 'submit_add_group' ) &&
     have_access( 'module' => $module, 'procedure' => 'ADD', 'user' => $user )
   )
{
  add_rec_group();
}

exit( 0 );


# -----------------------------------------------------------------------------
# непосредственное добавление группы записей в БД
# -----------------------------------------------------------------------------
sub add_rec_group
{
  my $conn;                 # дескриптор соединения с БД
  my $result;               # результат выполнения SQL-команды
  my $command;              # строка команды для добавления записи в таблицу
  my $num_recs = 0;         # счетчик добавленных записей
  my $feature_code;         # код признака
  
  # уникальный идентификатор объекта биб. описания
  my $obj_id            = $query->param( 'obj_id' );

  # строка, содержащая все введенные коды
  my $feature_codes_str = $query->param( 'feature_codes_str' );

  # комментарий (в этом случае он будет одинаковым для всех вводимых записей)
  my $comment           = $query->param( 'comment' );

  # имя окна браузера для вывода HTML-страницы
  my $target_win        = $query->param( 'target_win' );

  # формируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # если проверка введенных значений прошла успешно, выполним добавление
  if ( check_param_group() )
  {
    $conn = exec_connect_db();          # подключаемся к БД
    exec_transaction( $conn, "BEGIN" ); # выполним операции в рамках транзакции

    # заменим каждый символ-разделитель на один "пробел"
    # (в качестве разделителей могут служить любые нецифровые символы)
    $feature_codes_str =~ s/[^\d]/ /g;
    
    # разделим строку, содержащую все введенные коды признаков, на отдельные
    # числа
    foreach $feature_code ( split( /\s+/, $feature_codes_str ) )
    {
      # сначала сформируем строку-команду ...
      $command = "INSERT INTO Features ( obj_id, feature_code, comment, " .
                                        "who_chg ) " .
                 "VALUES ( $obj_id, $feature_code, '$comment', '$user' )";

      # DEBUG 
      print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

      # ... и вставим запись в таблицу Features
      $result = $conn->exec( $command );

      # если произошла ошибка при вставке записи в базу данных
      if ( PGRES_COMMAND_OK ne $result->resultStatus )
      {
        # сообщение для пользователя
        print "<p class=\"error\">$CFG{ ERROR_INSERT }</p>\n";
        print STDERR $command . "\n";           # выводим строку команды в лог
        die $conn->errorMessage;
      }
      else
      {
        $num_recs++;         # счетчик добавленных записей
      }
    }

    exec_transaction( $conn, "END" );    # конец транзакции
    
    print "<p class=\"info\">";
    print( "В базу данных добавлено записей: $num_recs</p>\n" );
  }

  close_bottom();      # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'feature_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код признака" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# проверка значений параметров формы при вводе группы записей
# -----------------------------------------------------------------------------
sub check_param_group
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # не введено ни одного числа или есть числа, содержащие 4 цифры и более
  if ( $query->param( 'feature_codes_str' ) !~ /\d+/ ||
       $query->param( 'feature_codes_str' ) =~ /\d{4,}/ )
  {
    push( @error_messages, "Не указан ни один код или указан неверный " .
          "код признака" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для ввода группы записей
# -----------------------------------------------------------------------------
sub do_add_group
{
  # имя окна браузера для вывода HTML-страницы
  my $target_win = $query->param( 'target_win' );

  # формируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # поскольку по умолчанию форма обрабатывается тем же URL-ом, которым она
  # была сформирована, то явно указываем URL скрипта для обработки данных,
  # собранных в полях формы, т.к. иначе останется PATH_INFO = 'add_group' от
  # предыдущего вызова скрипта, во время которого формируется форма, и этот
  # PATH_INFO будет обработан еще раз, чего уже не требуется
  print $query->start_form( -action => $script_name,
                            -target => $target_win
                          ), "\n";

  # создадим форму для ввода данных
  make_form_group();

  print $query->submit( -name => "submit_add_group",
                        -value => "Добавить группу записей",
                        -class => 'submit'
                      ), "\n";

  print $query->end_form();

  close_bottom();   # завершаем оформление нижней части окна

  print $query->end_html();

  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $feature_code;       # код признака
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id       = $query->param( 'obj_id' );
  $feature_code = $query->param( 'feature_code' );
  $target_win   = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}features " .
             "WHERE obj_id = '$obj_id' AND " .
                   "feature_code = '$feature_code'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех признаков для данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id     = $query->param( 'obj_id' );
  $bib_rec    = $query->param( 'bib_rec' );
  $target_win = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о признаках для данного объекта
  # (получим также наименования признака и группы признаков)
  # ПРИМЕЧАНИЕ. Сортировка - по коду признака. Посмотрим, удобно ли это?
  $command = "SELECT f.*, ft.feature_name, fg.feature_group_name " .
             "FROM ${db_schema}features f, ${db_schema}feature_groups fg, " .
                  "${db_schema}feature_types ft " .
             "WHERE f.obj_id = '$obj_id' AND " .
                   "f.feature_code = ft.feature_code AND " .
                   "ft.feature_group_code = fg.feature_group_code " .
             "ORDER BY f.feature_code";

  $result = exec_select( $command );   # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код признака</th>\n";
    print "<th>Наименование признака</th>\n";
    print "<th>Группа признаков</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код и наименование признака, наименование группы признаков
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'feature_code' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'feature_name' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'feature_group_name' ) ]</td>\n";

      # просмотр данной записи о признаке в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&feature_code=" . 
            $row[ $result->fnumber( 'feature_code' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о признаке
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&feature_code=" . 
            $row[ $result->fnumber( 'feature_code' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о признаке
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
    print "&nbsp;&nbsp;\n";
  
    # сформируем ссылку для добавления группы записей о признаках
    print qq{<a class="command_link" href="$script_name/add_group?$href"} .
          qq{ target="$target_win">Добавить группу записей</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $feature_code;       # код признака
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id       = $query->param( 'obj_id' );
  $feature_code = $query->param( 'feature_code' );
  $target_win   = $query->param( 'target_win' );

  # выбираем одну запись для признака
  $command = "SELECT f.*, ft.feature_name, fg.feature_group_name " .
             "FROM ${db_schema}features f, ${db_schema}feature_groups fg, " .
                  "${db_schema}feature_types ft " .
             "WHERE f.obj_id = '$obj_id' AND " .
                   "f.feature_code = '$feature_code' AND " .
                   "f.feature_code = ft.feature_code AND " .
                   "ft.feature_group_code = fg.feature_group_code " .
             "ORDER BY f.feature_code";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "код признака - $feature_code",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "features",
                       key => "obj_id, feature_code:feature_code_begin",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения ключевого поля из параметров скрипта
    $fields{ 'obj_id' } = $query->param( 'obj_id' );

    # значения по умолчанию

    # код признака
    $fields{ 'feature_code' } = $CFG{ FEATURES_FEATURE_CODE };
    
    # комментарий
    $fields{ 'comment' }      = $CFG{ FEATURES_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор объекта
  # библиографического описания" не подлежит редактированию, не будем позволять
  # его редактировать, но совсем убрать его нельзя, т.к. оно используются при
  # выполнении операции UPDATE в качестве ключа, а также при выполнении
  # операции INSERT (значение этого поля сохраняется таким же, каким оно было
  # при предыдущем запуске скрипта, поэтому можно не присваивать ему значения)
  print $query->hidden( -name => 'obj_id' ), "\n";

  # создадим скрытое поле для "Кода признака"
  # ПРИМЕЧАНИЕ. Идентификатор скрытого поля имеет постфикс 'begin' для того,
  #             чтобы можно было обновлять значение этого поля, поскольку оно
  #             является ключевым (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужно и исходное значение этого поля, и его новое
  #             значение (которое, возможно, было изменено пользователем).
  print $query->hidden( -name => 'feature_code_begin',
                        -default => $fields{ 'feature_code' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();           # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код признака</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник feature_types.pl
  create_selection_func( func_name => 'select_feature',
                         module_name => 'feature_types.pl',
                         caller_name => $module,
                         window_name => 'feature_types',
                         exp_imp => $query->param( 'exp_imp' ) );
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Наименование признака";
  # поле "Наименование группы признаков" просто обнуляем (наша функция 
  # create_hot_help() не позволяет предложить более интеллектуальную обработку
  # для этого поля)
  create_hot_help( "document.forms[ 0 ].feature_code",
                   "FEATURES_FEATURE_CODE", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].feature_name.value = str;
                   document.forms[ 0 ].feature_group_name.value = ''; " )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'feature_code',
                           -default => $fields{ 'feature_code' },
                           -size => $CFG{ FEATURES_FEATURE_CODE_SIZE },
                           -maxlength => 
                             $CFG{ FEATURES_FEATURE_CODE_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )"} .
        qq{ onclick="select_feature()">Список</a>},
        "</td>\n";
  print "</tr>\n";

  # выведем также наименования признака и группы признаков 
  # (без возможности редактирования)
  print "<tr>\n";
  print "<th>Наименование признака</th>\n";
  print "<td>",
        $query->textfield( -name => 'feature_name',
                           -default => $fields{ 'feature_name' },
                           -size => $CFG{ FEATURES_FEATURE_NAME_SIZE },
                           -maxlength => 
                             $CFG{ FEATURES_FEATURE_NAME_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Наименование группы признаков</th>\n";
  print "<td>",
        $query->textfield( -name => 'feature_group_name',
                           -default => $fields{ 'feature_group_name' },
                           -size => $CFG{ FEATURES_FEATURE_GROUP_NAME_SIZE },
                           -maxlength => 
                             $CFG{ FEATURES_FEATURE_GROUP_NAME_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "FEATURES_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ FEATURES_COMMENT_ROWS },
                          -columns => $CFG{ FEATURES_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# создание формы для добавления группы записей
# -----------------------------------------------------------------------------
sub make_form_group
{
  # возьмем значения ключевого поля из параметров скрипта
  my $obj_id = $query->param( 'obj_id' );

  # поскольку значение поля "Уникальный идентификатор объекта
  # библиографического описания" не подлежит редактированию, не будем позволять
  # его редактировать, но совсем убрать его нельзя, т.к. оно используются при
  # выполнении операции INSERT (значение этого поля сохраняется таким же, каким оно было
  # при предыдущем запуске скрипта, поэтому можно не присваивать ему значения)
  print $query->hidden( -name => 'obj_id' ), "\n";

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  print_add_info();             # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Коды признаков</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].feature_codes_str",
                   "FEATURES_FEATURE_CODES_STR" )
    if $CFG{ HOT_HELP_IS_ON };
  
  print $query->textfield( -name => 'feature_codes_str',
                           -default => $CFG{ FEATURES_FEATURE_CODES_STR },
                           -size => $CFG{ FEATURES_FEATURE_CODES_STR_SIZE },
                           -maxlength => 
                             $CFG{ FEATURES_FEATURE_CODES_STR_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарии</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "FEATURES_COMMENT_STR" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $CFG{ FEATURES_COMMENT_STR },
                          -rows => $CFG{ FEATURES_COMMENT_STR_ROWS },
                          -columns => $CFG{ FEATURES_COMMENT_STR_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Признаки объекта библиографического описания" );
  
  return ( 0 );
}
