#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Общие характеристики объектов библиографического описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'GEN_CHARS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Gen_chars
my @f_names =
   ( 'obj_id',           # Уникальный идентификатор объекта
                         # библиографического описания
     'gen_char',         # Общая характеристика источника 
     'useful_fragments', # Информация о фрагментах текста, заинтересовавших
                         # меня (номера страниц, абзацев, строк...)
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'gen_char' ) eq '' )
  {
    push( @error_messages, "Не заполнено поле \"Общая характеристика " .
          "источника\"" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $target_win;         # имя окна браузера для вывода HTML-страницы

  my $command;            # строка команды для работы с БД

  $obj_id     = $query->param( 'obj_id' );
  $target_win = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}gen_chars " .
             "WHERE obj_id = '$obj_id'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех общих характеристик для данного издания
# ПРИМЕЧАНИЕ. В действительности выводится только одна запись (см. первичный
#             ключ таблицы Gen_chars), но для упрощения кодирования
#             будем использовать типовую схему.
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  $obj_id     = $query->param( 'obj_id' );
  $bib_rec    = $query->param( 'bib_rec' );
  $target_win = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи об общих характеристиках для данного объекта
  # (фактически это будет не больше одной записи - см. табл. Gen_chars)
  $command = "SELECT * " .
             "FROM ${db_schema}gen_chars " .
             "WHERE obj_id = '$obj_id'";

  # сделаем выборку из БД
  $result = exec_select( $command );
  
  if ( $result )
  {
    # выберем запись из полученной выборки
    @row = $result->fetchrow;

    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    # расположим поля в столбик во всю ширину таблицы, т.к. возможен
    # большой объем текста
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Общая характеристика источника</th>\n";
    print "</tr>\n";

    print "<tr>\n";
    print "<td>$row[ $result->fnumber( 'gen_char' ) ]</td>\n";
    print "</tr>\n";

    print "<tr>\n";
    print "<th>Информация о полезных фрагментах текста</th>\n";
    print "</tr>\n";

    print "<tr>\n";
    # добавим "&nbsp;", т.к. это поле может быть пустым, и разделительные
    # линейки в таблице в таком случае не будут выводиться
    print "<td>$row[ $result->fnumber( 'useful_fragments' ) ]&nbsp;</td>\n";
    print "</tr>\n";

    # просмотр данной записи об общих характеристиках в виде формы
    print "<tr>\n<td>" if ( $can_view || $can_delete );
    print qq{<a class="command_link" href="$script_name/view?$href"} .
          qq{ target="$target_win">Просмотр</a>} if $can_view;
    print "\n" if $can_view;

    # удаление данной записи об общих характеристиках
    print qq{<a class="command_link" href="$script_name/delete?$href"} .
          qq{ onclick="$confirm_delete"} .
          qq{ target="$target_win">Удаление</a>} if $can_delete;
    print "</td>\n</tr>" if ( $can_view || $can_delete );

    print "</table>\n";
  }
  else     # нет ни одной записи, а допускается всего ОДНА (см. PRIMARY KEY)
  {
    # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
    if ( ! $query->param( 'exp_imp' ) &&
         have_access( 'module' => $module, 'procedure' => 'ADD', 
                      'user' => $user ) )
    {
      # сформируем ссылку для добавления записи об общей характеристике
      # объекта библиографического описания
      print "<br />\n";
      print qq{<a class="command_link" href="$script_name/add?$href"} .
            qq{ target="$target_win">Добавить запись</a>};
    }
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id     = $query->param( 'obj_id' );
  $target_win = $query->param( 'target_win' );

  # выбираем одну запись для общих характеристик
  $command = "SELECT * " .
             "FROM ${db_schema}gen_chars " .
             "WHERE obj_id = $obj_id";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "gen_chars",
                       key => "obj_id",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения ключевого поля из параметров скрипта
    $fields{ 'obj_id' } = $query->param( 'obj_id' );

    # значения по умолчанию

    # общая характеристика источника
    $fields{ 'gen_char' }         = $CFG{ GEN_CHARS_GEN_CHAR };

    # информация о фрагментах текста, заинтересовавших меня
    $fields{ 'useful_fragments' } = $CFG{ GEN_CHARS_USEFUL_FRAGMENTS };

    # комментарий
    $fields{ 'comment' }          = $CFG{ GEN_CHARS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор объекта
  # библиографического описания" не подлежит редактированию, не будем позволять
  # его редактировать, но совсем убрать его нельзя, т.к. оно используются при
  # выполнении операции UPDATE в качестве ключа, а также при выполнении
  # операции INSERT (значение этого поля сохраняется таким же, каким оно было
  # при предыдущем запуске скрипта, поэтому можно не присваивать ему значения)
  print $query->hidden( -name => 'obj_id' ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();           # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Общая характеристика источника</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].gen_char", "GEN_CHARS_GEN_CHAR" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'gen_char',
                          -default => $fields{ 'gen_char' },
                          -rows => $CFG{ GEN_CHARS_GEN_CHAR_ROWS },
                          -columns => $CFG{ GEN_CHARS_GEN_CHAR_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Информация об интересных или полезных фрагментах текста " .
            "(номера страниц, абзацев, строк...)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].useful_fragments",
                   "GEN_CHARS_USEFUL_FRAGMENTS" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'useful_fragments',
                          -default => $fields{ 'useful_fragments' },
                          -rows => $CFG{ GEN_CHARS_USEFUL_FRAGMENTS_ROWS },
                          -columns => $CFG{ GEN_CHARS_USEFUL_FRAGMENTS_COLS } 
                        ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "GEN_CHARS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ GEN_CHARS_COMMENT_ROWS },
                          -columns => $CFG{ GEN_CHARS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Общая характеристика объекта " .
                    "библиографического описания" );
  return ( 0 );
}
