#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   справочник "Ключевые слова"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'KEYWORDS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Keywords
my @f_names =
   ( 'keyword_code',      # Код ключевого слова
     'keyword_name',      # Ключевое слово (выражение)
     'keyword_code_orig', # Код ключевого слова, являющегося началом цепочки
                          # (для учета переименований, разночтений, сокращений,
                          # написаний на различных языках)
     'lang_code',         # Код языка (1 - русский язык)
     'comment'            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

# выбираем описания всех объектов, которым назначено выбранное ключевое слово
if ( $query->path_info =~ /^\/list_bib_recs$/ )
{
  do_list_bib_recs();
}


exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'keyword_code' ) eq '' )
  {
    push( @error_messages, "Не указан код ключевого слова" );
  }

  if ( $query->param( 'keyword_name' ) eq '' )
  {
    push( @error_messages, "Не указано ключевое слово (выражение)" );
  }

  # допускается пустое поле или цифры
  if ( $query->param( 'keyword_code_orig' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный код ключевого слова -- начала цепочки" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Ключевые слова',
                    user => $user,
                    make_letters => 'yes',
                    num_last_recs => $CFG{ KEYWORDS_NUM_LAST_RECS },
                    full_top => 'yes'
                  );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $keyword_code;          # код ключевого слова
  my $command;               # строка команды для работы с БД

  $keyword_code = $query->param( 'keyword_code' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}keywords " .
             "WHERE keyword_code = '$keyword_code'";

  # используем типовую процедуру
  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $keyword_code;          # код ключевого слова
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $keyword_code = $query->param( 'keyword_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Keywords WHERE keyword_code = $keyword_code";
  push( @table_cmd_pairs, [ 'Keywords', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $keyword_code;          # код ключевого слова
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $keyword_code = $query->param( 'keyword_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Keywords " .
             "WHERE keyword_code = $keyword_code";
  push( @table_cmd_pairs, [ 'Keywords', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# выборка и вывод биб. описаний всех объектов, которым назначено выбранное
# ключевое слово
# -----------------------------------------------------------------------------
sub do_list_bib_recs
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $keyword_code;               # код ключевого слова, выбранного 
                                  # пользователем
  my @keywords = ();              # массив кодов ключевых слов,
                                  # которые по "цепочке" связаны со словом,
                                  # выбранным пользователем
  my $headline;                   # заголовок списка биб. описаний
  my $keywords_info;              # список ключевых слов, выводимых на экран

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  $keyword_code = $query->param( 'keyword_code' );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выбираем записи из таблицы Keywords для четырех типов ключевых слов:
  # -- то слово, которое "кликнул" пользователь;
  # -- то слово, которое является для выбранного пользователем
  # слова началом цепочки (на основе поля keyword_code_orig);
  # -- слова, у которых выбранное пользователем слово является
  # началом цепочки (на основе поля keyword_code_orig);
  # -- слова, у которых началом цепочки является то же слово,
  # что и у того слова, которое выбрал пользователь
  # ПРИМЕЧАНИЕ. См. большой комментарий в такой же процедуре в модуле
  #             Persons.pl.
  $command = "SELECT k1.keyword_code, k1.keyword_name " .
             "FROM ${db_schema}keywords k1 " .
             "WHERE k1.keyword_code = $keyword_code OR " .
                   "k1.keyword_code = ( SELECT k2.keyword_code_orig " .
                                       "FROM ${db_schema}keywords k2 " .
                                       "WHERE k2.keyword_code = " .
                                             "$keyword_code ) OR " .
                   "k1.keyword_code_orig = $keyword_code OR " .
                   "k1.keyword_code_orig = ( SELECT k3.keyword_code_orig " .
                                            "FROM ${db_schema}keywords k3 " .
                                            "WHERE k3.keyword_code = " .
                                                  "$keyword_code ) " .
             "ORDER BY k1.keyword_name";

  $result = exec_select( $command );   # сделаем выборку из БД
  
  if ( $result )
  {
    # формируем заголовок списка биб. описаний и массив кодов ключевых слов
    while ( @row = $result->fetchrow )
    {
      $keywords_info .= $row[ $result->fnumber( 'keyword_name' ) ] . "; ";
      push( @keywords, $row[ $result->fnumber( 'keyword_code' ) ] );
    }

    $keywords_info =~ s/; $//;   # уберем последнюю точку с запятой: она лишняя
  }
  else
  {
    ;
    # TODO. Надо что-то делать или нет?
  }
  # ------------------------------------------------------------------------

  # выведем список ключевых слов в качестве заголовка
  print "<p class=\"bib_rec\">$keywords_info</p>\n";

  # сформируем заголовок модуля
  $headline = "<h2 class=\"module_title\">" .
              "Библиографические описания, связанные с данным " .
              "ключевым словом" .
              "</h2>\n";

  # выбираем теперь записи с биб. описаниями объектов, которым назначено
  # одно из ключевых слов, являющихся разночтениями, сокращениями, написаниями
  # на других языках и т. д. для ключевого слова, выбранного пользователем;
  # используем DISTINCT, т. к. одному объекту может быть назначено два и более
  # вариантов одного ключевого слова (например, полное написание и его
  # сокращенный вариант -- Российская Федерация и РФ)
  $command = "SELECT DISTINCT b.obj_id, b.bib_rec_num, " .
                    "CASE WHEN b.bib_rec_short IS NOT NULL " .
                         "THEN b.bib_rec_short " .
                         "ELSE b.bib_rec_full " .
                    "END AS bib_rec " .
             "FROM ${db_schema}obj_keywords ok, ${db_schema}bib_recs b " .
             "WHERE ok.obj_id = b.obj_id AND " .
                   "ok.keyword_code IN (" . join( ",", @keywords ) . ") " .
             "ORDER BY bib_rec";

  # вывод биб. описаний всех объектов, выбранных по запросу
  list_bib_recs_by_query( $query, $command, $headline );

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка тех ключевых слов (выражений), которые 
# начинаются с указанной буквы
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $letter;                     # первая буква ключевого слова
  my $letter_lower;               # первая буква -- в нижнем регистре

  # число последних записей, введенных в таблицу, которые нужно выводить
  # при обращении по специальной ссылке
  my $num_last_recs;

  # условие WHERE для команды выборки
  my $where;

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  $letter        = $query->param( 'letter' );
  $num_last_recs = $query->param( 'num_last_recs' );

  # если параметры не переданы, то просто завершаем работу досрочно - 
  # это происходит при первоначальном запуске скрипта
  # ПРИМЕЧАНИЕ. Проверка как на undef, так и на пустое значение '' выполнена
  #             на случай каких-либо проблем в CGI.pm.
  if ( ( $letter eq undef || $letter eq '' ) &&
       ( $num_last_recs eq undef || $num_last_recs eq '' )
     )
  {
    close_bottom();   # завершаем оформление нижней части окна
    print $query->end_html();
    return ( 0 );
  }

  # если пользователь выбрал ссылку для вывода только последних записей,
  # введенных в таблицу
  if ( $num_last_recs )
  {
    # ПРИМЕЧАНИЕ. Если даже требуемое число последних введенных записей будет
    #             больше числа всех записей, имеющихся в таблице, этот 
    #             оператор сработает. Если какие-то записи были удалены,
    #             вследствие чего образовались "дырки" в нумерации 
    #             keyword_code, то просто будет выведено меньшее число записей.
    $where = "WHERE k1.keyword_code >= ( SELECT max( keyword_code ) " .
                                        "FROM ${db_schema}keywords " .
                                      ") - $num_last_recs + 1 ";
  }
  # пользователь выбрал конкретную букву или другой символ
  elsif ( $letter )
  {
    # для русских букв может потребоваться особая предварительная обработка
    $letter = reveal_rus_letter( $letter );

    # воспользуемся возможностями локализации для перевода буквы, выбранной
    # пользователем, в нижний регистр
    $letter_lower = change_case( $letter, 'lower' );

    # если первый символ -- двойные кавычки, то не учитываем его
    $where = "WHERE k1.keyword_name ~ '^($letter|\"$letter|" .
                                        "$letter_lower|\"$letter_lower)' ";
  }

  # выбираем все записи о ключевых словах (а также краткое наименование языка,
  # на котором записано ключевые слово), которые удовлетворяют условию $where
  # (т. е. начинаются с указанной буквы или были введены в таблицу последними)
  # выберем также ключевое слово - начало цепочки
  $command = "SELECT k1.*, k2.keyword_name AS keyword_name_orig, " .
                    "l.lang_sname " .
             "FROM ( ${db_schema}keywords k1 " .
                  "LEFT JOIN ${db_schema}keywords k2 " .
                  "ON k1.keyword_code_orig = k2.keyword_code ) " .
                  "LEFT JOIN ${db_schema}langs l " .
                  "ON k1.lang_code = l.lang_code " .
             $where .
  # переведем значение поля в нижний регистр для того, чтобы регистр символов
  # не влиял на порядок сортировки (наверное, можно было переводить и 
  # в верхний регистр -- все равно)
  # если первый символ в поле -- кавычки, то не учитываем его, а берем
  # значение поля, начиная со второго символа;
             "ORDER BY CASE WHEN k1.keyword_name ~ '^\"' " .
                           "THEN lower( substr( k1.keyword_name, 2 ) ) " .
                           "ELSE lower( k1.keyword_name ) " .
                      "END";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если это список записей, введенных в таблицу последними, то укажем это
    if ( $num_last_recs )
    {
      print "<h3>Записи, введенные в базу данных последними (" . 
            $num_last_recs . " шт.)</h3>\n";
    }

    # если этот модуль вызван в режиме справочника, и при этом он вызван
    # "сам из себя", то запретим операции просмотра и удаления записей
    # (поскольку режим корректировки выполняется из режима просмотра,
    # то и он будет недоступен)
    if ( $query->param( 'enable_select' ) &&
         $query->param( 'caller_name') eq $module )
    {
      $can_view = 0;
      $can_delete = 0;
    }
    else
    {
      $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                               'user' => $user );
      $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                                 'user' => $user );
    }

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код ключевого слова</th>\n";
    print "<th>Ключевое слово (выражение)</th>\n";
    print "<th>Ключевое слово &mdash; начало цепочки</th>\n";
    print "<th>Язык</th>\n";
    print "<th>&nbsp;</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретного ключевого слова ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Keywords полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'OBJ_KEYWORDS' )
      {
        # ... полям keyword_code и keyword_name в модуле OBJ_KEYWORDS
        create_assign_values_func( 'keyword_code', 'keyword_name' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'KEYWORDS' )
      {
        # ... полям keyword_code_orig и keyword_name_orig в модуле KEYWORDS
        create_assign_values_func( 'keyword_code_orig', 'keyword_name_orig' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код ключевого слова, ключевое слово (выражение), 
      # код и само ключевое слова -- начало цепочки, язык, на котором написано
      # ключевое слово
      # (добавим символ &nbsp; к коду ключевого слова, являющемуся началом
      # цепочки, из соображений эстетики: если значение поля
      # 'keyword_code_orig' равно null, то не отображаются горизонтальные
      # строки-разделители)
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'keyword_code' ) ]</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретного ключевого слова, то значение поля
      # keyword_name выведем в виде ссылки

      # экранируем кавычки и другие "неудобные" символы
      # (см. комментарий в модуле obj_groups_descrs.pl)
      # выполним добавочную обработку значения переменной, т. к. того, что
      # делает функция escapeHTML(), недостаточно
      my $keyword_name = $row[ $result->fnumber( 'keyword_name' ) ];
      $keyword_name =~ s/\'/\\\'/g;
      $keyword_name = CGI::escapeHTML( $keyword_name );

      if ( $query->param( 'enable_select' ) )
      {
        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'keyword_code' ) ],
                                  '$keyword_name' )">
                   $row[ $result->fnumber( 'keyword_name' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'keyword_name' ) ]</td>\n";
      }

      # если ключевое слово, являющееся началом цепочки, отсутствует,
      # то выведем лишь символ пробела "&nbsp;"
      print "<td>",
            ( $row[ $result->fnumber( 'keyword_code_orig' ) ] ne undef ) ?
              "(" . $row[ $result->fnumber( 'keyword_code_orig' ) ] . ") " .
              $row[ $result->fnumber( 'keyword_name_orig' ) ] :
            "&nbsp;",
            "</td>\n";

      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "keyword_code=" . $row[ $result->fnumber( 'keyword_code' ) ] .
              $add_params;

      # просмотр биб. описаний тех объектов, которым назначено выбранное
      # ключевое слово
      print "<td>",
            qq{ <a class="command_link"} .
            qq{ href="$script_name/list_bib_recs?$href"} .
            qq{ target="_blank">Биб. описания</a> },
            "</td>\n";

      # просмотр ключевого слова (выражения) в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи о ключевом слове
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $keyword_code;               # код ключевого слова
  my $command;                    # строка команды для работы с БД

  $keyword_code = $query->param( 'keyword_code' );

  # выбираем одну запись о ключевом слове
  # выберем также ключевое слово - начало цепочки
  $command = "SELECT k1.*, k2.keyword_name AS keyword_name_orig " .
             "FROM ${db_schema}keywords k1 " .
                  "LEFT JOIN ${db_schema}keywords k2 " .
                  "ON k1.keyword_code_orig = k2.keyword_code " .
             "WHERE k1.keyword_code = '$keyword_code'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "ключевого слова " . $keyword_code,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
          );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "keywords",
                       key => "keyword_code",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны ключевые слова)
  my( $ref_values, $ref_labels ) = popup_langs_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'keyword_code' } = get_next_key_value( 'keywords',
                                                    'keyword_code' );
    # значения по умолчанию

    # ключевое слово (выражение)
    $fields{ 'keyword_name' }      = $CFG{ KEYWORDS_KEYWORD_NAME };

    # код ключевого слова, являющегося началом цепочки
    # (для учета переименований, разночтений, сокращений,
    # написаний на различных языках)
    $fields{ 'keyword_code_orig' } = $CFG{ KEYWORDS_KEYWORD_CODE_ORIG };

    # код языка (1 - русский язык)
    $fields{ 'lang_code' }         = $CFG{ KEYWORDS_LANG_CODE };

    # комментарий
    $fields{ 'comment' }           = $CFG{ KEYWORDS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Код ключевого слова" формируется автоматически
  # с использованием функции get_next_key_value(), не будем позволять его
  # редактировать, но совсем убрать его нельзя, т.к. оно используется
  # при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'keyword_code',
                        -default => $fields{ 'keyword_code' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код ключевого слова</th>\n";
  print "<td>", $fields{ 'keyword_code' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Ключевое слово (выражение)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].keyword_name",
                   "KEYWORDS_KEYWORD_NAME" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'keyword_name',
                           -default => $fields{ 'keyword_name' },
                           -size => $CFG{ KEYWORDS_KEYWORD_NAME_SIZE },
                           -maxlength =>
                             $CFG{ KEYWORDS_KEYWORD_NAME_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Ключевое слово &mdash; начало цепочки</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник keywords.pl
  create_selection_func( func_name => 'select_orig_keyword',
                         module_name => 'keywords.pl',
                         caller_name => $module,
                         window_name => 'orig_keywords',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также ключевое слово (без возможности редактирования)
  print "<td>",
        $query->textfield( -name => 'keyword_code_orig',
                           -default => $fields{ 'keyword_code_orig' },
                           -size => $CFG{ KEYWORDS_KEYWORD_CODE_ORIG_SIZE },
                           -maxlength =>
                             $CFG{ KEYWORDS_KEYWORD_CODE_ORIG_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'keyword_name_orig',
                           -default => $fields{ 'keyword_name_orig' },
                           -size => $CFG{ KEYWORDS_KEYWORD_NAME_ORIG_SIZE },
                           -maxlength =>
                             $CFG{ KEYWORDS_KEYWORD_NAME_ORIG_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" onclick=} .
        qq{"select_orig_keyword()">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  
  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<th>Язык</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values,
                            -labels => { @{ $ref_labels } }
                          ),
        "</td>\n";
  print "</tr>\n";
  
  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "KEYWORDS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ KEYWORDS_COMMENT_ROWS },
                          -columns => $CFG{ KEYWORDS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}
