#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   работа со справочными таблицами, имеющими типовую структуру
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

# ПРИМЕЧАНИЕ. Типовая структура справочных таблиц (см. файл tables34.cfg):
#             - код (ключевое поле);
#             - наименование полное;
#             - наименование краткое (его может не быть);
#             - примечание.
#             Таким образом, таблица может содержать 3 или 4 поля (плюс
#             служебные поля, которые есть и во всех других таблицах).

# ПРИМЕЧАНИЕ. При инсталляции системы "Электронный архив" создаются жесткие
#             ссылки на этот модуль, что позволяет запускать один и тот же
#             программный файл под разными именами. Имя программного файла
#             совпадает с именем таблицы базы данных, которую (таблицу) этот
#             программный файл обрабатывает.

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;             # CGI-объект
my $script_name;       # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module;

my $table;             # имя обрабатываемой таблицы базы данных
my $table_type;        # тип таблицы: с тремя полями или с четырьмя
my @f_names = ();      # массив имен полей обрабатываемой таблицы
my $key_name;          # имя ключевого поля таблицы
my $short_name;        # имя поля для краткого наименования
my $full_name;         # имя поля для полного наименования
                       # ПРИМЕЧАНИЕ. При работе с "трехпольной" таблицей
                       #             будем использовать переменную $full_name.
my $title;             # заголовок, выводимый на экране

use constant TABLE_3_FIELDS   => 0;  # таблица с тремя полями
use constant TABLE_4_FIELDS   => 1;  # таблица с четырьмя полями

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# получим информацию для обрабатываемой таблицы из глобального хеш-массива
get_table_info();

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( $key_name ) eq '' )
  {
    push( @error_messages, "Не указано значение ключевого поля" );
  }

  # если таблица "четырехпольная"
  if ( $table_type == TABLE_4_FIELDS && 
       $query->param( $short_name ) eq '' )
  {
    push( @error_messages, "Не указано краткое наименование" );
  }

  if ( $query->param( $full_name ) eq '' )
  {
    # текст сообщения зависит от числа полей в таблице, поскольку переменная
    # $full_name используется как для "четырехпольной", так и для
    # "трехпольной" таблицы (а переменная $short_name используется только
    # для "четырехпольной" таблицы)
    push( @error_messages, ( $table_type == TABLE_4_FIELDS ) ?
                             "Не указано полное наименование" :
                             "Не указано наименование" );
  }

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }

  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => $title,
                    user => $user
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $command;               # строка команды для работы с БД

  my $key_value = $query->param( $key_name );  # значение ключевого поля
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}$table " .
             "WHERE $key_name = '$key_value'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  my $key_value = $query->param( $key_name );  # значение ключевого поля

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM $table " .
             "WHERE $key_name = '$key_value'";
  push( @table_cmd_pairs, [ $table, $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  my $key_value = $query->param( $key_name );  # значение ключевого поля

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.$table " .
             "WHERE $key_name = '$key_value'";
  push( @table_cmd_pairs, [ $table, $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех записей
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выбираем все записи из таблицы (сортируем по второму полю)
  $command = "SELECT * FROM ${db_schema}$table " .
             "ORDER BY $full_name";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код</th>\n";
    
    ( $table_type == TABLE_4_FIELDS ) ?
      print "<th>Наименование краткое</th>\n" :
      print "<th>Наименование</th>\n";

    # колонка без заголовков
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # код (ключевое поле - первичный ключ)
      # id возбмем один для всех типовых модулей -- lang_code
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( $key_name ) ]</td>\n";

      # если таблица "четырехпольная", то выводим краткое наименование
      if ( $table_type == TABLE_4_FIELDS )
      {
        print "<td>$row[ $result->fnumber( $short_name ) ]</td>\n";
      }
      # если таблица "трехпольная", то выводим "простое" наименование,
      # которое скрывается под именем $full_name
      else
      {
        print "<td>$row[ $result->fnumber( $full_name ) ]</td>\n";
      }

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "$key_name=" . $row[ $result->fnumber( $key_name ) ];

      # просмотр информации в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $command;                    # строка команды для работы с БД

  my $key_value = $query->param( $key_name );  # значение ключевого поля

  # выбираем одну запись
  $command = "SELECT * FROM ${db_schema}$table " .
             "WHERE $key_name = '$key_value'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи со " .
                     "значением ключа " . $key_value,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
          );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# получение информации для обрабатываемой таблицы из глобального хеш-массива
# -----------------------------------------------------------------------------
sub get_table_info
{
  # имя этого скрипта (не включая путь и расширение)
  # оно будет совпадать с именем таблицы БД
  my $script;

  my $in_table = 0;
  
  # выделим имя скрипта (без расширения .pl) из полного имени, включающего путь
  ( undef, $script ) = $script_name =~ /^(.+\/)(.+)\..+$/;

  # открываем конфигурационный файл с описаниями таблиц-справочников
  if ( ! open( TABLES, "< ./cfg/tables34.cfg" ) )
  {
    # выводим строку команды в лог
    print STDERR "Не удалось открыть файл tables34.cfg: $!\n"; 

    # завершаем работу скрипта
    die "Не удалось открыть файл tables34.cfg: $!\n";
  }

  while ( <TABLES> )
  {
    chomp;                  # уберем символ новой строки
    next if $_ eq '';       # пустые строки пропускаем
    next if $_ =~ /^\s*#/;  # строки-комментарии пропускаем

    s/^\s*//;               # отбросим лидирующие и ...
    s/\s*$//;               # ... замыкающие пробелы

    if ( /^TABLE\s+(.+)$/ )
    {
      if ( lc( $1 ) eq $script )
      {
        $in_table = 1;          # мы "внутри" определения таблицы
        $table = $1;            # имя обрабатываемой таблицы базы данных
        $module = uc( $table ); # условное наименование модуля
      }
    }
    elsif ( $in_table == 1 )
    {
      if ( /^TITLE\s+(.+)$/ )
      {
        $title = $1;            # заголовок, выводимый на экране
      }
      elsif ( /_code$/ )
      {
        $key_name = $_;         # имя ключевого поля таблицы      

        # формируем массив имен полей обрабатываемой таблицы
        push( @f_names, $key_name );
      }
      elsif ( /_fname$/ )
      {
        $full_name = $_;        # имя поля для полного наименования

        # формируем массив имен полей обрабатываемой таблицы
        push( @f_names, $full_name );

        # определим тип таблицы как "четырехпольную" по наличию имени поля
        # с постфиксом _fname
        $table_type = TABLE_4_FIELDS;
      }
      elsif ( /_sname$/ )
      {
        $short_name = $_;       # имя поля для краткого наименования

        # формируем массив имен полей обрабатываемой таблицы
        push( @f_names, $short_name );
      }
      elsif ( /_name$/ )
      {
        $full_name = $_;        # имя поля для полного наименования

        # формируем массив имен полей обрабатываемой таблицы
        push( @f_names, $full_name );

        # определим тип таблицы как "трехпольную" по наличию имени поля
        # с постфиксом _name
        $table_type = TABLE_3_FIELDS;
      }
      elsif ( /^comment$/ )     # это последнее поле в описании каждой таблицы
      {
        # формируем массив имен полей обрабатываемой таблицы
        push( @f_names, 'comment' );

        $in_table = 0;          # описание интересующей нас таблицы завершилось
      }
    }
    elsif ( $in_table == 0 )
    {
      next;
    }
  }

  # закрываем конфигурационный файл с описаниями таблиц-справочников
  if ( ! close( TABLES ) )
  {
    # выводим строку команды в лог
    print STDERR "Не удалось закрыть файл tables34.cfg: $!\n"; 

    # завершаем работу скрипта
    die "Не удалось закрыть файл tables34.cfg: $!";
  }

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => $table,
                       key => $key_name,
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )  # режим просмотра существующей записи из БД
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ $key_name } = get_next_key_value( $table, $key_name );
  }
  # -------------------------------------------------------------------

  # поскольку значение поля "Код ..." формируется автоматически
  # с использованием функции get_next_key_value(), не будем позволять его
  # редактировать, но совсем убрать его нельзя, т.к. оно используется
  # при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => $key_name,
                        -default => $fields{ $key_name }
                      ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";
 
  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код</th>\n";
  print "<td>", $fields{ $key_name }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";

  # число полей в таблице влияет на заголовок
  ( $table_type == TABLE_4_FIELDS ) ?
    print "<th>Полное наименование</th>\n" :
    print "<th>Наименование</th>\n";

  print "<td>",
        $query->textfield( -name => $full_name,
                           -default => $fields{ $full_name },
                           -size => $CFG{ LANGS_LANG_FNAME_SIZE },
                           -maxlength => $CFG{ LANGS_LANG_FNAME_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  if ( $table_type == TABLE_4_FIELDS )
  {
    print "<tr>\n";
    print "<th>Краткое наименование</th>\n";
    print "<td>",
          $query->textfield( -name => $short_name,
                             -default => $fields{ $short_name },
                             -size => $CFG{ LANGS_LANG_SNAME_SIZE },
                             -maxlength => $CFG{ LANGS_LANG_SNAME_MAXLENGTH }
                           ),
          "</td>\n";
    print "</tr>\n";
  }
  
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>",
        $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ LANGS_COMMENT_ROWS },
                          -columns => $CFG{ LANGS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "</table>\n";

  # --------------------------------------------------------------------
    
  return ( 0 );
}
