#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Состав временных группировок (списков) объектов
#           библиографического описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

# ПРИМЕЧАНИЕ. При формировании пользовательского интерфейса будем использовать
#             термин "тематический список" вместо термина "временная 
#             группировка". Теперь термин "тематический список" кажется более
#             информативным для пользователя. Переделывать же имена полей
#             таблиц базы данных и другие элементы исходных текстов
#             программной системы, которые не видны пользователю, не будем.

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'OBJ_GROUPS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Obj_groups
my @f_names =
   ( 'tmp_group_code',     # Уникальный код временной группировки
     'obj_id',             # Уникальный идентификатор объекта
                           # библиографического описания
     'bib_rec_num',        # Номер библиогр. записи для данного объекта
                           # (основная запись имеет номер 0)
     'tmp_group_inc_date', # Дата внесения объекта во временную группировку
     'comment'             # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'tmp_group_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный уникальный код " .
          "тематического списка" );
  }

  # НЕ допускается пустое поле
  if ( ! date_is_correct( $query->param( 'tmp_group_inc_date' ) ) )
  {
    push( @error_messages, "Не указана или неверная дата внесения объекта " .
          "в тематический список" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# ПРИМЕЧАНИЕ. Эта процедура - точная копия процедуры do_delete_from_group()
#             из модуля obj_groups_descrs.pl.
# TODO Подумать над принятием более грамотного решения с тем, чтобы
#      избежать подобного дублирования.
# -----------------------------------------------------------------------------
sub do_del
{
  my $tmp_group_code;        # уникальный код временной группировки
  my $obj_id;                # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;           # номер библиогр. записи для данного объекта
  my $target_win;            # имя окна браузера для вывода HTML-страницы
  my $command;               # строка команды для работы с БД

  $tmp_group_code = $query->param( 'tmp_group_code' );
  $obj_id         = $query->param( 'obj_id' );
  $bib_rec_num    = $query->param( 'bib_rec_num' );
  $target_win     = $query->param( 'target_win' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}obj_groups " .
             "WHERE tmp_group_code = '$tmp_group_code' AND " .
                   "obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' ";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка группировок, в которые включено данное издание
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о временных группировках для данного объекта
  $command = "SELECT og.*, ogd.tmp_group_name " .
             "FROM ${db_schema}obj_groups og, " .
                  "${db_schema}obj_groups_descrs ogd " .
             "WHERE og.obj_id = '$obj_id' AND " .
                   "og.bib_rec_num = '$bib_rec_num' AND " .
                   "og.tmp_group_code = ogd.tmp_group_code " .
             "ORDER BY ogd.tmp_group_name";

  $result = exec_select( $command );   # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Уникальный код тематического списка</th>\n";
    print "<th>Наименование тематического списка</th>\n";
    print "<th>Дата внесения объекта в тематический список</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим уникальный код и наименование временной группировки, дату
      # внесения объекта во временную группировку
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'tmp_group_code' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'tmp_group_name' ) ]</td>\n";
      print "<td class=\"center\">",
            date_for_display( $row[ $result->fnumber( 'tmp_group_inc_date' ) ]
                            ),
            "</td>\n";

      # просмотр данной записи о временной группировке в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&tmp_group_code=" .
            $row[ $result->fnumber( 'tmp_group_code' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о временной группировке
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&tmp_group_code=" .
            $row[ $result->fnumber( 'tmp_group_code' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о временной группировке
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $tmp_group_code;     # уникальный код временной группировки
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id         = $query->param( 'obj_id' );
  $bib_rec_num    = $query->param( 'bib_rec_num' );
  $tmp_group_code = $query->param( 'tmp_group_code' );
  $target_win     = $query->param( 'target_win' );

  # выбираем одну запись о временной группировке для данного объекта
  $command = "SELECT og.*, ogd.tmp_group_name " .
             "FROM ${db_schema}obj_groups og, " .
                  "${db_schema}obj_groups_descrs ogd " .
             "WHERE og.tmp_group_code = '$tmp_group_code' AND " .
                   "og.obj_id = '$obj_id' AND " .
                   "og.bib_rec_num = '$bib_rec_num' AND " .
                   "og.tmp_group_code = ogd.tmp_group_code";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "уникальный код тематического списка - $tmp_group_code",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "obj_groups",
                       key => "obj_id, bib_rec_num, " .
                              "tmp_group_code:tmp_group_code_begin",
                       f_names => \@f_names,
                       f_special => [ 'tmp_group_inc_date' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  my( $day, $mon, $year );  # для формирования текущей даты

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );

    # сформируем текущую дату
    ( $day, $mon, $year ) = ( localtime() )[ 3..5 ];

    $fields{ 'tmp_group_inc_date' } =
      sprintf( "%04d-%02d-%02d", $year + 1900, $mon + 1, $day );

    # значения по умолчанию

    # уникальный код временной группировки (тематического списка)
    $fields{ 'tmp_group_code' } = $CFG{ OBJ_GROUPS_TMP_GROUP_CODE };

    # примечание
    $fields{ 'comment' }        = $CFG{ OBJ_GROUPS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи для данного
  # объекта" не подлежат редактированию, не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа, а также при выполнении операции INSERT
  # (значения этих полей сохраняются такими же, какими они были при
  # предыдущем запуске скрипта, поэтому можно не присваивать им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";

  # создадим скрытое поле для "Уникального кода временной группировки"
  # ПРИМЕЧАНИЕ. Идентификатор скрытого поля имеет постфикс 'begin' для того,
  #             чтобы можно было обновлять значение этого поля, поскольку оно
  #             является ключевым (см. функцию make_cmd в пакете E_ARCH.pm.
  #             Т.е. нам нужно и исходное значение этого поля, и его новое
  #             значение (которое, возможно, было изменено пользователем).
  print $query->hidden( -name => 'tmp_group_code_begin',
                        -default => $fields{ 'tmp_group_code' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();          # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Уникальный код тематического списка</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник obj_groups_descrs.pl
  create_selection_func( func_name => 'select_tmp_group',
                         module_name => 'obj_groups_descrs.pl',
                         caller_name => $module,
                         window_name => 'obj_groups_descrs',
                         exp_imp => $query->param( 'exp_imp' ) );
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Наименование временной
  # группировки"
  create_hot_help( "document.forms[ 0 ].tmp_group_code",
                   "OBJ_GROUPS_TMP_GROUP_CODE", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                    document.forms[ 0 ].tmp_group_name.value = str; " )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'tmp_group_code',
                           -default => $fields{ 'tmp_group_code' },
                           -size => $CFG{ OBJ_GROUPS_TMP_GROUP_CODE_SIZE },
                           -maxlength =>
                             $CFG{ OBJ_GROUPS_TMP_GROUP_CODE_MAXLENGTH }
                          ),
        "&nbsp;&nbsp;",
        qq{<a class="command_link" href="javascript:void( 0 )" } .
        qq{onclick="select_tmp_group()">Список</a>},
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Наименование тематического списка</th>\n";
  print "<td>",
        $query->textarea( -name => 'tmp_group_name',
                          -default => $fields{ 'tmp_group_name' },
                          -rows => $CFG{ OBJ_GROUPS_TMP_GROUP_NAME_ROWS },
                          -columns => $CFG{ OBJ_GROUPS_TMP_GROUP_NAME_COLS }
                          -readonly => 'readonly'
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата внесения объекта в тематический список</th>\n";
  print "<td>",
        $query->textfield( -name => 'tmp_group_inc_date',
                           -default =>
                             date_for_display( $fields{ 'tmp_group_inc_date' }
                                             ),
                           -size => $CFG{ OBJ_GROUPS_TMP_GROUP_INC_DATE_SIZE },
                           -maxlength =>
                             $CFG{ OBJ_GROUPS_TMP_GROUP_INC_DATE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "OBJ_GROUPS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ OBJ_GROUPS_COMMENT_ROWS },
                          -columns => $CFG{ OBJ_GROUPS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Дата введения валюты
  # ПРИМЕЧАНИЕ. Эта проверка лишняя, т.к. другие поля в этой функции
  #             не обрабатываются. Но пусть пока что останется.
  if ( $fname eq 'tmp_group_inc_date' )
  {
    $value = date_for_db( $query->param( $fname ) );
  }

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Участие в тематических списках объектов " .
                    "библиографического описания" );
  return ( 0 );
}
