#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Описания временных группировок (списков) объектов 
#           библиографического описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

# ПРИМЕЧАНИЕ. При формировании пользовательского интерфейса будем использовать
#             термин "тематический список" вместо термина "временная 
#             группировка". Теперь термин "тематический список" кажется более
#             информативным для пользователя. Переделывать же имена полей
#             таблиц базы данных и другие элементы исходных текстов
#             программной системы, которые не видны пользователю, не будем.

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'OBJ_GROUPS_DESCRS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Obj_groups_descrs
my @f_names =
   ( 'tmp_group_code',     # Уникальный код временной группировки
     'tmp_group_name',     # Наименование временной группировки
     'tmp_group_descr',    # Описание временной группировки
     'tmp_group_date',     # Дата рождения временной группировки
     'comment'             # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

# выбираем описания всех объектов, входящих в данную временную группировку
if ( $query->path_info =~ /^\/list$/ )
{
  do_list();
}

# исключаем описание указанного объекта из данной временной группировки
elsif ( $query->path_info =~ /^\/delete_from_group$/ &&
        have_access( 'module' => $module, 'procedure' => 'DELETE_FROM_GROUP',
                     'user' => $user )
      )
{
  do_delete_from_group();
}

# добавление объектов в группировку (организация формы для ввода данных)
elsif ( $query->path_info =~ /^\/add_objs_into_group$/ &&
        have_access( 'module' => $module, 'procedure' => 'ADD_INTO_GROUP',
                     'user' => $user )
      )
{
  do_add_objs_into_group();
}

# непосредственный ввод группы записей об объектах, добавляемых во временную
# группировку (с проверкой правильности ввода)
if ( $query->param( 'submit_add_objs_into_group' ) &&
     have_access( 'module' => $module, 'procedure' => 'ADD_INTO_GROUP',
                  'user' => $user )
   )
{
  add_objs_into_group();
}

exit( 0 );


# -----------------------------------------------------------------------------
# непосредственное добавление группы записей в БД (т.е. объектов, добавляемых
# во временную группировку)
# -----------------------------------------------------------------------------
sub add_objs_into_group
{
  my( $conn, $result );    # для работы с БД
  my $command;             # строка команды для добавления записи в таблицу
  my $num_recs = 0;        # счетчик добавленных записей
  my $obj_id;              # уникальный идентификатор объекта биб. описания
  my @obj_ids = ();        # массив для хранения всех вводимых идентификаторов
  my( $first, $last );     # первый и последний идентификаторы диапазона

  # уникальный код временной группировки
  my $tmp_group_code = $query->param( 'tmp_group_code' );

  # дата внесения объектов во временную группировку
  my $tmp_group_inc_date = $query->param( 'tmp_group_inc_date' );

  # строка, содержащая все введенные идентификаторы добавляемых объектов
  my $obj_ids_str = $query->param( 'obj_ids_str' );

  # комментарий
  my $comment = $query->param( 'comment' );
  
  # формируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();          # начинаем оформление верхней части окна
  create_top_2();      # формируем верхнюю часть окна со списком "команд"
                       # для пользователя
  close_top();         # завершаем оформление верхней части окна

  open_bottom();       # начинаем оформление нижней части окна

  print_add_info();    # выведем дополнительную информацию на экран
  
  # если проверка введенных значений не прошла успешно
  if ( ! check_param_objs() )
  {
    close_bottom();    # завершаем оформление нижней части окна
    print $query->end_html();
    return ( 1 );      # возвращаемое значение пока что никак не используется
  }
  
  # проверка прошла успешно - выполним добавление записей

  $conn = exec_connect_db();           # подключаемся к БД
  exec_transaction( $conn, "BEGIN" );  # выполним операции в рамках транзакции

  # заменим каждый символ-разделитель на один "пробел"
  # (в качестве разделителей могут служить некоторые нецифровые символы)
  # учтем, что допустимо указывать диапазоны идентификаторов в форме,
  # например, 125-143, поэтому дефис "-" заменять "пробелом" не нужно
  $obj_ids_str =~ s/[^\d-]/ /g;

  # сформируем массив для хранения всех вводимых идентификаторов
  # (это делается с целью "развертывания" диапазонов в полные списки
  # идентификаторов)

  $obj_ids_str =~ s/^\s*//g;  # удалим начальные ...
  $obj_ids_str =~ s/\s*$//g;  # ... и замыкающие пробелы
  
  # разделим строку, содержащую все введенные идентификаторы объектов,
  # на отдельные числа (учтем возможное наличие диапазонов)
  foreach ( split( /\s+/, $obj_ids_str ) )
  {
    if ( /^\d+$/ )           # это одиночный идентификатор
    {
      push( @obj_ids, $_ );  # поместим его в массив
    }
    # диапазон идентификаторов (возможно более 1 дефиса)
    elsif ( /^(\d+)[-]+(\d+)$/ )
    {
      # первый и последний идентификаторы диапазона
      $first = $1;
      $last  = $2;
      
      # TODO Перенести эту проверку в check_param_objs().
      if ( $first > $last)
      {
        print "<p class=\"user_error\">";
        print "Неверный диапазон идентификаторов: $first, $last";
        print "</p>\n";

        close_bottom();  # завершаем оформление нижней части окна
        print $query->end_html();
        return ( 1 );    # возвращаемое значение пока что никак не используется
      }  
      else
      {
        for ( $obj_id = $first; $obj_id <= $last; $obj_id++ )
        {
          push( @obj_ids, $obj_id );  # поместим каждый идентификатор в массив
        }
      }
    }
    # на всякий случай сделаем эту ветвь (хотя она должна быть избыточной)
    # ПРИМЕЧАНИЕ. Эта ветвь пригодилась в процессе отладки модуля.
    else
    {
      print "<p class=\"user_error\">";
      print "Недопустимый способ задания идентификаторов объектов: $_";
      print "</p>\n";

      close_bottom();  # завершаем оформление нижней части окна
      print $query->end_html();
      return ( 1 );  # возвращаемое значение пока что никак не используется
    }
  }

  # теперь введем в таблицу Obj_groups записи для всех идентификаторов
  # объектов биб. описаний
  foreach $obj_id ( @obj_ids )
  {
    # сначала сформируем строку-команду ...
    # ПРИМЕЧАНИЕ. Поле bib_rec_num получит значение по умолчанию, равное 0.
    #             Будем считать, что при групповом вводе объектов во временную
    #             группировку такой подход оправдан.
    $command = "INSERT INTO Obj_groups ( tmp_group_code, obj_id, " .
                                        "tmp_group_inc_date, comment, " .
                                        "who_chg ) " .
               "VALUES ( $tmp_group_code, $obj_id, " .
                         "'" . date_for_db( $tmp_group_inc_date ) . "', " .
                         ( $comment ne '' ? "'$comment'" : "null" ) .
                         ", '$user' )";

    # DEBUG 
    print_debug_info( $command ) if $DEBUG;  # печатаем строку команды

    # ... и вставим запись в таблицу Obj_groups
    $result = $conn->exec( $command );

    # если произошла ошибка при вставке записи в базу данных
    if ( PGRES_COMMAND_OK ne $result->resultStatus )
    {
      # сообщение для пользователя
      print "<p class=\"error\">$CFG{ ERROR_INSERT }</p>\n";
      print STDERR $command . "\n";           # выводим строку команды в лог
      die $conn->errorMessage;
    }
    else
    {
      $num_recs++;         # счетчик добавленных записей
    }
  }

  exec_transaction( $conn, "END" );     # конец транзакции
    
  print "<p class=\"info\">";
  print "В базу данных добавлены записи: $num_recs шт.</p>\n";

  close_bottom();      # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'tmp_group_code' ) eq '' )
  {
    push( @error_messages, "Не указан код тематического списка" );
  }

  if ( $query->param( 'tmp_group_name' ) eq '' )
  {
    push( @error_messages, "Не указано наименование тематического списка" );
  }

  if ( $query->param( 'tmp_group_descr' ) eq '' )
  {
    push( @error_messages, "Не указано описание тематического списка" );
  }

  # НЕ допускается пустое поле
  if ( ! date_is_correct( $query->param( 'tmp_group_date' ) ) )
  {
    push( @error_messages, "Не указана или неверная дата рождения " .
          "тематического списка" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# проверка значений параметров формы при добавлении объектов
# во временную группировку
# -----------------------------------------------------------------------------
sub check_param_objs
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # строка со списком идентификаторов объектов
  if ( $query->param( 'obj_ids_str' ) eq '' )   # не допускается пустое поле
  {
    push( @error_messages, "Не введены идентификаторы объектов" );
  }
  # допускаются символы-разделители: ". , ; \ /", а для задания диапазонов
  # идентификаторов служит дефис "-"
  # ПРИМЕР допустимого, при этом весьма неряшливого, ввода:
  #              " 115 ; 134-137,  147--151 168  ".
  elsif ( $query->param( 'obj_ids_str' ) !~
          /^\s*                # возможные пробелы в начале строки
           (\d+|               # число или ...
             (\d+[-]+\d+)      # ... диапазон чисел (возможно более
           )                   # одного дефиса (на всякий случай))
           (\s*[\s\.,;\/\\]\s* # допустимые разделители
             (\d+|             # и опять число или ...
               (\d+[-]+\d+)    # ... диапазон ...
             )
           ){0,}               # и так ноль или более раз
           \s*$/x              # возможные пробелы в конце строки
        )
  {
    push( @error_messages, "Введены недопустимые символы или неверно " .
          "заданы диапазоны" );
  }

  # НЕ допускается пустое поле
  if ( ! date_is_correct( $query->param( 'tmp_group_inc_date' ) ) )
  {
    push( @error_messages, "Не указана или неверная дата ввода объектов " .
          "в тематический список" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя, при работе с описаниями группировок
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  # ПРИМЕЧАНИЕ. Заголовок модуля, выводимый на экран, сделаем отличным
  #             от заголовка, выбранного для программного модуля.
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Тематические списки ' .
                                    'библиографических описаний',
                    user => $user
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя, при работе с составом конкретной группировки
# -----------------------------------------------------------------------------
sub create_top_2
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Тематический список ' .
                                    'библиографических описаний',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация процедуры добавления объектов в группировку
# -----------------------------------------------------------------------------
sub do_add_objs_into_group
{
  # имя окна браузера для вывода HTML-страницы
  my $target_win = $query->param( 'target_win' );

  # формируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top_2();   # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # поскольку по умолчанию форма обрабатывается тем же URL-ом, которым она
  # была сформирована, то явно указываем URL скрипта для обработки данных,
  # собранных в полях формы, т.к. иначе останется PATH_INFO =
  # 'add_objs_into_group' от предыдущего вызова скрипта, во время которого
  # формируется форма, и этот PATH_INFO будет обработан еще раз, чего уже
  # не требуется
  print $query->start_form( -action => $script_name, -target => $target_win );

  # создадим форму для ввода идентификаторов объектов, добавляемых
  # во временную группировку
  make_form_objs();

  print $query->submit( -name => "submit_add_objs_into_group",
                        -value => "Добавить описания в тематический список",
                        -class => 'submit'
                      ), "\n";

  print $query->end_form();

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи -- описания временной группировки
# -----------------------------------------------------------------------------
sub do_del
{
  my $tmp_group_code;        # уникальный код временной группировки
  my $command;               # строка команды для работы с БД

  $tmp_group_code = $query->param( 'tmp_group_code' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}obj_groups_descrs " .
             "WHERE tmp_group_code = '$tmp_group_code'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# исключение описания указанного объекта из данной временной группировки
# ПРИМЕЧАНИЕ. Эта процедура - почти точная копия процедуры do_del()
#             из модуля obj_groups.pl.
# TODO Подумать над принятием более грамотного решения с тем, чтобы
#      избежать подобного дублирования.
# -----------------------------------------------------------------------------
sub do_delete_from_group
{
  my $tmp_group_code;        # уникальный код временной группировки
  my $obj_id;                # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;           # номер библиогр. записи для данного объекта
  my $command;               # строка команды для работы с БД

  $tmp_group_code = $query->param( 'tmp_group_code' );
  $obj_id         = $query->param( 'obj_id' );
  $bib_rec_num    = $query->param( 'bib_rec_num' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}obj_groups " .
             "WHERE tmp_group_code = '$tmp_group_code' AND " .
                   "obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' ";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top_2,
           target => "_self",
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование всех биб. описаний, входящих в данный тематический список
# (временную группировку)
# -----------------------------------------------------------------------------
sub do_export
{
  my $tmp_group_code;        # уникальный код временной группировки  
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $tmp_group_code = $query->param( 'tmp_group_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)

  # выбираем все биб. описания для данной временной группировки
  $command = "SELECT b.* " .
             "FROM Obj_groups og, Bib_recs b " .
             "WHERE og.tmp_group_code = '$tmp_group_code' AND " .
                   "og.obj_id = b.obj_id AND " .
                   "og.bib_rec_num = b.bib_rec_num";

  push( @table_cmd_pairs, [ 'Bib_recs', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование всех биб. описаний, входящих в данный тематический список
# (временную группировку)
# -----------------------------------------------------------------------------
sub do_import
{
  my $tmp_group_code;        # уникальный код временной группировки  
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $tmp_group_code = $query->param( 'tmp_group_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)

  # выбираем все биб. описания для данной временной группировки
  $command = "SELECT b.* " .
             "FROM exp_imp.Obj_groups og, exp_imp.Bib_recs b " .
             "WHERE og.tmp_group_code = '$tmp_group_code' AND " .
                   "og.obj_id = b.obj_id AND " .
                   "og.bib_rec_num = b.bib_rec_num";

  push( @table_cmd_pairs, [ 'Bib_recs', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка описаний всех объектов, входящих в данную временную группировку
# -----------------------------------------------------------------------------
sub do_list
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $href;                       # типовая часть HTML-ссылки
  my $href_bib_rec;               # типовая часть HTML-ссылки для биб. описаний
  my $tmp_group_code;             # уникальный код временной группировки
  my $tmp_group_name;             # наименование временной группировки
  my $tmp_group_descr;            # описание временной группировки
  my $tmp_group_date;             # дата создания временной группировки
  my @avail_objs;                 # список физ. форм, в которых присутствуют
                                  # наличные экземпляры объекта биб. описания,
                                  # хранимые у персоналии, имеющей
                                  # идентификатор, указанный в конфиг. файле

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query,
                     target => "_self" );

  $tmp_group_code  = $query->param( 'tmp_group_code' );
  $tmp_group_name  = $query->param( 'tmp_group_name' );
  $tmp_group_descr = $query->param( 'tmp_group_descr' );
  $tmp_group_date  = $query->param( 'tmp_group_date' );

  open_top();       # начинаем оформление верхней части окна
  create_top_2();   # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "tmp_group_code=$tmp_group_code" .
          "&tmp_group_name=" . CGI::escape( $tmp_group_name ) .
          "&tmp_group_descr=" . CGI::escape( $tmp_group_descr ) .
          "&tmp_group_date=$tmp_group_date";

  # выбираем все записи для данной временной группировки
  # ПРИМЕЧАНИЕ. Используем возможность PostgreSQL, позволяющую выбирать
  #             имя поля в зависимости от какого-то условия (это удобно,
  #             так как может быть заполнено только одно из полей
  #             'bib_rec_full' или 'bib_rec_short').
  $command = "SELECT CASE WHEN b.bib_rec_short IS NOT NULL " .
                         "THEN b.bib_rec_short " .
                         "ELSE b.bib_rec_full " .
                    "END AS bib_rec, " .
                    "og.* " .
             "FROM ${db_schema}obj_groups og, ${db_schema}bib_recs b " .
             "WHERE og.tmp_group_code = '$tmp_group_code' AND " .
                   "og.obj_id = b.obj_id AND " .
                   "og.bib_rec_num = b.bib_rec_num " .
             "ORDER BY 1";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # в качестве имени модуля для просмотра будет именно 'BIB_RECS',
    # а не кодовое имя того модуля, из которого вызывается эта процедура
    $can_view = have_access( 'module' => 'BIB_RECS', 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 
                               'procedure' => 'DELETE_FROM_GROUP',
                               'user' => $user );

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      # в качестве имени модуля будет именно 'BIB_RECS'
      $can_import = import_enabled() && 
                    have_access( 'module' => 'BIB_RECS',
                                 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => 'BIB_RECS',
                                 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Номер п/п</th>\n";
    print "<th>ID и номер биб. описания</th>\n";
    print "<th>Биб. описание члена тематического списка</th>\n";
    print "<th>Дата внесения описания в список</th>\n";
    print "<th>Наличие</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим порядковый номер, уник. код объекта биб. описания
      # и номер биб. записи, биб. описание члена группировки и дату внесения
      # объекта в группировку
      $i++;                      # порядковый номер
      print "<td class=\"center\">$i</td>\n";
      print "<td class=\"center\">$row[ $result->fnumber( 'obj_id' ) ]" .
            " (". $row[ $result->fnumber( 'bib_rec_num' ) ] . ")",
            "</td>\n";

      # будем заменять двойной дефис, которым мы имитируем тире при вводе
      # данных, символом тире &mdash;
      # см. TODO в функции list_bib_recs_by_query() модуля E_ARCH.pm.
      my $bib_rec = $row[ $result->fnumber( 'bib_rec' ) ];
      $bib_rec =~ s/--/&mdash;/g;

      print "<td>$bib_rec</td>\n";

      print "<td>",
            date_for_display( $row[ $result->fnumber( 'tmp_group_inc_date' ) ]
                            ),
            "</td>\n";

      # наличие экземпляров данного объекта у владельца, идентификатор
      # которого указан в конфиг. файле
      print "<td>\n";

      @avail_objs = get_available( $row[ $result->fnumber( 'obj_id' ) ] );

      # выведем все найденные экземпляры (точнее, физ. формы, в которых эти
      # экземпляры присутствуют) в одной ячейке
      foreach ( @avail_objs )
      {
        print $_ . "<br />\n";
      }
      print "</td>\n";

      # типовая часть HTML-ссылки для биб. описаний
      $href_bib_rec = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
                      "obj_id=" . $row[ $result->fnumber( 'obj_id' ) ] .
                      "&bib_rec_num=" . 
                      $row[ $result->fnumber( 'bib_rec_num' ) ];

      # просмотр одного биб. описания в виде HTML-формы (в новом окне)
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{<a class="command_link" } .
            qq{href="/$CFG\{ HOME_WEB_DIR \}/bib_recs.pl/view?$href_bib_rec} .
            qq{&target_win=_self"} .
            qq{ target="_blank">Просмотр биб. описания</a>} . "\n" 
        if $can_view;

      # удаление записи из временной группировки
      print qq{<a class="command_link" } .
            qq{href="$script_name/delete_from_group?$href} .
            "&obj_id=" . $row[ $result->fnumber( 'obj_id' ) ] .
            "&bib_rec_num=" .
            $row[ $result->fnumber( 'bib_rec_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Исключение из тематического списка</a> } . "\n"
        if $can_delete;

      # экспортирование одного биб. описания
      print qq{<a class="command_link" } .
            qq{href="/$CFG\{ HOME_WEB_DIR \}/bib_recs.pl/export?} .
            $href_bib_rec .
            qq{&target_win=_self"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт биб. описания</a>} . "\n"
        if $can_export;

      # импортирование одного биб. описания
      print qq{<a class="command_link" } .
            qq{href="/$CFG\{ HOME_WEB_DIR \}/bib_recs.pl/import?} .
            $href_bib_rec .
            qq{&target_win=_self"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт биб. описания</a>} . "\n"
        if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления объектов в группировку
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD_INTO_GROUP', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" } .
          qq{href="$script_name/add_objs_into_group?$href"} .
          qq{ target="_self">Добавить описания в тематический список</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех временных группировок
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $href;                       # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выбираем все записи о временных группировках
  $command = "SELECT * " .
             "FROM ${db_schema}obj_groups_descrs " .
             "ORDER BY tmp_group_name";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      # в качестве имени модуля будет именно 'BIB_RECS'
      $can_import = import_enabled() && 
                    have_access( 'module' => 'BIB_RECS',
                                 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => 'BIB_RECS',
                                 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Номер п/п</th>\n";
    print "<th>Уникальный код списка</th>\n";
    print "<th>Наименование списка</th>\n";
    print "<th>Дата рождения списка</th>\n";
    print "<th>&nbsp;</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретной временной группировки ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Obj_groups_descrs полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'OBJ_GROUPS' )
      {
        # ... полям tmp_group_code, tmp_group_name в модуле OBJ_GROUPS
        create_assign_values_func( 'tmp_group_code', 'tmp_group_name' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим порядковый номер, уникальный код, наименование
      # и дату рождения временной группировки
      $i++;                      # порядковый номер
      print "<td class=\"center\">$i</td>\n";
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'tmp_group_code' ) ]</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретной временной группировки, то значение
      # поля tmp_group_name выведем в виде ссылки

      # пришлось использовать функцию escapeHTML(), поскольку в противном
      # случае те значения, в которых содержались символы кавычек,
      # не обрабатывались корректно, т.е. значение из справочника
      # не переносилось в поле ВЫЗЫВАЮЩЕЙ формы и окно справочника
      # не закрывалось (наверное, JavaScript не "любит" кавычки и некоторые
      # другие символы)
      # ПРИМЕЧАНИЕ. Использование функции CGI::escape(), как и ожидалось,
      #             не дало положительного результата -- значение в поле
      #             переносилось, но оно было представлено в форме
      #             вида %20%F4%AD и т.д.

      # выполним добавочную обработку значения переменной, т. к. того, что
      # делает функция escapeHTML(), недостаточно
      my $tmp_group_name = $row[ $result->fnumber( 'tmp_group_name' ) ];
      $tmp_group_name =~ s/\'/\\\'/g;
      $tmp_group_name = CGI::escapeHTML( $tmp_group_name );

      if ( $query->param( 'enable_select' ) )
      {
        print "<td>",
              qq{<a class="command_link" href="javascript:void( 0 )" onclick=
                 "assign_values( $row[ $result->fnumber( 'tmp_group_code' ) ],
                                 '$tmp_group_name' )">
                 $row[ $result->fnumber( 'tmp_group_name' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'tmp_group_name' ) ]</td>\n";
      }

      print "<td>",
            date_for_display( $row[ $result->fnumber( 'tmp_group_date' ) ] ),
            "</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "tmp_group_code=" . $row[ $result->fnumber( 'tmp_group_code' ) ];

      # просмотр состава временной группировки
      print "<td>",
            qq{<a class="command_link" } .
            qq{href="$script_name/list?$href} .
            "&tmp_group_name=" .
            CGI::escape( $row[ $result->fnumber( 'tmp_group_name' ) ] ) .
            "&tmp_group_descr=" .
            CGI::escape( $row[ $result->fnumber( 'tmp_group_descr' ) ] ) .
            "&tmp_group_date=" .
            $row[ $result->fnumber( 'tmp_group_date' ) ] . "\"" .
            qq{ target="_blank">Состав</a> },
            "</td>\n";

      # просмотр описания временной группировки в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" } .
            qq{href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n"
        if $can_view;

      # удаление записи о временной группировке
      print qq{<a class="command_link" } .
            qq{href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete"} .
            qq{ target="_self">Удаление</a> } . "\n"
        if $can_delete;

      # экспортирование всех биб. описаний, входящих в группировку
      print qq{<a class="command_link" } .
            qq{href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт всего списка</a>} . "\n"
        if $can_export;

      # импортирование всех биб. описаний, входящих в группировку
      print qq{<a class="command_link" } .
            qq{href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт всего списка</a>} . "\n"
        if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы (описание одной группировки)
# -----------------------------------------------------------------------------
sub do_view
{
  my $tmp_group_code;        # уникальный код временной группировки
  my $command;               # строка команды для работы с БД

  $tmp_group_code = $query->param( 'tmp_group_code' );

  # выбираем одну запись о временной группировке
  $command = "SELECT * " .
             "FROM ${db_schema}obj_groups_descrs " .
             "WHERE tmp_group_code = '$tmp_group_code'"; 

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "тематического списка " . $tmp_group_code,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# (описание одной группировки)
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "obj_groups_descrs",
                       key => "tmp_group_code",
                       f_names => \@f_names,
                       f_special => [ 'tmp_group_date' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# (описание одной группировки)
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  my( $day, $mon, $year );  # для формирования текущей даты

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'tmp_group_code' } = get_next_key_value( 'obj_groups_descrs',
                                                      'tmp_group_code' );
    # значения по умолчанию
    # сформируем текущую дату
    ( $day, $mon, $year ) = ( localtime() )[ 3..5 ];

    # дата рождения временной группировки
    $fields{ 'tmp_group_date' } =
      sprintf( "%04d-%02d-%02d", $year + 1900, $mon + 1, $day );

    # наименование временной группировки
    $fields{ 'tmp_group_name' }  = $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_NAME };

    # описание временной группировки
    $fields{ 'tmp_group_descr' } = $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_DESCR };

    # примечание
    $fields{ 'comment' }         = $CFG{ OBJ_GROUPS_DESCRS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный код временной группировки" формируется
  # автоматически с использованием функции get_next_key_value(), не будем
  # позволять его редактировать, но совсем убрать его нельзя, т.к. оно
  # используется при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'tmp_group_code',
                        -default => $fields{ 'tmp_group_code' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Уникальный код тематического списка</th>\n";
  print "<td>", $fields{ 'tmp_group_code' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Наименование тематического списка</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].tmp_group_name",
                   "OBJ_GROUPS_DESCRS_TMP_GROUP_NAME" ) 
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'tmp_group_name',
                          -default => $fields{ 'tmp_group_name' },
                          -rows => 
                            $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_NAME_ROWS },
                          -columns => 
                            $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_NAME_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Описание тематического списка</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].tmp_group_descr",
                   "OBJ_GROUPS_DESCRS_TMP_GROUP_DESCR" ) 
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'tmp_group_descr',
                          -default => $fields{ 'tmp_group_descr' },
                          -rows => 
                            $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_DESCR_ROWS },
                          -columns => 
                            $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_DESCR_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата рождения тематического списка</th>\n";
  print "<td>",
        $query->textfield( -name => 'tmp_group_date',
                           -default =>
                              date_for_display( $fields{ 'tmp_group_date' } ),
                           -size =>
                             $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_DATE_SIZE },
                           -maxlength =>
                             $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_DATE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "OBJ_GROUPS_DESCRS_COMMENT" ) 
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ OBJ_GROUPS_DESCRS_COMMENT_ROWS },
                          -columns => $CFG{ OBJ_GROUPS_DESCRS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# создание формы для добавления объектов в группировку
# -----------------------------------------------------------------------------
sub make_form_objs
{
  my $tmp_group_inc_date;   # дата внесения объектов во временную группировку
  my( $day, $mon, $year );  # для формирования текущей даты
  
  # возьмем значение ключевого поля "Уникальный код временной группировки"
  # из параметров скрипта
  my $tmp_group_code = $query->param( 'tmp_group_code' ); 

  # сформируем текущую дату
  ( $day, $mon, $year ) = ( localtime() )[ 3..5 ];

  $tmp_group_inc_date = sprintf( "%04d-%02d-%02d",
                                 $year + 1900, $mon + 1, $day );

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный код временной группировки"
  # не подлежит редактированию, не будем позволять его редактировать,
  # но совсем убрать его нельзя, т.к. оно используются при выполнении
  # операции INSERT (значение этого поля сохраняется таким же, каким оно было
  # при предыдущем запуске скрипта, поэтому можно не присваивать ему значения)
  print $query->hidden( -name => 'tmp_group_code' ), "\n";

  # эти скрытые поля призваны ретранслировать информацию о временной
  # группировке следующему вызову скрипта, когда будет работать функция
  # add_objs_into_group
  print $query->hidden( -name => 'tmp_group_name' ), "\n";
  print $query->hidden( -name => 'tmp_group_descr' ), "\n";
  print $query->hidden( -name => 'tmp_group_date' ), "\n";

  print_add_info(); # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Уникальный код тематического списка</th>\n";
  print "<td>", $tmp_group_code, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Идентификаторы объектов биб. описаний</th>\n";
  print "<td>",
        $query->textfield( -name => 'obj_ids_str',
                           -size => $CFG{ OBJ_GROUPS_DESCRS_OBJ_IDS_STR_SIZE },
                           -maxlength =>
                             $CFG{ OBJ_GROUPS_DESCRS_OBJ_IDS_STR_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номера библиогр. записей</th>\n";
  print "<td>", "0 для всех объектов (по умолчанию)", "</td>\n";
  print "</TR>\n";

  print "<tr>\n";
  print "<th>Дата внесения описаний в тематический список</th>\n";
  print "<td>",
        $query->textfield( -name => 'tmp_group_inc_date',
                           -default => date_for_display( $tmp_group_inc_date ),
                           -size => 
                             $CFG{ OBJ_GROUPS_DESCRS_TMP_GROUP_INC_DATE_SIZE },
                           -maxlength => $CFG{ 'OBJ_GROUPS_DESCRS_' .
                                               'TMP_GROUP_INC_DATE_MAXLENGTH' }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарии</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  # ПРИМЕЧАНИЕ. Шаблоны текста для "быстрой" вставки будут такими же,
  #             как и в функции make_form().
  create_hot_help( "document.forms[ 0 ].comment", "OBJ_GROUPS_DESCRS_COMMENT" ) 
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -rows => $CFG{ OBJ_GROUPS_DESCRS_COMMENT_ROWS },
                          -columns => $CFG{ OBJ_GROUPS_DESCRS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Дата введения валюты
  # ПРИМЕЧАНИЕ. Эта проверка лишняя, т.к. другие поля в этой функции
  #             не обрабатываются. Но пусть пока что останется.
  if ( $fname eq 'tmp_group_date' )
  {
    $value = date_for_db( $query->param( $fname ) );
  }

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  my $script_name = $query->script_name;

  my $result;               # для работы с БД
  my $command;              # строка команды для выборки из таблицы
  my @row;                  # для хранения одной записи из выборки
  my $tmp_group_name;       # наименование временной группировки
  my $tmp_group_descr;      # описание временной группировки

  # если имеются проблемы с русскими буквами в HTML-ссылках
  if ( $CFG{ HIDE_RUS_LETTERS } )
  {
    $command = "SELECT tmp_group_name, tmp_group_descr " .
               "FROM ${db_schema}Obj_groups_descrs " .
               "WHERE tmp_group_code = " . $query->param( 'tmp_group_code' );
  
    # сделаем выборку из БД
    $result = exec_select( $command );
  
    if ( $result )
    {
      # берем единственную запись из полученной выборки
      @row = $result->fetchrow;
      $tmp_group_name = $row[ $result->fnumber( 'tmp_group_name' ) ];
      $tmp_group_descr = $row[ $result->fnumber( 'tmp_group_descr' ) ];

      # будем заменять двойной дефис, которым мы имитируем тире при вводе
      # данных, символом тире &mdash;
      # см. TODO в функции list_bib_recs_by_query() модуля E_ARCH.pm.
      $tmp_group_name =~ s/--/&mdash;/g;
      $tmp_group_descr =~ s/--/&mdash;/g;
    }
  }
  else
  {
    # TODO. Нужно разобраться с использованием оператора s/--/&mdash;/g
    #       в этом модуле. Есть некоторая бессистемность!
    $tmp_group_name = $query->param( 'tmp_group_name' );
    $tmp_group_descr = $query->param( 'tmp_group_descr' );
  }

  # выведем уникальный код временной группировки, ее наименование, описание
  # и дату ее создания
  print "<p class=\"bib_rec\">";
  print "Уникальный код: " . $query->param( 'tmp_group_code' ) . "\n";
  print "&nbsp;&nbsp;\n";
  print "Дата создания: " .
        date_for_display( $query->param( 'tmp_group_date' ) ) . "<br />\n";
  print "Наименование: $tmp_group_name<br />\n";
  print "Описание: $tmp_group_descr\n";
  print "</p>\n";

  print "<h3>Состав</h3>\n";

  # создадим также ссылку для вывода списка биб. описаний, входящих
  # в групировку
  # эта ссылка дает точно такой же результат, что и ссылка в процедуре
  # do_query(); наличие этой ссылки избавляет от необходимости возвращаться
  # к списку временных группировок с целью вывода состава группировки после
  # удаления из нее одного из биб. описаний
  print qq{<a class="command_link" } .
        qq{href="$script_name/list?} .
        ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
        "tmp_group_code=" . $query->param( 'tmp_group_code' ) .
        "&tmp_group_name=" . CGI::escape( $query->param( 'tmp_group_name' ) ) .
        "&tmp_group_descr=" .
        CGI::escape( $query->param( 'tmp_group_descr' ) ) .
        "&tmp_group_date=" . $query->param( 'tmp_group_date' ) . "\"" .
        qq{ target="_self">Состав</a>\n};

  return ( 0 );
}
