#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Номера выпусков в сериях"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'OBJ_SERIES';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Obj_series
my @f_names =
   ( 'obj_id',           # Уникальный идентификатор объекта
                         # библиографического описания (составной части)
     'bib_rec_num',      # Номер библиогр. записи для данного объекта
                         # (основная запись имеет номер 0) 
     'series_id',        # Уникальный идентификатор серии/подсерии
     'ser_descr_num',    # Номер описания серии/подсерии
                         # (это поле, аналогичное полю bib_rec_num)
                         # (основное описание имеет номер 0)
     'issue_num',        # Номер выпуска в серии/подсерии
                         # (он может быть не указан, т.е. выпуски
                         # могут быть ненумерованными)
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # допускаются только цифры
  if ( $query->param( 'series_id' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный уникальный " .
          "идентификатор серии/подсерии" );
  }

  # допускаются только цифры
  if ( $query->param( 'ser_descr_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный номер описания " .
          "серии/подсерии" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $series_id;          # уникальный идентификатор серии/подсерии
  my $ser_descr_num;      # номер описания серии/подсерии
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id        = $query->param( 'obj_id' );
  $bib_rec_num   = $query->param( 'bib_rec_num' );
  $series_id     = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );
  $target_win    = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}obj_series " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "series_id = '$series_id' AND " .
                   "ser_descr_num = '$ser_descr_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех серий, в которые включено данное биб. описание
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о сериях/подсериях для данного биб. описания
  # (возьмем и описание серии/подсерии)
  $command = "SELECT os.*, sd.ser_descr " .
             "FROM ${db_schema}obj_series os, ${db_schema}ser_descrs sd " .
             "WHERE os.obj_id = '$obj_id' AND " .
                   "os.bib_rec_num = '$bib_rec_num' AND " .
                   "os.series_id = sd.series_id AND " .
                   "os.ser_descr_num = sd.ser_descr_num " .
             "ORDER BY sd.ser_descr";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Уникальный идентификатор серии/подсерии</th>\n";
    print "<th>Номер описания серии/подсерии</th>\n";
    print "<th>Описание серии/подсерии</th>\n";
    print "<th>Номер выпуска</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим уник. идентификатор серии/подсерии, номер описания
      # серии/подсерии, описание серии/подсерии и номер выпуска
      # (добавим символ &nbsp; к номеру выпуска из соображений эстетики:
      # если значение поля 'issu_num' равно null, то не отображаются
      # горизонтальные строки-разделители)
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'series_id' ) ]</td>\n";
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'ser_descr_num' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'ser_descr' ) ]</td>\n";
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'issue_num' ) ]&nbsp;</td>\n";

      # просмотр данной записи о серии/подсерии в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&series_id=" . $row[ $result->fnumber( 'series_id' ) ] .
            "&ser_descr_num=" . 
            $row[ $result->fnumber( 'ser_descr_num' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о серии/подсерии 
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&series_id=" . $row[ $result->fnumber( 'series_id' ) ] .
            "&ser_descr_num=" . 
            $row[ $result->fnumber( 'ser_descr_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о серии/подсерии для
  # данного биб. описания
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $series_id;          # уникальный идентификатор серии/подсерии
  my $ser_descr_num;      # номер описания серии/подсерии
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id        = $query->param( 'obj_id' );
  $bib_rec_num   = $query->param( 'bib_rec_num' );
  $series_id     = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );
  $target_win    = $query->param( 'target_win' );

  # выбираем одну запись о серии/подсерии для данного биб. описания
  # (возьмем и описание серии/подсерии)
  $command = "SELECT os.*, sd.ser_descr " .
             "FROM ${db_schema}obj_series os, ${db_schema}ser_descrs sd " .
             "WHERE os.obj_id = '$obj_id' AND " .
                   "os.bib_rec_num = '$bib_rec_num' AND " .
                   "os.series_id = '$series_id' AND " .
                   "os.ser_descr_num = '$ser_descr_num' AND " .
                   "os.series_id = sd.series_id AND " .
                   "os.ser_descr_num = sd.ser_descr_num";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "уникальный идентификатор серии/подсерии - " .
                     $series_id . ", " .
                     "номер описания серии/подсерии - $ser_descr_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "obj_series",
                       key => "obj_id, bib_rec_num, " .
                              "series_id:series_id_begin, " .
                              "ser_descr_num:ser_descr_num_begin",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );

    # комментарий
    $fields{ 'comment' } = $CFG{ OBJ_SERIES_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи
  # для данного объекта" не подлежат редактированию, не будем позволять их
  # редактировать, но совсем убрать их нельзя, т.к. они используются при
  # выполнении операции UPDATE в качестве ключа, а также при выполнении
  # операции INSERT (значения этих полей сохраняются такими же, какими они
  # были при предыдущем запуске скрипта, поэтому можно не присваивать
  # им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";

  # создадим скрытые поля для "Уникального идентификатора серии/подсерии" и
  # "Номера описания серии/подсерии"
  # ПРИМЕЧАНИЕ. Идентификаторы скрытых полей имеют постфикс 'begin' для того,
  #             чтобы можно было обновлять значения этих полей, поскольку они
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужны и исходные значения этих полей, и их новые
  #             значения (которые, возможно, были изменены пользователем).
  print $query->hidden( -name => 'series_id_begin',
                        -default => $fields{ 'series_id' }
                      ), "\n";
  print $query->hidden( -name => 'ser_descr_num_begin',
                        -default => $fields{ 'ser_descr_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();         # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Уникальный идентификатор серии/подсерии</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник ser_descrs.pl
  create_selection_func( func_name => 'select_series',
                         module_name => 'ser_descrs.pl',
                         caller_name => $module,
                         window_name => 'ser_descrs',
                         exp_imp => $query->param( 'exp_imp' ) );
  print "<td>",
        $query->textfield( -name => 'series_id',
                           -default => $fields{ 'series_id' },
                           -size => $CFG{ OBJ_SERIES_SERIES_ID_SIZE },
                           -maxlength => $CFG{ OBJ_SERIES_SERIES_ID_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )"} .
        qq{ onclick="select_series()">Список</a>},
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номер описания серии/подсерии</th>\n";
  print "<td>",
        $query->textfield( -name => 'ser_descr_num',
                           -default => $fields{ 'ser_descr_num' },
                           -size => $CFG{ OBJ_SERIES_SER_DESCR_NUM_SIZE },
                           -maxlength => 
                             $CFG{ OBJ_SERIES_SER_DESCR_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # выведем также описание серии/подсерии
  # (без возможности редактирования)
  print "<tr>\n";
  print "<th>Описание серии/подсерии</th>\n";
  print "<td>", 
        $query->textarea( -name => 'ser_descr',
                          -default => $fields{ 'ser_descr' },
                          -rows => $CFG{ OBJ_SERIES_SER_DESCR_ROWS },
                          -columns => $CFG{ OBJ_SERIES_SER_DESCR_COLS },
                          -readonly => 'readonly'
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номер выпуска в серии/подсерии<br />" .
        "(он может быть не указан, т. к. выпуски могут быть " .
        "ненумерованными)</th>\n";
  print "<td>",
        $query->textfield( -name => 'issue_num',
                           -default => $fields{ 'issue_num' },
                           -size => $CFG{ OBJ_SERIES_ISSUE_NUM_SIZE },
                           -maxlength => $CFG{ OBJ_SERIES_ISSUE_NUM_MAXLENGTH }
                          ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "OBJ_SERIES_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ OBJ_SERIES_COMMENT_ROWS },
                          -columns => $CFG{ OBJ_SERIES_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Номера выпусков в сериях" );
  
  return ( 0 );
}
