#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Организации, учреждения, временные коллективы"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'ORGS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Orgs
my @f_names =
   ( 'org_id',        # Уникальный идентификатор организации
     'org_fname',     # Наименование организации полное
     'org_sname',     # Наименование организации краткое
     'org_city_code', # Местонахождение организации (город)
     'org_addr',      # Адрес организации
     'org_email',     # Электронный адрес организации
     'org_descr',     # Краткое описание организации, доп. сведения
     'org_id_orig',   # Код организациии, являющейся началом цепочки
                      # (для учета разночтений, написаний на различных языках)
     'org_id_parent', # Код вышестоящей организации
     'org_tmp_flag',  # Признак временного коллектива (конференции)
     'lang_code',     # Код языка, на котором приведено наименование
                      # организации (1 - русский язык)
     'process_code',  # Код вида обработки наименования организации
     'comment'        # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# Если этот модуль был вызван в качестве справочника "сам из себя", то
# имя параметра 'caller_name' будет 'ORGS_PARENT' или 'ORGS_ORIG'. Для того
# чтобы процедура create_top_block() работала корректно, назначим условное имя
# модуля таким же, как и имя ВЫЗЫВАЮЩЕГО модуля.
# Этот модуль, в отличие от других модулей-справочников, может быть вызван
# "сам из себя" в двух случаях: для выбора вышестоящей организации
# или организации -- начала цепочки. Поэтому можно было либо модифицировать
# функцию create_selection_func() путем добавления еще одного параметра --
# имени поля в вызывающем модуле, либо модифицировать имя самого вызывающего
# модуля в зависимости от того, для какого поля вызывается справочник. 
# Я выбрал второй вариант. Подмена имени ВЫЗЫВАЮЩЕГО модуля производится 
# при вызове функции create_selection_func() из функции make_form().
# Имя ВЫЗЫВАЮЩЕГО модуля после подмены будет 'ORGS_PARENT' или 'ORGS_ORIG'.
# TODO. Необходимо более основательно оценить это решение с точки зрения
#       безопасности.
if ( $query->param( 'caller_name' ) =~ /^(ORGS_ORIG|ORGS_PARENT)$/ )
{
  $module = $query->param( 'caller_name' );
}

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'org_id' ) eq '' )
  {
    push( @error_messages, "Не указан уникальный идентификатор организации" );
  }

  if ( $query->param( 'org_fname' ) eq '' )
  {
    push( @error_messages, "Не указано полное наименование организации" );
  }

  if ( $query->param( 'org_sname' ) eq '' )
  {
    push( @error_messages, "Не указано краткое наименование организации" );
  }

  if ( $query->param( 'org_city_code' ) eq '' )
  {
    push( @error_messages, "Не указан код города -- местонахождения " .
          "организации" );
  }

  # допускается либо пустое поле, либо только цифры
  if ( $query->param( 'org_id_orig' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный код организациии, являющейся началом " .
          "цепочки" );
  }

  # допускается либо пустое поле, либо только цифры
  if ( $query->param( 'org_id_parent' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный код вышестоящей организациии" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  if ( $query->param( 'process_code' ) eq '' )
  {
    push( @error_messages, "Не указан код вида обработки наименования " .
          "организации" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Организации, учреждения, ' .
                                    'временные коллективы',
                    user => $user,
                    make_letters => 'yes',
                    num_last_recs => $CFG{ ORGS_NUM_LAST_RECS },
                    full_top => 'yes'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $org_id;                # уникальный идентификатор организации
  my $command;               # строка команды для работы с БД

  $org_id = $query->param( 'org_id' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}orgs " .
             "WHERE org_id = '$org_id'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $org_id;                # уникальный идентификатор организации
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $org_id = $query->param( 'org_id' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Orgs WHERE org_id = $org_id";
  push( @table_cmd_pairs, [ 'Orgs', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $org_id;                # уникальный идентификатор организации
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $org_id = $query->param( 'org_id' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Orgs WHERE org_id = $org_id";
  push( @table_cmd_pairs, [ 'Orgs', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех организаций, наименования которых начинаются
# с указанной буквы
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $letter;                     # первая буква наименования организации
  my $letter_lower;               # первая буква -- в нижнем регистре

  # число последних записей, введенных в таблицу, которые нужно выводить
  # при обращении по специальной ссылке
  my $num_last_recs;

  # условие WHERE для команды выборки
  my $where;
  
  # дополнительное условие WHERE для команды выборки
  my $where_add;
  
  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  $letter        = $query->param( 'letter' );
  $num_last_recs = $query->param( 'num_last_recs' );

  # если параметры не переданы, то просто завершаем работу досрочно - 
  # это происходит при первоначальном запуске скрипта
  # ПРИМЕЧАНИЕ. Проверка как на undef, так и на пустое значение '' выполнена
  #             на случай каких-либо проблем в CGI.pm.
  if ( ( $letter eq undef || $letter eq '' ) &&
       ( $num_last_recs eq undef || $num_last_recs eq '' )
     )
  {
    close_bottom();   # завершаем оформление нижней части окна
    print $query->end_html();
    return ( 0 );
  }

  # если пользователь выбрал ссылку для вывода только последних записей,
  # введенных в таблицу
  if ( $num_last_recs )
  {
    # ПРИМЕЧАНИЕ. Если даже требуемое число последних введенных записей будет
    #             больше числа всех записей, имеющихся в таблице, этот 
    #             оператор сработает. Если какие-то записи были удалены,
    #             вследствие чего образовались "дырки" в нумерации org_id,
    #             то просто будет выведено меньшее число записей.
    $where = "WHERE o1.org_id >= ( SELECT max( org_id ) " .
                                  "FROM ${db_schema}orgs ) - " .
                                  "$num_last_recs + 1 ";
  }
  # пользователь выбрал конкретную букву или другой символ
  elsif ( $letter )
  {
    # для русских букв может потребоваться особая предварительная обработка
    $letter = reveal_rus_letter( $letter );

    # воспользуемся возможностями локализации для перевода буквы, выбранной
    # пользователем, в нижний регистр
    $letter_lower = change_case( $letter, 'lower' );

    # если первый символ -- двойные кавычки, то не учитываем его
    $where = "WHERE o1.org_sname ~ '^($letter|\"$letter|" .
                                     "$letter_lower|\"$letter_lower)' ";
  }

  # при вызове этого модуля в качестве справочника из модуля confs.pl
  # будем выбирать только ВРЕМЕННЫЕ коллективы
  $where_add = ( $query->param( 'caller_name' ) eq 'CONFS' ) ? 
               " AND o1.org_tmp_flag = 't' " :
               "";

  # выбираем все записи об организациях, которые удовлетворяют условию $where
  # (т. е. краткие наименования которых начинаются с указанной буквы или
  # которые были введены в таблицу последними)
  # выбираем также краткое наименование языка, на котором записано название
  # организации, и города, где она находится;
  # выбираем также наименования вышестоящей организации и организации,
  # являющейся началом цепочки
  $command = "SELECT o1.*, o2.org_sname AS org_sname_orig, " .
                    "o3.org_sname AS org_sname_parent, " .
                    "l.lang_sname, c.city_sname " .
             "FROM ((( ${db_schema}orgs o1 LEFT JOIN ${db_schema}orgs o2 " .
                  "ON o1.org_id_orig = o2.org_id ) " .
                  "LEFT JOIN ${db_schema}orgs o3 " .
                  "ON o1.org_id_parent = o3.org_id ) " .
                  "LEFT JOIN ${db_schema}langs l " .
                  "ON o1.lang_code = l.lang_code ) " .
                  "LEFT JOIN ${db_schema}cities c " .
                  "ON o1.org_city_code = c.city_code " .
              $where . $where_add .
  # переведем значение поля в нижний регистр для того, чтобы регистр символов
  # не влиял на порядок сортировки (наверное, можно было переводить и 
  # в верхний регистр -- все равно)
  # если первый символ в поле -- кавычки, то не учитываем его, а берем
  # значение поля, начиная со второго символа;
             "ORDER BY CASE WHEN o1.org_sname ~ '^\"' " .
                           "THEN lower( substr( o1.org_sname, 2 ) ) " .
                           "ELSE lower( o1.org_sname ) " .
                      "END";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если это список записей, введенных в таблицу последними, то укажем это
    if ( $num_last_recs )
    {
      print "<h3>Записи, введенные в базу данных последними (" . 
            $num_last_recs . " шт.)</h3>\n";
    }

    # если этот модуль вызван в режиме справочника, и при этом он вызван
    # "сам из себя", то запретим операции просмотра и удаления записей
    # (поскольку режим корректировки выполняется из режима просмотра,
    # то и он будет недоступен)
    if ( $query->param( 'enable_select' ) &&
         $query->param( 'caller_name') eq $module )
    {
      $can_view = 0;
      $can_delete = 0;
    }
    else
    {
      $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                               'user' => $user );
      $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                                 'user' => $user );
    }

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код организации</th>\n";
    print "<th>Наименование краткое</th>\n";
    print "<th>Город</th>\n";
    print "<th>Организация &mdash; начало цепочки (код и наименование)</th>\n";
    print "<th>Вышестоящая организация (код и наименование)</th>\n";
    print "<th>Язык</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретной организации ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Orgs полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'RESP_STAT' )
      {
        # ... полям resp_org_id и org_sname в модуле RESP_STAT
        create_assign_values_func( 'resp_org_id', 'org_sname' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'PUB_PRINTER' )
      {
        # ... полям pub_org_id и org_sname в модуле PUB_PRINTER
        create_assign_values_func( 'pub_org_id', 'org_sname' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'STOR_OBJS' )
      {
        # ... полям owner_org_id и org_sname в модуле STOR_OBJS
        create_assign_values_func( 'owner_org_id', 'org_sname' );
      }
      # модуль orgs.pl вызывается в двух случаях "сам из себя", поэтому
      # приходится модифицировать его имя
      elsif ( $query->param( 'caller_name' ) eq 'ORGS_ORIG' )
      {
        # ... полям org_id_orig и org_sname_orig в модуле ORGS_ORIG
        create_assign_values_func( 'org_id_orig', 'org_sname_orig' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'ORGS_PARENT' )
      {
        # ... полям org_id_parent и org_sname_parent в модуле ORGS_PARENT
        create_assign_values_func( 'org_id_parent', 'org_sname_parent' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'CONFS' )
      {
        # ... полям conf_code и conf_fname в модуле CONFS
        create_assign_values_func( 'conf_code', 'conf_fname' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'ORGS_ACT' )
      {
        # ... полям org_id и org_fname в модуле ORGS_ACT
        create_assign_values_func( 'org_id', 'org_fname' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код организации, краткое наименование, город, код 
      # и наименование организации -- начало цепочки, код и наименование 
      # вышестоящей организации, язык, на котором написано наименование
      # (добавим символ &nbsp; к коду организации, являющейся началом цепочки,
      # и к коду вышестоящей организации из соображений эстетики: если значение
      # поля 'org_id_orig' или 'org_id_parent' равно null, то не отображаются
      # горизонтальные строки-разделители)
      print "<td class=\"center\">$row[ $result->fnumber( 'org_id' ) ]</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретной организации, то значение поля
      # org_sname выведем в виде ссылки

      # экранируем кавычки и другие "неудобные" символы
      # (см. комментарий в модуле obj_groups_descrs.pl)
      # выполним добавочную обработку значения переменной, т. к. того, что
      # делает функция escapeHTML(), недостаточно
      my $org_sname = $row[ $result->fnumber( 'org_sname' ) ];
      $org_sname =~ s/\'/\\\'/g;
      $org_sname = CGI::escapeHTML( $org_sname );

      if ( $query->param( 'enable_select' ) )
      {
        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'org_id' ) ],
                                  '$org_sname' )">
                   $row[ $result->fnumber( 'org_sname' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'org_sname' ) ]</td>\n";
      }

      print "<td>$row[ $result->fnumber( 'city_sname' ) ]</td>\n";

      # если организация, являющаяся началом цепочки, отсутствует, то выведем
      # лишь символ пробела "&nbsp;"
      print "<td>",
            ( $row[ $result->fnumber( 'org_id_orig' ) ] ne undef ) ?
              "(" . $row[ $result->fnumber( 'org_id_orig' ) ] . ") " .
              $row[ $result->fnumber( 'org_sname_orig' ) ] :
            "&nbsp;",
            "</td>\n";

      # если вышестоящая организация, отсутствует, то выведем
      # лишь символ пробела "&nbsp;"
      print "<td>",
            ( $row[ $result->fnumber( 'org_id_parent' ) ] ne undef ) ?
              "(" . $row[ $result->fnumber( 'org_id_parent' ) ] . ") " .
              $row[ $result->fnumber( 'org_sname_parent' ) ] :
            "&nbsp;",
            "</td>\n";

      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "org_id=" . $row[ $result->fnumber( 'org_id' ) ] .
              $add_params;

      # просмотр описания организации в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи об организации 
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $org_id;                # уникальный идентификатор организации
  my $command;               # строка команды для работы с БД

  $org_id = $query->param( 'org_id' );

  # выбираем одну запись об организации
  # выбираем также наименования вышестоящей организации и организации, являющейся
  # началом цепочки
  $command = "SELECT o1.*, o2.org_sname AS org_sname_orig, " .
                    "o3.org_sname AS org_sname_parent " .
             "FROM ( ${db_schema}orgs o1 LEFT JOIN ${db_schema}orgs o2 " .
                  "ON o1.org_id_orig = o2.org_id ) " .
                  "LEFT JOIN ${db_schema}orgs o3 " .
                  "ON o1.org_id_parent = o3.org_id " .
             "WHERE o1.org_id = '$org_id'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "организации " . $org_id,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
          );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "orgs",
                       key => "org_id",
                       f_names => \@f_names,
                       f_special => [ 'org_tmp_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны наименования организаций)
  my( $ref_values_langs, $ref_labels_langs ) = popup_langs_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов языковой обработки наименований организаций)
  my( $ref_values_procs, $ref_labels_procs ) =
    popup_process_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'org_id' } = get_next_key_value( 'orgs', 'org_id' );

    # значения по умолчанию

    # наименование организации полное
    $fields{ 'org_fname' }     = $CFG{ ORGS_ORG_FNAME };

    # наименование организации краткое
    $fields{ 'org_sname' }     = $CFG{ ORGS_ORG_SNAME };

    # местонахождение организации (город)
    $fields{ 'org_city_code' } = $CFG{ ORGS_ORG_CITY_CODE };

    # адрес организации
    $fields{ 'org_addr' }      = $CFG{ ORGS_ORG_ADDR };

    # электронный адрес организации
    $fields{ 'org_email' }     = $CFG{ ORGS_ORG_EMAIL };

    # краткое описание организации, доп. сведения
    $fields{ 'org_descr' }     = $CFG{ ORGS_ORG_DESCR };

    # код организациии, являющейся началом цепочки
    # (для учета разночтений, написаний на различных языках)
    $fields{ 'org_id_orig' }   = $CFG{ ORGS_ORG_ID_ORIG };

    # код вышестоящей организации
    $fields{ 'org_id_parent' } = $CFG{ ORGS_ORG_ID_PARENT };

    # признак временного коллектива (конференции)
    $fields{ 'org_tmp_flag' }  = $CFG{ ORGS_ORG_TMP_FLAG };

    # код языка, на котором приведено наименование организации 
    $fields{ 'lang_code' }     = $CFG{ ORGS_LANG_CODE };

    # код вида обработки наименования организации
    $fields{ 'process_code' }  = $CFG{ ORGS_PROCESS_CODE };

    # комментарий
    $fields{ 'comment' }       = $CFG{ ORGS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор организации" формируется
  # автоматически с использованием функции get_next_key_value(), не будем
  # позволять его редактировать, но совсем убрать его нельзя, т.к. оно
  # используется при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'org_id',
                        -default => $fields{ 'org_id' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код организации</th>\n";
  print "<td>", $fields{ 'org_id' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Полное наименование</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].org_fname", "ORGS_ORG_FNAME" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'org_fname',
                          -default => $fields{ 'org_fname' },
                          -rows => $CFG{ ORGS_ORG_FNAME_ROWS },
                          -columns => $CFG{ ORGS_ORG_FNAME_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Краткое наименование</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].org_sname", "ORGS_ORG_SNAME" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'org_sname',
                           -default => $fields{ 'org_sname' },
                           -size => $CFG{ ORGS_ORG_SNAME_SIZE },
                           -maxlength => $CFG{ ORGS_ORG_SNAME_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_cities_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<th>Город</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник cities.pl
  create_selection_func( func_name => 'select_city',
                         module_name => 'cities.pl',
                         caller_name => $module,
                         window_name => 'cities',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также краткое наименование города и страны
  # (без возможности редактирования)
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Название города";
  # название страны просто обнуляем (наша функция create_hot_help() не 
  # позволяет предложить более интеллектуальную обработку для этого поля)
  create_hot_help( "document.forms[ 0 ].org_city_code",
                   "ORGS_ORG_CITY_CODE", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].org_city_sname.value = str;
                   document.forms[ 0 ].org_country_sname_city.value = ''; " )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'org_city_code',
                           -default => $fields{ 'org_city_code' },
                           -size => $CFG{ ORGS_ORG_CITY_CODE_SIZE },
                           -maxlength => $CFG{ ORGS_ORG_CITY_CODE_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'org_city_sname',
                           -default => $fields{ 'org_city_sname' },
                           -size => $CFG{ ORGS_ORG_CITY_SNAME_SIZE },
                           -maxlength => $CFG{ ORGS_ORG_CITY_SNAME_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
       "&nbsp;&nbsp;",
        $query->textfield( -name => 'org_country_sname_city',
                           -default => $fields{ 'org_country_sname_city' },
                           -size => $CFG{ ORGS_ORG_COUNTRY_SNAME_CITY_SIZE },
                           -maxlength =>
                             $CFG{ ORGS_ORG_COUNTRY_SNAME_CITY_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" onclick=} .
        qq{"select_city()">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Адрес организации</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].org_addr", "ORGS_ORG_ADDR" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'org_addr',
                          -default => $fields{ 'org_addr' },
                          -rows => $CFG{ ORGS_ORG_ADDR_ROWS },
                          -columns => $CFG{ ORGS_ORG_ADDR_COLS }
                        ),  
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Электронный адрес организации</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].org_email", "ORGS_ORG_EMAIL" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'org_email',
                          -default => $fields{ 'org_email' },
                          -rows => $CFG{ ORGS_ORG_EMAIL_ROWS },
                          -columns => $CFG{ ORGS_ORG_EMAIL_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Краткое описание организации, допополнительные сведения</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].org_descr", "ORGS_ORG_DESCR" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'org_descr',
                          -default => $fields{ 'org_descr' },
                          -rows => $CFG{ ORGS_ORG_DESCR_ROWS },
                          -columns => $CFG{ ORGS_ORG_DESCR_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Организация &mdash; начало цепочки (код и наименование)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник orgs.pl
  # модифицируем наименование модуля, т.к. модуль orgs.pl вызывается в качестве
  # справочника дважды "сам из себя" для заполнения разных полей
  create_selection_func( func_name => 'select_orig_org',
                         module_name => 'orgs.pl',
                         caller_name => $module . "_ORIG",
                         window_name => 'orig_orgs',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также краткое наименование организации - начала цепочки
  # (без возможности редактирования)
  # TODO. Возможно, имеет смысл сделать блокировку поля "Код вышестоящей
  #       организации", если в поле "Код организации -- начало цепочки"
  #       введено непустое значение. Ограничение CHECK на таблицу Orgs
  #       запрещает делать ссылку на вышестоящую организацию из записи,
  #       не являющейся началом цепочки эквивалентных записей. В настоящее
  #       время в случае нарушения пользователем этого требования на экран
  #       будет выведено системное сообщение, сформированное СУБД, что может
  #       смутить (confuse) пользователя.
  print "<td>",
        $query->textfield( -name => 'org_id_orig',
                           -default => $fields{ 'org_id_orig' },
                           -size => $CFG{ ORGS_ORG_ID_ORIG_SIZE },
                           -maxlength => $CFG{ ORGS_ORG_ID_ORIG_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'org_sname_orig',
                           -default => $fields{ 'org_sname_orig' },
                           -size => $CFG{ ORGS_ORG_SNAME_ORIG_SIZE },
                           -maxlength => $CFG{ ORGS_ORG_SNAME_ORIG_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" onclick=} .
        qq{"select_orig_org()">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Вышестоящая организация (код и наименование)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник orgs.pl
  # модифицируем наименование модуля, т.к. модуль orgs.pl вызывается в качестве
  # справочника дважды "сам из себя" для заполнения разных полей
  create_selection_func( func_name => 'select_parent_org',
                         module_name => 'orgs.pl',
                         caller_name => $module . "_PARENT",
                         window_name => 'parent_orgs',
                         exp_imp => $query->param( 'exp_imp' ) );

  print "<td>",
        $query->textfield( -name => 'org_id_parent',
                           -default => $fields{ 'org_id_parent' },
                           -size => $CFG{ ORGS_ORG_ID_PARENT_SIZE },
                           -maxlength => $CFG{ ORGS_ORG_ID_PARENT_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'org_sname_parent',
                           -default => $fields{ 'org_sname_parent' },
                           -size => $CFG{ ORGS_ORG_SNAME_PARENT_SIZE },
                           -maxlength =>
                             $CFG{ ORGS_ORG_SNAME_PARENT_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" onclick=} .
        qq{"select_parent_org()">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак временного коллектива (конференции)</th>\n";
  print "<td>",
        $query->checkbox( -name => 'org_tmp_flag',
                          -checked => 
                             $fields{ 'org_tmp_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  
  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<th>Язык</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_process_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<th>Вид языковой обработки</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'process_code',
                            -default => $fields{ 'process_code' },
                            -values => $ref_values_procs,
                            -labels => { @{ $ref_labels_procs } }
                          ),
        "</td>\n";
  print "</tr>\n";
  
  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "ORGS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ ORGS_COMMENT_ROWS },
                          -columns => $CFG{ ORGS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Признак временного коллектива (конференции)
  if ( $fname eq 'org_tmp_flag' )
  {
    # это поле имеет тип bool, поэтому проверяем значение параметра
    # только на непустоту
    $value = $query->param( $fname ) ? 'yes' : 'no';
  }

  return ( $value );  
}
