#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Персоналии"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'PERSONS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Persons
my @f_names =
   ( 'pers_id',         # Уникальный идентификатор персоналии
     'pers_name',       # Полное имя, отчество, фамилия
                        # ПРИМЕЧАНИЕ. В поле записывать первой
                        #             фамилию и ставить запятую.
     'pers_life_years', # Годы жизни
     'pers_add_info',   # Дополнительные сведения (младший/старший,
                        # отец/сын, ... род занятий, ...)
     'pers_id_orig',    # Код персоналии, являющейся началом цепочки
                        # (для учета разночтений, написаний на различных
                        # языках)
     'lang_code',       # Код языка, на котором приведено написание
                        # Ф. И. О. персоналии (1 - русский язык)
     'process_code',    # Код вида обработки имени персоналии
                        # (1 - оригинальное, т.е. в том виде, как
                        # приведено в источнике библиограф. описания)
     'pers_addr',       # Адрес
     'pers_email',      # Электронный адрес
     'comment'          # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

# выбираем описания всех объектов, в области сведений
# об ответственности которых фигурирует конкретная персоналия
if ( $query->path_info =~ /^\/list_bib_recs$/ )
{
  do_list_bib_recs();
}

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'pers_id' ) eq '' )
  {
    push( @error_messages, "Не указан уникальный идентификатор персоналии" );
  }

  if ( $query->param( 'pers_name' ) eq '' )
  {
    push( @error_messages, "Не указаны полное имя, отчество, фамилия " .
          "персоналии" );
  }

  # допускается либо пустое поле, либо только цифры
  if ( $query->param( 'pers_id_orig' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный код персоналиии, являющийся началом " .
          "цепочки" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  if ( $query->param( 'process_code' ) eq '' )
  {
    push( @error_messages, "Не указан код вида обработки имени персоналии" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Персоналии',
                    user => $user,
                    make_letters => 'yes',
                    num_last_recs => $CFG{ PERSONS_NUM_LAST_RECS },
                    full_top => 'yes'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $pers_id;               # уникальный идентификатор персоналии
  my $command;               # строка команды для работы с БД

  $pers_id = $query->param( 'pers_id' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}persons " .
             "WHERE pers_id = '$pers_id'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $pers_id;               # уникальный идентификатор персоналии
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $pers_id = $query->param( 'pers_id' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Persons WHERE pers_id = $pers_id";
  push( @table_cmd_pairs, [ 'Persons', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $pers_id;               # уникальный идентификатор персоналии
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $pers_id = $query->param( 'pers_id' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Persons WHERE pers_id = $pers_id";
  push( @table_cmd_pairs, [ 'Persons', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# выборка и вывод биб. описаний всех объектов, в области сведений
# об ответственности которых фигурирует конкретная персоналия
# -----------------------------------------------------------------------------
sub do_list_bib_recs
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $pers_id;                    # уникальный идентификатор персоналии,
                                  # выбранной пользователем
  my @persons = ();               # массив идентификаторов персоналий,
                                  # которые по "цепочке" связаны с персоналией,
                                  # выбранной пользователем
  my $headline;                   # заголовок списка биб. описаний
  my $personal_info;              # информация о персоналии, выводимая на экран
  
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  $pers_id = $query->param( 'pers_id' );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выбираем записи из таблицы Persons для четырех типов персоналий:
  # -- та персоналия, которую "кликнул" пользователь;
  # -- та персоналия, которая является для выбранной пользователем
  # персоналии началом цепочки (на основе поля pers_id_orig);
  # -- персоналии, у которых выбранная пользователем персоналия является
  # началом цепочки (на основе поля pers_id_orig);
  # -- персоналии, у которых началом цепочки является та же персоналия,
  # что и у той, которую выбрал пользователь
  # ПРИМЕЧАНИЕ. Можно было бы передавать этой процедуре в качестве параметров
  #             еще имя персоналии и идентификатор персоналии-родоначальницы,
  #             изменив соответственно текст SQL-команды. SQL-команда немного
  #             упростилась бы. Но тогда пришлось бы при формировании
  #             заголовка списка биб. описаний и массива идентификаторов
  #             персоналий не только обрабатывать в цикле (по единой схеме)
  #             результаты выборки из БД, но также каким-то образом включать
  #             в обработку и данные, полученные в качестве параметров скрипта,
  #             т. е. имя персоналии и ее идентификатор. В результате дело
  #             усложнилось бы, т. к. мы выполняем сортировку имен персоналий
  #             средствами СУБД (ORDER BY), а пришлось бы вставлять в 
  #             отсортированный список имя той персоналии, которую выбрал 
  #             пользователь и которую мы и передали в качестве параметров.
  $command = "SELECT p1.pers_id, p1.pers_name " .
             "FROM ${db_schema}persons p1 " .
             "WHERE p1.pers_id = $pers_id OR " .
                   "p1.pers_id = ( SELECT p2.pers_id_orig " .
                                  "FROM ${db_schema}persons p2 " .
                                  "WHERE p2.pers_id = $pers_id ) OR " .
                   "p1.pers_id_orig = $pers_id OR " .
                   "p1.pers_id_orig = ( SELECT p3.pers_id_orig " .
                                       "FROM ${db_schema}persons p3 " .
                                       "WHERE p3.pers_id = $pers_id ) " .
             "ORDER BY p1.pers_name";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # формируем персональную информацию и массив идентификаторов персоналий
    while ( @row = $result->fetchrow )
    {
      $personal_info .= $row[ $result->fnumber( 'pers_name' ) ] . "; ";
      push( @persons, $row[ $result->fnumber( 'pers_id' ) ] );
    }

    $personal_info =~ s/; $//;   # уберем последнюю точку с запятой: она лишняя
  }
  else
  {
    ;
    # TODO. Надо что-то делать или нет?
  }
  # ------------------------------------------------------------------------

  # выбираем наименования ролей, которые "играла" указанная персоналия
  # во всех объектах биб. описания, описанных в БД
  # (с учетом всех разночтений имени персоналии, т. е. на основе списка 
  # в массиве @persons)
  # используем DISTINCT, т. к. роли могут повторяться для разных объектов
  $command = "SELECT DISTINCT r.role_fname " .
             "FROM ${db_schema}roles r, ${db_schema}resp_stat rs " .
             "WHERE rs.resp_pers_id IN (" . join( ",", @persons ) . ") AND " .
                   "rs.role_code = r.role_code " .
             "ORDER BY 1";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # продолжаем формирование персональной информации
    $personal_info .= "<br />\nРоли: ";
    while ( @row = $result->fetchrow )
    {
      $personal_info .= $row[ $result->fnumber( 'role_fname' ) ] . ", ";
    }

    $personal_info =~ s/, $//;   # уберем последнюю запятую: она лишняя
  }
  else
  {
    ;
    # TODO. Надо что-то делать или нет?
  }
  # ------------------------------------------------------------------------

  # выведем персональную информацию в качестве заголовка
  print "<p class=\"bib_rec\">$personal_info</p>\n";

  # сформируем заголовок модуля
  $headline = "<h2 class=\"module_title\">" .
              "Библиографические описания, связанные с данной персоналией" .
              "</h2>\n";

  # выбираем теперь записи с биб. описаниями объектов, в судьбе которых
  # принимала участие персоналия, выбранная пользователем (используем
  # сформированный список всех разночтений имени персоналии)
  # используем DISTINCT, т. к. одна персоналия может "играть" более одной
  # роли в судьбе конкретного объекта (например, быть автором предисловия
  # и переводчиком)
  $command = "SELECT DISTINCT b.obj_id, b.bib_rec_num, " .
                    "CASE WHEN b.bib_rec_short IS NOT NULL " .
                         "THEN b.bib_rec_short " .
                         "ELSE b.bib_rec_full " .
                    "END AS bib_rec " .
             "FROM ${db_schema}bib_recs b, ${db_schema}resp_stat r " .
             "WHERE b.obj_id = r.obj_id AND " .
                   "r.resp_pers_id IN (" . join( ",", @persons ) . ") " .
             "ORDER BY bib_rec";

  # вывод биб. описаний всех объектов, выбранных по запросу
  list_bib_recs_by_query( $query, $command, $headline );

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех персоналий, которые начинаются с указанной буквы
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $letter;                     # первая буква персоналии
  my $letter_lower;               # первая буква -- в нижнем регистре

  # число последних записей, введенных в таблицу, которые нужно выводить
  # при обращении по специальной ссылке
  my $num_last_recs;

  # условие WHERE для команды выборки
  my $where;
  
  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки
  
  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  $letter        = $query->param( 'letter' );
  $num_last_recs = $query->param( 'num_last_recs' );

  # если параметры не переданы, то просто завершаем работу досрочно - 
  # это происходит при первоначальном запуске скрипта
  # ПРИМЕЧАНИЕ. Проверка как на undef, так и на пустое значение '' выполнена
  #             на случай каких-либо проблем в CGI.pm.
  if ( ( $letter eq undef || $letter eq '' ) &&
       ( $num_last_recs eq undef || $num_last_recs eq '' )
     )
  {
    close_bottom();   # завершаем оформление нижней части окна
    print $query->end_html();
    return ( 0 );
  }

  # если пользователь выбрал ссылку для вывода только последних записей,
  # введенных в таблицу
  if ( $num_last_recs )
  {
    # ПРИМЕЧАНИЕ. Если даже требуемое число последних введенных записей будет
    #             больше числа всех записей, имеющихся в таблице, этот 
    #             оператор сработает. Если какие-то записи были удалены,
    #             вследствие чего образовались "дырки" в нумерации pers_id,
    #             то просто будет выведено меньшее число записей.
    $where = "WHERE p1.pers_id >= ( SELECT max( pers_id ) " .
                                   "FROM ${db_schema}persons ) - " .
                                   "$num_last_recs + 1 ";
  }
  # пользователь выбрал конкретную букву или другой символ
  elsif ( $letter )
  {
    # для русских букв может потребоваться особая предварительная обработка
    $letter = reveal_rus_letter( $letter );

    # воспользуемся возможностями локализации для перевода буквы, выбранной
    # пользователем, в нижний регистр
    $letter_lower = change_case( $letter, 'lower' );

    $where = "WHERE substr( p1.pers_name, 1, 1 ) IN " .
                   "( '$letter', '$letter_lower' ) ";
  }

  # выбираем все записи о персоналиях, которые удовлетворяют условию $where
  # (т. е. начинаются с указанной буквы или были введены в таблицу последними)
  # (а также краткое наименование языка, на котором записано имя персоналии)
  # выберем также Ф.И.О. для персоналии, являющейся началом цепочки
  $command = "SELECT p1.*, p2.pers_name AS pers_name_orig, l.lang_sname " .
             "FROM ( ${db_schema}persons p1 " .
                    "LEFT JOIN ${db_schema}persons p2 " .
                    "ON p1.pers_id_orig = p2.pers_id " .
                  ") JOIN ${db_schema}langs l " .
                  "ON p1.lang_code = l.lang_code " .
             $where .
             "ORDER BY p1.pers_name";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если это список записей, введенных в таблицу последними, то укажем это
    if ( $num_last_recs )
    {
      print "<h3>Записи, введенные в базу данных последними (" . 
            $num_last_recs . " шт.)</h3>\n";
    }

    # если этот модуль вызван в режиме справочника, и при этом он вызван
    # "сам из себя", то запретим операции просмотра и удаления записей
    # (поскольку режим корректировки выполняется из режима просмотра,
    # то и он будет недоступен)
    if ( $query->param( 'enable_select' ) &&
         $query->param( 'caller_name') eq $module )
    {
      $can_view = 0;
      $can_delete = 0;
    }
    else
    {
      $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                               'user' => $user );
      $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                                 'user' => $user );
    }

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код персоналии</th>\n";
    print "<th>Ф. И. О. персоналии</th>\n";
    print "<th>Персоналия &mdash; начало цепочки (код и Ф. И. О.)</th>\n";
    print "<th>Язык</th>\n";
    print "<th>&nbsp;</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретной персоналии ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Persons полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'RESP_STAT' )
      {
        # ... полям resp_pers_id и pers_name в модуле RESP_STAT
        create_assign_values_func( 'resp_pers_id', 'pers_name' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'PUB_PRINTER' )
      {
        # ... полям pub_pers_id и pers_name в модуле PUB_PRINTER
        create_assign_values_func( 'pub_pers_id', 'pers_name' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'STOR_OBJS' )
      {
        # ... полям owner_pers_id и pers_name в модуле STOR_OBJS
        create_assign_values_func( 'owner_pers_id', 'pers_name' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'PERSONS' )
      {
        # ... полям pers_id_orig и pers_name_orig в модуле PERSONS
        create_assign_values_func( 'pers_id_orig', 'pers_name_orig' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код персоналии, Ф. И. О., код и Ф. И. О. персоналии -- начало 
      # цепочки, язык, на котором написано Ф. И. О. персоналии
      # (добавим символ &nbsp; к коду персоналии, являющемуся началом цепочки,
      # из соображений эстетики: если значение поля 'pers_id_orig' равно null,
      # то не отображаются горизонтальные строки-разделители)
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'pers_id' ) ]</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретной персоналии, то значение поля
      # pers_name выведем в виде ссылки

      # экранируем кавычки и другие "неудобные" символы
      # (см. комментарий в модуле obj_groups_descrs.pl)
      # выполним добавочную обработку значения переменной, т. к. того, что
      # делает функция escapeHTML(), недостаточно
      my $pers_name = $row[ $result->fnumber( 'pers_name' ) ];
      $pers_name =~ s/\'/\\\'/g;
      $pers_name = CGI::escapeHTML( $pers_name );

      if ( $query->param( 'enable_select' ) )
      {
        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'pers_id' ) ],
                                  '$pers_name' )">
                   $row[ $result->fnumber( 'pers_name' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'pers_name' ) ]</td>\n";
      }

      # если персоналия, являющаяся началом цепочки, отсутствует, то выведем
      # лишь символ пробела "&nbsp;"
      print "<td>",
            ( $row[ $result->fnumber( 'pers_id_orig' ) ] ne undef ) ?
              "(" . $row[ $result->fnumber( 'pers_id_orig' ) ] . ") " .
              $row[ $result->fnumber( 'pers_name_orig' ) ] :
            "&nbsp;",
            "</td>\n";
      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "pers_id=" . $row[ $result->fnumber( 'pers_id' ) ] .
              $add_params;

      # просмотр биб. описаний тех объектов, в области сведений
      # об ответственности которых фигурирует эта персоналия
      print "<td>",
            qq{<a class="command_link" } .
            qq{href="$script_name/list_bib_recs?$href"} .
            qq{ target="_blank">Биб. описания</a>},
            "</td>\n";

      # просмотр описания персоналии в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи о персоналии 
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $pers_id;               # уникальный идентификатор персоналии
  my $command;               # строка команды для работы с БД

  $pers_id = $query->param( 'pers_id' );

  # выбираем одну запись о персоналии
  # выберем также Ф.И.О. для персоналии, являющейся началом цепочки
  $command = "SELECT p1.*, p2.pers_name AS pers_name_orig " .
             "FROM ${db_schema}persons p1 LEFT JOIN ${db_schema}persons p2 " .
                  "ON p1.pers_id_orig = p2.pers_id " .
             "WHERE p1.pers_id = '$pers_id'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "персоналии " . $pers_id,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "persons",
                       key => "pers_id",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны наименования персоналий)
  my( $ref_values_langs, $ref_labels_langs ) =
    popup_langs_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов языковой обработки наименований персоналий)
  my( $ref_values_procs, $ref_labels_procs ) =
    popup_process_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'pers_id' } = get_next_key_value( 'persons', 'pers_id' );

    # значения по умолчанию

    # полное имя, отчество, фамилия
    # ПРИМЕЧАНИЕ. В поле записывать первой фамилию и ставить запятую
    $fields{ 'pers_name' }       = $CFG{ PERSONS_PERS_NAME };

    # годы жизни
    $fields{ 'pers_life_years' } = $CFG{ PERSONS_PERS_LIFE_YEARS };

    # дополнительные сведения (младший/старший, отец/сын, ... род занятий, ...)
    $fields{ 'pers_add_info' }   = $CFG{ PERSONS_PERS_ADD_INFO };

    # код персоналии, являющейся началом цепочки (для учета разночтений,
    # написаний на различных языках)
    $fields{ 'pers_id_orig' }    = $CFG{ PERSONS_PERS_ID_ORIG };

    # код языка, на котором приведено написание Ф. И. О. персоналии
    # (1 - русский язык)
    $fields{ 'lang_code' }       = $CFG{ PERSONS_LANG_CODE };

    # код вида обработки имени персоналии
    # (1 - оригинальное, т.е. в том виде, как приведено в источнике
    # библиограф. описания)
    $fields{ 'process_code' }    = $CFG{ PERSONS_PROCESS_CODE };

    # адрес
    $fields{ 'pers_addr' }       = $CFG{ PERSONS_PERS_ADDR };

    # электронный адрес
    $fields{ 'pers_email' }      = $CFG{ PERSONS_PERS_EMAIL };

    # комментарий
    $fields{ 'comment' }         = $CFG{ PERSONS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор персоналии" формируется
  # автоматически с использованием функции get_next_key_value(), не будем
  # позволять его редактировать, но совсем убрать его нельзя, т.к. оно
  # используется при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'pers_id',
                        -default => $fields{ 'pers_id' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код персоналии</th>\n";
  print "<td>", $fields{ 'pers_id' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Полные фамилия, имя, отчество<br />" .
            "(после фамилии ставить запятую)</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pers_name",
                   "PERSONS_PERS_NAME" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'pers_name',
                           -default => $fields{ 'pers_name' },
                           -size => $CFG{ PERSONS_PERS_NAME_SIZE },
                           -maxlength => $CFG{ PERSONS_PERS_NAME_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Годы жизни</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pers_life_years",
                   "PERSONS_PERS_LIFE_YEARS" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'pers_life_years',
                           -default => $fields{ 'pers_life_years' },
                           -size => $CFG{ PERSONS_PERS_LIFE_YEARS_SIZE },
                           -maxlength =>
                             $CFG{ PERSONS_PERS_LIFE_YEARS_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дополнительные сведения<br />" .
            "(младший/старший, отец/сын, род занятий, ...)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pers_add_info",
                   "PERSONS_PERS_ADD_INFO" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'pers_add_info',
                          -default => $fields{ 'pers_add_info' },
                          -rows => $CFG{ PERSONS_PERS_ADD_INFO_ROWS },
                          -columns => $CFG{ PERSONS_PERS_ADD_INFO_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Персоналия &mdash; начало цепочки (код и Ф. И. О.)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник persons.pl
  create_selection_func( func_name => 'select_orig_person',
                         module_name => 'persons.pl',
                         caller_name => $module,
                         window_name => 'orig_persons',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также имя персоналии (без возможности редактирования)
  print "<td>",
        $query->textfield( -name => 'pers_id_orig',
                           -default => $fields{ 'pers_id_orig' },
                           -size => $CFG{ PERSONS_PERS_ID_ORIG_SIZE },
                           -maxlength => $CFG{ PERSONS_PERS_ID_ORIG_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'pers_name_orig',
                           -default => $fields{ 'pers_name_orig' },
                           -size => $CFG{ PERSONS_PERS_NAME_ORIG_SIZE },
                           -maxlength =>
                             $CFG{ PERSONS_PERS_NAME_ORIG_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" onclick=} .
        qq{"select_orig_person()">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  
  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<th>Язык</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_process_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<th>Вид языковой обработки</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'process_code',
                            -default => $fields{ 'process_code' },
                            -values => $ref_values_procs,
                            -labels => { @{ $ref_labels_procs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Адрес персоналии</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pers_addr",
                   "PERSONS_PERS_ADDR" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'pers_addr',
                          -default => $fields{ 'pers_addr' },
                          -rows => $CFG{ PERSONS_PERS_ADDR_ROWS },
                          -columns => $CFG{ PERSONS_PERS_ADDR_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Электронный адрес</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pers_email",
                   "PERSONS_PERS_EMAIL" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'pers_email',
                          -default => $fields{ 'pers_email' },
                          -rows => $CFG{ PERSONS_PERS_EMAIL_ROWS },
                          -columns => $CFG{ PERSONS_PERS_EMAIL_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment",
                   "PERSONS_COMMENT" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ PERSONS_COMMENT_ROWS },
                          -columns => $CFG{ PERSONS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}
