#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Дата издания, печатания, изготовления"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'PUB_PRINT_DATE';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Pub_print_date
my @f_names =
   ( 'obj_id',           # Уникальный идентификатор объекта
                         # библиографического описания
     'pub_date_begin',   # Дата начала издания
                         # ПРИМЕЧАНИЕ. Для дат выбран тип данных 
                         #             text, т.к. возможны даты вида
                         #             [1988], 198?, 197-, ...
     'dash_flag',        # Флаг наличия тире, которое ставится между
                         # датой начала и окончания издания
     'pub_date_end',     # Дата окончания издания
     'print_date',       # Дата печатания, изготовления
     'sine_anno_flag',   # Признак "без года (sine anno)"
     'pub_print_date_sure_flag', # Флаг точно установленной даты
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'pub_date_begin' ) eq '' &&
       $query->param( 'pub_date_end' ) ne '' )
  {
    push( @error_messages, "Указана дата окончания издания, а дата начала " .
          "не указана" );
  }

  # попытаемся сделать проверку в соответствии с ограничением CHECK,
  # наложенным на таблицу Pub_print_date
  if ( ! ( ( $query->param( 'sine_anno_flag' ) ne '' &&
             $query->param( 'pub_date_begin' ) eq '' &&
             $query->param( 'dash_flag' ) eq '' &&
             $query->param( 'pub_date_end' ) eq '' &&
             $query->param( 'print_date' ) eq ''
           ) ||
           ( $query->param( 'sine_anno_flag' ) eq '' &&
             ( $query->param( 'pub_date_begin' ) ne '' ||
               $query->param( 'pub_date_end' ) ne '' ||
               $query->param( 'print_date' ) ne ''
             )  
           )
         )
     )
  {
    push( @error_messages, "Неверная комбинация значений полей" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id     = $query->param( 'obj_id' );
  $target_win = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}pub_print_date " .
             "WHERE obj_id = '$obj_id'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех дат издания для данного издания
# ПРИМЕЧАНИЕ. В действительности выводится только одна запись (см. первичный
#             ключ таблицы Pub_print_date), но для упрощения кодирования
#             будем использовать типовую схему.
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id     = $query->param( 'obj_id' );
  $bib_rec    = $query->param( 'bib_rec' );
  $target_win = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки (поле bib_rec_num не требуется)
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о датах издания для данного объекта
  # (фактически это будет не больше одной записи - см. табл. Pub_print_date)
  $command = "SELECT * " .
             "FROM ${db_schema}pub_print_date " .
             "WHERE obj_id = $obj_id";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Дата начала издания</th>\n";
    print "<th>Тире-разделитель</th>\n";
    print "<th>Дата окончания издания</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим даты издания
      # (добавим символ &nbsp; к датам из соображений эстетики: если значение
      # полей 'pub_date_begin' и 'pub_date_end' равны null, то не отображаются
      # горизонтальные строки-разделители)
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'pub_date_begin' ) ]&nbsp;</td>\n";
      print "<td class=\"center\">",
            $row[ $result->fnumber( 'dash_flag' ) ] eq 't' ? "&mdash;" :
            "&nbsp;",
            "</td>\n";
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'pub_date_end' ) ]&nbsp;</td>\n";

      # просмотр данной записи о дате издания в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href"} .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о дате издания 
      print qq{<a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }
  else     # нет ни одной записи, а допускается всего ОДНА (см. PRIMARY KEY)
  {
    # сформируем ссылку для добавления записи о дате издания
    # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
    if ( ! $query->param( 'exp_imp' ) &&
         have_access( 'module' => $module, 'procedure' => 'ADD', 
                      'user' => $user ) )
    {
      print "<br />\n";
      print qq{<a class="command_link" href="$script_name/add?$href"} .
            qq{ target="$target_win">Добавить запись</a>};
    }
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id     = $query->param( 'obj_id' );
  $target_win = $query->param( 'target_win' );

  # выбираем одну запись о дате издания
  $command = "SELECT * " .
             "FROM ${db_schema}pub_print_date " .
             "WHERE obj_id = $obj_id";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "pub_print_date",
                       key => "obj_id",
                       f_names => \@f_names,
                       f_special => [ 'dash_flag', 'sine_anno_flag',
                                      'pub_print_date_sure_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения ключевого поля из параметров скрипта
    $fields{ 'obj_id' } = $query->param( 'obj_id' );

    # значения по умолчанию

    # дата начала издания
    $fields{ 'pub_date_begin' } = $CFG{ PUB_PRINT_DATE_PUB_DATE_BEGIN };

    # флаг наличия тире, которое ставится между
    # датой начала и окончания издания
    $fields{ 'dash_flag' }      = $CFG{ PUB_PRINT_DATE_DASH_FLAG };

    # дата окончания издания
    $fields{ 'pub_date_end' }   = $CFG{ PUB_PRINT_DATE_PUB_DATE_END };

    # дата печатания, изготовления
    $fields{ 'print_date' }     = $CFG{ PUB_PRINT_DATE_PRINT_DATE };
    
    # признак "без года (sine anno)"
    $fields{ 'sine_anno_flag' } = $CFG{ PUB_PRINT_DATE_SINE_ANNO_FLAG };

    # флаг точно установленной даты
    $fields{ 'pub_print_date_sure_flag' } =
           $CFG{ PUB_PRINT_DATE_PUB_PRINT_DATE_SURE_FLAG };

    # комментарий
    $fields{ 'comment' } = $CFG{ PUB_PRINT_DATE_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор объекта
  # библиографического описания" не подлежит редактированию, не будем
  # позволять его редактировать, но совсем убрать его нельзя, т.к. оно
  # используется при выполнении операции UPDATE в качестве ключа, а также при
  # выполнении операции INSERT (значение этого поля сохраняется таким же,
  # каким оно было при предыдущем запуске скрипта, поэтому можно не
  # присваивать ему значения)
  print $query->hidden( -name => 'obj_id' ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();           # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Дата начала издания</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pub_date_begin",
                   "PUB_PRINT_DATE_PUB_DATE_BEGIN", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'pub_date_begin',
                           -default => $fields{ 'pub_date_begin' },
                           -size => $CFG{ PUB_PRINT_DATE_PUB_DATE_BEGIN_SIZE },
                           -maxlength => 
                             $CFG{ PUB_PRINT_DATE_PUB_DATE_BEGIN_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак наличия тире-разделителя<br />" .
        "(оно ставится между датой начала и датой окончания издания)</th>\n";
  print "<td>",
        $query->checkbox( -name => 'dash_flag',
                          -checked => 
                             $fields{ 'dash_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата окончания издания</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pub_date_end",
                   "PUB_PRINT_DATE_PUB_DATE_END", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'pub_date_end',
                           -default => $fields{ 'pub_date_end' },
                           -size => $CFG{ PUB_PRINT_DATE_PUB_DATE_END_SIZE },
                           -maxlength => 
                             $CFG{ PUB_PRINT_DATE_PUB_DATE_END_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата печатания, изготовления</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].print_date",
                   "PUB_PRINT_DATE_PRINT_DATE", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'print_date',
                           -default => $fields{ 'print_date' },
                           -size => $CFG{ PUB_PRINT_DATE_PRINT_DATE_SIZE },
                           -maxlength => 
                             $CFG{ PUB_PRINT_DATE_PRINT_DATE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак \"без года (sine anno)\"</th>\n";
  print "<td>",
        $query->checkbox( -name => 'sine_anno_flag',
                          -checked => 
                             $fields{ 'sine_anno_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак точно установленной даты<br />" .
        "(если дата не точная, тогда при указании даты используются " .
        "скобки &mdash; [])</th>\n";
  print "<td>",
        $query->checkbox( -name => 'pub_print_date_sure_flag',
                          -checked => 
                             $fields{ 'pub_print_date_sure_flag' } eq 't' ?
                             '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment",
                   "PUB_PRINT_DATE_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ PUB_PRINT_DATE_COMMENT_ROWS },
                          -columns => $CFG{ PUB_PRINT_DATE_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;

  # поскольку эта функция обрабатывает только поля-флаги 'dash_flag',
  # 'sine_anno_flag', 'pub_print_date_sure_flag', имеющие тип bool,
  # то проверяем значение параметра только на непустоту
  $value = $query->param( $fname ) ? 'yes' : 'no';

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Дата издания, печатания, изготовления" );
  
  return ( 0 );
}
