#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Место издания, изготовления"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'PUB_PRINT_PLACE';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Pub_print_place
my @f_names =
   ( 'obj_id',            # Уникальный идентификатор объекта
                          # библиографического описания
     'bib_rec_num',       # Номер библиогр. записи для данного объекта
                          # (основная запись имеет номер 0)
     'group_num',         # Номер группы сведений о месте издания
     'place_num',         # Номер места издания в пределах группы
     'city_country_flag', # Флаг "город/страна" (1 - город; 2 - страна)
     'city_code',         # Код города (код места издания)
     'country_code',      # Код страны (он нужен, когда не известно
                          # место издания)
     'pub_print_flag',    # Признак "издатель/изготовитель"
                          # (1 - издатель; 2 - изготовитель)
     'sine_loco_flag',    # Признак "без места (sine loco)"
     'place_sure_flag',   # Флаг точно установленного места издания
     'place_add_info',    # Дополнительные сведения к месту издания/печатания
     'comment'            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'group_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указано или неверное значение номера " .
          "группы сведений о месте издания" );
  }

  if ( $query->param( 'place_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный номер места издания " .
          "в пределах группы" );
  }

  if ( ! ( $query->param( 'city_country_flag' ) eq '' ||
           $query->param( 'city_country_flag' ) eq '1' ||
           $query->param( 'city_country_flag' ) eq '2'
         )
     )    
  {
    push( @error_messages, "Неверное значение флага \"город/страна\"" );
  }

  if ( $query->param( 'city_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код города " .
          "(код места издания)" );
  }

  if ( $query->param( 'country_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код страны " .
          "(он нужен, когда не известно место издания)" );
  }

  if ( ! ( $query->param( 'pub_print_flag' ) eq '1' ||
           $query->param( 'pub_print_flag' ) eq '2'
         )
     )    
  {
    push( @error_messages, "Неверное значение признака " .
          "\"издатель/изготовитель\"" );
  }

  # ПРИМЕЧАНИЕ. Решил пока что не делать проверку, соответствующую ограничению
  #             CHECK таблицы Pub_print_place. Пусть в случае ошибки,
  #             допущенной при заполнении полей формы, работает СУБД.

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $group_num;          # номер группы сведений о месте издания
  my $place_num;          # номер места издания в пределах группы
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $group_num   = $query->param( 'group_num' );
  $place_num   = $query->param( 'place_num' );
  $target_win  = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}pub_print_place " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "group_num = '$group_num' AND " .
                   "place_num = '$place_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка мест издания, изготовления данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки
  my $place;              # место издания

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о местах издания, изготовления для данного объекта
  # (получим также краткие наименования города и страны, причем страну выберем
  # дважды: один раз - на основе кода страны из таблицы Pub_print_place,
  # в второй - на основе кода страны из таблицы Cities, поскольку в таблице
  # Pub_print_place город и страна НЕ СВЯЗАНЫ между собой)
  $command = "SELECT p.*, ct.city_sname, " .
                    "cntr1.country_sname as country_sname_pub, " .
                    "cntr2.country_sname as country_sname_city " .
             "FROM ${db_schema}pub_print_place p, ${db_schema}cities ct, " .
                  "${db_schema}countries cntr1, ${db_schema}countries cntr2 " .
             "WHERE p.obj_id = $obj_id AND " .
                   "p.bib_rec_num = $bib_rec_num AND " .
                   "p.city_code = ct.city_code AND " .
                   "ct.country_code = cntr2.country_code AND " .
                   "p.country_code = cntr1.country_code " .
             "ORDER BY p.group_num, p.place_num";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Номер группы сведений о месте издания</th>\n";
    print "<th>Номер места издания в пределах группы</th>\n";
    print "<th>Место издания (печатания)</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим номер группы и номер места издания в группе, наименование
      # места издания
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'group_num' ) ]</td>\n";
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'place_num' ) ]</td>\n";

      # сформируем место издания
      if ( $row[ $result->fnumber( 'city_country_flag' ) ] eq '' )
      {
        $place = "-";
      }
      # город (в скобках укажем страну)
      elsif ( $row[ $result->fnumber( 'city_country_flag' ) ] == 1 )
      {
        # берем страну в соответствии с кодом города
        $place = $row[ $result->fnumber( 'city_sname' ) ] . " (" .
                 $row[ $result->fnumber( 'country_sname_city' ) ] . ")";
      }
      # страна
      else       # ( $row[ $result->fnumber( 'city_country_flag' ) ] == 2 )
      {
        $place = $row[ $result->fnumber( 'country_sname_pub' ) ];
      }
      
      # место издания установлено не точно
      if ( $row[ $result->fnumber( 'place_sure_flag' ) ] ne 't' )
      {
        $place = "[" . $place . "]";   # возьмем место издания в скобки
      }

      print "<td>$place</td>\n";
      
      # просмотр данной записи о месте издания в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&group_num=" . $row[ $result->fnumber( 'group_num' ) ] .
            "&place_num=" . $row[ $result->fnumber( 'place_num' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о месте издания
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&group_num=" . $row[ $result->fnumber( 'group_num' ) ] .
            "&place_num=" . $row[ $result->fnumber( 'place_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о месте издания
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $group_num;          # номер группы сведений о месте издания
  my $place_num;          # номер места издания в пределах группы
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $group_num   = $query->param( 'group_num' );
  $place_num   = $query->param( 'place_num' );
  $target_win  = $query->param( 'target_win' );

  # выбираем одну запись о месте издания
  # (получим также краткие наименования города и страны, причем страну выберем
  # дважды: один раз - на основе кода страны из таблицы Pub_print_place,
  # в второй - на основе кода страны из таблицы Cities, поскольку в таблице
  # Pub_print_place город и страна НЕ СВЯЗАНЫ между собой)
  $command = "SELECT p.*, ct.city_sname, " .
                    "cntr1.country_sname as country_sname_pub, " .
                    "cntr2.country_sname as country_sname_city " .
             "FROM ${db_schema}pub_print_place p, ${db_schema}cities ct, " .
                  "${db_schema}countries cntr1, ${db_schema}countries cntr2 " .
             "WHERE p.obj_id = $obj_id AND " .
                   "p.bib_rec_num = $bib_rec_num AND " .
                   "p.group_num = '$group_num' AND " .
                   "p.place_num = '$place_num' AND " .
                   "p.city_code = ct.city_code AND " .
                   "ct.country_code = cntr2.country_code AND " .
                   "p.country_code = cntr1.country_code";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "номер группы сведений о месте издания - $group_num, " .
                     "номер места издания в пределах группы - $place_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "pub_print_place",
                       key => "obj_id, bib_rec_num, " .
                              "group_num:group_num_begin, " .
                              "place_num:place_num_begin",
                       f_names => \@f_names,
                       f_special => [ 'sine_loco_flag', 'place_sure_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );

    # значения по умолчанию

    # номер группы сведений о месте издания
    $fields{ 'group_num' }         = $CFG{ PUB_PRINT_PLACE_GROUP_NUM };

    # номер места издания в пределах группы
    $fields{ 'place_num' }         = $CFG{ PUB_PRINT_PLACE_PLACE_NUM };

    # флаг "город/страна"
    $fields{ 'city_country_flag' } = $CFG{ PUB_PRINT_PLACE_CITY_COUNTRY_FLAG };

    # код города (код места издания)
    $fields{ 'city_code' }         = $CFG{ PUB_PRINT_PLACE_CITY_CODE };

    # код страны
    $fields{ 'country_code' }      = $CFG{ PUB_PRINT_PLACE_COUNTRY_CODE };

    # признак "издатель/изготовитель"
    $fields{ 'pub_print_flag' }    = $CFG{ PUB_PRINT_PLACE_PUB_PRINT_FLAG };

    # признак "без места (sine loco)"
    $fields{ 'sine_loco_flag' }    = $CFG{ PUB_PRINT_PLACE_SINE_LOCO_FLAG };

    # флаг точно установленного места издания
    $fields{ 'place_sure_flag' }   = $CFG{ PUB_PRINT_PLACE_PLACE_SURE_FLAG };

    # дополнительные сведения к месту издания/печатания (например, когда
    # одноименные города есть в разных странах)
    $fields{ 'place_add_info' }    = $CFG{ PUB_PRINT_PLACE_PLACE_ADD_INFO };

    # комментарий
    $fields{ 'comment' }           = $CFG{ PUB_PRINT_PLACE_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript">
  <!--
  // функция для координирования значений ряда полей и доступа к ним
  // в зависимости от значения поля "Флаг 'город/страна'"
  function on_change_city_country_flag()
  {
    // если выбран "город"
    if ( document.forms[ 0 ].city_country_flag.selectedIndex == 1 )
    {
      document.forms[ 0 ].country_code.value = 0;     // 0 -- фиктивная страна
      document.forms[ 0 ].country_sname_pub.value = '';      // очистим поле
      document.forms[ 0 ].country_code.disabled = true;      // запретим ввод
      document.forms[ 0 ].country_sname_pub.disabled = true; // запретим ввод
      document.forms[ 0 ].city_code.disabled = false;        // разрешим ввод
      document.forms[ 0 ].city_sname.disabled = false;       // разрешим ввод
      document.forms[ 0 ].country_sname_city.disabled = false; // разрешим ввод
      // Признак 'без места (sine loco)'
      document.forms[ 0 ].sine_loco_flag.checked = false;
      // избыточная команда, но пусть пока что останется
      document.forms[ 0 ].sine_loco_flag.disabled = true;
      // Флаг точно установленного места издания
      document.forms[ 0 ].place_sure_flag.checked = true;
      document.forms[ 0 ].place_sure_flag.disabled = false;
    }
    // если выбрана "страна"
    else if ( document.forms[ 0 ].city_country_flag.selectedIndex == 2 )
    {
      document.forms[ 0 ].city_code.value = 0;        // 0 -- фиктивный город
      document.forms[ 0 ].city_sname.value = '';              // очистим поле
      document.forms[ 0 ].country_sname_city.value = '';      // очистим поле
      document.forms[ 0 ].city_code.disabled = true;          // запретим ввод
      document.forms[ 0 ].city_sname.disabled = true;         // запретим ввод
      document.forms[ 0 ].country_sname_city.disabled = true; // запретим ввод
      document.forms[ 0 ].country_code.disabled = false;      // разрешим ввод
      document.forms[ 0 ].country_sname_pub.disabled = false; // разрешим ввод
      // Признак 'без места (sine loco)'
      document.forms[ 0 ].sine_loco_flag.checked = false;
      // избыточная команда, но пусть пока что останется
      document.forms[ 0 ].sine_loco_flag.disabled = true;
      // Флаг точно установленного места издания
      document.forms[ 0 ].place_sure_flag.checked = true;
      document.forms[ 0 ].place_sure_flag.disabled = false;
    }
    else   // место издания не известно
    {
      document.forms[ 0 ].city_code.value = 0;        // 0 -- фиктивный город
      document.forms[ 0 ].city_sname.value = '';              // очистим поле
      document.forms[ 0 ].country_sname_city.value = '';      // очистим поле
      document.forms[ 0 ].country_code.value = 0;     // 0 -- фиктивная страна
      document.forms[ 0 ].country_sname_pub.value = '';       // очистим поле
      document.forms[ 0 ].city_code.disabled = true;          // запретим ввод
      document.forms[ 0 ].city_sname.disabled = true;         // запретим ввод
      document.forms[ 0 ].country_sname_city.disabled = true; // запретим ввод
      document.forms[ 0 ].country_code.disabled = true;       // запретим ввод
      document.forms[ 0 ].country_sname_pub.disabled = true;  // запретим ввод
      // Признак 'без места (sine loco)'
      document.forms[ 0 ].sine_loco_flag.checked = true;
      // избыточная команда, но пусть пока что останется
      document.forms[ 0 ].sine_loco_flag.disabled = true;
      // Флаг точно установленного места издания
      document.forms[ 0 ].place_sure_flag.checked = false;
      document.forms[ 0 ].place_sure_flag.disabled = true;
    }
  }
  //-->
  </script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи для данного
  # объекта" не подлежат редактированию, не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа, а также при выполнении операции INSERT
  # (значения этих полей сохраняются такими же, какими они были при
  # предыдущем запуске скрипта, поэтому можно не присваивать им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";

  # создадим скрытые поля для "Номера группы сведений о месте издания"
  # и "Номера места издания в пределах группы"
  # ПРИМЕЧАНИЕ. Идентификаторы скрытых полей имеют постфикс 'begin' для того,
  #             чтобы можно было обновлять значения этих полей, поскольку они
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm.
  #             Т.е. нам нужны и исходные значения этих полей, и их новые
  #             значения (которые, возможно, были изменены пользователем).
  print $query->hidden( -name => 'group_num_begin',
                        -default => $fields{ 'group_num' }
                      ), "\n";
  print $query->hidden( -name => 'place_num_begin',
                        -default => $fields{ 'place_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();         # выведем дополнительную информацию на экран
  
  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Номер группы сведений о месте издания</th>\n";
  print "<td>",
        $query->textfield( -name => 'group_num',
                           -default => $fields{ 'group_num' },
                           -size => $CFG{ PUB_PRINT_PLACE_GROUP_NUM_SIZE },
                           -maxlength => 
                             $CFG{ PUB_PRINT_PLACE_GROUP_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номер места издания в пределах группы</th>\n";
  print "<td>",
        $query->textfield( -name => 'place_num',
                           -default => $fields{ 'place_num' },
                           -size => $CFG{ PUB_PRINT_PLACE_PLACE_NUM_SIZE },
                           -maxlength => 
                             $CFG{ PUB_PRINT_PLACE_PLACE_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак \"город/страна\"</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'city_country_flag',
                            -default => $fields{ 'city_country_flag' },
                            -values => [ '', '1', '2' ],
                            -labels => { '' => '-', '1' => 'город',
                                         '2' => 'страна' },
                            -onChange => "on_change_city_country_flag()"
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Город &mdash; место издания (код, наименование, страна)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник cities.pl
  create_selection_func( func_name => 'select_city',
                         module_name => 'cities.pl',
                         caller_name => $module,
                         window_name => 'cities',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также краткое наименование города и страны
  # (без возможности редактирования)
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Название города";
  # название страны просто обнуляем (наша функция create_hot_help() не 
  # позволяет предложить более интеллектуальную обработку для этого поля)
  create_hot_help( "document.forms[ 0 ].city_code",
                   "PUB_PRINT_PLACE_CITY_CODE", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].city_sname.value = str;
                   document.forms[ 0 ].country_sname_city.value = ''; " )
    if $CFG{ HOT_HELP_IS_ON };

  # массивы параметров для полей city_code, city_sname и country_sname_city
  my @city_code_params = ( '-name', 'city_code',
                           '-default', $fields{ 'city_code' },
                           '-size', $CFG{ PUB_PRINT_PLACE_CITY_CODE_SIZE },
                           '-maxlength', 
                             $CFG{ PUB_PRINT_PLACE_CITY_CODE_MAXLENGTH }
                         );
  my @city_sname_params = ( '-name', 'city_sname',
                            '-default', $fields{ 'city_sname' },
                            '-size', $CFG{ PUB_PRINT_PLACE_CITY_SNAME_SIZE },
                            '-maxlength', 
                              $CFG{ PUB_PRINT_PLACE_CITY_SNAME_MAXLENGTH },
                            '-readonly', 'readonly'
                          );
  my @country_sname_city_params = 
                          ( '-name', 'country_sname_city',
                            '-default', $fields{ 'country_sname_city' },
                            '-size', 
                              $CFG{ PUB_PRINT_PLACE_COUNTRY_SNAME_CITY_SIZE },
                            '-maxlength', 
                            # используем конкатенацию для формирования ключа 
                            # хеш-массива
                            $CFG{ 'PUB_PRINT_PLACE_' .
                                  'COUNTRY_SNAME_CITY_MAXLENGTH' },
                            '-readonly', 'readonly'
                          );
  # если место издания -- "страна" (2), то запрещаем ввод в эти поля
  if ( $fields{ 'city_country_flag' } == 2 )
  {
    push( @city_code_params, '-disabled', 'disabled' );
    push( @city_sname_params, '-disabled', 'disabled' );
    push( @country_sname_city_params, '-disabled', 'disabled' );
  }

  print $query->textfield( @city_code_params ), "&nbsp;&nbsp;",
        $query->textfield( @city_sname_params ), "&nbsp;&nbsp;",
        $query->textfield( @country_sname_city_params ), "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" 
            onclick=
            "if ( document.forms[ 0 ].city_code.disabled == false )
               select_city();
             else
               window.alert( 'Поле заблокировано. Ввод невозможен' );
            ">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Страна (код, наименование)<br />" .
        "(указывается, когда не известен город &mdash; место издания)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник countries.pl
  create_selection_func( func_name => 'select_country',
                         module_name => 'countries.pl',
                         caller_name => $module,
                         window_name => 'countries',
                         exp_imp => $query->param( 'exp_imp' ) );
  # выведем также краткое наименование страны (без возможности редактирования)
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Название страны" 
  create_hot_help( "document.forms[ 0 ].country_code",
                   "PUB_PRINT_PLACE_COUNTRY_CODE", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].country_sname_pub.value = str; " )
    if $CFG{ HOT_HELP_IS_ON };

  # массивы параметров для полей country_code и country_sname_pub
  my @country_code_params = ( '-name', 'country_code',
                              '-default', $fields{ 'country_code' },
                              '-size', 
                                $CFG{ PUB_PRINT_PLACE_COUNTRY_CODE_SIZE },
                              '-maxlength', 
                                $CFG{ PUB_PRINT_PLACE_COUNTRY_CODE_MAXLENGTH }
                            );
  my @country_sname_pub_params = ( '-name', 'country_sname_pub',
                                   '-default', $fields{ 'country_sname_pub' },
                                   '-size',
                                   # используем конкатенацию для формирования
                                   # ключа хеш-массива
                                     $CFG{ 'PUB_PRINT_PLACE_' .
                                           'COUNTRY_SNAME_PUB_SIZE' },
                                   '-maxlength',
                                     $CFG{ 'PUB_PRINT_PLACE_' .
                                           'COUNTRY_SNAME_PUB_MAXLENGTH' },
                                   '-readonly', 'readonly'
                                 );
  # если место издания -- "город" (1), то запрещаем ввод в эти поля
  if ( $fields{ 'city_country_flag' } == 1 )
  {
    push( @country_code_params, '-disabled', 'disabled' );
    push( @country_sname_pub_params, '-disabled', 'disabled' );
  }

  print $query->textfield( @country_code_params ), "&nbsp;&nbsp;",
        $query->textfield( @country_sname_pub_params ), "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" 
            onclick=
            "if ( document.forms[ 0 ].country_code.disabled == false )
               select_country();
             else
               window.alert( 'Поле заблокировано. Ввод невозможен' );
            ">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак \"издатель/изготовитель\"</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'pub_print_flag',
                            -default => $fields{ 'pub_print_flag' },
                            -values => [ '1', '2' ],
                            -labels => { '1' => 'издатель',
                                         '2' => 'изготовитель' }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак \"без места (sine loco)\"</th>\n";
  print "<td>",
        $query->checkbox( -name => 'sine_loco_flag',
                          -checked => 
                             $fields{ 'sine_loco_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => '',
  # значения этого признака изменяются только с помощью кода на языке
  # JavaScript
                          -disabled => 'disabled'
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак точно установленного места издания<br />" .
        "(если место не точное, тогда при указании места используются " .
        "скобки &mdash; [])</th>\n";
  print "<td>",
        $query->checkbox( -name => 'place_sure_flag',
                          -checked => 
                             $fields{ 'place_sure_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дополнительные сведения к месту издания/печатания <br />" .
            "(например, когда одноименные города есть в разных " .
            "странах)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].place_add_info",
                   "PUB_PRINT_PLACE_PLACE_ADD_INFO" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'place_add_info',
                          -default => $fields{ 'place_add_info' },
                          -rows => $CFG{ PUB_PRINT_PLACE_PLACE_ADD_INFO_ROWS },
                          -columns => 
                            $CFG{ PUB_PRINT_PLACE_PLACE_ADD_INFO_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "PUB_PRINT_PLACE_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ PUB_PRINT_PLACE_COMMENT_ROWS },
                          -columns => $CFG{ PUB_PRINT_PLACE_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;

  # поскольку эта функция обрабатывает только поля-флаги 'sine_loco_flag',
  # 'place_sure_flag', имеющие тип bool, то проверяем значение параметра
  # только на непустоту
  $value = $query->param( $fname ) ? 'yes' : 'no';

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Место издания, изготовления" );
  
  return ( 0 );
}
