#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Издатель, изготовитель, распространитель"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'PUB_PRINTER';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Pub_printer
my @f_names =
   ( 'obj_id',              # Уникальный идентификатор объекта
                            # библиографического описания
     'bib_rec_num',         # Номер библиогр. записи для данного объекта
                            # (основная запись имеет номер 0)
     'group_num',           # Номер группы сведений об издателе
     'pub_num',             # Номер издателя, изготовителя в пределах группы
     'pers_org_flag',       # Флаг "персоналия/организация"
                            # (1 - персоналия; 2 - организация)
     'pub_org_id',          # Код издателя, изготовителя ... (организация)
     'pub_pers_id',         # Код издателя, изготовителя ... (персоналия)
     'pub_print_flag',      # Признак "издатель/изготовитель"
     'pub_func',            # Функция издателя (изготовителя)
     'sine_nomine_flag',    # Признак "без издателя (sine nomine)"
     'pub_print_sure_flag', # Флаг точно установленного издателя
     'pub_print_add_info',  # Дополнительные сведения к имени издателя/
                            # изготовителя
     'comment'              # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'group_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указано или неверное значение номера " .
          "группы сведений об издателе" );
  }

  if ( $query->param( 'pub_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный номер издателя, " .
          "изготовителя в пределах группы" );
  }

  if ( ! ( $query->param( 'pers_org_flag' ) eq '' ||
           $query->param( 'pers_org_flag' ) eq '1' ||
           $query->param( 'pers_org_flag' ) eq '2'
         )
     )    
  {
    push( @error_messages, "Неверное значение флага " .
          "\"персоналия/организация\"" );
  }

  if ( $query->param( 'pub_org_id' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код издателя, " .
          "изготовителя ... (организация)" );
  }

  if ( $query->param( 'pub_pers_id' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код издателя, " .
          "изготовителя ... (персоналия)" );
  }

  if ( ! ( $query->param( 'pub_print_flag' ) eq '1' ||
           $query->param( 'pub_print_flag' ) eq '2' ||
           $query->param( 'pub_print_flag' ) eq '3'
         )
     )    
  {
    push( @error_messages, "Неверное значение признака " .
          "\"издатель/изготовитель\"" );
  }

  # ПРИМЕЧАНИЕ. Решил пока что не делать проверку, соответствующую ограничению
  #             CHECK таблицы Pub_printer. Пусть в случае ошибки,
  #             допущенной при заполнении полей формы, работает СУБД.

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $group_num;          # номер группы сведений об издателе
  my $pub_num;            # номер издателя, изготовителя в пределах группы
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $group_num   = $query->param( 'group_num' );
  $pub_num     = $query->param( 'pub_num' );
  $target_win  = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}pub_printer " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "group_num = '$group_num' AND " .
                   "pub_num = '$pub_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка мест издания, изготовления данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки
  my $publisher;          # издатель/изготовитель

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи об издателях/изготовителях для данного объекта
  # (получим также краткое наименования организации-издателя и наименование
  # персоналии-издателя)
  $command = "SELECT p.*, pers.pers_name, o.org_sname " .
             "FROM ${db_schema}pub_printer p, ${db_schema}persons pers, " .
                  "${db_schema}orgs o " .
             "WHERE p.obj_id = $obj_id AND " .
                   "p.bib_rec_num = $bib_rec_num AND " .
                   "p.pub_pers_id = pers.pers_id AND " .
                   "p.pub_org_id = o.org_id " .
             "ORDER BY p.group_num, p.pub_num";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Номер группы сведений об издателе/изготовителе</th>\n";
    print "<th>Номер издателя/изготовителя в пределах группы</th>\n";
    print "<th>Издатель/изготовитель</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим номер группы и номер издателя в группе, наименование
      # издателя
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'group_num' ) ]</td>\n";
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'pub_num' ) ]</td>\n";

      # сформируем издателя/изготовителя
      if ( $row[ $result->fnumber( 'pers_org_flag' ) ] eq '' )
      {
        $publisher = "без издателя";
      }
      # организация
      elsif ( $row[ $result->fnumber( 'pers_org_flag' ) ] == 2 )
      {
        # берем наименование организации
        $publisher = $row[ $result->fnumber( 'org_sname' ) ];
      }
      # персоналия
      else       # ( $row[ $result->fnumber( 'pers_org_flag' ) ] == 1 )
      {
        $publisher = $row[ $result->fnumber( 'pers_name' ) ];
      }
      
      # издатель/изготовитель установлен не точно
      if ( $row[ $result->fnumber( 'pub_print_sure_flag' ) ] ne 't' )
      {
        # возьмем издателя/изготовителя в скобки
        $publisher = "[" . $publisher . "]";
      }

      print "<td>$publisher</td>\n";

      # просмотр данной записи об издателе/изготовителе в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&group_num=" . $row[ $result->fnumber( 'group_num' ) ] .
            "&pub_num=" . $row[ $result->fnumber( 'pub_num' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи об издателе/изготовителе
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&group_num=" . $row[ $result->fnumber( 'group_num' ) ] .
            "&pub_num=" . $row[ $result->fnumber( 'pub_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи об издателе
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $group_num;          # номер группы сведений об издателе
  my $pub_num;            # номер издателя, изготовителя в пределах группы
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $group_num   = $query->param( 'group_num' );
  $pub_num     = $query->param( 'pub_num' );
  $target_win  = $query->param( 'target_win' );

  # выбираем одну запись об издателе/изготовителе
  # (получим также краткое наименования организации-издателя и наименование
  # персоналии-издателя)
  $command = "SELECT p.*, pers.pers_name, o.org_sname " .
             "FROM ${db_schema}pub_printer p, ${db_schema}persons pers, " .
                  "${db_schema}orgs o " .
             "WHERE p.obj_id = $obj_id AND " .
                   "p.bib_rec_num = $bib_rec_num AND " .
                   "p.group_num = '$group_num' AND " .
                   "p.pub_num = '$pub_num' AND " .
                   "p.pub_pers_id = pers.pers_id AND " .
                   "p.pub_org_id = o.org_id";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "номер группы сведений об издателе - $group_num, " .
                     "номер издателя в пределах группы - $pub_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "pub_printer",
                       key => "obj_id, bib_rec_num, " .
                              "group_num:group_num_begin, " .
                              "pub_num:pub_num_begin",
                       f_names => \@f_names,
                       f_special => [ 'sine_nomine_flag',
                                      'pub_print_sure_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );

    # значения по умолчанию

    # номер группы сведений об издателе
    $fields{ 'group_num' }           = $CFG{ PUB_PRINTER_GROUP_NUM };

    # номер издателя, изготовителя в пределах группы
    $fields{ 'pub_num' }             = $CFG{ PUB_PRINTER_PUB_NUM };

    # флаг "персоналия/организация"
    $fields{ 'pers_org_flag' }       = $CFG{ PUB_PRINTER_PERS_ORG_FLAG };

    # код издателя, изготовителя (организация)
    $fields{ 'pub_org_id' }          = $CFG{ PUB_PRINTER_PUB_ORG_ID };

    # код издателя, изготовителя (персоналия)
    $fields{ 'pub_pers_id' }         = $CFG{ PUB_PRINTER_PUB_PERS_ID };

    # признак "издатель/изготовитель"
    $fields{ 'pub_print_flag' }      = $CFG{ PUB_PRINTER_PUB_PRINT_FLAG };

    # функция издателя (изготовителя)
    $fields{ 'pub_func' }            = $CFG{ PUB_PRINTER_PUB_FUNC };

    # признак "без издателя (sine nomine)"
    $fields{ 'sine_nomine_flag' }    = $CFG{ PUB_PRINTER_SINE_NOMINE_FLAG };

    # флаг точно установленного издателя
    $fields{ 'pub_print_sure_flag' } = $CFG{ PUB_PRINTER_PUB_PRINT_SURE_FLAG };

    # дополнительные сведения к имени издателя/изготовителя
    $fields{ 'pub_print_add_info' }  = $CFG{ PUB_PRINTER_PUB_PRINT_ADD_INFO };

    # комментарий
    $fields{ 'comment' }             = $CFG{ PUB_PRINTER_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript">
  <!--
  // функция для координирования значений ряда полей и доступа к ним
  // в зависимости от значения поля "Флаг 'персоналия/организация'"
  function on_change_pers_org_flag()
  {
    // если выбрана "персоналия"
    if ( document.forms[ 0 ].pers_org_flag.selectedIndex == 1 )
    {
      document.forms[ 0 ].pub_org_id.value = 0;   // 0 -- фиктивная организация
      document.forms[ 0 ].org_sname.value = '';         // очистим поле
      document.forms[ 0 ].pub_org_id.disabled = true;   // запретим ввод
      document.forms[ 0 ].org_sname.disabled = true;    // запретим ввод
      document.forms[ 0 ].pub_pers_id.disabled = false; // разрешим ввод
      document.forms[ 0 ].pers_name.disabled = false;   // разрешим ввод
      // Признак 'без издателя (sine nomine)'
      document.forms[ 0 ].sine_nomine_flag.checked = false;
      // избыточная команда, но пусть пока что останется
      document.forms[ 0 ].sine_nomine_flag.disabled = true;
      // Флаг точно установленного издателя
      document.forms[ 0 ].pub_print_sure_flag.checked = true;
      document.forms[ 0 ].pub_print_sure_flag.disabled = false;
    }
    // если выбрана "организация"
    else if ( document.forms[ 0 ].pers_org_flag.selectedIndex == 2 )
    {
      document.forms[ 0 ].pub_pers_id.value = 0;   // 0 -- фиктивная персоналия
      document.forms[ 0 ].pers_name.value = '';         // очистим поле
      document.forms[ 0 ].pub_pers_id.disabled = true;  // запретим ввод
      document.forms[ 0 ].pers_name.disabled = true;    // запретим ввод
      document.forms[ 0 ].pub_org_id.disabled = false;  // разрешим ввод
      document.forms[ 0 ].org_sname.disabled = false;   // разрешим ввод
      // Признак 'без издателя (sine nomine)'
      document.forms[ 0 ].sine_nomine_flag.checked = false;
      // избыточная команда, но пусть пока что останется
      document.forms[ 0 ].sine_nomine_flag.disabled = true;
      // Флаг точно установленного издателя
      document.forms[ 0 ].pub_print_sure_flag.checked = true;
      document.forms[ 0 ].pub_print_sure_flag.disabled = false;
    }
    else   // издатель не известен
    {
      document.forms[ 0 ].pub_pers_id.value = 0;  // 0 -- фиктивная персоналия
      document.forms[ 0 ].pers_name.value = '';   // очистим поле
      document.forms[ 0 ].pub_org_id.value = 0;   // 0 -- фиктивная организация
      document.forms[ 0 ].org_sname.value = '';   // очистим поле
      document.forms[ 0 ].pub_pers_id.disabled = true;  // запретим ввод
      document.forms[ 0 ].pers_name.disabled = true;    // запретим ввод
      document.forms[ 0 ].pub_org_id.disabled = true;   // запретим ввод
      document.forms[ 0 ].org_sname.disabled = true;    // запретим ввод
      // Признак 'без издателя (sine nomine)'
      document.forms[ 0 ].sine_nomine_flag.checked = true;
      // избыточная команда, но пусть пока что останется
      document.forms[ 0 ].sine_nomine_flag.disabled = true;
      // Флаг точно установленного издателя
      document.forms[ 0 ].pub_print_sure_flag.checked = false;
      document.forms[ 0 ].pub_print_sure_flag.disabled = true;
    }
  }
  //-->
  </script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи для данного
  # объекта" не подлежат редактированию, не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа, а также при выполнении операции INSERT
  # (значения этих полей сохраняются такими же, какими они были при
  # предыдущем запуске скрипта, поэтому можно не присваивать им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";

  # создадим скрытые поля для "Номера группы сведений об издателе"
  # и "Номера издателя, изготовителя в пределах группы"
  # ПРИМЕЧАНИЕ. Идентификаторы скрытых полей имеют постфикс 'begin' для того,
  #             чтобы можно было обновлять значения этих полей, поскольку они
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm.
  #             Т.е. нам нужны и исходные значения этих полей, и их новые
  #             значения (которые, возможно, были изменены пользователем).
  print $query->hidden( -name => 'group_num_begin',
                        -default => $fields{ 'group_num' }
                      ), "\n";
  print $query->hidden( -name => 'pub_num_begin',
                        -default => $fields{ 'pub_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();          # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Номер группы сведений об издателе/изготовителе</th>\n";
  print "<td>",
        $query->textfield( -name => 'group_num',
                           -default => $fields{ 'group_num' },
                           -size => $CFG{ PUB_PRINTER_GROUP_NUM_SIZE },
                           -maxlength => 
                             $CFG{ PUB_PRINTER_GROUP_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номер издателя/изготовителя в пределах группы</th>\n";
  print "<td>",
        $query->textfield( -name => 'pub_num',
                           -default => $fields{ 'pub_num' },
                           -size => $CFG{ PUB_PRINTER_PUB_NUM_SIZE },
                           -maxlength => $CFG{ PUB_PRINTER_PUB_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак \"персоналия/организация\"</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'pers_org_flag',
                            -default => $fields{ 'pers_org_flag' },
                            -values => [ '', '1', '2' ],
                            -labels => { '' => '-', '1' => 'персоналия',
                                         '2' => 'организация' },
                            -onChange => "on_change_pers_org_flag()"
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Организация &mdash; издатель/изготовитель " .
        "(код и наименование)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник orgs.pl
  create_selection_func( func_name => 'select_org',
                         module_name => 'orgs.pl',
                         caller_name => $module,
                         window_name => 'orgs',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также краткое наименование организации
  # (без возможности редактирования)
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Название организации"
  create_hot_help( "document.forms[ 0 ].pub_org_id",
                   "PUB_PRINTER_PUB_ORG_ID", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].org_sname.value = str; " )
    if $CFG{ HOT_HELP_IS_ON };

  # массивы параметров для полей pub_org_id и org_sname
  # если тип издателя - "персоналия" или издатель не известен, то значением
  # поля будет 0
  my @pub_org_id_params = ( '-name', 'pub_org_id',
                            '-default', $fields{ 'pers_org_flag' } == '2' ? 
                                        $fields{ 'pub_org_id' } :
                                        0,
                            '-size', $CFG{ PUB_PRINTER_PUB_ORG_ID_SIZE },
                            '-maxlength', 
                              $CFG{ PUB_PRINTER_PUB_ORG_ID_MAXLENGTH }
                          );
  my @org_sname_params = ( '-name', 'org_sname',
                           '-default', $fields{ 'org_sname' },
                           '-size', $CFG{ PUB_PRINTER_ORG_SNAME_SIZE },
                           '-maxlength', 
                             $CFG{ PUB_PRINTER_ORG_SNAME_MAXLENGTH },
                           '-readonly', 'readonly'
                         );
  # если издатель -- "персоналия" (1) или издатель не известен, то запрещаем
  # ввод в эти поля
  if ( $fields{ 'pers_org_flag' } == 1 || $fields{ 'pers_org_flag' } eq '' )
  {
    push( @pub_org_id_params, '-disabled', 'disabled' );
    push( @org_sname_params, '-disabled', 'disabled' );
  }

  print $query->textfield( @pub_org_id_params ), "&nbsp;&nbsp;",
        $query->textfield( @org_sname_params ), "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" 
            onclick=
            "if ( document.forms[ 0 ].pub_org_id.disabled == false )
               select_org();
             else
               window.alert( 'Поле заблокировано. Ввод невозможен' );
            ">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Персоналия &mdash; издатель/изготовитель (код и Ф. И. О.)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник persons.pl
  create_selection_func( func_name => 'select_person',
                         module_name => 'persons.pl',
                         caller_name => $module,
                         window_name => 'persons',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также наименование персоналии (без возможности редактирования)
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Имя персоналии" 
  create_hot_help( "document.forms[ 0 ].pub_pers_id",
                   "PUB_PRINTER_PUB_PERS_ID", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].pers_name.value = str; " )
    if $CFG{ HOT_HELP_IS_ON };

  # массивы параметров для полей pub_pers_id и pers_name
  # если тип издателя - "организация" или издатель не известен, то значением
  # поля будет 0
  my @pub_pers_id_params = ( '-name', 'pub_pers_id',
                             '-default', $fields{ 'pers_org_flag' } == '1' ? 
                                         $fields{ 'pub_pers_id' } :
                                         0,
                             '-size', $CFG{ PUB_PRINTER_PUB_PERS_ID_SIZE },
                             '-maxlength', 
                               $CFG{ PUB_PRINTER_PUB_PERS_ID_MAXLENGTH }
                           );
  my @pers_name_params = ( '-name', 'pers_name',
                           '-default', $fields{ 'pers_name' },
                           '-size', $CFG{ PUB_PRINTER_PERS_NAME_SIZE },
                           '-maxlength', 
                             $CFG{ PUB_PRINTER_PERS_NAME_MAXLENGTH },
                           '-readonly', 'readonly'
                         );  
  # если издатель -- "организация" (2) или издатель не известен, то запрещаем
  # ввод в эти поля
  if ( $fields{ 'pers_org_flag' } == 2 || $fields{ 'pers_org_flag' } eq '' )
  {
    push( @pub_pers_id_params, '-disabled', 'disabled' );
    push( @pers_name_params, '-disabled', 'disabled' );
  }
  
  print $query->textfield( @pub_pers_id_params ), "&nbsp;&nbsp;",
        $query->textfield( @pers_name_params ), "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" 
            onclick=
            "if ( document.forms[ 0 ].pub_pers_id.disabled == false )
               select_person();
             else
               window.alert( 'Поле заблокировано. Ввод невозможен' );
            ">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак \"издатель/изготовитель\"</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'pub_print_flag',
                            -default => $fields{ 'pub_print_flag' },
                            -values => [ '1', '2', '3' ],
                            -labels => { '1' => 'издатель',
                                         '2' => 'изготовитель',
                                         '3' => 'другое' }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Функция издателя/изготовителя</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pub_func", "PUB_PRINTER_PUB_FUNC" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'pub_func',
                           -default => $fields{ 'pub_func' },
                           -size => $CFG{ PUB_PRINTER_PUB_FUNC_SIZE },
                           -maxlength =>
                             $CFG{ PUB_PRINTER_PUB_FUNC_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак \"без издателя (sine nomine)\"</th>\n";
  print "<td>",
        $query->checkbox( -name => 'sine_nomine_flag',
                          -checked => 
  # будем ориентироваться на значение поля "Флаг 'персоналия/организация'"
  # TODO Подумать над тем, чтобы вообще убрать из конфиг. файла e_arch.cfg
  #      параметр PUB_PRINTER_SINE_NOMINE_FLAG
                            $fields{ 'pers_org_flag' } eq '' ? '1' : '0',
                          -value => 'yes',
                          -label => '',
  # значения этого признака изменяются только с помощью кода на языке
  # JavaScript
                          -disabled => 'disabled'
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак точно установленного издателя/изготовителя<br />" .
        "(если издатель/изготовитель установлен не точно, тогда при " .
        "указании издателя/изготовителя используются скобки &mdash; [])" .
        "</th>\n";
  print "<td>";

  # массив параметров для поля pub_print_sure_flag
  # будем ориентироваться на значение поля "Флаг 'персоналия/организация'"
  my @pub_print_sure_flag_params =
     ( '-name', 'pub_print_sure_flag',
       '-checked', $fields{ 'pers_org_flag' } ne '' ?
                   ( $fields{ 'pub_print_sure_flag' } eq 't' ? '1' : '0' ) :
                   0,
       '-value', 'yes',
       '-label', ''
     );
  # если тип издателя не известен, то запрещаем ввод в это поле
  if ( $fields{ 'pers_org_flag' } eq '' )
  {
    push( @pub_print_sure_flag_params, '-disabled', 'disabled' );
  }

  print $query->checkbox( @pub_print_sure_flag_params ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дополнительные сведения к имени издателя/изготовителя<br />" .
            "(например, когда одноименные издатели есть в разных странах)" .
        "</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].pub_print_add_info",
                   "PUB_PRINTER_PUB_PRINT_ADD_INFO" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'pub_print_add_info',
                          -default => $fields{ 'pub_print_add_info' },
                          -rows => $CFG{ PUB_PRINTER_PUB_PRINT_ADD_INFO_ROWS },
                          -columns => 
                            $CFG{ PUB_PRINTER_PUB_PRINT_ADD_INFO_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "PUB_PRINTER_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ PUB_PRINTER_COMMENT_ROWS },
                          -columns => $CFG{ PUB_PRINTER_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;

  # поскольку эта функция обрабатывает только поля-флаги 'sine_nomine_flag',
  # 'pub_print_sure_flag', имеющие тип bool, то проверяем значение параметра
  # только на непустоту
  $value = $query->param( $fname ) ? 'yes' : 'no';

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Издатель, изготовитель, распространитель" );
  
  return ( 0 );
}
