#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Содержание справочных материалов объектов библиографического
#            описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'REFS_CONTENTS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Refs_contents
my @f_names =
   ( 'obj_id',             # Уникальный идентификатор объекта
                           # библиографического описания
     'bib_rec_num',        # Номер библиогр. записи для данного объекта
                           # (основная запись имеет номер 0)
     'ref_num',            # Порядковый номер (уникальный код) справочного 
                           # материала в пределах данного объекта биб. описания
     'ref_elem',           # Элемент справочного материала
                           # (например, термин из предметного указателя)
     'ref_elem_descr',     # Описание элемента справочного материала
                           # (например, номера страниц, на которых
                           # упоминается персоналия, или толкование
                           # термина из глоссария)
     'lang_code',          # Код языка, на котором записан элемент
                           # справочного материала (1 - русский язык)
     'process_code',       # Код вида языковой обработки элемента
                           # справочного материала (1 - оригинальный,
                           # т.е. в том виде, как он приведен в источнике
                           # биб. описания)
     'comment',            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'ref_elem' ) eq '' )
  {
    push( @error_messages, "Не указан элемент справочного материала" );
  }

  # почему-то проверка сделана только на наличие каких-либо данных в поле,
  # но тип данных не проверяется, хотя аналогичные числовые поля имеют
  # проверку типа !~ /^\d+$/
  # TODO. Привести подобные проверки к единой схеме во всех модулях.
  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  if ( $query->param( 'process_code' ) eq '' )
  {
    push( @error_messages, "Не указан код вида языковой обработки элемента " .
          "справочного материала" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $ref_num;            # порядковый номер справочного материала
                          # в пределах данного объекта биб. описания
  my $ref_elem;           # элемент справочного материала
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $ref_num     = $query->param( 'ref_num' );
  $ref_elem    = $query->param( 'ref_elem' );
  $target_win  = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}refs_contents " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "ref_num = '$ref_num' AND " .
                   "ref_elem = '$ref_elem'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();        # начинаем оформление верхней части окна
  create_top();      # формируем верхнюю часть окна
  close_top();       # завершаем оформление верхней части окна

  print_add_info();  # выведем дополнительную информацию на экран

  open_bottom();     # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();    # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех элементов (содержания) конкретного справочного материала
# для данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля refs_descrs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $ref_num;            # порядковый номер справочного материала
                          # в пределах данного объекта биб. описания
  my $ref_descr;          # описание справочного материала
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки
  my $ref_elem_descr;     # описание элемента справочного материала
                          # (переменная создана для удобства)
  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $ref_num     = $query->param( 'ref_num' );
  $ref_descr   = $query->param( 'ref_descr' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec   = CGI::escape( $bib_rec );
  $ref_descr = CGI::escape( $ref_descr );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&ref_num=$ref_num" .
          "&ref_descr=$ref_descr" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи об элементах конкретного справочного материала 
  # для данного объекта
  # (получим также краткое наименования языка, на котором записан элемент)
  $command = "SELECT rc.*, l.lang_sname " .
             "FROM ${db_schema}refs_contents rc, ${db_schema}langs l " .
             "WHERE rc.obj_id = $obj_id AND " .
                   "rc.bib_rec_num = $bib_rec_num AND " .
                   "rc.ref_num = $ref_num AND " .
                   "rc.lang_code = l.lang_code " .
             "ORDER BY rc.ref_elem";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Элемент</th>\n";
    print "<th>Описание элемента</th>\n";
    print "<th>Язык элемента</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим элемент, его описание (первые символы, число которых
      # определяется в конфиг. файле) и язык, на котором записан элемент
      print "<td>$row[ $result->fnumber( 'ref_elem' ) ]</td>\n";
      print "<td>";

      # используем переменную для упрощения кода
      $ref_elem_descr = $row[ $result->fnumber( 'ref_elem_descr' ) ];

      # если параметр в конфиг. файле не указан или разрешенное число 
      # выводимых символов не меньше длины всего описания, то выводим описание
      # полностью
      if ( ! $CFG{ REFS_CONTENTS_REF_ELEM_DESCR_DISPLAY_LEN } ||
           length( $ref_elem_descr ) <= 
             $CFG{ REFS_CONTENTS_REF_ELEM_DESCR_DISPLAY_LEN }
         )
      {
        print $ref_elem_descr;
      }
      elsif ( length( $ref_elem_descr ) >
              $CFG{ REFS_CONTENTS_REF_ELEM_DESCR_DISPLAY_LEN }
            )
      {
        print substr( $ref_elem_descr, 0, 
                      $CFG{ REFS_CONTENTS_REF_ELEM_DESCR_DISPLAY_LEN } );
        print "...";     # добавим признак незавершенности выведенных данных
      }      
      print "</td>\n";
      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # просмотр данной записи об элементе справочного материала в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&ref_elem=" . 
              CGI::escape( $row[ $result->fnumber( 'ref_elem' ) ] ) . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи об элементе справочного материала
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&ref_elem=" . 
              CGI::escape( $row[ $result->fnumber( 'ref_elem' ) ] ) . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи об элементе справочного материала
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $ref_num;            # порядковый номер справочного материала
                          # в пределах данного объекта биб. описания
  my $ref_elem;           # элемент справочного материала
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $ref_num     = $query->param( 'ref_num' );
  $ref_elem    = $query->param( 'ref_elem' );
  $target_win  = $query->param( 'target_win' );

  # выбираем одну запись об элементе справочного материала для данного объекта
  # (получим также краткое наименования языка, на котором записан элемент,
  # и вид языковой обработки этого элемента)
  $command = "SELECT rc.*, l.lang_sname, p.process_sname " .
             "FROM ${db_schema}refs_contents rc, ${db_schema}langs l, " .
                  "${db_schema}process_types p " .
             "WHERE rc.obj_id = $obj_id AND " .
                   "rc.bib_rec_num = $bib_rec_num AND " .
                   "rc.ref_num = $ref_num AND " .
                   "rc.ref_elem = '$ref_elem' AND " .
                   "rc.lang_code = l.lang_code AND " .
                   "rc.process_code = p.process_code";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "номер описания справочного материала - $ref_num, " .
                     "элемент справочного материала - $ref_elem",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "refs_contents",
                       key => "obj_id, bib_rec_num, ref_num," .
                              "ref_elem:ref_elem_begin",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны элементы справочных материалов)
  my( $ref_values_langs, $ref_labels_langs ) = popup_langs_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов языковой обработки элементов справочных материалов)
  my( $ref_values_procs, $ref_labels_procs ) =
    popup_process_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения трех ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );
    $fields{ 'ref_num' }     = $query->param( 'ref_num' );

    # значения по умолчанию

    # элемент справочного материала
    $fields{ 'ref_elem' }       = $CFG{ REFS_CONTENTS_REF_ELEM };

    # описание элемента справочного материала
    $fields{ 'ref_elem_descr' } = $CFG{ REFS_CONTENTS_REF_ELEM_DESCR };

    # код языка
    $fields{ 'lang_code' }      = $CFG{ REFS_CONTENTS_LANG_CODE };

    # код вида языковой обработки описания справочного материала
    $fields{ 'process_code' }   = $CFG{ REFS_CONTENTS_PROCESS_CODE };

    # комментарий
    $fields{ 'comment' }        = $CFG{ REFS_CONTENTS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания", "Номер библиогр. записи для данного
  # объекта" и "Порядковый номер справочного материала" не подлежат 
  # редактированию, не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа, а также при выполнении операции INSERT
  # (значения этих полей сохраняются такими же, какими они были при
  # предыдущем запуске скрипта, поэтому можно не присваивать им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";
  print $query->hidden( -name => 'ref_num' ), "\n";

  # создадим скрытое поле для "Элемента справочного материала"
  # ПРИМЕЧАНИЕ. Идентификаторы скрытых полей имеют постфикс 'begin' для того,
  #             чтобы можно было обновлять значения этих полей, поскольку они
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужны и исходные значения этих полей, и их новые
  #             значения (которые, возможно, были изменены пользователем).
  print $query->hidden( -name => 'ref_elem_begin',
                        -default => $fields{ 'ref_elem' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание и описание
  # справочного материала следующему вызову скрипта, когда будет работать
  # функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";
  print $query->hidden( -name => 'ref_descr' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();           # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Элемент справочного материала</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].ref_elem",
                   "REFS_CONTENTS_REF_ELEM" ) if $CFG{ HOT_HELP_IS_ON };
  
  print $query->textarea( -name => 'ref_elem',
                          -default => $fields{ 'ref_elem' },
                          -rows => $CFG{ REFS_CONTENTS_REF_ELEM_ROWS },
                          -columns => $CFG{ REFS_CONTENTS_REF_ELEM_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Описание элемента справочного материала</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].ref_elem_descr",
                   "REFS_CONTENTS_REF_ELEM_DESCR" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'ref_elem_descr',
                          -default => $fields{ 'ref_elem_descr' },
                          -rows => $CFG{ REFS_CONTENTS_REF_ELEM_DESCR_ROWS },
                          -columns => 
                            $CFG{ REFS_CONTENTS_REF_ELEM_DESCR_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Язык элемента справочного материала</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_process_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид языковой обработки элемента справочного материала</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'process_code',
                            -default => $fields{ 'process_code' },
                            -values => $ref_values_procs,
                            -labels => { @{ $ref_labels_procs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "REFS_CONTENTS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ REFS_CONTENTS_COMMENT_ROWS },
                          -columns => $CFG{ REFS_CONTENTS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Содержание справочного материала объекта " .
                    "библиографического описания", $module );
  return ( 0 );
}
