#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Описания справочных материалов объектов библиографического 
#            описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'REFS_DESCRS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Refs_descrs
my @f_names =
   ( 'obj_id',             # Уникальный идентификатор объекта
                           # библиографического описания
     'bib_rec_num',        # Номер библиогр. записи для данного объекта
                           # (основная запись имеет номер 0)
     'ref_num',            # Порядковый номер (уникальный код) справочного 
                           # материала в пределах данного объекта биб. описания
     'ref_type_code',      # Код вида справочного материала
     'ref_descr',          # Описание (название) справочного материала,
                           # взятое из объекта биб. описания
                           # (если материал сформирован пользователем,
                           # то и описание должен сформировать он сам)
     'ref_vol',            # Объем справочного материала или номера страниц,
                           # на которых он размещен
     'ref_original_flag',  # Признак оригинального материала
                           # (если материал сформирован пользователем,
                           # то будет значение 'no')
     'ref_complete_flag',  # Признак полноты представления справочного
                           # материала в базе данных
     'lang_code',          # Код языка, на котором приведено описание
                           # справочного материала (1 - русский язык)
     'process_code',       # Код вида языковой обработки описания
                           # справочного материала (1 - оригинальное,
                           # т.е. в том виде, как приведено в источнике
                           # биб. описания или как сформировано
                           # пользователем)
     'comment',            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # порядковый номер справочного материала в пределах данного объекта
  # биб. описания
  if ( $query->param( 'ref_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный порядковый номер " .
          "справочного материала" );
  }
  elsif ( $query->param( 'ref_num' ) <= 0 )
  {
    push( @error_messages, "Недопустимое значение порядкового номера " .
          "справочного материала" );
  }

  # код вида справочного материала
  if ( $query->param( 'ref_type_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код вида справочного " .
          "материала" );
  }

  if ( $query->param( 'ref_descr' ) eq '' )
  {
    push( @error_messages, "Не указано описание (название) справочного " .
          "материала" );
  }

  # почему-то проверка сделана только на наличие каких-либо данных в поле,
  # но тип данных не проверяется, хотя аналогичные числовые поля имеют
  # проверку типа !~ /^\d+$/
  # TODO. Привести подобные проверки к единой схеме во всех модулях.
  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  if ( $query->param( 'process_code' ) eq '' )
  {
    push( @error_messages, "Не указан код вида языковой обработки описания " .
          "справочного материала" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $ref_num;            # порядковый номер справочного материала
                          # в пределах данного объекта биб. описания
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $ref_num     = $query->param( 'ref_num' );
  $target_win  = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}refs_descrs " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "ref_num = '$ref_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех справочных материалов для данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи об описаниях справочных материалов для данного объекта
  # (получим также краткое наименования языка, на котором записано описание)
  # подсчитаем число элементов в каждом справочном материале
  $command = "SELECT rd.obj_id, rd.bib_rec_num, rd.ref_num, rd.ref_descr, " .
                    "rt.ref_type_sname, l.lang_sname, " .
                    "COUNT( rc.* ) AS num_elems " .
             "FROM ${db_schema}refs_descrs rd " .
                  "LEFT JOIN ${db_schema}refs_contents rc ON " .
                  "rd.obj_id = rc.obj_id AND " .
                  "rd.bib_rec_num = rc.bib_rec_num AND " .
                  "rd.ref_num = rc.ref_num, " .
                  "${db_schema}ref_types rt, ${db_schema}langs l " .
             "WHERE rd.obj_id = $obj_id AND " .
                   "rd.bib_rec_num = $bib_rec_num AND " .
                   "rd.ref_type_code = rt.ref_type_code AND " .
                   "rd.lang_code = l.lang_code " .
             "GROUP BY rd.obj_id, rd.bib_rec_num, rd.ref_num, rd.ref_descr, " .
                      "rt.ref_type_sname, l.lang_sname " .
             "ORDER BY rd.ref_num";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Номер п/п</th>\n";
    print "<th>Вид материала</th>\n";
    print "<th>Описание материала</th>\n";
    print "<th>Число элементов</th>\n";
    print "<th>Язык описания</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим порядковый номер справочного материала, его вид, описание,
      # число элементов в данном справочном материале и язык, на котором
      # сделано описание материала
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'ref_num' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'ref_type_sname' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'ref_descr' ) ]</td>\n";
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'num_elems' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # просмотр содержания данного справочного материала в новом окне;
      # просмотр описания данного справочного материала в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" } .
            qq{href="/$CFG\{ HOME_WEB_DIR \}/refs_contents.pl/query?$href} .
            "&ref_num=" . $row[ $result->fnumber( 'ref_num' ) ] .
            "&ref_descr=" .
            CGI::escape( $row[ $result->fnumber( 'ref_descr' ) ] ) . "\"" .
            qq{ target="_blank">Содержание</a>}, "\n",
            qq{<a class="command_link" href="$script_name/view?$href} .
            "&ref_num=" . $row[ $result->fnumber( 'ref_num' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о справочном материале
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&ref_num=" . $row[ $result->fnumber( 'ref_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о справочном материале
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $ref_num;            # порядковый номер справочного материала
                          # в пределах данного объекта биб. описания
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $ref_num     = $query->param( 'ref_num' );
  $target_win  = $query->param( 'target_win' );

  # выбираем одну запись об описании справочного материала для данного объекта
  # (получим также краткое наименования языка, на котором записано описание,
  # и вид языковой обработки этого описания)
  $command = "SELECT rd.*, rt.ref_type_sname, l.lang_sname, p.process_sname " .
             "FROM ${db_schema}refs_descrs rd, ${db_schema}ref_types rt, " .
                  "${db_schema}langs l, ${db_schema}process_types p " .
             "WHERE rd.obj_id = $obj_id AND " .
                   "rd.bib_rec_num = $bib_rec_num AND " .
                   "rd.ref_num = $ref_num AND " .
                   "rd.ref_type_code = rt.ref_type_code AND " .
                   "rd.lang_code = l.lang_code AND " .
                   "rd.process_code = p.process_code";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "номер описания справочного материала - $ref_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "refs_descrs",
                       key => "obj_id, bib_rec_num, " .
                              "ref_num:ref_num_begin",
                       f_names => \@f_names,
                       f_special => [ 'ref_original_flag',
                                      'ref_complete_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # видов справочных материалов)
  my( $ref_values_ref_types, $ref_labels_ref_types ) =
    popup_ref_types_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны описания справочных материалов)
  my( $ref_values_langs, $ref_labels_langs ) = popup_langs_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов языковой обработки описаний справочных материалов)
  my( $ref_values_procs, $ref_labels_procs ) =
    popup_process_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );

    # значения по умолчанию

    # номер справочного материала
    if ( $CFG{ REFS_DESCRS_REF_NUM } )  # есть параметр в конфиг. файле
    {
      $fields{ 'ref_num' } = $CFG{ REFS_DESCRS_REF_NUM };
    }
    else
    {
      # будем предлагать пользователю значение поля ref_num для новой
      # записи, которое на 1 больше максимального значения этого поля 
      # для данного номера биб. записи данного объекта библиогр. описания
      # (для значений полей obj_id и bib_rec_num в таблице Refs_descrs)
      $fields{ 'ref_num' } = 
        get_next_key_value( 'refs_descrs', 'ref_num',
                            'obj_id = ' . $query->param( 'obj_id' ) . ' AND ' .
                            'bib_rec_num = ' . $query->param( 'bib_rec_num' )
                          );
    }

    # код вида справочного материала
    $fields{ 'ref_type_code' }     = $CFG{ REFS_DESCRS_REF_TYPE_CODE };

    # описание (название) справочного материала,
    # взятое из объекта биб. описания
    $fields{ 'ref_descr' }         = $CFG{ REFS_DESCRS_REF_DESCR };

    # объем справочного материала или номера страниц, на которых он размещен
    $fields{ 'ref_vol' }           = $CFG{ REFS_DESCRS_REF_VOL };

    # признак оригинального материала
    $fields{ 'ref_original_flag' } = $CFG{ REFS_DESCRS_REF_ORIGINAL_FLAG };

    # признак полноты представления справочного материала в базе данных
    $fields{ 'ref_complete_flag' } = $CFG{ REFS_DESCRS_REF_COMPLETE_FLAG };

    # код языка
    $fields{ 'lang_code' }         = $CFG{ REFS_DESCRS_LANG_CODE };

    # код вида языковой обработки описания справочного материала
    $fields{ 'process_code' }      = $CFG{ REFS_DESCRS_PROCESS_CODE };

    # комментарий
    $fields{ 'comment' }           = $CFG{ REFS_DESCRS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи для данного
  # объекта" не подлежат редактированию, не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа, а также при выполнении операции INSERT
  # (значения этих полей сохраняются такими же, какими они были при
  # предыдущем запуске скрипта, поэтому можно не присваивать им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";

  # создадим скрытое поле для "Порядкового номера справочного материала ..."
  # ПРИМЕЧАНИЕ. Идентификаторы скрытых полей имеют постфикс 'begin' для того,
  #             чтобы можно было обновлять значения этих полей, поскольку они
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужны и исходные значения этих полей, и их новые
  #             значения (которые, возможно, были изменены пользователем).
  print $query->hidden( -name => 'ref_num_begin',
                        -default => $fields{ 'ref_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();            # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Порядковый номер справочного материала</th>\n";
  print "<td>",
        $query->textfield( -name => 'ref_num',
                           -default => $fields{ 'ref_num' },
                           -size => $CFG{ REFS_DESCRS_REF_NUM_SIZE },
                           -maxlength => $CFG{ REFS_DESCRS_REF_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_ref_types_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид справочного материала</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'ref_type_code',
                            -default => $fields{ 'ref_type_code' },
                            -values => $ref_values_ref_types,
                            -labels => { @{ $ref_labels_ref_types } }
                          ),
        "</td>\n";
  print "</tr>\n";


  print "<tr>\n";
  print "<th>Описание (название) справочного материала</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].ref_descr",
                   "REFS_DESCRS_REF_DESCR" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'ref_descr',
                          -default => $fields{ 'ref_descr' },
                          -rows => $CFG{ REFS_DESCRS_REF_DESCR_ROWS },
                          -columns => $CFG{ REFS_DESCRS_REF_DESCR_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Объем справочного материала или номера страниц, на которых " .
            "он размещен</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].ref_vol",
                   "REFS_DESCRS_REF_VOL" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'ref_vol',
                           -default => $fields{ 'ref_vol' },
                           -size => $CFG{ REFS_DESCRS_REF_VOL_SIZE },
                           -maxlength => $CFG{ REFS_DESCRS_REF_VOL_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак оригинального материала<br />" .
            "(если материал сформирован пользователем, то поле будет " .
            "пустым)</th>\n";
  print "<td>",
        $query->checkbox( -name => 'ref_original_flag',
                          -checked => 
                             $fields{ 'ref_original_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак полноты представления справочного " .
            "материала в базе данных</th>\n";
  print "<td>",
        $query->checkbox( -name => 'ref_complete_flag',
                          -checked => 
                             $fields{ 'ref_complete_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Язык описания справочного материала</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_process_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид языковой обработки описания справочного материала</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'process_code',
                            -default => $fields{ 'process_code' },
                            -values => $ref_values_procs,
                            -labels => { @{ $ref_labels_procs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "REFS_DESCRS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ REFS_DESCRS_COMMENT_ROWS },
                          -columns => $CFG{ REFS_DESCRS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Признак оригинального материала
  if ( $fname eq 'ref_original_flag' )
  {
    # это поле имеет тип bool, поэтому проверяем значение параметра
    # только на непустоту
    $value = $query->param( $fname ) ? 'yes' : 'no';
  }
  # Признак полноты представления справочного материала в базе данных
  elsif ( $fname eq 'ref_complete_flag' )
  {
    # это поле имеет тип bool, поэтому проверяем значение параметра
    # только на непустоту
    $value = $query->param( $fname ) ? 'yes' : 'no';
  }
  
  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Описания справочных материалов объекта " .
                    "библиографического описания" );
  return ( 0 );
}
