#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   справочник "Научные специальности"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'SCI_SPECS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Sci_specs
my @f_names =
   ( 'sci_spec_code',     # Код научной специальности
     'sci_spec_cipher',   # Шифр научной специальности
     'sci_spec_name',     # Наименование научной специальности
     'sci_spec_date',     # Дата введения научной специальности
     'sci_spec_term',     # Срок действия научной специальности
     'comment'            # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'sci_spec_code' ) eq '' )
  {
    push( @error_messages, "Не указан код научной специальности" );
  }

  # не допускается пустое поле; значение должно иметь формат вида "05.13.01"
  if ( $query->param( 'sci_spec_cipher' ) !~ /^\d\d\.\d\d\.\d\d$/ )
  {
    push( @error_messages, "Не указан или неверный шифр научной " .
          "специальности" );
  }

  if ( $query->param( 'sci_spec_name' ) eq '' )
  {
    push( @error_messages, "Не указано наименование научной специальности" );
  }

  # НЕ допускается пустое поле
  if ( ! date_is_correct( $query->param( 'sci_spec_date' ) ) )
  {
    push( @error_messages, "Не введена либо неверная дата введения " .
          "научной специальности" );
  }

  # НЕ допускается пустое поле
  if ( ! date_is_correct( $query->param( 'sci_spec_term' ) ) )
  {
    push( @error_messages, "Не введен либо неверный срок действия " .
          "научной специальности" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Научные специальности',
                    user => $user
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $sci_spec_code;         # код научной специальности
  my $command;               # строка команды для работы с БД

  $sci_spec_code = $query->param( 'sci_spec_code' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}sci_specs " .
             "WHERE sci_spec_code = '$sci_spec_code'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  my $sci_spec_code;         # код научной специальности
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $sci_spec_code = $query->param( 'sci_spec_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Sci_specs WHERE sci_spec_code = $sci_spec_code";
  push( @table_cmd_pairs, [ 'Sci_specs', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  my $sci_spec_code;         # код научной специальности
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $sci_spec_code = $query->param( 'sci_spec_code' );

  # сформируем строку-команду для выборки данных из таблицы и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Sci_specs " .
             "WHERE sci_spec_code = $sci_spec_code";
  push( @table_cmd_pairs, [ 'Sci_specs', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех научных специальностей
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выбираем все записи о научных специальностях
  $command = "SELECT * " .
             "FROM ${db_schema}sci_specs " .
             "ORDER BY sci_spec_cipher, sci_spec_date";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Код научной специальности</th>\n";
    print "<th>Шифр научной специальности</th>\n";
    print "<th>Наименование</th>\n";
    print "<th>Дата введения</th>\n";
    print "<th>Срок действия</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # ПРИМЕЧАНИЕ. Этот фрагмент добавлен в расчете на будущее: этот модуль
    #             предположительно будет использоваться в режиме справочника.

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретного города (места издания) ...
    #if ( $query->param( 'enable_select' ) )
    #{
    #  # ... то сформируем функцию JavaScript для присваивания значений полей
    #  # из текущей записи таблицы Sci_specs полям в ВЫЗЫВАЮЩЕМ окне ...
    #  if ( $query->param( 'caller_name' ) eq '__________' )
    #  {
    #    # ... полям sci_spec_code, sci_spec_cipher и sci_spec_name
    #    # в модуле ______________
    #    create_assign_values_func( 'sci_spec_code', 'sci_spec_cipher',
    #                               'sci_spec_name' );
    #  }
    #}

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим код, шифр, наименование, дату введения и срок действия
      # научной специальности
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'sci_spec_code' ) ]</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретного города (места издания), то значение
      # поля sci_spec_cipher выведем в виде ссылки
      if ( $query->param( 'enable_select' ) )
      {
        print "<td class=\"center\">",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( 
                     $row[ $result->fnumber( 'sci_spec_code' ) ],
                     '$row[ $result->fnumber( 'sci_spec_cipher' ) ]',
                     '$row[ $result->fnumber( 'sci_spec_name' ) ]'
                   )">
                   $row[ $result->fnumber( 'city_fname' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td class=\"center\">" .
              "$row[ $result->fnumber( 'sci_spec_cipher' ) ]</td>\n";
      }

      print "<td>$row[ $result->fnumber( 'sci_spec_name' ) ]</td>\n";
      print "<td>" .
            date_for_display( $row[ $result->fnumber( 'sci_spec_date' ) ] ) .
            "</td>\n";
      print "<td>" .
            date_for_display( $row[ $result->fnumber( 'sci_spec_term' ) ] ) .
            "</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "sci_spec_code=" . $row[ $result->fnumber( 'sci_spec_code' ) ] .
              $add_params;

      # просмотр описания научной специальности в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи о научной специальности
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $sci_spec_code;         # код научной специальности
  my $command;               # строка команды для работы с БД

  $sci_spec_code = $query->param( 'sci_spec_code' );

  # выбираем одну запись о научной специальности
  $command = "SELECT * " .
             "FROM ${db_schema}sci_specs " .
             "WHERE sci_spec_code = '$sci_spec_code'"; 

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "научной специальности " . $sci_spec_code,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  # параметр
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "sci_specs",
                       key => "sci_spec_code",
                       f_names => \@f_names,
                       f_special => [ 'sci_spec_date', 'sci_spec_term' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'sci_spec_code' } = get_next_key_value( 'sci_specs',
                                                     'sci_spec_code' );
    # значения по умолчанию

    # шифр научной специальности
    $fields{ 'sci_spec_cipher' } = $CFG{ SCI_SPECS_SCI_SPEC_CIPHER };

    # наименование научной специальности
    $fields{ 'sci_spec_name' }   = $CFG{ SCI_SPECS_SCI_SPEC_NAME };

    # дата введения научной специальности
    $fields{ 'sci_spec_date' }   = $CFG{ SCI_SPECS_SCI_SPEC_DATE };

    # срок действия научной специальности
    $fields{ 'sci_spec_term' }   = $CFG{ SCI_SPECS_SCI_SPEC_TERM };

    # примечание
    $fields{ 'comment' }         = $CFG{ SCI_SPECS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Код научной специальности" формируется
  # автоматически с использованием функции get_next_key_value(), не будем
  # позволять его редактировать, но совсем убрать его нельзя, т.к. оно
  # используется при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'sci_spec_code',
                        -default => $fields{ 'sci_spec_code' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Код научной специальности</th>\n";
  print "<td>", $fields{ 'sci_spec_code' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Шифр научной специальности</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].sci_spec_cipher",
                   "SCI_SPECS_SCI_SPEC_CIPHER" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'sci_spec_cipher',
                           -default => $fields{ 'sci_spec_cipher' },
                           -size => $CFG{ SCI_SPECS_SCI_SPEC_CIPHER_SIZE },
                           -maxlength =>
                             $CFG{ SCI_SPECS_SCI_SPEC_CIPHER_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Наименование</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].sci_spec_name",
                   "SCI_SPECS_SCI_SPEC_NAME" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'sci_spec_name',
                          -default => $fields{ 'sci_spec_name' },
                          -rows => $CFG{ SCI_SPECS_SCI_SPEC_NAME_ROWS },
                          -columns => $CFG{ SCI_SPECS_SCI_SPEC_NAME_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата введения</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].sci_spec_date",
                   "SCI_SPECS_SCI_SPEC_DATE", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'sci_spec_date',
                           -default =>
                              date_for_display( $fields{ 'sci_spec_date' } ),
                           -size => $CFG{ SCI_SPECS_SCI_SPEC_DATE_SIZE },
                           -maxlength =>
                             $CFG{ SCI_SPECS_SCI_SPEC_DATE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Срок действия</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].sci_spec_term",
                   "SCI_SPECS_SCI_SPEC_TERM", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'sci_spec_term',
                           -default =>
                              date_for_display( $fields{ 'sci_spec_term' } ),
                           -size => $CFG{ SCI_SPECS_SCI_SPEC_TERM_SIZE },
                           -maxlength =>
                             $CFG{ SCI_SPECS_SCI_SPEC_TERM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "SCI_SPECS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ SCI_SPECS_COMMENT_ROWS },
                          -columns => $CFG{ SCI_SPECS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Дата введения валюты
  if ( $fname eq 'sci_spec_date' || $fname eq 'sci_spec_term' )
  {
    $value = date_for_db( $query->param( $fname ) );
  }

  return ( $value );  
}
