#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Описания серий/подсерий"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'SER_DESCRS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Ser_descrs
my @f_names =
   ( 'series_id',        # Уникальный идентификатор серии/подсерии
     'ser_descr_num',    # Номер описания серии/подсерии
                         # (это поле, аналогичное полю bib_rec_num)
                         # (основное описание имеет номер 0)
     'ser_descr',        # Описание серии/подсерии
     'lang_code',        # Код языка, на котором составлено
                         # описание серии/подсерии (1 - русский язык)
     'process_code',     # Код вида обработки описания серии/подсерии
                         # (1 - оригинальное, т.е. в том виде, как
                         # приведено в источнике библиограф. описания)
     'reason_add_descr', # Назначение дополнит. описания (или причина
                         # его составления)
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

# выбираем описания всех объектов, входящих в выбранную серию
# (с учетом ее конкретного описания)
if ( $query->path_info =~ /^\/list_bib_recs$/ )
{
  do_list_bib_recs();
}

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # должны быть только цифры (пустое поле не допускается)
  if ( $query->param( 'series_id' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный уникальный идентификатор " .
          "серии/подсерии" );
  }

  # должны быть только цифры (пустое поле не допускается)
  if ( $query->param( 'ser_descr_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный номер описания " .
          "серии/подсерии" );
  }

  if ( $query->param( 'ser_descr' ) eq '' )
  {
    push( @error_messages, "Не приведено описание серии/подсерии" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  if ( $query->param( 'process_code' ) eq '' )
  {
    push( @error_messages, "Не указан код вида обработки описания " .
          "серии/подсерии" );
  }

  # DEBUG
  # print_params( $query );
  
  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }

  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Описания серий/подсерий',
                    user => $user,
                    make_letters => 'yes',
                    num_last_recs => $CFG{ SER_DESCRS_NUM_LAST_RECS },
                    full_top => 'yes'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $series_id;         # уникальный идентификатор серии/подсерии
  my $ser_descr_num;     # номер описания серии/подсерии
  my $command;           # строка команды для работы с БД

  $series_id     = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}ser_descrs " .
             "WHERE series_id = '$series_id' AND " .
                   "ser_descr_num = '$ser_descr_num'";

  # используем типовую процедуру
  # TODO Видимо, придется написать нестандартную процедуру удаления, которая
  #      учитывала бы ситуации, когда удаляется последнее описание для
  #      данной серии/подсерии или не последнее описание. В первом случае нужно
  #      удалять также и запись из таблицы Series, а во втором - не нужно.
  #      При использовании же типовой процедуры запись из таблицы Series
  #      удаляться не будет никогда (что не всегда верно).
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  # ВАЖНО! Будем экспортировать только указанное описание для данной серии.
  
  my $series_id;             # уникальный идентификатор серии/подсерии
  my $ser_descr_num;         # номер описания серии/подсерии
  my $command;               # строка команды для работы с БД
  my $dep_table;             # имя зависимой таблицы
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $series_id = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );

  # сначала сформируем строку-команду для таблицы Ser_descrs и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Ser_descrs " .
             "WHERE series_id = $series_id AND " .
                   "ser_descr_num = $ser_descr_num";
  push( @table_cmd_pairs, [ 'Ser_descrs', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  # ВАЖНО! Будем импортировать только указанное описание для данной серии.
  
  my $series_id;             # уникальный идентификатор серии/подсерии
  my $ser_descr_num;         # номер описания серии/подсерии
  my $command;               # строка команды для работы с БД
  my $dep_table;             # имя зависимой таблицы
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $series_id = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );

  # сначала сформируем строку-команду для таблицы Ser_descrs и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Ser_descrs " .
             "WHERE series_id = $series_id AND " .
                   "ser_descr_num = $ser_descr_num";
  push( @table_cmd_pairs, [ 'Ser_descrs', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# выборка и вывод биб. описаний всех объектов, входящих в выбранную серию
# (с учетом ее конкретного описания)
# -----------------------------------------------------------------------------
sub do_list_bib_recs
{
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $series_id;                  # уникальный идентификатор серии/подсерии,
                                  # выбранной пользователем
  my $series_issn;                # ISSN серии/подсерии
  my $ser_descr_num;              # номер описания серии/подсерии
  my $ser_descr;                  # описание серии/подсерии
  my $headline;                   # заголовок списка биб. описаний

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  $series_id     = $query->param( 'series_id' );
  $series_issn   = $query->param( 'series_issn' );
  $ser_descr_num = $query->param( 'ser_descr_num' );
  $ser_descr     = $query->param( 'ser_descr' );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  # выведем уникальный идентификатор серии/подсерии и номер описания
  # серии/подсерии в качестве заголовка
  print "<p class=\"bib_rec\">";
  print "Идентификатор серии/подсерии: $series_id\n";
  print "&nbsp;&nbsp;\n";
  print "Номер описания для данной серии/подсерии: $ser_descr_num</p>\n";

  # выведем также описание серии/подсерии и ISSN (если есть)
  print "<p class=\"bib_rec\">$ser_descr<br />\n";
  print "ISSN: " . ( ( $series_issn ) ? $series_issn : "нет" ) . "</p>\n";

  # сформируем заголовок модуля
  $headline = "<h2 class=\"module_title\">Состав серии/подсерии</h2>\n";

  # выбираем записи с биб. описаниями объектов, привязанными к выбранному
  # описанию серии/подсерии (т. е. номеру серии/подсерии и номеру описания 
  # серии/подсерии)
  $command = "SELECT b.obj_id, b.bib_rec_num, " .
                    "CASE WHEN b.bib_rec_short IS NOT NULL " .
                         "THEN b.bib_rec_short " .
                         "ELSE b.bib_rec_full " .
                    "END AS bib_rec " .
             "FROM ${db_schema}obj_series os, ${db_schema}bib_recs b " .
             "WHERE os.series_id = $series_id AND " .
                   "os.ser_descr_num = $ser_descr_num AND " .
                   "os.obj_id = b.obj_id AND " .
                   "os.bib_rec_num = b.bib_rec_num " .
             "ORDER BY bib_rec";

  # вывод биб. описаний всех объектов, выбранных по запросу
  list_bib_recs_by_query( $query, $command, $headline );

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка тех описаний серий/подсерий, которые 
# начинаются с указанной буквы
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД
  my $letter;                     # первая буква описания серии/подсерии
  my $letter_lower;               # первая буква -- в нижнем регистре

  # число последних записей, введенных в таблицу, которые нужно выводить
  # при обращении по специальной ссылке
  my $num_last_recs;

  # условие WHERE для команды выборки
  my $where;

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки

  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();        # начинаем оформление верхней части окна
  create_top();      # формируем верхнюю часть окна
  close_top();       # завершаем оформление верхней части окна

  open_bottom();     # начинаем оформление нижней части окна

  $letter        = $query->param( 'letter' );
  $num_last_recs = $query->param( 'num_last_recs' );

  # если параметры не переданы, то просто завершаем работу досрочно - 
  # это происходит при первоначальном запуске скрипта
  # ПРИМЕЧАНИЕ. Проверка как на undef, так и на пустое значение '' выполнена
  #             на случай каких-либо проблем в CGI.pm.
  if ( ( $letter eq undef || $letter eq '' ) &&
       ( $num_last_recs eq undef || $num_last_recs eq '' )
     )
  {
    close_bottom();   # завершаем оформление нижней части окна
    print $query->end_html();
    return ( 0 );
  }

  # если пользователь выбрал ссылку для вывода только последних записей,
  # введенных в таблицу
  if ( $num_last_recs )
  {
    # ПРИМЕЧАНИЕ. Если даже требуемое число последних введенных записей будет
    #             больше числа всех записей, имеющихся в таблице, этот 
    #             оператор сработает. Если какие-то записи были удалены,
    #             вследствие чего образовались "дырки" в нумерации series_id,
    #             то просто будет выведено меньшее число записей.
    # ПРИМЕЧАНИЕ. В этом условии не учтен тот факт, что есть еще поле
    #             ser_descr_num, значение которого практически всегда равно
    #             нулю. Но если значение этого поля для каких-то описаний
    #             окажется не равным нулю (т. е. для некоторой серии имеется
    #             более одного описания), тогда возможны два исхода:
    #             - запись, введенная в таблицу одной из ПОСЛЕДНИХ, не попадает
    #             в выборку, если значение поля series_id для этого описания
    #             НЕ ПОПАДАЕТ в диапазон, формируемый параметром
    #             $num_last_recs, однако число записей в выборке будет таким,
    #             какого требует параметр $num_last_recs;
    #             - число записей в выборке будет больше, чем требует параметр
    #             $num_last_recs, если значение поля series_id для этого
    #             описания ПОПАДАЕТ в диапазон, формируемый параметром
    #             $num_last_recs
    $where = "WHERE d.series_id >= " .
                    "( SELECT max( series_id ) " .
                      "FROM ${db_schema}ser_descrs ) - " .
                    "$num_last_recs + 1 AND " .
                    "d.series_id = s.series_id ";
  }
  # пользователь выбрал конкретную букву или другой символ
  elsif ( $letter )
  {
    # ВАЖНО! Не удалять этот фрагмент. Пусть останется в исторических целях.

    # ПРИМЕЧАНИЕ. См. комментарий в модуле bib_recs.pl.
    #{
    #  use Encode qw( from_to );
    #  # если длина переменной $letter более одного байта (в частности, два 
    #  # байта), то считаем, что это UTF-8
    #  from_to( $letter, "utf-8", "koi8-r" ) if ( length( $letter ) > 1 );
    #}

    # для русских букв может потребоваться особая предварительная обработка
    $letter = reveal_rus_letter( $letter );

    # воспользуемся возможностями локализации для перевода буквы, выбранной
    # пользователем, в нижний регистр
    $letter_lower = change_case( $letter, 'lower' );

    # если первый символ -- двойные кавычки, то не учитываем его
    $where = "WHERE d.ser_descr ~ '^($letter|\"$letter|" .
                                    "$letter_lower|\"$letter_lower)' AND " .
                   "d.series_id = s.series_id ";
  }

  # выбираем все записи, которые удовлетворяют условию $where
  # (т. е. те, в которых значение поля ser_descr начинается с указанной буквы
  # или те, что были введены в таблицу последними);
  # при этом регистр символов не имеет значения (первая буква может быть 
  # и строчной);
  # выберем также ISSN серии
  $command = "SELECT d.*, s.series_issn " .
             "FROM ${db_schema}ser_descrs d, ${db_schema}series s " .
             $where .
  # переведем значение поля в нижний регистр для того, чтобы регистр символов
  # не влиял на порядок сортировки (наверное, можно было переводить и 
  # в верхний регистр -- все равно)
             "ORDER BY lower( substr( d.ser_descr, 1, 100 ) )";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если это список записей, введенных в таблицу последними, то укажем это
    if ( $num_last_recs )
    {
    print "<h3>Записи, введенные в базу данных последними (" . 
          $num_last_recs . " шт.)</h3>\n";
    }

    # если этот модуль вызван в режиме справочника, и при этом он вызван
    # "сам из себя", то запретим операции просмотра и удаления записей
    # (поскольку режим корректировки выполняется из режима просмотра,
    # то и он будет недоступен)
    if ( $query->param( 'enable_select' ) &&
         $query->param( 'caller_name') eq $module )
    {
      $can_view = 0;
      $can_delete = 0;
    }
    else
    {
      $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                               'user' => $user );
      $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                                 'user' => $user );
    }

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>ID</th>\n";
    print "<th>ISSN</th>\n";
    print "<th>Номер описания</th>\n";
    print "<th>Описание серии/подсерии</th>\n";
    print "<th>&nbsp;</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе конкретной серии/подсерии ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Ser_descrs полям в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'OBJ_SERIES' )
      {
        # ... полям series_id, ser_descr_num и ser_descr в модуле OBJ_SERIES
        # ПРИМЕЧАНИЕ. В данном случае специфика фактических аргументов
        #             не совсем соответствует назначениям формальных параметров
        #             (см. функцию assign_values() в пакете E_ARCH.pm).
        create_assign_values_func( 'series_id', 'ser_descr_num', 'ser_descr' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим уникальный идентификатор серии/подсерии, ISSN, номер описания
      # серии/подсерии и описание серии/подсерии
      print "<td class=\"center\">" .
            $row[ $result->fnumber( 'series_id' ) ] . "</td>\n";
      print "<td>$row[ $result->fnumber( 'series_issn' ) ]</td>\n";
      print "<td class=\"center\">" .
            $row[ $result->fnumber( 'ser_descr_num' ) ] . "</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретной серии/подсерии, то значение поля
      # ser_descr выведем в виде ссылки

      if ( $query->param( 'enable_select' ) )
      {
        # экранируем кавычки и другие "неудобные" символы
        # (см. комментарий в модуле obj_groups_descrs.pl)
        # выполним добавочную обработку значения переменной, т. к. того, что
        # делает функция escapeHTML(), недостаточно
        my $ser_descr = $row[ $result->fnumber( 'ser_descr' ) ];
        $ser_descr =~ s/\'/\\\'/g;
        $ser_descr = CGI::escapeHTML( $ser_descr );

        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( $row[ $result->fnumber( 'series_id' ) ],
                                  $row[ $result->fnumber( 'ser_descr_num' ) ],
                                  '$ser_descr' )">
                   $row[ $result->fnumber( 'ser_descr' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'ser_descr' ) ]</td>\n";
      }

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "series_id=" . $row[ $result->fnumber( 'series_id' ) ] .
              "&ser_descr_num=" . $row[ $result->fnumber( 'ser_descr_num' ) ] .
              $add_params;

      # просмотр биб. описаний объектов, входящих в выбранную серию/подсерию
      # (с учетом ее конкретного описания)
      print "<td>",
            qq{<a class="command_link" } .
            qq{href="$script_name/list_bib_recs?$href} .
            "&series_issn=" . $row[ $result->fnumber( 'series_issn' ) ] .
            "&ser_descr=" . 
            CGI::escape( $row[ $result->fnumber( 'ser_descr' ) ] ) . "\"" .
            qq{ target="_blank">Состав</a>},
            "</td>\n";

      # просмотр описания в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a>} . "\n" if $can_view;

      # удаление записи 
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a>} . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $series_id;             # уникальный идентификатор серии/подсерии
  my $ser_descr_num;         # номер описания серии/подсерии
  my $command;               # строка команды для работы с БД

  $series_id     = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );

  # выбираем одно описание серии/подсерии 
  # выберем также ISSN серии
  $command = "SELECT d.*, s.series_issn " .
             "FROM ${db_schema}ser_descrs d, ${db_schema}series s " .
             "WHERE d.series_id = '$series_id' AND " .
                   "d.ser_descr_num = '$ser_descr_num' AND " . 
                   "d.series_id = s.series_id";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с " .
                     "уникальным идентификатором серии/подсерии " . $series_id .
                     " и номером описания " . $ser_descr_num,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "ser_descrs",
                       key => "series_id:series_id_begin, " .
                              "ser_descr_num:ser_descr_num_begin",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых выполнено биб. описание)
  my( $ref_values_langs, $ref_labels_langs ) = popup_langs_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов языковой обработки биб. описания)
  my( $ref_values_procs, $ref_labels_procs ) =
    popup_process_types_list( $db_schema );

  # типовая часть HTML-ссылок для запуска режимов работы с "деталировками"
  # описаний серий/подсерий
  my $href;
  
  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # значения по умолчанию

    # номер описания серии/подсерии (это поле, аналогичное полю bib_rec_num)
    # (основное описание имеет номер 0)
    $fields{ 'ser_descr_num' }    = $CFG{ SER_DESCRS_SER_DESCR_NUM };

    # описание серии/подсерии
    $fields{ 'ser_descr' }        = $CFG{ SER_DESCRS_SER_DESCR };

    # код языка, на котором составлено описание серии/подсерии
    # (1 - русский язык)
    $fields{ 'lang_code' }        = $CFG{ SER_DESCRS_LANG_CODE };

    # код вида обработки описания серии/подсерии
    # (1 - оригинальное, т.е. в том виде, как приведено в источнике 
    # библиограф. описания)
    $fields{ 'process_code' }     = $CFG{ SER_DESCRS_PROCESS_CODE };

    # назначение дополнит. описания (или причина его составления)
    $fields{ 'reason_add_descr' } = $CFG{ SER_DESCRS_REASON_ADD_DESCR };

    # примечание
    $fields{ 'comment' }          = $CFG{ SER_DESCRS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # создадим скрытые поля для "Уникального идентификатора серии/подсерии" и
  # "Номер описания серии/подсерии"
  # ПРИМЕЧАНИЕ. Идентификаторы скрытых полей имеют постфикс 'begin' для того,
  #             чтобы можно было обновлять значения этих полей, поскольку они
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужны и исходные значения этих полей, и их новые
  #             значения (которые, возможно, были изменены пользователем).
  print $query->hidden( -name => 'series_id_begin',
                        -default => $fields{ 'series_id' }
                      ), "\n";
  print $query->hidden( -name => 'ser_descr_num_begin',
                        -default => $fields{ 'ser_descr_num' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Уникальный идентификатор серии/подсерии</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль series.pl в качестве модуля-справочника  
  create_selection_func( func_name => 'select_series',
                         module_name => 'series.pl',
                         caller_name => $module,
                         window_name => 'series',
                         exp_imp => $query->param( 'exp_imp' ) );

  print "<td>",
        $query->textfield( -name => 'series_id',
                           -default => $fields{ 'series_id' },
                           -size => $CFG{ SER_DESCRS_SERIES_ID_SIZE },
                           -maxlength => $CFG{ SER_DESCRS_SERIES_ID_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )"} .
        qq{ onclick="select_series()">Список</a>},
        "</td>\n";
  print "</tr>\n";

  # выведем также ISSN серии (без возможности редактирования)
  print "<tr>\n";
  print "<th>ISSN, присвоенный серии/подсерии</th>\n";
  print "<td>",
        $query->textfield( -name => 'series_issn',
                           -default => $fields{ 'series_issn' },
                           -size => $CFG{ SER_DESCRS_SERIES_ISSN_SIZE },
                           -maxlength => 
                             $CFG{ SER_DESCRS_SERIES_ISSN_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Номер описания серии/подсерии</th>\n";
  print "<td>",
        $query->textfield( -name => 'ser_descr_num',
                           -default => $fields{ 'ser_descr_num' },
                           -size => $CFG{ SER_DESCRS_SER_DESCR_NUM_SIZE },
                           -maxlength => 
                             $CFG{ SER_DESCRS_SER_DESCR_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Описание серии/подсерии</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].ser_descr", "SER_DESCRS_SER_DESCR" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'ser_descr',
                          -default => $fields{ 'ser_descr' },
                          -rows => $CFG{ SER_DESCRS_SER_DESCR_ROWS },
                          -columns => $CFG{ SER_DESCRS_SER_DESCR_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Язык</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_process_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид языковой обработки</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'process_code',
                            -default => $fields{ 'process_code' },
                            -values => $ref_values_procs,
                            -labels => { @{ $ref_labels_procs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Назначение дополнительного описания " .
            "(или причина его составления)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].reason_add_descr",
                   "SER_DESCRS_REASON_ADD_DESCR" )
    if $CFG{ HOT_HELP_IS_ON };

  print  $query->textfield( -name => 'reason_add_descr',
                            -default => $fields{ 'reason_add_descr' },
                            -size => $CFG{ SER_DESCRS_REASON_ADD_DESCR_SIZE },
                            -maxlength => 
                              $CFG{ SER_DESCRS_REASON_ADD_DESCR_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "SER_DESCRS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ SER_DESCRS_COMMENT_ROWS },
                          -columns => $CFG{ SER_DESCRS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, значит, мы находимся в режиме просмотра/корректировки
  # и можем создать ссылки для ввода записей в таблицы, связанные с таблицей
  # Ser_descrs
  if ( $ref_row ne undef )
  {
    # типовая часть HTML-ссылок для запуска режимов работы с "деталировками"
    # описаний серий/подсерий (будем включать в ссылку не только ключевые
    # поля, но также и описание серии/подсерии - для наглядности и удобства
    # ввода "деталировок")
    # ПРИМЕЧАНИЕ. В описании серии/подсерии могут быть символы, которые нужно
    #             заэкранировать - что мы и делаем.
    $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
            "series_id=$fields{ 'series_id' }" .
            "&ser_descr_num=$fields{ 'ser_descr_num' }" .
            "&ser_descr=" . CGI::escape( $fields{ 'ser_descr' } ) .
            "&target_win=_self";

    # заглавия серий/подсерий
    print qq{<a class="command_link"} .
          qq{ href="/$CFG\{ HOME_WEB_DIR \}/ser_titles.pl/query?$href" } .
          qq{ target="_self">Заглавия серий/подсерий</a><br /><br />\n};
  }

  return ( 0 );
}
