#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Заглавия серий/подсерий"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'SER_TITLES';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Ser_titles
my @f_names =
   ( 'series_id',        # Уникальный идентификатор серии/подсерии
     'ser_descr_num',    # Номер описания серии/подсерии
                         # (это поле, аналогичное полю bib_rec_num)
                         # (основное описание имеет номер 0)
     'ser_title_num',    # Номер заглавия
                         # (0 - основное; 1, 2, ... - параллельные)
     'ser_title',        # Заглавие серии/подсерии
     'ser_title_info',   # Сведения, относящиеся к заглавию
     'lang_code',        # Код языка, на котором записано заглавие
                         # (1 - русский язык)
     'process_code',     # Код вида обработки заглавия
                         # (1 - оригинальное)
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'ser_title_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный номер заглавия" );
  }

  if ( $query->param( 'ser_title' ) eq '' )
  {
    push( @error_messages, "Не указано заглавие серии/подсерии" );
  }

  if ( $query->param( 'lang_code' ) eq '' )
  {
    push( @error_messages, "Не указан код языка" );
  }

  if ( $query->param( 'process_code' ) eq '' )
  {
    push( @error_messages, "Не указан код вида обработки заглавия" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Описания серий/подсерий',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $series_id;          # уникальный идентификатор серии/подсерии
  my $ser_descr_num;      # номер описания серии/подсерии
  my $ser_title_num;      # номер заглавия
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $series_id     = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );
  $ser_title_num = $query->param( 'ser_title_num' );
  $target_win    = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}ser_titles " .
             "WHERE series_id = '$series_id' AND " .
                   "ser_descr_num = '$ser_descr_num' AND " .
                   "ser_title_num = '$ser_title_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех заглавий для данной серии/подсерии
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля ser_descrs.pl.

  my $series_id;          # уникальный идентификатор серии/подсерии
  my $ser_descr_num;      # номер описания серии/подсерии
  my $ser_descr;          # описание серии/подсерии
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  $series_id     = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );
  $ser_descr     = $query->param( 'ser_descr' );
  $target_win    = $query->param( 'target_win' );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $ser_descr = CGI::escape( $ser_descr );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "series_id=$series_id" .
          "&ser_descr_num=$ser_descr_num" .
          "&ser_descr=$ser_descr" .
          "&target_win=$target_win";

  # выбираем все записи о заглавиях для данной серии/подсерии
  # (получим также краткое наименования языка, на котором записано заглавие)
  $command = "SELECT t.*, l.lang_sname " .
             "FROM ${db_schema}ser_titles t, ${db_schema}langs l " .
             "WHERE t.series_id = $series_id AND " .
                   "t.ser_descr_num = $ser_descr_num AND " .
                   "t.lang_code = l.lang_code " .
             "ORDER BY t.ser_title_num";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Номер заглавия</th>\n";
    print "<th>Заглавие</th>\n";
    print "<th>Язык заглавия</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим номер заглавия, заглавие и язык заглавия
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'ser_title_num' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'ser_title' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'lang_sname' ) ]</td>\n";

      # просмотр данной записи о заглавии в виде формы
      # добавим параметр - биб. описание
      print "<td>" if ( $can_view || $can_delete );
      print qq{ <a class="command_link" href="$script_name/view?$href} .
            "&ser_title_num=" . 
            $row[ $result->fnumber( 'ser_title_num' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a> } if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о заглавии
      print qq{ <a class="command_link" href="$script_name/delete?$href} .
            "&ser_title_num=" . 
            $row[ $result->fnumber( 'ser_title_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete" } .
            qq{ target="$target_win">Удаление</a> } if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления заглавия серии/подсерии 
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $series_id;          # уникальный идентификатор серии/подсерии
  my $ser_descr_num;      # номер описания серии/подсерии
  my $ser_title_num;      # номер заглавия
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $series_id     = $query->param( 'series_id' );
  $ser_descr_num = $query->param( 'ser_descr_num' );
  $ser_title_num = $query->param( 'ser_title_num' );
  $target_win    = $query->param( 'target_win' );

  # выбираем одну запись о заглавии
  # (возьмем также краткие наименования языка заглавия и вида обработки
  # заглавия)
  $command = "SELECT t.*, l.lang_sname, p.process_sname " .
             "FROM ${db_schema}ser_titles t, ${db_schema}langs l, " .
                  "${db_schema}process_types p " .
             "WHERE t.series_id = $series_id AND " .
                   "t.ser_descr_num = $ser_descr_num AND " .
                   "t.ser_title_num = '$ser_title_num' AND " .
                   "t.lang_code = l.lang_code AND " .
                   "t.process_code = p.process_code";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор серии/подсерии - " .
                     $series_id . ", " .
                     "номер описания для данной серии/подсерии - " .
                     $ser_descr_num . ", " .
                     "номер заглавия - $ser_title_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "ser_titles",
                       key => "series_id, ser_descr_num, " .
                              "ser_title_num:ser_title_num_begin",
                       f_names => \@f_names,
                       f_special => undef,
                       func_special => undef,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # языков, на которых записаны заглавия)
  my( $ref_values_langs, $ref_labels_langs ) = popup_langs_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов языковой обработки заглавий)
  my( $ref_values_procs, $ref_labels_procs ) =
    popup_process_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'series_id' }     = $query->param( 'series_id' );
    $fields{ 'ser_descr_num' } = $query->param( 'ser_descr_num' );

    # номер заглавия (0 - основное; 1, 2, ... - параллельные)
    $fields{ 'ser_title_num' } = $CFG{ SER_TITLES_SER_TITLE_NUM };
    
    # код языка, на котором записано заглавие (1 - русский язык)
    $fields{ 'lang_code' }     = $CFG{ SER_TITLES_LANG_CODE };

    # код вида обработки заглавия (1 - в оригинале)
    $fields{ 'process_code' }  = $CFG{ SER_TITLES_PROCESS_CODE };
  }
  # -------------------------------------------------------------------

  # поскольку значения полей "Уникальный идентификатор серии/подсерии"
  # "Номер описания серии/подсерии" не подлежат редактированию, не будем
  # позволять их редактировать, но совсем убрать их нельзя, т.к. они
  # используются при выполнении операции UPDATE в качестве ключа, а также при
  # выполнении операции INSERT (значения этих полей сохраняются такими же,
  # какими они были при предыдущем запуске скрипта, поэтому можно не
  # присваивать им значений)
  print $query->hidden( -name => 'series_id' ), "\n";
  print $query->hidden( -name => 'ser_descr_num' ), "\n";

  # создадим скрытое поле для "Номер заглавия"
  # ПРИМЕЧАНИЕ. Идентификатор скрытого поля имеет постфикс 'begin' для того,
  #             чтобы можно было обновлять значение этого поля, поскольку оно
  #             является ключевым (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужно и исходное значение этого поля, и его новое
  #             значение (которое, возможно, было изменено пользователем).
  print $query->hidden( -name => 'ser_title_num_begin',
                        -default => $fields{ 'ser_title_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать описание серии/подсерии
  # следующему вызову скрипта, когда будет работать функция add_rec() 
  # или update_rec()
  print $query->hidden( -name => 'ser_descr' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info(); # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Номер заглавия</th>\n";
  print "<td>",
        $query->textfield( -name => 'ser_title_num',
                           -default => $fields{ 'ser_title_num' },
                           -size => $CFG{ SER_TITLES_SER_TITLE_NUM_SIZE },
                           -maxlength => 
                             $CFG{ SER_TITLES_SER_TITLE_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Заглавие серии/подсерии</th>\n";
  print "<td>",
        $query->textarea( -name => 'ser_title',
                          -default => $fields{ 'ser_title' },
                          -rows => $CFG{ SER_TITLES_SER_TITLE_ROWS },
                          -columns => $CFG{ SER_TITLES_SER_TITLE_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Сведения, относящиеся к заглавию<br />" .
            "(разделять знаком \":\")</th>\n";
  print "<td>",
        $query->textarea( -name => 'ser_title_info',
                          -default => $fields{ 'ser_title_info' },
                          -rows => $CFG{ SER_TITLES_SER_TITLE_INFO_ROWS },
                          -columns => $CFG{ SER_TITLES_SER_TITLE_INFO_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_langs_list() формируется не хеш,
  # а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Язык заглавия</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'lang_code',
                            -default => $fields{ 'lang_code' },
                            -values => $ref_values_langs,
                            -labels => { @{ $ref_labels_langs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_process_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид языковой обработки</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'process_code',
                            -default => $fields{ 'process_code' },
                            -values => $ref_values_procs,
                            -labels => { @{ $ref_labels_procs } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "SER_TITLES_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ SER_TITLES_COMMENT_ROWS },
                          -columns => $CFG{ SER_TITLES_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем описание серии/подсерии и заголовок модуля
  print_ser_descr( $query, "Заглавия серий/подсерий" );
  
  return ( 0 );
}
