#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Серии/подсерии"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'SERIES';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Series
my @f_names =
   ( 
     'series_id',        # Уникальный идентификатор серии/подсерии
     'series_issn',      # ISSN, присвоенный серии/подсерии
     'subser_flag',      # Признак подсерии
     'series_id_parent', # Идентификатор вышестоящей серии (иерархия)
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              do_find => \&do_find,
              do_export => \&do_export,
              do_import => \&do_import,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # допускается либо пустое поле, либо только цифры
  if ( $query->param( 'series_id_parent' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный идентификатор вышестоящей серии" );
  }

  if ( $query->param( 'series_id_parent' ) == $query->param( 'series_id' ) )
  {
    push( @error_messages, "Идентификаторы серии и вышестоящей серии " .
          "не должны совпадать" );
  }
  
  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Серии/подсерии',
                    user => $user
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $series_id;             # уникальный идентификатор серии/подсерии
  my $command;               # строка команды для работы с БД

  $series_id = $query->param( 'series_id' );
  
  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}series " .
             "WHERE series_id = '$series_id'";

  # используем типовую процедуру
  del_rec( query => $query,
           create_top => \&create_top,
           target => "_self",
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# экспортирование записи
# -----------------------------------------------------------------------------
sub do_export
{
  # ВАЖНО!!! Будем экспортировать все описания для данной серии.
  #          В настоящий момент (15.04.2012) такая стратегия представляется
  #          разумной. Поэтому нам не нужно использовать значение поля
  #          ser_descr_num из таблицы Ser_descrs. Достаточно только поля
  #          series_id. При этом выборку будем делать не из таблицы Series,
  #          а из таблицы Ser_descrs. Так будет удобнее с точки зрения
  #          конфигурации ссылочных и зависимых таблиц, которая представлена
  #          в конфиг. файле экспорта/импорта e_arch_exp.cfg.
  
  my $series_id;             # уникальный идентификатор серии/подсерии

  # ПРИМЕЧАНИЕ. Оставим этот фрагмент в качестве напоминания.
  # my $ser_descr_num;       # номер описания серии/подсерии
  
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $series_id = $query->param( 'series_id' );

  # сначала сформируем строку-команду для таблицы Ser_descrs и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM Ser_descrs WHERE series_id = $series_id";
  push( @table_cmd_pairs, [ 'Ser_descrs', $command ] );

  # вызываем стандартную процедуру экспорта
  export_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# организация формы для выполнения поиска информации
# -----------------------------------------------------------------------------
sub do_find
{
  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна
  
  open_bottom();    # начинаем оформление нижней части окна
  
  print "<p class=\"info\">В настоящее время поиск не реализован</p>\n";

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# импортирование записи
# -----------------------------------------------------------------------------
sub do_import
{
  # ВАЖНО!!! Будем импортировать все описания для данной серии.
  #          В настоящий момент (15.04.2012) такая стратегия представляется
  #          разумной. Поэтому нам не нужно использовать значение поля
  #          ser_descr_num из таблицы Ser_descrs. Достаточно только поля
  #          series_id. При этом выборку будем делать не из таблицы Series,
  #          а из таблицы Ser_descrs. Так будет удобнее с точки зрения
  #          конфигурации ссылочных и зависимых таблиц, которая представлена
  #          в конфиг. файле экспорта/импорта e_arch_exp.cfg.
  
  my $series_id;             # уникальный идентификатор серии/подсерии

  # ПРИМЕЧАНИЕ. Оставим этот фрагмент в качестве напоминания.
  # my $ser_descr_num;       # номер описания серии/подсерии
  
  my $command;               # строка команды для работы с БД
  my @table_cmd_pairs = ();  # массив ссылок на двухэлементные массивы,
                             # содержащие имя таблицы и команду для нее

  $series_id = $query->param( 'series_id' );

  # сначала сформируем строку-команду для таблицы Ser_descrs и поместим
  # ее в массив (используем ссылку на массив из двух элементов, первым
  # из которых является имя таблицы)
  $command = "SELECT * FROM exp_imp.Ser_descrs WHERE series_id = $series_id";
  push( @table_cmd_pairs, [ 'Ser_descrs', $command ] );

  # вызываем стандартную процедуру импорта
  import_data( query => $query,
               create_top => \&create_top,
               table_cmd_pairs => \@table_cmd_pairs
             );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех серий/подсерий
# -----------------------------------------------------------------------------
sub do_query
{
  my( $result, $i );              # для работы с БД
  my @row;                        # для хранения одной записи из выборки
  my $command;                    # строка команды для работы с БД

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # имеет ли пользователь право на экспортирование/импортирование записи
  my $can_export = 0;
  my $can_import = 0;

  # дополнительные параметры для скрипта при выполнении режимов просмотра
  # и удаления (цель -- ретранслировать эти параметры от одного запуска
  # скрипта к другому)
  my $add_params = '';

  # основное описание серии/подсерии (используется для формирования
  # HTML-ссылки при вызове модуля в качестве справочника)
  my $basic_ser_descr;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  my $href;    # типовая часть HTML-ссылки
  
  # при использовании данного модуля в режиме справочника будут существовать
  # параметры enable_select и caller_name (имя вызывающего модуля)
  if ( $query->param( 'enable_select' ) )
  {
    $add_params = "&enable_select=1&caller_name=" .
                  $query->param( 'caller_name' );
  }

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => "_self" );

  # временное описание новой серии/подсерии
  my $faked_ser_descr = $CFG{ SERIES_FAKED_DESCR };

  $faked_ser_descr = '--- Новая серия ---' unless $faked_ser_descr;

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна
  
  # выбираем все записи о сериях/подсериях
  # выберем также основное описание серии/подсерии и основное описание 
  # и ISSN вышестоящей серии 
  # ПРИМЕЧАНИЕ. Конструкция CASE WHEN ... используется для того, чтобы можно
  #             было отличить вновь введенную серию, еще не имеющую ни одного
  #             описания, в общем списке серий.
  #             Основное описание серии/подсерии в таблице Ser_descrs имеет 
  #             значение поля ser_descr_num равное 0. 
  #             Условие d1.ser_descr_num IS NULL включено в предложение WHERE,
  #             чтобы можно было видеть вновь введенную серию, у которой еще
  #             нет ни одного описания.
  #             Если условие d2.ser_descr_num IS NULL не включать в предложение
  #             WHERE, то в выборку попадают только серии, у которых есть
  #             вышестоящие серии (причем, будет включена только одна запись --
  #             для основного описания вышестоящей серии). 
  #             Если исключить условие ( d2.ser_descr_num = 0 OR 
  #             d2.ser_descr_num IS NULL ), то в выборку попадают все серии,
  #             однако для тех серий, у которых есть вышестоящие серии,
  #             в выборку будет включено столько записей, сколько имеется
  #             описаний для вышестоящей серии. Чтобы проверить это все
  #             практически, нужно для одной из серий ввести хотя бы два
  #             описания (в модуле "Описания серий/подсерий"), а затем
  #             "поиграть" с вышеописанными фрагментами из предложения WHERE.
  $command = "SELECT s1.*, s2.series_issn AS parent_ser_issn, " .
                    "CASE WHEN d1.ser_descr IS NOT NULL " .
                         "THEN d1.ser_descr " .
                         "ELSE '$faked_ser_descr' " .
                     "END AS basic_ser_descr, " .
                    "d2.ser_descr AS parent_ser_descr " .
             "FROM ( ( ${db_schema}series s1 " .
                      "LEFT JOIN ${db_schema}series s2 " .
                      "ON s1.series_id_parent = s2.series_id " .
                    ") LEFT JOIN ${db_schema}ser_descrs d1 " .
                    "ON s1.series_id = d1.series_id " .
                  ") LEFT JOIN ${db_schema}ser_descrs d2 " .
                  "ON s2.series_id = d2.series_id " .
             "WHERE ( d1.ser_descr_num IS NULL OR " .
                     "d1.ser_descr_num = 0 ) AND " .
                   "( d2.ser_descr_num = 0 OR d2.ser_descr_num IS NULL ) " .
             "ORDER BY basic_ser_descr";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    # если этот модуль вызван в режиме справочника, и при этом он вызван
    # "сам из себя", то запретим операции просмотра и удаления записей
    # (поскольку режим корректировки выполняется из режима просмотра,
    # то и он будет недоступен)
    if ( $query->param( 'enable_select' ) &&
         $query->param( 'caller_name') eq $module )
    {
      $can_view = 0;
      $can_delete = 0;
    }
    else
    {
      $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                               'user' => $user );
      $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                                 'user' => $user );
    }

    # работаем со схемой БД "экспорт/импорт", поэтому экспорт не имеет смысла
    if ( $query->param( 'exp_imp' ) )
    {
      $can_import = import_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'IMPORT',
                                 'user' => $user );
    }
    # работаем со схемой public -- можем экспортировать данные,
    # а импортировать -- нет
    else
    {
      $can_export = export_enabled() && 
                    have_access( 'module' => $module, 'procedure' => 'EXPORT',
                                 'user' => $user );
    }

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>ID</th>\n";
    print "<th>ISSN</th>\n";
    print "<th>Основное описание серии/подсерии</th>\n";
    print "<th>Признак подсерии</th>\n";
    print "<th>ID, ISSN и основное описание вышестоящей серии</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete || $can_export ||
                                   $can_import );
    print "</tr>\n";

    # если модуль вызван в режиме справочника для оказания помощи пользователю
    # при выборе вышестоящей серии (или серии, для которой необходимо
    # ввести описание) ...
    if ( $query->param( 'enable_select' ) )
    {
      # ... то сформируем функцию JavaScript для присваивания значений полей
      # из текущей записи таблицы Series (а также таблицы Ser_descrs) полям
      # в ВЫЗЫВАЮЩЕМ окне ...
      if ( $query->param( 'caller_name' ) eq 'SERIES' )
      {
        # ... полям series_id_parent, parent_ser_issn и parent_ser_descr
        # в модуле SERIES
        create_assign_values_func( 'series_id_parent', 'parent_ser_issn',
                                   'parent_ser_descr' );
      }
      elsif ( $query->param( 'caller_name' ) eq 'SER_DESCRS' )
      {
        # ... полям series_id и series_issn в модуле SER_DESCRS
        create_assign_values_func( 'series_id', 'series_issn' );
      }
    }

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим уникальный идентификатор серии/подсерии, ISSN, основное
      # описание серии/подсерии, признак подсерии, идентификатор, ISSN и
      # основное описание вышестоящей серии
      # (добавим символ &nbsp; к значениям полей из соображений эстетики:
      # если значение поля 'series_issn' или 'series_id_parent' равно null,
      # то не отображаются горизонтальные строки-разделители)
      print "<td class=\"center\">" .
            $row[ $result->fnumber( 'series_id' ) ] . "</td>\n";
      print "<td>$row[ $result->fnumber( 'series_issn' ) ]&nbsp;</td>\n";

      # если модуль вызван в режиме справочника для оказания помощи
      # пользователю при выборе конкретной вышестоящей серии, то значение
      # поля basic_ser_descr выведем в виде ссылки
      if ( $query->param( 'enable_select' ) )
      {
        # заэкранируем одинарные и двойные кавычки в основном описании
        # серии/подсерии 
        # выполним добавочную обработку значения переменной, т. к. того, что
        # делает функция escapeHTML(), недостаточно
        # ПРИМЕЧАНИЕ. Она заменяет символ "одинарная кавычка" на выражение
        #             "&#39;", которое затем при вызове функции assign_values()
        #             заменяется опять одинарной кавычкой, но параметры этой
        #             функции также заключены в одинарные кавычки -- возникает
        #             ошибка.
        # TODO. Надо разобраться с этим подробнее.
        # TODO. В том случае, когда в составе фрагмента текста есть
        #       символ "'" (одинарная кавычка), представленная здесь
        #       программная конструкция не работает, поэтому следующая строка
        #       закомментирована, но оставлена для последующего 
        #       разбирательства.
        # $basic_ser_descr =~ s/\'/\\\'/g;
        $basic_ser_descr = 
          CGI::escapeHTML( $row[ $result->fnumber( 'basic_ser_descr' ) ] );

        print "<td>",
              qq{ <a class="command_link" href="javascript:void( 0 )" onclick=
                  "assign_values( 
                     $row[ $result->fnumber( 'series_id' ) ],
                     '$row[ $result->fnumber( 'series_issn' ) ]',
                     '$basic_ser_descr'
                   ); 
                   parent.opener.document.forms[ 0 ].subser_flag.checked =
                     true;
                  ">
                   $row[ $result->fnumber( 'basic_ser_descr' ) ]</a> },
              "</td>\n";
      }
      else
      {
        print "<td>$row[ $result->fnumber( 'basic_ser_descr' ) ]&nbsp;</td>\n";
      }
      print "<td>",
            $row[ $result->fnumber( 'subser_flag' ) ] eq 't' ? 'Да' : 'Нет',
            "</td>\n";

      # если есть родительская серия, то выводим ее идентификатор в скобках
      # если есть ISSN этой родительской серии, то и его выводим в скобках
      print "<td>" .
            (
              ( $row[ $result->fnumber( 'series_id_parent' ) ] ) ? 
              "(" . $row[ $result->fnumber( 'series_id_parent' ) ] . ")" :
              ""
            ) . "&nbsp;\n" .
            (
              ( $row[ $result->fnumber( 'parent_ser_issn' ) ] ) ?
              "(" . $row[ $result->fnumber( 'parent_ser_issn' ) ] . ")" :
              ""
            ) . "&nbsp;\n" .
            "$row[ $result->fnumber( 'parent_ser_descr' ) ]&nbsp;\n" .
            "</td>\n";

      # типовая часть HTML-ссылки
      $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
              "series_id=" . $row[ $result->fnumber( 'series_id' ) ] .
              $add_params;

      # просмотр записи о серии/подсерии в виде формы
      print "<td>" if ( $can_view || $can_delete || $can_export ||
                        $can_import );
      print qq{ <a class="command_link" href="$script_name/view?$href"} .
            qq{ target="_self">Просмотр</a> } . "\n" if $can_view;

      # удаление записи о серии/подсерии
      print qq{ <a class="command_link" href="$script_name/delete?$href"} .
            qq{ onclick="$confirm_delete" } .
            qq{ target="_self">Удаление</a> } . "\n" if $can_delete;

      # экспортирование записи 
      print qq{ <a class="command_link" href="$script_name/export?$href"} .
            qq{ onclick="$confirm_export"} .
            qq{ target="_blank">Экспорт</A> } . "\n" if $can_export;

      # импортирование записи 
      print qq{ <a class="command_link" href="$script_name/import?$href"} .
            qq{ onclick="$confirm_import"} .
            qq{ target="_blank">Импорт</A> } . "\n" if $can_import;

      print "</td>" if ( $can_view || $can_delete || $can_export ||
                         $can_import );
      print "</tr>\n";
    }
    print "</table>\n";
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $series_id;             # уникальный идентификатор серии/подсерии
  my $command;               # строка команды для работы с БД

  $series_id = $query->param( 'series_id' );

  # выбираем одну запись о серии/подсерии
  $command = "SELECT s1.*, s2.series_issn AS parent_ser_issn, " .
                    "d1.ser_descr AS basic_ser_descr, " .
                    "d2.ser_descr AS parent_ser_descr " .
             "FROM ( ( ${db_schema}series s1 " .
                      "LEFT JOIN ${db_schema}series s2 " .
                      "ON s1.series_id_parent = s2.series_id " .
                    ") LEFT JOIN ${db_schema}ser_descrs d1 " .
                    "ON s1.series_id = d1.series_id " .
                  ") LEFT JOIN ${db_schema}ser_descrs d2 " .
                  "ON s2.series_id = d2.series_id " .
             "WHERE ( d1.ser_descr_num IS NULL OR " .
                     "d1.ser_descr_num = 0 ) AND " .
                   "( d2.ser_descr_num = 0 OR " .
                     "d2.ser_descr_num IS NULL ) AND " .
                   "s1.series_id = '$series_id'"; 

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => "_self",
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с кодом " .
                     "серии/подсерии " . $series_id,
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "series",
                       key => "series_id",
                       f_names => \@f_names,
                       f_special => [ 'subser_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # будем предлагать пользователю значение ключевого поля для новой записи,
    # которое на 1 больше максимального значения этого поля в данной таблице
    $fields{ 'series_id' } = get_next_key_value( 'series', 'series_id' );

    # значения по умолчанию

    # примечание
    $fields{ 'comment' } = $CFG{ SERIES_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор серии/подсерии"
  # формируется автоматически с использованием функции get_next_key_value(),
  # не будем позволять его редактировать, но совсем убрать его нельзя,
  # т.к. оно используется при выполнении операции UPDATE в качестве ключа
  print $query->hidden( -name => 'series_id',
                        -default => $fields{ 'series_id' }
                      ), "\n";

  # если модуль был вызван в режиме справочника, то наличие этих полей 
  # позволяет при повторном вызове этого же скрипта сохранить информацию о том,
  # что модуль был вызван именно в режиме справочника, а также имя
  # того модуля, из которого справочник был вызван
  # ПРИМЕЧАНИЕ. См. примечание в модуле cities.pl.
  print $query->hidden( -name => 'enable_select' ), "\n";
  print $query->hidden( -name => 'caller_name' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Уникальный идентификатор серии/подсерии</th>\n";
  print "<td>", $fields{ 'series_id' }, "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>ISSN, присвоенный серии/подсерии</th>\n";
  print "<td>",
        $query->textfield( -name => 'series_issn',
                           -default => $fields{ 'series_issn' },
                           -size => $CFG{ SERIES_ISSN_SIZE },
                           -maxlength => $CFG{ SERIES_ISSN_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # формируем это поле только в режиме просмотра записи из БД, а в режиме
  # ввода -- нет
  if ( $ref_row ne undef )
  {
    print "<tr>\n";
    print "<th>Основное описание серии/подсерии</th>\n";
    print "<td>",
          $query->textarea( -name => 'basic_ser_descr',
                            -default => $fields{ 'basic_ser_descr' },
                            -rows => $CFG{ SERIES_BASIC_SER_DESCR_ROWS },
                            -columns => $CFG{ SERIES_BASIC_SER_DESCR_COLS },
                            -readonly => 'readonly'
                          ),
          "</td>\n";
    print "</tr>\n";
  }

  print "<tr>\n";
  print "<th>Признак подсерии</th>\n";
  print "<td>",
        $query->checkbox( -name => 'subser_flag',
                          -checked => 
                             $fields{ 'subser_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Идентификатор вышестоящей серии (иерархия)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль series.pl в качестве модуля-справочника  
  create_selection_func( func_name => 'select_parent_series',
                         module_name => 'series.pl',
                         caller_name => $module,
                         window_name => 'parent_series',
                         exp_imp => $query->param( 'exp_imp' ) );
  print "<td>",
        $query->textfield( -name => 'series_id_parent',
                           -default => $fields{ 'series_id_parent' },
                           -size => $CFG{ SERIES_ID_PARENT_SIZE },
                           -maxlength => $CFG{ SERIES_ID_PARENT_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )"} .
        qq{ onclick="select_parent_series()">Список</a>},
        "</td>\n";
  print "</tr>\n";

  # выведем также ISSN и основное описание вышестоящей серии
  # (без возможности редактирования)
  print "<tr>\n";
  print "<th>ISSN вышестоящей серии</th>\n";
  print "<td>",
        $query->textfield( -name => 'parent_ser_issn',
                           -default => $fields{ 'parent_ser_issn' },
                           -size => $CFG{ SERIES_PARENT_SER_ISSN_SIZE },
                           -maxlength => 
                             $CFG{ SERIES_PARENT_SER_ISSN_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Основное описание вышестоящей серии</th>\n";
  print "<td>",
        $query->textarea( -name => 'parent_ser_descr',
                          -default => $fields{ 'parent_ser_descr' },
                          -rows => $CFG{ SERIES_PARENT_SER_DESCR_ROWS },
                          -columns => $CFG{ SERIES_PARENT_SER_DESCR_COLS },
                          -readonly => 'readonly'
                        ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "SERIES_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ SERIES_COMMENT_ROWS },
                          -columns => $CFG{ SERIES_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;

  # поскольку эта функция обрабатывает только поле-флаг 'subser_flag',
  # имеющее тип bool, то проверяем значение параметра только на непустоту
  $value = $query->param( $fname ) ? 'yes' : 'no';

  return ( $value );  
}
