#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Специфическое обозначение материала и объем;
#           сопроводительный материал"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'SPEC_MATER';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Spec_mater
my @f_names =
   ( 'obj_id',            # Уникальный идентификатор объекта
                          # библиографического описания
     'bib_rec_num',       # Номер библиогр. записи для данного объекта
                          # (основная запись имеет номер 0)
     'spec_mat_num',      # Порядковый номер записи о материале для данного
                          # объекта библиограф. описания (для основного
                          # материала и для сопроводительного материала
                          # вводить отдельные записи в эту таблицу)
     'spec_mat_code',     # Код специфического обозначения материала
     'total_extent',      # Суммарный объем материала данного вида
                          # (например, "378" (страниц и т.д.))
     'orig_extent',       # Объем в том виде, как он представлен
                          # в источнике (например, XXV с., 354 с., [10 с.]) 
     'phys_add_info',     # Другие сведения о физической характеристике
     'orig_dimensions',   # Размеры (как они представлены в источнике)
     'size_sheet',        # Размер листа (для книг) (например, 60x90)
     'part_sheet',        # Доля листа (для книг) (например, 1/16)
     'accomp_flag',       # Признак сопроводительного материала
     'accomp_title',      # Наименование сопроводительного материала
     'accomp_obj_ID',     # Уникальный идентификатор объекта библиогр.
                          # описания, если на сопроводительный материал
                          # составлено самостоятельное описание
     'pack_code',         # Код вида упаковки
     'comment'            # Примечание
   );

$query = CGI->new();               # создадим новый CGI-объект

my $db_schema;                     # схема БД (для экспорта/импорта)

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'spec_mat_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указано или неверное значение порядкового " .
          "номера записи о материале" );
  }

  if ( $query->param( 'spec_mat_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код специфического " .
          "обозначения материала" );
  }

  # допускается пустое поле или цифры
  if ( $query->param( 'total_extent' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный суммарный объем материала данного вида" );
  }

  if ( $query->param( 'orig_extent' ) eq '' )
  {
    push( @error_messages, "Не указан объем (в том виде, как он представлен " .
          "в источнике)" );
  }

  if ( ( $query->param( 'size_sheet' ) !~ /^\d+x\d+$/ ) &&
       ( $query->param( 'size_sheet' ) !~ /^$/ )
     )
  {
    push( @error_messages, "Размер листа может быть пустым полем или должен " .
          "иметь значение формата, например, 60x90" );
  }

  if ( ( $query->param( 'part_sheet' ) !~ /^1\/\d+$/ ) &&
       ( $query->param( 'part_sheet' ) !~ /^$/ )
     )
  {
    push( @error_messages, "Доля листа может быть пустым полем или должна " .
          "иметь значение формата, например, 1/16" );
  }

  # допускается пустое поле или цифры
  if ( $query->param( 'accomp_obj_id' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный уникальный идентификатор объекта " .
          "биб. описания, если на сопроводительный материал составлено " .
          "самостоятельное описание" );
  }

  if ( $query->param( 'pack_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код вида упаковки" );
  }

  # DEBUG
  # print_params( $query );
  
  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }

  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $spec_mat_num;       # порядковый номер записи о материале
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id       = $query->param( 'obj_id' );
  $bib_rec_num  = $query->param( 'bib_rec_num' );
  $spec_mat_num = $query->param( 'spec_mat_num' );
  $target_win   = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}spec_mater " .
             "WHERE obj_id = '$obj_id' AND " .
                   "bib_rec_num = '$bib_rec_num' AND " .
                   "spec_mat_num = '$spec_mat_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка обозначений материала и объема, а также сопроводительных
# материалов для данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id      = $query->param( 'obj_id' );
  $bib_rec_num = $query->param( 'bib_rec_num' );
  $bib_rec     = $query->param( 'bib_rec' );
  $target_win  = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec_num=$bib_rec_num" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о материалах для данного объекта
  # (получим также краткие наименования специфич. обозн. материала, группы
  # материалов, а также вида упаковки)
  $command = "SELECT m.*, mt.spec_mat_sname, mg.spec_mat_group_sname, " .
                    "p.pack_sname " .
             "FROM ${db_schema}spec_mater m, " .
                  "${db_schema}spec_mater_types mt, " .
                  "${db_schema}spec_mater_groups mg, " .
                  "${db_schema}pack_types p " .
             "WHERE m.obj_id = $obj_id AND " .
                   "m.bib_rec_num = $bib_rec_num AND " .
                   "m.spec_mat_code = mt.spec_mat_code AND " .
                   "mt.spec_mat_group_code = mg.spec_mat_group_code AND " .
                   "m.pack_code = p.pack_code " .
             "ORDER BY m.spec_mat_num";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Порядковый номер записи о материале</th>\n";
    print "<th>Специфическое обозначение материала</th>\n";
    print "<th>Объем в оригинальном виде</th>\n";
    print "<th>Вид упаковки</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим порядковый номер записи о материале, специфическое 
      # обозначение материала и группу, объем в оригинальном виде
      # и вид упаковки
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'spec_mat_num' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'spec_mat_sname' ) ]" .
            " (" . $row[ $result->fnumber( 'spec_mat_group_sname' ) ] . ")" .
            "</td>\n";
      print "<td>$row[ $result->fnumber( 'orig_extent' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'pack_sname' ) ]</td>\n";

      # просмотр данной записи о материале в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&spec_mat_num=" . 
            $row[ $result->fnumber( 'spec_mat_num' ) ]  . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о материале
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&spec_mat_num=" . 
            $row[ $result->fnumber( 'spec_mat_num' ) ]  . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о материале
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна  
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec_num;        # номер библиогр. записи для данного объекта
  my $spec_mat_num;       # порядковый номер записи о материале
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id       = $query->param( 'obj_id' );
  $bib_rec_num  = $query->param( 'bib_rec_num' );
  $spec_mat_num = $query->param( 'spec_mat_num' );
  $target_win   = $query->param( 'target_win' );

  # выбираем одну запись о материале
  $command = "SELECT * " .
             "FROM ${db_schema}spec_mater " .
             "WHERE obj_id = $obj_id AND " .
                   "bib_rec_num = $bib_rec_num AND " .
                   "spec_mat_num = '$spec_mat_num'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "номер библиогр. записи для данного объекта - " .
                     $bib_rec_num . ", " .
                     "порядковый номер записи о материале - $spec_mat_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "spec_mater",
                       key => "obj_id, bib_rec_num, " .
                              "spec_mat_num:spec_mat_num_begin",
                       f_names => \@f_names,
                       f_special => [ 'accomp_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # видов специфич. обознач. материала)
  my( $ref_values_mat, $ref_labels_mat ) =
    popup_spec_mater_types_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # видов упаковки)
  my( $ref_values_pack, $ref_labels_pack ) =
    popup_pack_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения двух ключевых полей из параметров скрипта
    $fields{ 'obj_id' }      = $query->param( 'obj_id' );
    $fields{ 'bib_rec_num' } = $query->param( 'bib_rec_num' );

    # значения по умолчанию

    # порядковый номер записи о материале для данного объекта библиограф.
    # описания
    if ( $CFG{ SPEC_MATER_SPEC_MAT_NUM } )  # есть параметр в конфиг. файле
    {
      $fields{ 'spec_mat_num' } = $CFG{ SPEC_MATER_SPEC_MAT_NUM };
    }
    else
    {
      # будем предлагать пользователю значение поля spec_mat_num для новой
      # записи, которое на 1 больше максимального значения этого поля 
      # для данного номера биб. записи данного объекта библиогр. описания
      # (для значений полей obj_id и bib_rec_num в таблице Spec_mater)
      $fields{ 'spec_mat_num' } = 
        get_next_key_value( 'spec_mater', 'spec_mat_num',
                            'obj_id = ' . $query->param( 'obj_id' ) . ' AND ' .
                            'bib_rec_num = ' . $query->param( 'bib_rec_num' )
                          );
    }

    # код специфического обозначения материала
    $fields{ 'spec_mat_code' }   = $CFG{ SPEC_MATER_SPEC_MAT_CODE };

    # объем в том виде, как он представлен в источнике 
    # (например, XXV с., 354 с., [10 с.])
    $fields{ 'orig_extent' }     = $CFG{ SPEC_MATER_ORIG_EXTENT };

    # другие сведения о физической характеристике
    $fields{ 'phys_add_info' }   = $CFG{ SPEC_MATER_PHYS_ADD_INFO };

    # размеры (как они представлены в источнике)
    $fields{ 'orig_dimensions' } = $CFG{ SPEC_MATER_ORIG_DIMENSIONS };

    # размер листа (для книг) (например, 60x90)
    $fields{ 'size_sheet' }      = $CFG{ SPEC_MATER_SIZE_SHEET };

    # доля листа (для книг) (например, 1/16)
    $fields{ 'part_sheet' }      = $CFG{ SPEC_MATER_PART_SHEET };

    # признак сопроводительного материала
    $fields{ 'accomp_flag' }     = $CFG{ SPEC_MATER_ACCOMP_FLAG };

    # наименование сопроводительного материала
    $fields{ 'accomp_title' }    = $CFG{ SPEC_MATER_ACCOMP_TITLE };

    # код вида упаковки
    $fields{ 'pack_code' }       = $CFG{ SPEC_MATER_PACK_CODE };

    # комментарий
    $fields{ 'comment' }         = $CFG{ SPEC_MATER_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значения полей "Уникальный идентификатор объекта
  # библиографического описания" и "Номер библиогр. записи для данного
  # объекта" не подлежат редактированию, не будем позволять их редактировать,
  # но совсем убрать их нельзя, т.к. они используются при выполнении
  # операции UPDATE в качестве ключа, а также при выполнении операции INSERT
  # (значения этих полей сохраняются такими же, какими они были при
  # предыдущем запуске скрипта, поэтому можно не присваивать им значений)
  print $query->hidden( -name => 'obj_id' ), "\n";
  print $query->hidden( -name => 'bib_rec_num' ), "\n";

  # создадим скрытое поле для "Порядкового номера записи о материале"
  # ПРИМЕЧАНИЕ. Идентификатор скрытого поля имеет постфикс 'begin' для того,
  #             чтобы можно было обновлять значение этого поля, поскольку оно
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm.
  #             Т.е. нам нужно и исходное значение этого поля, и его новое
  #             значение (которое, возможно, было изменено пользователем).
  print $query->hidden( -name => 'spec_mat_num_begin',
                        -default => $fields{ 'spec_mat_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();        # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Порядковый номер записи о материале</th>\n";
  print "<td>",
        $query->textfield( -name => 'spec_mat_num',
                           -default => $fields{ 'spec_mat_num' },
                           -size => $CFG{ SPEC_MATER_SPEC_MAT_NUM_SIZE },
                           -maxlength => 
                             $CFG{ SPEC_MATER_SPEC_MAT_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_spec_mater_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Специфическое обозначение материала</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'spec_mat_code',
                            -default => $fields{ 'spec_mat_code' },
                            -values => $ref_values_mat,
                            -labels => { @{ $ref_labels_mat } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Суммарный объем материала данного вида<br />" .
            "(например, \"378\" (страниц и т. д.))</th>\n";
  print "<td>",
        $query->textfield( -name => 'total_extent',
                           -default => $fields{ 'total_extent' },
                           -size => $CFG{ SPEC_MATER_TOTAL_EXTENT_SIZE },
                           -maxlength => 
                             $CFG{ SPEC_MATER_TOTAL_EXTENT_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Объем в том виде, как он представлен в источнике<br />" .
            "(например, XXV с., 354 с., [10 с.])</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].orig_extent",
                   "SPEC_MATER_ORIG_EXTENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'orig_extent',
                           -default => $fields{ 'orig_extent' },
                           -size => $CFG{ SPEC_MATER_ORIG_EXTENT_SIZE },
                           -maxlength => 
                             $CFG{ SPEC_MATER_ORIG_EXTENT_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Другие сведения о физической характеристике</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].phys_add_info",
                   "SPEC_MATER_PHYS_ADD_INFO" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'phys_add_info',
                           -default => $fields{ 'phys_add_info' },
                           -size => $CFG{ SPEC_MATER_PHYS_ADD_INFO_SIZE },
                           -maxlength => 
                             $CFG{ SPEC_MATER_PHYS_ADD_INFO_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Размеры<br />" .
        "(как они представлены в источнике)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].orig_dimensions",
                   "SPEC_MATER_ORIG_DIMENSIONS" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'orig_dimensions',
                           -default => $fields{ 'orig_dimensions' },
                           -size => $CFG{ SPEC_MATER_ORIG_DIMENSIONS_SIZE },
                           -maxlength => 
                             $CFG{ SPEC_MATER_ORIG_DIMENSIONS_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Размер листа (для книг)<br />" .
        "(например, 60x90)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].size_sheet",
                   "SPEC_MATER_SIZE_SHEET", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'size_sheet',
                           -default => $fields{ 'size_sheet' },
                           -size => $CFG{ SPEC_MATER_SIZE_SHEET_SIZE },
                           -maxlength => 
                             $CFG{ SPEC_MATER_SIZE_SHEET_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Доля листа (для книг)<br />" .
        "(например, 1/16)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].part_sheet",
                   "SPEC_MATER_PART_SHEET", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'part_sheet',
                           -default => $fields{ 'part_sheet' },
                           -size => $CFG{ SPEC_MATER_PART_SHEET_SIZE },
                           -maxlength => 
                             $CFG{ SPEC_MATER_PART_SHEET_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак сопроводительного материала</th>\n";
  print "<td>",
        $query->checkbox( -name => 'accomp_flag',
                          -checked => 
                             $fields{ 'accomp_flag' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Наименование сопроводительного материала</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].accomp_title",
                   "SPEC_MATER_ACCOMP_TITLE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'accomp_title',
                          -default => $fields{ 'accomp_title' },
                          -rows => $CFG{ SPEC_MATER_ACCOMP_TITLE_ROWS },
                          -columns => $CFG{ SPEC_MATER_ACCOMP_TITLE_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Уникальный идентификатор объекта биб. описания, " .
            "если на сопроводительный материал составлено ".
            "самостоятельное описание</th>\n";
  print "<td>",
        $query->textfield( -name => 'accomp_obj_id',
                           -default => $fields{ 'accomp_obj_id' },
                           -size => $CFG{ SPEC_MATER_ACCOMP_OBJ_ID_SIZE },
                           -maxlength => 
                             $CFG{ SPEC_MATER_ACCOMP_OBJ_ID_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_pack_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид упаковки</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'pack_code',
                            -default => $fields{ 'pack_code' },
                            -values => $ref_values_pack,
                            -labels => { @{ $ref_labels_pack } }
                          ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment",
                   "SPEC_MATER_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ SPEC_MATER_COMMENT_ROWS },
                          -columns => $CFG{ SPEC_MATER_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
    
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;

  # поскольку эта функция обрабатывает только поле-флаг 'accomp_flag',
  # имеющее тип bool, то проверяем значение параметра только на непустоту
  $value = $query->param( $fname ) ? 'yes' : 'no';

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Специфическое обозначение материала и объем; " .
                            "сопроводительный материал" );
  return ( 0 );
}
