#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Стандартные номера (ISBN, ISSN и др.), присвоенные объектам
#           библиографического описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'STD_NUMS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Std_nums
my @f_names =
   ( 'obj_id',           # Уникальный идентификатор объекта
                         # библиографического описания
     'std_num_code',     # Код вида стандартного номера
     'std_num_val',      # Стандартный номер (ISBN, ISSN ...)
     'std_num_note',     # Дополнительные сведения к стандартному номеру
     'std_num_wrong',    # Флаг неверного стандартного номера (ISBN или ISSN)
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # код вида стандартного номера (допускаются только цифры)
  if ( $query->param( 'std_num_code' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код вида стандартного " .
          "номера" );
  }

  if ( $query->param( 'std_num_val' ) eq '' )
  {
    push( @error_messages, "Не указан стандартный номер (ISBN, ISSN ...)" );
  }

  # DEBUG
  # print_params( $query );

  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }
  
  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $std_num_val;        # стандартный номер (ISBN, ISSN ...)
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $std_num_val = $query->param( 'std_num_val' );
  $target_win  = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}std_nums " .
             "WHERE obj_id = '$obj_id' AND " .
                   "std_num_val = '$std_num_val'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех стандартных номеров для данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id     = $query->param( 'obj_id' );
  $bib_rec    = $query->param( 'bib_rec' );
  $target_win = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о стандартных номерах для данного объекта
  # (получим также краткое наименования вида стандартного номера)
  $command = "SELECT sn.*, st.std_num_name " .
             "FROM ${db_schema}std_nums sn, ${db_schema}std_num_types st " .
             "WHERE sn.obj_id = $obj_id AND " .
                   "sn.std_num_code = st.std_num_code " .
             "ORDER BY st.std_num_name";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Вид стандартного номера</th>\n";
    print "<th>Стандартный номер</th>\n";
    print "<th>Дополнительные сведения к стандартному номеру</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим вид стандартного номера и сам номер, а также доп. сведения
      # (добавим символ &nbsp; к доп. сведениям к станд. номеру
      # из соображений эстетики: если значение поля 'std_num_note' равно
      # null, то не отображаются горизонтальные строки-разделители)
      print "<td>$row[ $result->fnumber( 'std_num_name' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'std_num_val' ) ]</td>\n";
      print "<td>$row[ $result->fnumber( 'std_num_note' ) ]&nbsp;</td>\n";

      # просмотр данной записи о стандартном номере в виде формы
      print "<td>" if ( $can_view || $can_delete );
      # заэкранируем буквы русского алфавита, которые могут оказаться
      # в поле std_num_val (например, авторский знак содержит русскую букву)
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&std_num_val=" .
            CGI::escape( $row[ $result->fnumber( 'std_num_val' ) ] ) . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о стандартном номере
      # заэкранируем буквы русского алфавита, которые могут оказаться
      # в поле std_num_val (например, авторский знак содержит русскую букву)
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&std_num_val=" .
            CGI::escape( $row[ $result->fnumber( 'std_num_val' ) ] ) . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о стандартном номере
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $std_num_val;        # стандартный номер (ISBN, ISSN ...)
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $std_num_val = $query->param( 'std_num_val' );
  $target_win  = $query->param( 'target_win' );

  # выбираем одну запись для стандартного номера
  $command = "SELECT * " .
             "FROM ${db_schema}std_nums " .
             "WHERE obj_id = $obj_id AND " .
                   "std_num_val = '$std_num_val'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "стандартный номер (ISBN, ISSN ...) - $std_num_val",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "std_nums",
                       key => "obj_id, std_num_val:std_num_val_begin",
                       f_names => \@f_names,
                       f_special => [ 'std_num_wrong' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # видов стандартных номеров)
  my( $ref_values, $ref_labels ) = popup_std_num_types_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения ключевого поля из параметров скрипта
    $fields{ 'obj_id' } = $query->param( 'obj_id' );

    # вид стандартного номера
    $fields{ 'std_num_code' } = $CFG{ STD_NUMS_STD_NUM_CODE };

    # если в конфиг. файле не указано начальное значение этого параметра,
    # то назначим 1 для определенности, иначе при вводе новой записи имеем
    # такую картину: в списке выбора в HTML-форме стоит опция "ISBN" (просто
    # она идет первой в массиве-параметре values), а поле "Флаг неверного
    # стандартного номера (ISBN или ISSN)" тем не менее заблокировано
    # (поскольку ключа 'std_num_code' в хеш-массиве $fields при создании
    # формы для ввода новой записи еще нет)
    $fields{ 'std_num_code' } = 1 if $fields{ 'std_num_code' } eq undef;
    
    # стандартный номер (ISBN, ISSN ...)
    $fields{ 'std_num_val' }  = $CFG{ STD_NUMS_STD_NUM_VAL };

    # доп. сведения к стандартному номеру
    $fields{ 'std_num_note' } = $CFG{ STD_NUMS_STD_NUM_NOTE };

    # комментарий
    $fields{ 'comment' }      = $CFG{ STD_NUMS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript">
  <!--
  // функция для управления доступом к полю "Флаг неверного стандартного
  // номера (ISBN или ISSN)" в зависимости от значения поля "Вид стандартного
  // номера"
  function on_change_std_num_code()
  {
    // получим массив опций меню и индекс текущей опции
    var opts = document.forms[ 0 ].std_num_code.options;
    var ind = document.forms[ 0 ].std_num_code.selectedIndex;
    
    // если выбрано значение "ISBN" или "ISSN"
    if ( opts[ ind ].text == "ISBN" || opts[ ind ].text == "ISSN" )
    {    
      // разрешим доступ к полю "Флаг неверного стандартного номера"
      document.forms[ 0 ].std_num_wrong.disabled = false;
    }
    else
    {
      // запретим доступ к полю "Флаг ..." и сбросим этот флаг
      document.forms[ 0 ].std_num_wrong.disabled = true;
      document.forms[ 0 ].std_num_wrong.checked = false;
    }
  }
  //-->
  </script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор объекта
  # библиографического описания" не подлежит редактированию, не будем позволять
  # его редактировать, но совсем убрать его нельзя, т.к. оно используются при
  # выполнении операции UPDATE в качестве ключа, а также при выполнении
  # операции INSERT (значение этого поля сохраняется таким же, каким оно было
  # при предыдущем запуске скрипта, поэтому можно не присваивать ему значения)
  print $query->hidden( -name => 'obj_id' ), "\n";

  # создадим скрытое поле для "Стандартного номера (ISBN, ISSN ...)"
  # ПРИМЕЧАНИЕ. Идентификаторы скрытых полей имеют постфикс 'begin' для того,
  #             чтобы можно было обновлять значения этих полей, поскольку они
  #             являются ключевыми (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужны и исходные значения этих полей, и их новые
  #             значения (которые, возможно, были изменены пользователем).
  print $query->hidden( -name => 'std_num_val_begin',
                        -default => $fields{ 'std_num_val' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();         # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_std_num_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Вид стандартного номера</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'std_num_code',
                            -default => $fields{ 'std_num_code' },
                            -values => $ref_values,
                            -labels => { @{ $ref_labels } },
                            -onchange => "on_change_std_num_code()"
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Стандартный номер (ISBN, ISSN ...)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].std_num_val", "STD_NUMS_STD_NUM_VAL" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'std_num_val',
                           -default => $fields{ 'std_num_val' },
                           -size => $CFG{ STD_NUMS_STD_NUM_VAL_SIZE },
                           -maxlength => $CFG{ STD_NUMS_STD_NUM_VAL_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дополнительные сведения к стандартному номеру</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].std_num_note",
                   "STD_NUMS_STD_NUM_NOTE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'std_num_note',
                          -default => $fields{ 'std_num_note' },
                          -rows => $CFG{ STD_NUMS_STD_NUM_NOTE_ROWS },
                          -columns => $CFG{ STD_NUMS_STD_NUM_NOTE_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак неверного стандартного номера (ISBN или ISSN)</th>\n";
  print "<td>";

  # массив параметров для поля std_num_wrong
  my @std_num_wrong_params = ( '-name', 'std_num_wrong',
                               '-checked', 
                                  $fields{ 'std_num_wrong' } eq 't' ? 
                                           '1' : '0',
                               '-value', 'yes',
                               '-label', ''
                             );
  # DEBUG
  # не удалять этот фрагмент отладочного кода (он показывает, что можно
  # вместо использования ссылки на хеш { @{ $ref_labels } } воспользоваться
  # вспомогательным хеш-массивом %tmp, тогда код получится более читабельным;
  # однако оставлена более сложная в синтаксическом отношении конструкция:
  # для тренировки в написании подобных "монстров")
  # print "\$fields{ 'std_num_code' } = $fields{ 'std_num_code' }" . "<BR>\n";
  # my %tmp = @{ $ref_labels };  
  # print "\$tmp{ \$fields{ 'std_num_code' } } = " .
  #       "$tmp{ $fields{ 'std_num_code' } }" . "<BR>\n"; 
  # if ( $tmp{ $fields{ 'std_num_code' } } ne "ISBN" &&
  #      $tmp{ $fields{ 'std_num_code' } } ne "ISSN" )

  # если значение поля "Вид стандартного номера" не равно "ISBN" или "ISSN",
  # то запрещаем ввод в поле "Флаг неверного стандартного номера"
  if ( { @{ $ref_labels } }->{ $fields{ 'std_num_code' } } ne "ISBN" &&
       { @{ $ref_labels } }->{ $fields{ 'std_num_code' } } ne "ISSN"
     )
  {
    push( @std_num_wrong_params, '-disabled', 'disabled' );
  }

  print $query->checkbox( @std_num_wrong_params ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "STD_NUMS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ STD_NUMS_COMMENT_ROWS },
                          -columns => $CFG{ STD_NUMS_COMMENT_COLS }
                        ),
        "</td\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Флаг неверного стандартного номера (ISBN или ISSN)
  if ( $fname eq 'std_num_wrong' )
  {
    # это поле имеет тип bool, поэтому проверяем значение параметра
    # только на непустоту
    $value = $query->param( $fname ) ? 'yes' : 'no';
  }

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Стандартные номера (ISBN, ISSN и др.), " .
                    "присвоенные объекту библиографического описания" );
  return ( 0 );
}
