#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Экземпляры (единицы хранения)"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'STOR_OBJS';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Stor_objs
my @f_names =
   ( 'obj_id',               # Уникальный идентификатор объекта
                             # библиографического описания
     'stor_obj_num',         # Порядковый номер единицы хранения для
                             # данного объекта библиогр. описания
     'pers_org_flag',        # Флаг "персоналия/организация"
                             # (1 - персоналия; 2 - организация)
     'owner_pers_id',        # Код владельца (персоналии)
     'owner_org_id',         # Код владельца (организации)
     'phys_form_code',       # Код физической формы, в которой
                             # присутствует единица хранения
     'stor_obj_code',        # Код (шифр) единицы хранения
                             # (например, номер экземпляра книги в библиотеке)
     'stor_place',           # Место хранения (например, файл на CD-ROM
                             # таком-то (идентификатор этого диска)
     'stor_obj_add_info',    # Доп. сведения о физической форме и объеме
                             # имеющейся части объекта описания
     'stor_obj_defects',     # Дефекты единицы хранения
     'stor_obj_phys_cond',   # Оценка общего физического состояния
                             # единицы хранения по 10-балльной шкале
     'stor_obj_complete',    # Признак полноты комплектации
     'acq_date',             # Дата приобретения
     'acq_date_precise',     # Признак точной даты приобретения
     'acq_date_note',        # Пояснения к дате приобретения
     'acq_place_city',       # Место приобретения (код населенного пункта)
     'acq_place_country',    # Место приобретения (код страны)
                             # (указывается, когда не известно точное
                             # наименование населенного пункта)
     'acq_way_code',         # Код канала поступления
     'stor_obj_price_curr',  # Цена (фактическая) в иностранной валюте (если
                             # эта единица хранения приобретена за валюту)
     'currency_code',        # Код валюты
     'exchange_rate',        # Обменный курс валюты к рублю
                             # (т.е. количество рублей, которое
                             # приравнивается к 1 единице иностранной валюты)
     'stor_obj_price',       # Цена в рублях (фактическая цена за данную
                             # единицу хранения)
     'stor_obj_price_note',  # Дополнительные сведения к цене
     'acq_note',             # Доп. сведения о приобретении
     'remove_stor_obj_flag', # Флаг выбытия единицы хранения
     'remove_stor_obj_note', # Доп. информация о выбытии единицы хранения
     'comment'               # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  if ( $query->param( 'stor_obj_num' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указано или неверное значение порядкового " .
          "номера единицы хранения для данного объекта библиогр. описания" );
  }

  if ( ! ( $query->param( 'pers_org_flag' ) eq '1' ||
           $query->param( 'pers_org_flag' ) eq '2'
         )
     )    
  {
    push( @error_messages, "Неверное значение флага \"персоналия/" .
          "организация\"" );
  }

  if ( $query->param( 'owner_org_id' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код владельца " .
          "(организация)" );
  }

  if ( $query->param( 'owner_pers_id' ) !~ /^\d+$/ )
  {
    push( @error_messages, "Не указан или неверный код владельца " .
          "(персоналия)" );
  }

  if ( $query->param( 'phys_form_code' ) eq '' )
  {
    push( @error_messages, "Не указан код физической формы, в которой " .
          "присутствует единица хранения" );
  }

  if ( ! ( $query->param( 'stor_obj_phys_cond' ) =~ /^\d{1,2}$/ &&
           $query->param( 'stor_obj_phys_cond' ) >= 1 &&
           $query->param( 'stor_obj_phys_cond' ) <= 10
         )  
     )
  {
    push( @error_messages, "Не указана или неверная оценка общего " .
          "физического состояния единицы хранения по 10-балльной шкале" );
  }

  # допускается пустое поле
  if ( $query->param( 'acq_date' ) ne '' &&
       ! date_is_correct( $query->param( 'acq_date' ) ) )
  {
    push( @error_messages, "Неверная дата введения данной валюты" );
  }

  # допускается пустое поле
  if ( $query->param( 'acq_place_city' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный код места приобретения " .
          "(код населенного пункта)" );
  }

  # допускается пустое поле
  if ( $query->param( 'acq_place_country' ) !~ /^\d*$/ )
  {
    push( @error_messages, "Неверный код места приобретения (код страны)" );
  }

  if ( $query->param( 'acq_way_code' ) eq '' )
  {
    push( @error_messages, "Не указан код канала поступления" );
  }

  # пустое поле или число, возможно, с дробной частью (не более двух цифр)
  if ( $query->param( 'stor_obj_price_curr' ) !~ /^(\d*|\d+\.\d{1,2})$/ )
  {
    push( @error_messages, "Неверная цена (фактическая) в иностранной " .
          "валюте (если эта единица хранения приобретена за валюту)" );
  }

  if ( $query->param( 'currency_code' ) eq '' )
  {
    push( @error_messages, "Не указан код валюты" );
  }

  # пустое поле или число, возможно, с дробной частью (не более пяти цифр)
  if ( $query->param( 'exchange_rate' ) !~ /^(\d*|\d+\.\d{1,5})$/ )
  {
    push( @error_messages, "Неверное значение обменного курса валюты " .
          "к рублю" );
  }

  # пустое поле или число, возможно, с дробной частью (не более двух цифр)
  if ( $query->param( 'stor_obj_price' ) !~ /^(\d*|\d+\.\d{1,2})$/ )
  {
    push( @error_messages, "Неверная цена (фактическая) в рублях" );
  }

  # ПРИМЕЧАНИЕ. Решил пока что не делать проверку, соответствующую ограничению
  #             CHECK таблицы Stor_objs. Пусть в случае ошибки,
  #             допущенной при заполнении полей формы, работает СУБД.

  # DEBUG
  # print_params( $query );
  
  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }

  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $stor_obj_num;       # порядковый номер единицы хранения для
                          # данного объекта библиогр. описания
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id       = $query->param( 'obj_id' );
  $stor_obj_num = $query->param( 'stor_obj_num' );
  $target_win   = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}stor_objs " .
             "WHERE obj_id = '$obj_id' AND " .
                   "stor_obj_num = '$stor_obj_num'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка экземпляров (единиц хранения) для данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки
  my $owner;              # владелец экземпляра хранения

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id     = $query->param( 'obj_id' );
  $bib_rec    = $query->param( 'bib_rec' );
  $target_win = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи об экземплярах хранения для данного объекта
  # (получим также краткое наименования организации-владельца, наименование
  # персоналии-владельца и краткое наименование физ. формы хранения)
  $command = "SELECT s.*, p.pers_name, o.org_sname, f.phys_form_sname " .
             "FROM ${db_schema}stor_objs s, ${db_schema}persons p, " .
                  "${db_schema}orgs o, ${db_schema}phys_form_types f " .
             "WHERE s.obj_id = $obj_id AND " .
                   "s.owner_pers_id = p.pers_id AND " .
                   "s.owner_org_id = o.org_id AND " .
                   "s.phys_form_code = f.phys_form_code " .
             "ORDER BY s.stor_obj_num";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Порядковый номер единицы хранения</th>\n";
    print "<th>Владелец</th>\n";
    print "<th>Физическая форма</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим номер единицы хранения, наименование владельца и наименование
      # физ. формы единицы хранения
      print "<td class=\"center\">" .
            "$row[ $result->fnumber( 'stor_obj_num' ) ]</td>\n";

      # сформируем владельца
      
      # организация
      if ( $row[ $result->fnumber( 'pers_org_flag' ) ] == 2 )
      {
        $owner = $row[ $result->fnumber( 'org_sname' ) ];
      }
      # персоналия
      else       # ( $row[ $result->fnumber( 'pers_org_flag' ) ] == 1 )
      {
        $owner = $row[ $result->fnumber( 'pers_name' ) ];
      }

      print "<td>$owner</td>\n";      
      print "<td>$row[ $result->fnumber( 'phys_form_sname' ) ]</td>\n";

      # просмотр данной записи об экземпляре хранения в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&stor_obj_num=" .
            $row[ $result->fnumber( 'stor_obj_num' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи об экземпляре хранения
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&stor_obj_num=" .
            $row[ $result->fnumber( 'stor_obj_num' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи об экземпляре хранения
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $stor_obj_num;       # порядковый номер единицы хранения для
                          # данного объекта библиогр. описания
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id       = $query->param( 'obj_id' );
  $stor_obj_num = $query->param( 'stor_obj_num' );
  $target_win   = $query->param( 'target_win' );

  # выбираем одну запись об экземпляре хранения
  # (получим также краткое наименования организации-владельца и наименование
  # персоналии-владельца; краткие наименования города и страны, причем страну
  # выберем дважды: один раз - на основе кода страны из таблицы Stor_objs,
  # а второй - на основе кода страны из таблицы Cities, поскольку в таблице
  # Stor_objs город и страна НЕ СВЯЗАНЫ между собой)
  $command = "SELECT s.*, p.pers_name, o.org_sname, ci.city_sname, " .
                    "co1.country_sname as country_sname_acq, " .
                    "co2.country_sname as country_sname_city " .
             "FROM ${db_schema}stor_objs s, ${db_schema}persons p, " .
                  "${db_schema}orgs o, ${db_schema}cities ci, " .
                  "${db_schema}countries co1, ${db_schema}countries co2 " .
             "WHERE s.obj_id = '$obj_id' AND " .
                   "s.stor_obj_num = '$stor_obj_num' AND " .
                   "s.owner_pers_id = p.pers_id AND " .
                   "s.owner_org_id = o.org_id AND " .
                   "s.acq_place_city = ci.city_code AND " .
                   "s.acq_place_country = co1.country_code AND " .
                   "ci.country_code = co2.country_code";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "порядковый номер единицы хранения - $stor_obj_num",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "stor_objs",
                       key => "obj_id, stor_obj_num:stor_obj_num_begin",
                       f_names => \@f_names,
                       f_special => [ 'acq_date', 'stor_obj_complete',
                                      'remove_stor_obj_flag' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  # получим две ссылки на массивы (они используются при формировании списка
  # видов физической формы, в которой присутствует единица хранения)
  my( $ref_values_phys, $ref_labels_phys ) =
    popup_phys_form_types_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # каналов поступления единиц хранения)
  my( $ref_values_acq, $ref_labels_acq ) = popup_acq_ways_list( $db_schema );

  # получим две ссылки на массивы (они используются при формировании списка
  # валют)
  my( $ref_values_cur, $ref_labels_cur ) = popup_currencies_list( $db_schema );

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
    # "Признак точной даты приобретения" требует особой обработки, если
    # значение этого поля в таблице равно null
    $fields{ 'acq_date_precise' } = '' 
      if $fields{ 'acq_date_precise' } eq undef;
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения ключевого поля из параметров скрипта
    $fields{ 'obj_id' } = $query->param( 'obj_id' );

    # значения по умолчанию

    # порядковый номер единицы хранения для данного объекта библиогр. описания
    if ( $CFG{ STOR_OBJS_STOR_OBJ_NUM } )  # есть параметр в конфиг. файле
    {
      $fields{ 'stor_obj_num' } = $CFG{ STOR_OBJS_STOR_OBJ_NUM };
    }
    else
    {
      # будем предлагать пользователю значение поля stor_obj_num для новой
      # записи, которое на 1 больше максимального значения этого поля 
      # для данного объекта библиогр. описания (для значения поля obj_id 
      # в таблице stor_objs)
      $fields{ 'stor_obj_num' } = 
        get_next_key_value( 'stor_objs', 'stor_obj_num',
                            'obj_id = ' . $query->param( 'obj_id' ) );
    }

    # флаг "персоналия/организация"
    $fields{ 'pers_org_flag' }        = $CFG{ STOR_OBJS_PERS_ORG_FLAG };

    # код владельца (персоналии)
    $fields{ 'owner_pers_id' }        = $CFG{ STOR_OBJS_OWNER_PERS_ID };

    # код владельца (организации)
    $fields{ 'owner_org_id' }         = $CFG{ STOR_OBJS_OWNER_ORG_ID };

    # код физической формы, в которой присутствует единица хранения
    $fields{ 'phys_form_code' }       = $CFG{ STOR_OBJS_PHYS_FORM_CODE };

    # код (шифр) единицы хранения (например, номер экземпляра книги
    # в библиотеке)
    $fields{ 'stor_obj_code' }        = $CFG{ STOR_OBJS_STOR_OBJ_CODE };

    # место хранения
    $fields{ 'stor_place' }           = $CFG{ STOR_OBJS_STOR_PLACE };

    # доп. сведения о физ. форме и объеме имеющейся части объекта описания
    $fields{ 'stor_obj_add_info' }    = $CFG{ STOR_OBJS_STOR_OBJ_ADD_INFO };

    # дефекты единицы хранения
    $fields{ 'stor_obj_defects' }     = $CFG{ STOR_OBJS_STOR_OBJ_DEFECTS };

    # оценка общего физ. состояния единицы хранения по 10-балльной шкале
    $fields{ 'stor_obj_phys_cond' }   = $CFG{ STOR_OBJS_STOR_OBJ_PHYS_COND };

    # признак полноты комплектации
    $fields{ 'stor_obj_complete' }    = $CFG{ STOR_OBJS_STOR_OBJ_COMPLETE };

    # дата приобретения
    $fields{ 'acq_date' }             = $CFG{ STOR_OBJS_ACQ_DATE };

    # признак точной даты приобретения
    $fields{ 'acq_date_precise' }     = $CFG{ STOR_OBJS_ACQ_DATE_PRECISE };

    # пояснения к дате приобретения
    $fields{ 'acq_date_note' }        = $CFG{ STOR_OBJS_ACQ_DATE_NOTE };

    # место приобретения (код населенного пункта)
    $fields{ 'acq_place_city' }       = $CFG{ STOR_OBJS_ACQ_PLACE_CITY };

    # место приобретения (код страны)
    $fields{ 'acq_place_country' }    = $CFG{ STOR_OBJS_ACQ_PLACE_COUNTRY };

    # код канала поступления
    $fields{ 'acq_way_code' }         = $CFG{ STOR_OBJS_ACQ_WAY_CODE };

    # цена (фактическая) в иностранной валюте (если эта единица хранения
    # приобретена за валюту)
    $fields{ 'stor_obj_price_curr' }  = $CFG{ STOR_OBJS_STOR_OBJ_PRICE_CURR };

    # код валюты
    $fields{ 'currency_code' }        = $CFG{ STOR_OBJS_CURRENCY_CODE };

    # обменный курс валюты к рублю
    $fields{ 'exchange_rate' }        = $CFG{ STOR_OBJS_EXCHANGE_RATE };

    # цена в рублях (фактическая цена за данную единицу хранения)
    $fields{ 'stor_obj_price' }       = $CFG{ STOR_OBJS_STOR_OBJ_PRICE };

    # дополнительные сведения к цене
    $fields{ 'stor_obj_price_note' }  = $CFG{ STOR_OBJS_STOR_OBJ_PRICE_NOTE };

    # доп. сведения о приобретении
    $fields{ 'acq_note' }             = $CFG{ STOR_OBJS_ACQ_NOTE };

    # флаг выбытия единицы хранения
    $fields{ 'remove_stor_obj_flag' } = $CFG{ STOR_OBJS_REMOVE_STOR_OBJ_FLAG };

    # доп. информация о выбытии единицы хранения
    $fields{ 'remove_stor_obj_note' } = $CFG{ STOR_OBJS_REMOVE_STOR_OBJ_NOTE };

    # комментарий
    $fields{ 'comment' }              = $CFG{ STOR_OBJS_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript">
  <!--
  // функция для координирования значений ряда полей и доступа к ним
  // в зависимости от значения поля "Флаг 'персоналия/организация'"
  function on_change_pers_org_flag()
  {
    // если выбрана "персоналия" (значение поля -- 1)
    if ( document.forms[ 0 ].pers_org_flag.selectedIndex == 0 )
    {
      document.forms[ 0 ].owner_org_id.value = 0; // 0 -- фиктивная организация
      document.forms[ 0 ].org_sname.value = '';           // очистим поле
      document.forms[ 0 ].owner_org_id.disabled = true;   // запретим ввод
      document.forms[ 0 ].org_sname.disabled = true;      // запретим ввод
      document.forms[ 0 ].owner_pers_id.disabled = false; // разрешим ввод
      document.forms[ 0 ].pers_name.disabled = false;     // разрешим ввод
    }
    // если выбрана "организация" (значение поля -- 2)
    else if ( document.forms[ 0 ].pers_org_flag.selectedIndex == 1 )
    {
      document.forms[ 0 ].owner_pers_id.value = 0; // 0 -- фиктивная персоналия
      document.forms[ 0 ].pers_name.value = '';           // очистим поле
      document.forms[ 0 ].owner_pers_id.disabled = true;  // запретим ввод
      document.forms[ 0 ].pers_name.disabled = true;      // запретим ввод
      document.forms[ 0 ].owner_org_id.disabled = false;  // разрешим ввод
      document.forms[ 0 ].org_sname.disabled = false;     // разрешим ввод
    }
  }
  //-->
  </script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор объекта
  # библиографического описания" не подлежит редактированию, не будем позволять
  # его редактировать, но совсем убрать его нельзя, т.к. оно используются при
  # выполнении операции UPDATE в качестве ключа, а также при выполнении
  # операции INSERT (значение этого поля сохраняется таким же, каким оно было
  # при предыдущем запуске скрипта, поэтому можно не присваивать ему значения)
  print $query->hidden( -name => 'obj_id' ), "\n";

  # создадим скрытое поле для "Порядкового номера единицы хранения для
  # данного объекта библиогр. описания"
  # ПРИМЕЧАНИЕ. Идентификатор скрытого поля имеет постфикс 'begin' для того,
  #             чтобы можно было обновлять значение этого поля, поскольку оно
  #             является ключевым (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужно и исходное значение этого поля, и его новое
  #             значение (которое, возможно, было изменено пользователем).
  print $query->hidden( -name => 'stor_obj_num_begin',
                        -default => $fields{ 'stor_obj_num' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();         # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Порядковый номер единицы хранения</th>\n";
  print "<td>",
        $query->textfield( -name => 'stor_obj_num',
                           -default => $fields{ 'stor_obj_num' },
                           -size => $CFG{ STOR_OBJS_STOR_OBJ_NUM_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_STOR_OBJ_NUM_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак \"персоналия/организация\"</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'pers_org_flag',
                            -default => $fields{ 'pers_org_flag' },
                            -values => [ '1', '2' ],
                            -labels => { '1' => 'персоналия',
                                         '2' => 'организация' },
                            -onchange => "on_change_pers_org_flag()"
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Владелец &mdash; персоналия (код и Ф. И. О.)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник persons.pl
  create_selection_func( func_name => 'select_person',
                         module_name => 'persons.pl',
                         caller_name => $module,
                         window_name => 'persons',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также наименование персоналии (без возможности редактирования)
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Имя персоналии" 
  create_hot_help( "document.forms[ 0 ].owner_pers_id",
                   "STOR_OBJS_OWNER_PERS_ID", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].pers_name.value = str; " )
    if $CFG{ HOT_HELP_IS_ON };

  # массивы параметров для полей owner_pers_id и pers_name
  my @owner_pers_id_params = ( '-name', 'owner_pers_id',
                               '-default', $fields{ 'owner_pers_id' },
                               '-size', $CFG{ STOR_OBJS_OWNER_PERS_ID_SIZE },
                               '-maxlength', 
                                 $CFG{ STOR_OBJS_OWNER_PERS_ID_MAXLENGTH }
                             );
  my @pers_name_params = ( '-name', 'pers_name',
                           '-default', $fields{ 'pers_name' },
                           '-size', $CFG{ STOR_OBJS_PERS_NAME_SIZE },
                           '-maxlength', $CFG{ STOR_OBJS_PERS_NAME_MAXLENGTH },
                           '-readonly', 'readonly'
                         );  
  # если издатель -- "организация" (2), то запрещаем ввод в эти поля
  if ( $fields{ 'pers_org_flag' } == 2 )
  {
    push( @owner_pers_id_params, '-disabled', 'disabled' );
    push( @pers_name_params, '-disabled', 'disabled' );
  }

  print $query->textfield( @owner_pers_id_params ), "&nbsp;&nbsp;",
        $query->textfield( @pers_name_params ), "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" 
            onclick=
            "if ( document.forms[ 0 ].owner_pers_id.disabled == false )
               select_person();
             else
               window.alert( 'Поле заблокировано. Ввод невозможен' );
            ">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Владелец &mdash; организация (код и наименование)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник orgs.pl
  create_selection_func( func_name => 'select_org',
                         module_name => 'orgs.pl',
                         caller_name => $module,
                         window_name => 'orgs',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также краткое наименование организации
  # (без возможности редактирования)
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Название организации"
  create_hot_help( "document.forms[ 0 ].owner_org_id",
                   "STOR_OBJS_OWNER_ORG_ID", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].org_sname.value = str; " )
    if $CFG{ HOT_HELP_IS_ON };

  # массивы параметров для полей owner_org_id и org_sname
  my @owner_org_id_params = ( '-name', 'owner_org_id',
                              '-default', $fields{ 'owner_org_id' },
                               '-size', $CFG{ STOR_OBJS_OWNER_ORG_ID_SIZE },
                               '-maxlength', 
                                 $CFG{ STOR_OBJS_OWNER_ORG_ID_MAXLENGTH }
                            );
  my @org_sname_params = ( '-name', 'org_sname',
                           '-default', $fields{ 'org_sname' },
                           '-size', $CFG{ STOR_OBJS_OWNER_ORG_SNAME_SIZE },
                           '-maxlength', 
                             $CFG{ STOR_OBJS_OWNER_ORG_SNAME_MAXLENGTH },
                           '-readonly', 'readonly'
                         );
  # если издатель -- "персоналия" (1), то запрещаем ввод в эти поля
  if ( $fields{ 'pers_org_flag' } == 1 )
  {
    push( @owner_org_id_params, '-disabled', 'disabled' );
    push( @org_sname_params, '-disabled', 'disabled' );
  }

  print $query->textfield( @owner_org_id_params ), "&nbsp;&nbsp;",
        $query->textfield( @org_sname_params ), "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )"
            onclick=
            "if ( document.forms[ 0 ].owner_org_id.disabled == false )
               select_org();
             else
               window.alert( 'Поле заблокировано. Ввод невозможен' );
            ">} . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_phys_form_types_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Физическая форма, в которой " .
            "присутствует единица хранения</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'phys_form_code',
                            -default => $fields{ 'phys_form_code' },
                            -values => $ref_values_phys,
                            -labels => { @{ $ref_labels_phys } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Код (шифр) единицы хранения<br />" .
            "(например, номер экземпляра книги в библиотеке)<br />" .
            "(этого кода может и не быть)</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].stor_obj_code",
                   "STOR_OBJS_STOR_OBJ_CODE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'stor_obj_code',
                           -default => $fields{ 'stor_obj_code' },
                           -size => $CFG{ STOR_OBJS_STOR_OBJ_CODE_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_STOR_OBJ_CODE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Место хранения<br />" .
            "(идентификатор DVD-диска, на котором хранится файл;<br />" .
            "шифр отдела в библиотеке и т. п.)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].stor_place", "STOR_OBJS_STOR_PLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'stor_place',
                          -default => $fields{ 'stor_place' },
                          -rows => $CFG{ STOR_OBJS_STOR_PLACE_ROWS },
                          -columns => $CFG{ STOR_OBJS_STOR_PLACE_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дополнительные сведения о физической форме и объеме " .
            "имеющейся части объекта описания<br />" .
            "(например, формат файла и его размер; полнота ксерокопии, " .
            "т. е. весь документ или его часть)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].stor_obj_add_info",
                   "STOR_OBJS_STOR_OBJ_ADD_INFO" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'stor_obj_add_info',
                          -default => $fields{ 'stor_obj_add_info' },
                          -rows => $CFG{ STOR_OBJS_STOR_OBJ_ADD_INFO_ROWS },
                          -columns => $CFG{ STOR_OBJS_STOR_OBJ_ADD_INFO_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дефекты единицы хранения<br />" .
            "(порванные страницы, наличие пометок в тексте, " .
            "отсутствие страниц и т. п.)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].stor_obj_defects",
                   "STOR_OBJS_STOR_OBJ_DEFECTS" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'stor_obj_defects',
                          -default => $fields{ 'stor_obj_defects' },
                          -rows => $CFG{ STOR_OBJS_STOR_OBJ_DEFECTS_ROWS },
                          -columns => $CFG{ STOR_OBJS_STOR_OBJ_DEFECTS_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Оценка общего физического состояния единицы хранения</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'stor_obj_phys_cond',
                            -default => $fields{ 'stor_obj_phys_cond' },
                            -values => [ '1', '2', '3', '4', '5',
                                         '6', '7', '8', '9', '10' ]
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак полноты комплектации<br />" .
            "('нет' - если, например, отсутствует часть страниц; " .
            "детальные сведения отражаются в поле \"Дефекты единицы " .
            "хранения\")</th>\n";
  print "<td>",
        $query->checkbox( -name => 'stor_obj_complete',
                          -checked => 
                             $fields{ 'stor_obj_complete' } eq 't' ? '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата приобретения<br />" .
            "(если дата известна с точностью до месяца, " .
            "то ставить первое число этого месяца, если с точностью " .
            "до года &mdash; первое число этого года, если она " .
            "не известна, то оставить пустое поле)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля)
  create_hot_help( "document.forms[ 0 ].acq_date",
                   "STOR_OBJS_ACQ_DATE", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'acq_date',
                           -default =>
                              date_for_display( $fields{ 'acq_date' } ),
                           -size => $CFG{ STOR_OBJS_ACQ_DATE_SIZE },
                           -maxlength => $CFG{ STOR_OBJS_ACQ_DATE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак точной даты приобретения</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'acq_date_precise',
                            -default => $fields{ 'acq_date_precise' },
                            -values => [ 't', 'f', '' ],
                            -labels => { 't' => 'дата точная',
                                         'f' => 'дата неточная',
                                         ''  => 'дата не известна' }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Пояснения к дате приобретения</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].acq_date_note",
                   "STOR_OBJS_ACQ_DATE_NOTE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'acq_date_note',
                           -default => $fields{ 'acq_date_note' },
                           -size => $CFG{ STOR_OBJS_ACQ_DATE_NOTE_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_ACQ_DATE_NOTE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Место приобретения (код и наименование населенного пункта, " .
        "наименование страны)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник cities.pl
  create_selection_func( func_name => 'select_city',
                         module_name => 'cities.pl',
                         caller_name => $module,
                         window_name => 'cities',
                         exp_imp => $query->param( 'exp_imp' ) );

  # выведем также краткое наименование города и страны
  # (без возможности редактирования)
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Название города";
  # название страны просто обнуляем (наша функция create_hot_help() не 
  # позволяет предложить более интеллектуальную обработку для этого поля);
  # кроме того, обнуляем поля "Место приобретения (код страны)" и
  # "Наименование страны" (когда не известен город)
  create_hot_help( "document.forms[ 0 ].acq_place_city",
                   "STOR_OBJS_ACQ_PLACE_CITY", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].city_sname.value = str;
                   document.forms[ 0 ].country_sname_city.value = '';
                   document.forms[ 0 ].acq_place_country.value = 0;
                   document.forms[ 0 ].country_sname_acq.value = ''; " )
    if $CFG{ HOT_HELP_IS_ON };
  
  print $query->textfield( -name => 'acq_place_city',
                           -default => $fields{ 'acq_place_city' },
                           -size => $CFG{ STOR_OBJS_ACQ_PLACE_CITY_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_ACQ_PLACE_CITY_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'city_sname',
                           -default => $fields{ 'city_sname' },
                           -size => $CFG{ STOR_OBJS_CITY_SNAME_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_CITY_SNAME_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'country_sname_city',
                           -default => $fields{ 'country_sname_city' },
                           -size => $CFG{ STOR_OBJS_COUNTRY_SNAME_CITY_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_COUNTRY_SNAME_CITY_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )" 
             onclick="select_city();
                      document.forms[ 0 ].acq_place_country.value = 0;
                      document.forms[ 0 ].country_sname_acq.value = ''; ">
          } . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Место приобретения (код и наименование страны)<br />" .
            "(указывается, когда не известно точное " .
            "наименование населенного пункта)</th>\n";

  # сформируем текст функции на языке JavaScript, которая вызывает
  # модуль-справочник countries.pl
  create_selection_func( func_name => 'select_country',
                         module_name => 'countries.pl',
                         caller_name => $module,
                         window_name => 'countries',
                         exp_imp => $query->param( 'exp_imp' ) );
  # выведем также краткое наименование страны (без возможности редактирования)
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля);
  # фрагмент JavaScript-кода означает следующее: если функция create_hot_help()
  # создаст HTML-кнопку, то у нее есть атрибут VALUE, а если будет создана
  # HTML-ссылка, то у нее такого атрибута нет, поэтому мы используем 
  # атрибут NAME для записи его значения в поле "Название страны";
  # кроме того, обнуляем поля "Место приобретения (код населенного пункта)",
  # "Наименование города" и "Наименование страны" (для этого города)
  create_hot_help( "document.forms[ 0 ].acq_place_country",
                   "STOR_OBJS_ACQ_PLACE_COUNTRY", "REPLACE",
                   "var str; if (this.value) str = this.value;
                             else str = this.name;
                   document.forms[ 0 ].country_sname_acq.value = str;
                   document.forms[ 0 ].acq_place_city.value = 0;
                   document.forms[ 0 ].city_sname.value = '';
                   document.forms[ 0 ].country_sname_city.value = ''; " )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'acq_place_country',
                           -default => $fields{ 'acq_place_country' },
                           -size => $CFG{ STOR_OBJS_ACQ_PLACE_COUNTRY_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_ACQ_PLACE_COUNTRY_MAXLENGTH }
                         ),
        "&nbsp;&nbsp;",
        $query->textfield( -name => 'country_sname_acq',
                           -default => $fields{ 'country_sname_acq' },
                           -size => $CFG{ STOR_OBJS_COUNTRY_SNAME_ACQ_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_COUNTRY_SNAME_ACQ_MAXLENGTH },
                           -readonly => 'readonly'
                         ),
        "&nbsp;&nbsp;",
        qq{ <a class="command_link" href="javascript:void( 0 )"
             onclick="select_country(); 
                      document.forms[ 0 ].acq_place_city.value = 0;
                      document.forms[ 0 ].city_sname.value = '';
                      document.forms[ 0 ].country_sname_city.value = '';">
          } . "Список</a>",
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_acq_ways_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Канал поступления единицы хранения</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'acq_way_code',
                            -default => $fields{ 'acq_way_code' },
                            -values => $ref_values_acq,
                            -labels => { @{ $ref_labels_acq } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Цена (фактическая) в иностранной валюте<br />" .
            "(если эта единица хранения приобретена за валюту)</th>\n";
  print "<td>";
  
  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля)
  create_hot_help( "document.forms[ 0 ].stor_obj_price_curr",
                   "STOR_OBJS_STOR_OBJ_PRICE_CURR", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };
  
  print $query->textfield( -name => 'stor_obj_price_curr',
                           -default => $fields{ 'stor_obj_price_curr' },
                           -size => $CFG{ STOR_OBJS_STOR_OBJ_PRICE_CURR_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_STOR_OBJ_PRICE_CURR_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  # для получения ссылки на хеш в параметре labels используем сложную
  # конструкцию, поскольку в функции popup_currencies_list() формируется
  # не хеш, а массив - заготовка для хеша
  print "<tr>\n";
  print "<th>Валюта</th>\n";
  print "<td>",
        $query->popup_menu( -name => 'currency_code',
                            -default => $fields{ 'currency_code' },
                            -values => $ref_values_cur,
                            -labels => { @{ $ref_labels_cur } }
                          ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Обменный курс валюты к рублю<br />" .
            "(сколько рублей стоит одна единица валюты)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля)
  create_hot_help( "document.forms[ 0 ].exchange_rate",
                   "STOR_OBJS_EXCHANGE_RATE", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'exchange_rate',
                           -default => $fields{ 'exchange_rate' },
                           -size => $CFG{ STOR_OBJS_EXCHANGE_RATE_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_EXCHANGE_RATE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Цена в рублях<br />" .
            "(фактическая цена за данную единицу хранения)</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране (точнее, замены значения поля)
  create_hot_help( "document.forms[ 0 ].stor_obj_price",
                   "STOR_OBJS_STOR_OBJ_PRICE", "REPLACE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'stor_obj_price',
                           -default => $fields{ 'stor_obj_price' },
                           -size => $CFG{ STOR_OBJS_STOR_OBJ_PRICE_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_STOR_OBJ_PRICE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дополнительные сведения к цене</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].stor_obj_price_note",
                   "STOR_OBJS_STOR_OBJ_PRICE_NOTE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textfield( -name => 'stor_obj_price_note',
                           -default => $fields{ 'stor_obj_price_note' },
                           -size => $CFG{ STOR_OBJS_STOR_OBJ_PRICE_NOTE_SIZE },
                           -maxlength =>
                             $CFG{ STOR_OBJS_STOR_OBJ_PRICE_NOTE_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дополнительные сведения о приобретении</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].acq_note", "STOR_OBJS_ACQ_NOTE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'acq_note',
                          -default => $fields{ 'acq_note' },
                          -rows => $CFG{ STOR_OBJS_ACQ_NOTE_ROWS },
                          -columns => $CFG{ STOR_OBJS_ACQ_NOTE_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Признак выбытия единицы хранения</th>\n";
  print "<td>",
        $query->checkbox( -name => 'remove_stor_obj_flag',
                          -checked => 
                             $fields{ 'remove_stor_obj_flag' } eq 't' ?
                                                               '1' : '0',
                          -value => 'yes',
                          -label => ''
                        ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дополнительная информация о выбытии единицы хранения</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].remove_stor_obj_note",
                   "STOR_OBJS_REMOVE_STOR_OBJ_NOTE" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'remove_stor_obj_note',
                          -default => $fields{ 'remove_stor_obj_note' },
                          -rows => $CFG{ STOR_OBJS_REMOVE_STOR_OBJ_NOTE_ROWS },
                          -columns =>
                            $CFG{ STOR_OBJS_REMOVE_STOR_OBJ_NOTE_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment", "STOR_OBJS_COMMENT" )
    if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ STOR_OBJS_COMMENT_ROWS },
                          -columns => $CFG{ STOR_OBJS_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Дата приобретения
  if ( $fname eq 'acq_date' )
  {
    $value = date_for_db( $query->param( $fname ) );
  }
  # Признак полноты комплектации и Флаг выбытия единицы хранения
  elsif ( $fname eq 'stor_obj_complete' || $fname eq 'remove_stor_obj_flag' )
  {
    # эти поля имеют тип bool, поэтому проверяем значение параметра
    # только на непустоту
    $value = $query->param( $fname ) ? 'yes' : 'no';
  }

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Экземпляры (единицы хранения)" );
  
  return ( 0 );
}
