#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Система:  "Электронный архив"
# Модуль:   "Проработка содержания объектов библиографического описания"
# Автор:    Моргунов Е. П.
# Дата:     15.04.2012
# Версия:   0.6
# -----------------------------------------------------------------------------

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use Pg;                          # для работы с PostgreSQL

use E_ARCH;                      # пакет функций, написанных автором

# будем выводить стек вызовов функций
$SIG{ __DIE__ } = \&CGI::Carp::confess;

my $query;                       # CGI-объект
my $script_name;                 # имя этого скрипта (включая путь)

# условное наименование модуля (используется при определении прав 
# на выполнение тех или иных процедур)
my $module = 'STUDYING';

# имя пользователя, запустившего скрипт (пока это просто REMOTE_USER)
my $user = get_user_name(); 

# массив имен полей таблицы Studying
my @f_names =
   ( 'obj_id',           # Уникальный идентификатор объекта
                         # библиографического описания
     'study_begin',      # Дата начала проработки
     'study_end',        # Дата окончания проработки
     'study_descr',      # Описание вида проработки
     'study_result',     # Описание результата проработки
     'comment'           # Примечание
   );

my $db_schema;                     # схема БД (для экспорта/импорта)

$query = CGI->new();               # создадим новый CGI-объект

# отменим буферизацию на устройствах
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

$script_name = $query->script_name;

# в имя схемы экспорта/импорта включим и точку, отделяющую имя таблицы
# от имени схемы; при отсутствии имени схемы в SQL-команде не окажется
# этой точки, что нам и нужно
$db_schema = $query->param( 'exp_imp' ) ? "exp_imp." : "";

# организуем выполнение стандартного набора режимов работы
common_modes( query => $query,
              module => $module,
              user => $user,
              make_form => \&make_form,
              make_command => \&make_command,
              check_param => \&check_param,
              create_top => \&create_top,
              do_query => \&do_query,
              do_view => \&do_view,
              do_add => \&do_add,
              do_del => \&do_del,
              add_rec => \&add_rec,
              update_rec => \&update_rec,
              print_add_info => \&print_add_info,
              action => $script_name
            );

exit( 0 );


# -----------------------------------------------------------------------------
# проверка значений параметров формы
# -----------------------------------------------------------------------------
sub check_param
{
  my $no_errors = 1;        # флаг отсутствия ошибок в параметрах
  my @error_messages = ();  # массив для сбора сообщений об ошибках при вводе

  # НЕ допускается пустое поле
  if ( ! date_is_correct( $query->param( 'study_begin' ) ) )
  {
    push( @error_messages, "Не указана или неверная дата начала проработки" );
  }

  # НЕ допускается пустое поле
  if ( ! date_is_correct( $query->param( 'study_end' ) ) )
  {
    push( @error_messages, "Не указана или неверная дата окончания " .
          "проработки" );
  }

  # DEBUG
  # print_params( $query );
  
  # выведем сообщения об ошибках, если они есть
  if ( scalar @error_messages != 0 )
  {
    print_user_err_msgs( \@error_messages );
    $no_errors = 0;
  }

  return ( $no_errors );
}

# -----------------------------------------------------------------------------
# формирование верхней части окна, содержащей заголовок модуля и список
# "команд" для пользователя
# -----------------------------------------------------------------------------
sub create_top
{
  # используем типовую процедуру
  create_top_block( query => $query,
                    module => $module,
                    module_title => 'Библиографическое описание (запись)',
                    user => $user,
                    full_top => 'no'
                  );
  return ( 0 );
}

# -----------------------------------------------------------------------------
# удаление записи
# -----------------------------------------------------------------------------
sub do_del
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $study_begin;        # дата начала проработки
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $study_begin = $query->param( 'study_begin' );
  $target_win  = $query->param( 'target_win' );

  # должна быть удалена только ОДНА запись с таким ключом
  $command = "DELETE FROM ${db_schema}studying " .
             "WHERE obj_id = '$obj_id' AND " .
                   "study_begin = '$study_begin'";

  # используем типовую процедуру
  del_rec( query => $query,
           target => $target_win,
           create_top => \&create_top,
           print_add_info => \&print_add_info,
           command => $command
         );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# вывод списка всех проработок содержания для данного издания
# -----------------------------------------------------------------------------
sub do_query
{
  # ПРИМЕЧАНИЕ. Эта процедура запускается при вызове данного модуля из
  #             модуля bib_recs.pl.

  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $bib_rec;            # биб. описание
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my( $result, $i );      # для работы с БД
  my @row;                # для хранения одной записи из выборки
  my $command;            # строка команды для работы с БД
  my $href;               # типовая часть HTML-ссылки

  # имеет ли пользователь право на просмотр записи в виде HTML-формы
  # и удаление записи
  my $can_view = 0;
  my $can_delete = 0;

  # признак того, что текущую строку таблицы нужно выводить другим цветом
  my $alt_row = 1;

  $obj_id     = $query->param( 'obj_id' );
  $bib_rec    = $query->param( 'bib_rec' );
  $target_win = $query->param( 'target_win' );

  # сформируем HTTP- и HTML-заголовки
  http_html_headers( query => $query, target => $target_win );

  open_top();       # начинаем оформление верхней части окна
  create_top();     # формируем верхнюю часть окна
  close_top();      # завершаем оформление верхней части окна

  open_bottom();    # начинаем оформление нижней части окна

  print_add_info(); # выведем дополнительную информацию на экран

  # заэкранируем символы, которые имеют специальное значение
  $bib_rec = CGI::escape( $bib_rec );

  # типовая часть HTML-ссылки
  $href = ( $query->param( 'exp_imp' ) ? "exp_imp=1&" : "" ) .
          "obj_id=$obj_id" .
          "&bib_rec=$bib_rec" .
          "&target_win=$target_win";

  # выбираем все записи о проработках содержания для данного объекта
  $command = "SELECT * " .
             "FROM ${db_schema}studying " .
             "WHERE obj_id = '$obj_id' " .
             "ORDER BY study_begin";

  $result = exec_select( $command );    # сделаем выборку из БД
  
  if ( $result )
  {
    $can_view = have_access( 'module' => $module, 'procedure' => 'VIEW', 
                             'user' => $user );
    $can_delete = have_access( 'module' => $module, 'procedure' => 'DELETE',
                               'user' => $user );

    # выведем информацию в виде таблицы
    print "<table class=\"list\">\n";
    print "<tr>\n";
    print "<th>Дата начала проработки</th>\n";
    print "<th>Дата окончания проработки</th>\n";
    print "<th>Описание вида проработки</th>\n";
    print "<th>&nbsp;</th>\n" if ( $can_view || $can_delete );
    print "</tr>\n";

    # выводим все записи из полученной выборки
    while ( @row = $result->fetchrow )
    {
      # если включен флаг альтернативной раскраски строк в таблице
      if ( $CFG{ ALT_ROWS_IS_ON } )
      {
        $alt_row = ! $alt_row;
        print ( ( $alt_row ) ? "<tr class=\"alt_row\">\n" : "<tr>\n" );
      }
      else
      {
        print "<tr>\n";
      }

      # выводим даты начала и окончания проработки, вид проработки
      print "<td class=\"center\">",
            date_for_display( $row[ $result->fnumber( 'study_begin' ) ] ),
            "</td>\n";
      print "<td class=\"center\">",
            date_for_display( $row[ $result->fnumber( 'study_end' ) ] ),
            "</td>\n";
      print "<td>$row[ $result->fnumber( 'study_descr' ) ]</td>\n";

      # просмотр данной записи о проработке содержания в виде формы
      print "<td>" if ( $can_view || $can_delete );
      print qq{<a class="command_link" href="$script_name/view?$href} .
            "&study_begin=" .
            $row[ $result->fnumber( 'study_begin' ) ] . "\"" .
            qq{ target="$target_win">Просмотр</a>} if $can_view;
      print "\n" if $can_view;

      # удаление данной записи о проработке содержания
      print qq{<a class="command_link" href="$script_name/delete?$href} .
            "&study_begin=" .
            $row[ $result->fnumber( 'study_begin' ) ] . "\"" .
            qq{ onclick="$confirm_delete"} .
            qq{ target="$target_win">Удаление</a>} if $can_delete;
      print "</td>\n" if ( $can_view || $can_delete );

      print "</tr>\n";
    }
    print "</table>\n";
  }

  # сформируем ссылку для добавления записи о проработке содержания
  # не разрешаем добавление записей при работе со схемой БД "экспорт/импорт"
  if ( ! $query->param( 'exp_imp' ) &&
       have_access( 'module' => $module, 'procedure' => 'ADD', 
                    'user' => $user ) )
  {
    print "<br />\n";
    print qq{<a class="command_link" href="$script_name/add?$href"} .
          qq{ target="$target_win">Добавить запись</a>};
  }

  close_bottom();   # завершаем оформление нижней части окна
  print $query->end_html();
  return ( 0 );
}

# -----------------------------------------------------------------------------
# просмотр одной записи в виде формы
# -----------------------------------------------------------------------------
sub do_view
{
  my $obj_id;             # уникальный идентификатор объекта биб. описания
  my $study_begin;        # дата начала проработки
  my $target_win;         # имя окна браузера для вывода HTML-страницы
  my $command;            # строка команды для работы с БД

  $obj_id      = $query->param( 'obj_id' );
  $study_begin = $query->param( 'study_begin' );
  $target_win  = $query->param( 'target_win' );

  # выбираем одну запись для детальной характеристики
  $command = "SELECT * " .
             "FROM ${db_schema}studying " .
             "WHERE obj_id = '$obj_id' AND " .
                   "study_begin = '$study_begin'";

  # используем типовую процедуру
  view_rec( query => $query,
            create_top => \&create_top,
            make_form => \&make_form,
            action => $script_name,
            target => $target_win,
            submit_name => "submit_update",
            submit_value => "Записать изменения в базу данных",
            command => $command,
            error => "В базе данных имеется более одной записи с ключом: " .
                     "уникальный идентификатор объекта биб. описания - " .
                     $obj_id . ", " .
                     "дата начала проработки - $study_begin",
            can_update => have_access( 'module' => $module, 
                                       'procedure' => 'UPDATE', 
                                       'user' => $user )
        );

  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub make_command
{
  my $operation = shift; # какая команда - вставка или корректировка?

  my $command;           # строка-команда

  # используем типовую процедуру
  # параметры: в том числе
  # - ссылка на массив имен полей этой таблицы БД, требующих специальной
  #   обработки (f_special);
  # - ссылка на функцию для обработки таких полей (func_special)
  $command = make_cmd( query => $query,
                       operation => $operation,
                       table => "studying",
                       key => "obj_id, study_begin:study_begin_begin",
                       f_names => \@f_names,
                       f_special => [ 'study_begin', 'study_end' ],
                       func_special => \&non_std_fields,
                       user => $user
                     );

  return ( $command );
}

# -----------------------------------------------------------------------------
# создание формы для просмотра (корректировки) и добавления записей
# -----------------------------------------------------------------------------
sub make_form
{
  # параметры
  my $ref_row = shift;  # ссылка на массив, содержащий одну запись из выборки
  my $res = shift;      # результат выборки

  my $i;
  my %fields = ();  # хеш-массив с информацией о начальных значениях полей

  my( $day, $mon, $year );  # для формирования текущей даты

  # если передан параметр - ссылка на массив значений полей одной записи
  # из выборки, ...
  if ( $ref_row ne undef )
  {
    # ... то сформируем хеш-массив, в котором...
    for $i ( 0 .. $res->nfields - 1 )
    {
      # ... каждому полю соответствует значение ключа, равное имени этого поля
      # (имя поля определим по его номеру - см. Pg.pm)
      $fields{ $res->fname( $i ) } = $$ref_row[ $i ];
    }
  }
  else                      # режим ввода новой записи в БД
  {
    # возьмем значения ключевого поля из параметров скрипта
    $fields{ 'obj_id' } = $query->param( 'obj_id' );

    # сформируем текущую дату
    ( $day, $mon, $year ) = ( localtime() )[ 3..5 ];

    $fields{ 'study_begin' } =
      sprintf( "%04d-%02d-%02d", $year + 1900, $mon + 1, $day );

    # по умолчанию проработка завершается на следующий день
    # TODO Это очень грубое решение - надо переделать!
    $fields{ 'study_end' } =
      sprintf( "%04d-%02d-%02d", $year + 1900, $mon + 1, $day + 1);

    # значения по умолчанию
    
    # описание вида проработки
    $fields{ 'study_descr' }  = $CFG{ STUDYING_STUDY_DESCR };
    
    # описание результата проработки
    $fields{ 'study_result' } = $CFG{ STUDYING_STUDY_RESULT };
    
    # комментарий
    $fields{ 'comment' }      = $CFG{ STUDYING_COMMENT };
  }
  # -------------------------------------------------------------------

  # включим в текст HTML-документа обращение к исходному файлу с кодом функции
  # insert_str_into_field(), которая позволяет вставить фрагмент текста
  # в поле HTML-формы
  print <<END_OF_SCRIPT;
  <script language="JavaScript" type="text/javascript"
          src="/$CFG{ HOME_WEB_JS_DIR }/insert_text.js"></script>
END_OF_SCRIPT
  ;

  # поскольку значение поля "Уникальный идентификатор объекта
  # библиографического описания" не подлежит редактированию, не будем позволять
  # его редактировать, но совсем убрать его нельзя, т.к. оно используются при
  # выполнении операции UPDATE в качестве ключа, а также при выполнении
  # операции INSERT (значение этого поля сохраняется таким же, каким оно было
  # при предыдущем запуске скрипта, поэтому можно не присваивать ему значения)
  print $query->hidden( -name => 'obj_id' ), "\n";

  # создадим скрытое поле для "Даты начала проработки"
  # ПРИМЕЧАНИЕ. Идентификатор скрытого поля имеет постфикс 'begin' для того,
  #             чтобы можно было обновлять значение этого поля, поскольку оно
  #             является ключевым (см. функцию make_cmd в пакете E_ARCH.pm).
  #             Т.е. нам нужно и исходное значение этого поля, и его новое
  #             значение (которое, возможно, было изменено пользователем).
  # ПРИМЕЧАНИЕ. Это поле в таблице БД уже имеет постфикс 'begin',
  #             поэтому постфикс для скрытого поля удваивается.
  print $query->hidden( -name => 'study_begin_begin',
                        -default => $fields{ 'study_begin' }
                      ), "\n";

  # это скрытое поле призвано ретранслировать биб. описание следующему
  # вызову скрипта, когда будет работать функция add_rec() или update_rec()
  print $query->hidden( -name => 'bib_rec' ), "\n";

  # ретранслируем параметр exp_imp, если он был, для следующего вызова
  # этого скрипта (он нужен для определения схемы БД: exp_imp или public)
  print $query->hidden( -name => 'exp_imp' ), "\n";

  print_add_info();        # выведем дополнительную информацию на экран

  print "<table class=\"form\">\n";

  print "<tr>\n";
  print "<th>Дата начала проработки</th>\n";
  # используем параметр override, т.к. это поле уже существовало на момент
  # создания этой формы и имело определенное значение, которое нужно заменить
  # (параметр study_begin передается скрипту в процедуре do_query() при
  # вызове скрипта с PATH_INFO равным 'view')
  # ПРИМЕЧАНИЕ. Если убрать параметр override, то дата в окне формы в режиме
  #             просмотра записи из БД будет представлена в формате
  #             гггг-мм-дд. Получится, что дата -- правильная, но формат
  #             ее представления -- не тот, который требуется.
  print "<td>",
        $query->textfield( -name => 'study_begin',
                           -default =>
                              date_for_display( $fields{ 'study_begin' } ),
                           -override => 1,
                           -size => $CFG{ STUDYING_STUDY_BEGIN_SIZE },
                           -maxlength => $CFG{ STUDYING_STUDY_BEGIN_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Дата окончания проработки</th>\n";
  print "<td>",
        $query->textfield( -name => 'study_end',
                           -default =>
                              date_for_display( $fields{ 'study_end' } ),
                           -size => $CFG{ STUDYING_STUDY_END_SIZE },
                           -maxlength => $CFG{ STUDYING_STUDY_END_MAXLENGTH }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Описание вида проработки</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].study_descr",
                   "STUDYING_STUDY_DESCR" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'study_descr',
                          -default => $fields{ 'study_descr' },
                          -rows => $CFG{ STUDYING_STUDY_DESCR_ROWS },
                          -columns => $CFG{ STUDYING_STUDY_DESCR_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<th>Описание результата проработки</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].study_result",
                   "STUDYING_STUDY_RESULT" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'study_result',
                          -default => $fields{ 'study_result' },
                          -rows => $CFG{ STUDYING_STUDY_RESULT_ROWS },
                          -columns => $CFG{ STUDYING_STUDY_RESULT_COLS }
                         ),
        "</td>\n";
  print "</tr>\n";

  # --------------------------------------------------------------------
  print "<tr>\n";
  print "<th>Комментарий</th>\n";
  print "<td>";

  # создадим ссылки (кнопки) для быстрой вставки фрагментов текста
  # в поле на экране
  create_hot_help( "document.forms[ 0 ].comment",
                   "STUDYING_COMMENT" ) if $CFG{ HOT_HELP_IS_ON };

  print $query->textarea( -name => 'comment',
                          -default => $fields{ 'comment' },
                          -rows => $CFG{ STUDYING_COMMENT_ROWS },
                          -columns => $CFG{ STUDYING_COMMENT_COLS }
                        ),
        "</td>\n";
  print "</tr>\n";
  # --------------------------------------------------------------------

  print "</table>\n";
  return ( 0 );
}

# -----------------------------------------------------------------------------
# формирование значений полей, требующих нестандартной обработки при
# формировании строки-команды для выполнения добавления и корректировки записи
# -----------------------------------------------------------------------------
sub non_std_fields
{
  # параметры:
  my $fname = shift;  # имя поля
                      # второй параметр - вид операции (добавление записи
                      # или корректировка). Этот параметр нам не нужен 
  my $value;
  
  # Дата начала проработки и Дата окончания проработки
  # ПРИМЕЧАНИЕ. Эта проверка излишняя, т.к. данная функция предназначена
  #             для обработки только полей типа date
  if ( $fname eq 'study_begin' || $fname eq 'study_end' )
  {
    $value = date_for_db( $query->param( $fname ) );
  }

  return ( $value );  
}

# -----------------------------------------------------------------------------
# вывод дополнительной информации на экран при выполнении ряда режимов работы
# -----------------------------------------------------------------------------
sub print_add_info
{
  # выведем биб. описание и заголовок модуля
  print_bib_record( $query, "Проработка содержания объекта " .
                    "библиографического описания" );
  return ( 0 );
}
